# -*- coding: utf-8 -*-
from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta, decode_string
from sqlalchemy.sql.expression import and_
import pprint
import json
from httplib2 import Http
from urllib import urlencode
import unidecode


@DBSwitch('cp')
class CpBotAdapter:
    '''
    Esta clase se encarga de interactuar con la base de datos MySQL de CallProcessor.
    En un futuro se adaptara para trabajar con clientes de trackphone, que tienen su base de datos en Postgres.
    '''

    def __init__(self):
        pass

    def obtener_direcciones(self, telefono_usuario, limite):
        '''
        Esta funcion se encarga de obtener mediante el telefono del usuario, las direcciones mas utilizadas en caso
        de tenerlas del callprocessor.
        las devuevlve ordenadas por ranking, y devuelve la cantidad determinada por el parametro limite

        :param telefono: El telefono del usuario que mando el mensaje
        :return: (respuesta) Una respuesta con estado y datos de direcciones en caso de tenerlo
        '''

        # Abro una sesion con la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla clientes_direcciones
        clientes_direcciones = self.cp.tabla('clientes_direcciones')

        # Hago la query
        q_obj = session.query(
            clientes_direcciones
        )

        # Filtro por el telefono del usuario
        registros = q_obj.filter(telefono_usuario == clientes_direcciones.Telefono,
                                 clientes_direcciones.Latitud != 0,
                                 clientes_direcciones.Longitud != 0)

        # Ordeno segun el ranking
        registros = registros.order_by(clientes_direcciones.Ranking.desc()).limit(limite)

        # Cierro la sesion
        session.close()

        registros = registros.all()

        # Verifico que no esta vacia la query
        if len(registros) == 0:
            # Devuelvo la respuesta con False y un mensaje
            return respuesta(False, u'No se obtuvieron direcciones para el telefono dado')

        # Inicializo el diccionario que devolvere
        dic = {}

        # Inicializo un contador
        count = 0

        # Itero a traves de los registros
        for registro in registros:
            count += 1  # Aumento el contador
            dic[count] = registro  # Guardo el registro en la key con el contador

        return respuesta(True, u'Se encontraron direcciones para el telefono', dato=dic)

    def cargar_servicio(self, empresa, hash, telefono, nombre, direccion, telefonista, latitud, longitud, host,
                        filtros=None):
        '''
        Esta funcion se encarga de cargar un servicio en CallProcessor utilizando una request POST.

        :param empresa (str): El id de la empresa
        :param hash (str):
        :param telefono (str):
        :param nombre (str):
        :param direccion (str):
        :param telefonista (str):
        :param latitud (int):
        :param longitud (int):
        :return: Una respuesta http
        '''

        http = Http(disable_ssl_certificate_validation=True)  # El objeto Http para hacer la request

        headers = {'Content-type': 'application/x-www-form-urlencoded'}  # Los headers necesarios para la request

        funcion = '/grabar_servicio'  # La ruta en la que haremos la request

        url = host + funcion  # Creamos la url

        # Iniciamos un diccionario que enviaremos por http
        dic = {}
        dic['empresa'] = empresa
        dic['hash'] = hash  # Validar HASH, se debe armar un login
        servicio = {}
        servicio['Telefono'] = telefono
        servicio['Nombre'] = nombre
        servicio['Direccion'] = direccion
        servicio['Telefonista'] = telefonista
        servicio['Latitud'] = latitud
        servicio['Longitud'] = longitud
        servicio['Filtros'] = filtros
        dic['dic_servicios'] = servicio
        parametros = urlencode(dic)  # Codificamos el diccionario

        print("\n\nvoy a cargar el servicio:")
        pprint.pprint(dic)
        resp, content = http.request(
            url, "POST", headers=headers, body=parametros
        )

        rta = json.loads(content)
        print("respuesta de cargar_servicio:", rta)
        return rta

    def obtener_servicio(self, id_servicio):
        '''
        Esta funcion se encarga de obtener todos los datos de un servicio ya cargado, mediante el id_servicio

        :param id_servicio (int):
        :return: (dict) Una respuesta con Estado y de ser posible un dato.
        '''

        # Abro la sesion con la base de datos
        session = self.cp.binded_session()

        # Conecto con la tabla 'actual'
        actual = self.cp.tabla('actual')

        # Hago la query
        q_obj = session.query(
            actual
        )

        # Filtro por el id_servicio
        q_obj = q_obj.filter(actual.IdServicio == id_servicio)

        # Obtengo el registro
        registro = q_obj.first()

        # Cierro la sesion con la base de datos
        session.close()

        # Si no pude obtener el registro
        if registro is None:
            return respuesta(False, u'No se encontro el servicio correspondiente a: %s' % id_servicio)

        dic = registro.__dict__  # Convierto el registro en un diccionario

        # Devuelvo la respuesta junto con el diccionario
        return respuesta(True, u'Se obtuvieron los datos del servicio con exito', dato=dic)

    def obtener_estado_servicio(self, id_servicio):
        '''
        Esta funcion se encarga de obtener los datos de un servicio que ya esta cargado.

        :param id_servicio:
        :return:
        '''

        # Abro la sesion con la base de datos
        session = self.cp.binded_session()

        # Conecto con la tabla 'actual'
        actual = self.cp.tabla('actual')

        # Hago la query
        q_obj = session.query(
            actual
        )

        # Filtro por el id_servicio
        q_obj = q_obj.filter(id_servicio == actual.IdServicio)

        # Intento obtener el primer registro
        registro = q_obj.first()

        session.close()
        # Si no pude obtener el registro
        if registro is None:
            return respuesta(False, u'No se encontro el servicio con id: %s' % id_servicio)

        # Inicio el diccionario a devolver
        dic = {}
        dic['despachado'] = self.esta_servicio_despachado(registro.Prioridad)
        # Si el registro del DNIChofer no es None, lo guardamos, sino guardamos None
        dic['dni_chofer'] = registro.DNIChofer if registro.DNIChofer is not None else None
        # Si el registro de distancia no es None, lo guardamos, sino guardamos None
        dic['distancia'] = registro.Distancia if registro.Distancia is not None else None
        dic['informacion_servicio'] = registro.__dict__  # Guardamos todos los datos del servicio

        try:
            dic['informacion_servicio'].pop('_sa_instance_state')  # Borramos la clase de SQLAlchemy del diccionario
        except:
            pass

        # Devuelvo la respuesta con True y el diccionario como dato
        return respuesta(True, u'Se obtuvieron los datos del servicio con exito', dato=dic)

    def esta_servicio_despachado(self, prioridad):
        """
        entra:prioridad, None o numero
        el estado 0 es despachado, el estado 8 es asignado al movil y puede que se aleje o no, pero no importa
        EN EL CALLPROCESOR FIGURA EL 8 COMO EL MOVIL SE ALEJA O COMO SERVICIO ACEPTADO SEGUN SE_ALEJA_EL_MOVIL
        return:True o False
        """
        return prioridad == 0 or prioridad == 8

    def obtener_datos_chofer(self, dni_chofer):
        '''
        Esta funcion se encarga de obtener el nombre y apellido del chofer.

        :param dni_chofer (int): El dni del chofer
        :return: (dict) Una respuesta con Estado, un mensaje y de ser posible un dato.
        '''

        # Abro la sesion con la base de datos
        session = self.cp.binded_session()

        # Conecto con la tabla 'choferes'
        choferes = self.cp.tabla('choferes')

        # Hago la query solamente al apellido y el nombre
        q_obj = session.query(
            choferes.Nombre,
            choferes.Apellido
        )

        # Filtro por el id chofer
        q_obj = q_obj.filter(choferes.DNI == dni_chofer)

        # Intento obtener el primer registro
        registro = q_obj.first()

        session.close()
        # Si no pude obtener el chofer
        if registro is None:
            return respuesta(False, u'No se pudo obtener los datos del chofer: %s' % dni_chofer)

        # Convierto el registro en diccionario
        dic = {
            "nombre": unidecode.unidecode(registro.Nombre.decode('latin-1')),
            "apellido": unidecode.unidecode(registro.Apellido.decode('latin-1'))
        }

        # Devuelvo la respuesta con True y el registro como dato
        return respuesta(True, u'Se obtuvieron los datos del chofer', dato=dic)

    def cancelar_servicio(self, id_servicio, host):
        '''
        Esta funcion se encarga de cancelar un servicio utilizando una request POST al CallProcessorWeb

        :param id_servicio (int): La id del servicio a cancelar
        :return: (dict) Una respuesta con estado, un mensaje y de ser necesario un dato.
        '''

        from srvweb.utiles.utiles import numero_empresa

        print("numero_empresa:", numero_empresa)
        # Obtenemos todos los datos relacionados al servicio utilizando la id_servicio
        rta_obtener_servicio = self.obtener_servicio(id_servicio)

        if rta_obtener_servicio['Estado'] is False:  # Si no pude obtener los datos del servicio
            return rta_obtener_servicio  # Devuelvo la respuesta

        # Guardo los datos que necesito en un diccionario
        dic = {
            'fecha': rta_obtener_servicio['Dato']['FechaServicio'],
            'telefono': rta_obtener_servicio['Dato']['Telefono'],
            'hora': rta_obtener_servicio['Dato']['HoraServicio'],
            'empresa': numero_empresa,
            'usuario': 'whatsapp'
        }

        http = Http(disable_ssl_certificate_validation=True)  # El objeto Http para hacer la request

        headers = {'Content-type': 'application/x-www-form-urlencoded'}  # Los headers necesarios para la request

        funcion = "/anular_o_reactivar_servicio"

        url = "http://" + host + funcion

        parametros = urlencode(dic)

        resp, content = http.request(
            url, "POST", headers=headers, body=parametros
        )

        if resp['status'] == '200':  # Si la request fue exitosa

            rta = json.loads(content)  # Convierto la respuesta en diccionario

            if rta['Estado']:  # Si la respuesta vino con estado True
                return respuesta(True, u'Se cancelo el servicio %s con exito' % id_servicio, dato='')

            return respuesta(False, u'No se pudo cancelar el servicio %s' % id_servicio, dato='')  # Sino devuelvo False


        else:

            return respuesta(False, u'Error en cancelar servicio', '')

    def obtener_datos_cliente(self, telefono):
        """
        funcion que trae los datos del cliente segun el telefono
        entrada:telefono:string
        salida:dict con los datos de la bd de mysql cp2000
        """
        # Abro la sesion con la base de datos
        session = self.cp.binded_session()

        try:
            # Conecto con la tabla 'clientes'
            clientes = self.cp.tabla('clientes')

            # Hago la query solamente al apellido y el nombre
            q_obj = session.query(clientes)

            # Filtro por el id chofer
            q_obj = q_obj.filter(clientes.Telefono == telefono)

            # Intento obtener el primer registro
            registro = q_obj.first()

            # Si pude obtener el cliente
            if registro:
                # Convierto el registro en diccionario
                dic = registro.__dict__
                rta = respuesta(True, u'Se obtuvieron los datos del cliente', dato=dic)
            else:  # si no hay filtros
                rta = respuesta(False, u'No se pudo obtener los datos del cliente del telefono: %s' % telefono)
        except Exception as error:
            rta = respuesta(False, u'No se pudo obtener los datos del cliente del telefono: %s' % telefono, error)
        finally:
            session.close()

        return rta


@DBSwitch('gis2')
class CallePostgres:
    '''
    Esta clase se encarga de la obtencion de calles que se encuentra en la base de datos de PostGres 'gis2'
    '''

    def __init__(self):
        pass

    def buscar_calle(self, calle_sin_numero, id_ciudad):
        '''
        Esta funcion se encarga de buscar en la base de datos si existe una calle en una ciudad dada.

        :param calle_sin_numero (str): Una direccion sin el numero
        :param id_ciudad (int): La id de una ciudad
        :return: (dict) Una respuesta con Estado, un mensaje y de ser posible un dato.
        '''

        # Abro la sesion
        session = self.gis2.binded_session()

        # Conecto con la tabla 'calles_nombres'
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Hago la query
        q_obj = session.query(
            calles_nombres
        )

        # Filtro por el id_ciudad y el nombre de la calle
        q_obj = q_obj.filter(calles_nombres.id_ciudad == id_ciudad, calles_nombres.nombre == calle_sin_numero)

        # Guardo el registro
        registro = q_obj.first()

        # Cierro la sesion con la base de datos
        session.close()

        # Si no pude obtener ningun registro
        if registro is None:
            # Devolvemos una respuesta con False y un mensaje
            return respuesta(False, u'No se encontro la calle %s en gis2' % calle_sin_numero)

        # Convierto el registro obtenido en un diccionario
        dic = registro.__dict__

        # Devuelvo una respuesta con True y el diccionario como dato
        return respuesta(True, u'Se obtuvo la calle con exito', dato=dic)

    def obtener_nombre_ciudad(self, id_ciudad):
        '''
        Esta funcion se encarga de obtener el nombre de una ciudad mediante su id.

        :param id_ciudad (int): La id de una ciudad
        :return: (dict) Una respuesta con estado, un mensaje y de ser posible un dato.
        '''

        # Inicio la sesion con la base de datos
        session = self.gis2.binded_session()

        # Obtengo la tabla 'ciudades'
        ciudades = self.gis2.tabla('ciudades')

        # Hago la query
        q_obj = session.query(
            ciudades.nombre
        )

        # Filtro por el id_ciudad
        q_obj = q_obj.filter(ciudades.id == id_ciudad)

        # Obtengo el primer registro
        registro = q_obj.first()

        # Si no pude obtener el nombre de la ciudad
        if registro is None:
            # Devolvemos la respuesta con False y un mensaje
            return respuesta(False, u'No se pudo obtener el nombre de la ciudad: %s' % id_ciudad)

        # Devuelvo la respuesta con True y el nombre de la ciudad como dato
        return respuesta(True, u'Se obtuvo el nombre de la ciudad con exito', dato=registro.nombre)


def probar():
    cpadapt = CpBotAdapter()
    # callepg = CallePostgres()
    dicky = cpadapt.obtener_estado_servicio(671)
    pprint.pprint(dicky['Dato'])
    return
