# -*- coding: utf-8 -*-

from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta
from sqlalchemy.exc import IntegrityError

from srvweb.utiles.utiles import numero_empresa
@DBSwitch('cpbot')
class Tags:

    def __init__(self, datos=None):
        '''
        El init toma como parametro los datos de la request.
        '''
        self.datos = datos

    def registro_a_diccionario(self, registro):
        '''
        Esta funcion se encarga de transformar un objeto de SQLAlchemy a un diccionario.
        '''

        dic = registro.__dict__  # Transformamos el registro en diccionario

        dic.pop('_sa_instance_state')  # Borramos la entrada que contiene la instanciacion de la clase de SQLAlchemy

        return dic  # Devolvemos el diccionario

    def campos_a_actualizar(self, tipo_tag):
        '''
        Esta funcion se encarga de evaluar el tipo de tag y devolver los campos que se pueden modificar.
        '''

        if tipo_tag == 'sistema':  # Si el tipo de tag es sistema
            return ['texto']  # Solo puede modificarse el texto del mismo

        if tipo_tag == 'funcional':  # Si el tipo de tag es funcional
            return []  # No se puede modificar ningun campo

        if tipo_tag == 'usuario':  # Si el tipo de tag es uno creado por el usuario
            return ['id_tag', 'tag', 'texto']  # Puede modificar todos los campos

    def obtener(self):
        '''
        Esta funcion se encarga de obtener todos los tags de la base de datos, para luego convertirlos en un json.
        '''

        # Abrimos la session
        session = self.cpbot.binded_session()

        # Obtenemos la tabla tags
        tags = self.cpbot.tabla('tags')

        # Hacemos la query
        q_obj = session.query(tags)

        # Obtenemos todos los objetos
        registros = q_obj.all()

        session.close()
        if len(registros) == 0:  # Si no pudimos obtener los registros
            # Devuelvo una respuesta con False y un mensaje
            return respuesta(False, u'No se encontraron tags')

        lista = []

        for registro in registros:  # Iteramos a traves de los registros

            # Convertimos el registro en diccionario
            reg_dict = self.registro_a_diccionario(registro)

            # Insertamos en la lista el registro
            lista.append(reg_dict)

        # Devolvemos una respuesta con True, un mensaje y el diccionario como dato.
        return respuesta(True, u'Se obtuvieron los tags con exito', dato=lista)

    def actualizar(self):
        '''
        Esta funcion se encarga de modificar un tag especifico.
        '''

        if 'id_tag_viejo' not in self.datos:  # Si no tenemos el id_tag_viejo como dato
            return respuesta(False, u'Falta la key id_tag_viejo en el diccionario')

        if 'tipo' not in self.datos:  # Si no tenemos el tipo como dato
            return respuesta(False, u'Falta la key tipo en el diccionario')

        if 'id_tag' not in self.datos:  # Si no tenemos el id_tag como dato
            return respuesta(False, u'Falta la key id_tag en el diccionario')

        # Abrimos la sesion
        session = self.cpbot.binded_session()

        # Conectamos con la tabla tags
        tags = self.cpbot.tabla('tags')

        # Hacemos la query
        q_obj = session.query(tags).filter(tags.id_tag == self.datos['id_tag_viejo'])

        if q_obj.count() <= 0:  # Si no se encontro ningun tag con esa id

            # Cerramos la sesion
            session.close()

            return respuesta(False, u'No se encontro un tag con la id: %s' % self.datos['id_tag'])

        # Verificamos los campos que pueden ser modificados
        campos = self.campos_a_actualizar(self.datos['tipo'])

        if len(campos) == 0:  # Si no podemos modificar ningun campo para ese tipo de tag
            session.close()
            return respuesta(False, u'Los tags funcionales no pueden ser modificados')

        # Creamos el diccionario que utilizaremos para la update
        dic = {key: value for (key, value) in self.datos.items() if key in campos}

        rta_update = q_obj.update(dic)  # Hacemos la update y guardamos la respuesta

        # Si no se actualizó ningún registro
        if rta_update == 0:
            # Cierra la sesión con la base de datos
            session.close()

            # Devolvemos la respuesta con False y un mensaje
            return respuesta(False, u'Hubo un error al actualizar el tag')

        # Si se actualizaron registros
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Devolvemos la respuesta con True y un mensaje
        return respuesta(True, u'Se modifico el tag %s con exito' % self.datos['id_tag'])

    def borrar(self):
        '''
        Esta funcion se encarga de eliminar un tag de la base de datos.
        '''

        # Abro la sesion con la base de datos
        session = self.cpbot.binded_session()

        # Obtengo la tabla tags
        tags = self.cpbot.tabla('tags')

        # Hacemos la query
        q_obj = session.query(tags).filter(tags.id_tag == self.datos['id_tag'])

        # Si no se encontro el tag
        if q_obj.count() == 0:
            # Cerramos la sesion con la base de datos
            session.close()

            return respuesta(False, u'No se encontro el tag: %s' % self.datos['id_tag'])

        # Si el tipo de tag no es del usuario
        if self.datos['tipo'] != 'usuario':
            session.close()
            # Devolvemos una respuesta con False y un mensaje
            return respuesta(False, u'Solo los tags del usuario pueden ser borrados')

        # Borramos el registro de la base de datos
        q_obj.delete(synchronize_session=False)  # Borramos el tag de la base de datos

        # Guardamos los cambios a la tabla
        session.commit()

        # Cierro la conexion con la base de datos
        session.close()

        # Devolvemos la respuesta con True y un mensaje
        return respuesta(True, u'Se borro el tag %s con exito' % self.datos['id_tag'])

    def agregar(self):
        '''
        Esta funcion se encarga de agregar un tag nuevo a la base de datos.
        '''

        if self.datos['tipo'] != 'usuario':  # Si nos pasan otro tipo para agregar
            return respuesta(False, u'Error al agregar el tag, su tipo debe ser usuario')

        dic = {
            'id_tag': self.datos['id_tag'],
            'texto': self.datos['texto'],
            'tag': self.datos['tag'],
            'tipo': self.datos['tipo']
        }

        # Insertamos el registro en la base de datos
        rta = self.cpbot.insertar('tags', 'id_tag', **dic)

        if rta['Estado'] is False:  # Si fallamos al insertar en la base de dato

            # Si el error se debe a que ya existe el tag en la bd
            if rta['Respuesta'].startswith("(psycopg2.IntegrityError) duplicate key"):
                # Devolvemos la respuesta con False y un mensaje
                return respuesta(False, u'El tag %s ya existe y no pudo ser agregado' % self.datos['id_tag'])

            # Si la web nos mando algun campo con None
            if rta['Respuesta'].startswith("(psycopg2.IntegrityError) null value in column"):
                # Devolvemos la respuesta con False y un mensaje
                return respuesta(False, u'Algun campo era nulo y no se pudo insertar el tag %s' % self.datos['id_tag'])

            # Ante cualquier otro caso devolvemos False y un mensaje
            return respuesta(False, u'Error al ingresar el tag en la base de datos: %s' % rta['Respuesta'])

        # Si se inserto correctamente devolvemos la respuesta con True y un mensaje
        return respuesta(True, u'Se inserto con exito el tag %s' % self.datos['id_tag'])

