# -*- coding: utf-8 -*-

import cjson
from sqlalchemy.sql.expression import and_

from at.map import _
from at.map.calle import CallePostgres
from at.map.ciudad import CiudadPostgres
from at.map.provincia import ProvinciaPostgres
from at.map.mapas import Mapas
from at.map.area_generator import generar_area_version
from at.orm.decorators import DBSwitch
from at.utiles.conversiones import respuesta
from at.utiles.servidores import DateTimeHelper
from at.utiles.procesos import ejecutar_proceso
from at.utiles import validar


@DBSwitch('gis2')
class Actualizar():
    '''
        Clase relacionada con las actualizaciones.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()
        self.calle_postgres = CallePostgres()
        self.ciudad_postgres = CiudadPostgres()
        self.provincia_postgres = ProvinciaPostgres()
        self.mapas = Mapas()

    def obtener_version_actual(self):

        # Abro la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla versiones.
        tabla_versiones = self.gis2.tabla('versiones')

        # Determino los campos que quiero obtener.
        q_obj = session.query(
            tabla_versiones.id,
            tabla_versiones.fecha_hora
        )

        # Ordeno por fecha y hora de forma ascendente.
        q_obj = q_obj.order_by(tabla_versiones.id.desc())

        # Si no obtengo registros, retorno.
        if int(q_obj.count()) == 0:

            # Armo la respuesta.
            rta = respuesta(
                False, _(u'Error al obtener la versión actual.')
            )

            # Cierro sesión.
            session.close()

            # Retorno.
            return rta

        # Obtengo la versión actual.
        version_actual = q_obj.first()

        # Cierro sesión.
        session.close()

        # Paso a string la fecha.
        fecha_hora = version_actual.fecha_hora.strftime('%d/%m/%Y %H:%M:%S')

        # Armo la versión.
        dic_version = {
            'id': version_actual.id,
            'fecha_hora': fecha_hora
        }

        # Armo la respuesta.
        rta = respuesta(True, dic_version)

        # Retorno.
        return rta

    def eliminar_calle(self, operacion):

        '''
        Esta función eliminá una calle.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto original.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo el id_calle
        id_calle = objeto_original['id']

        # Trato de eliminar la calle.
        rta_borrar = self.calle_postgres.borrar(id_calle)

        rta_borrar['operacion'] = operacion

        return rta_borrar

    def modificar_calle(self, operacion):
        '''
        Esta función modifica una calle.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo los datos.
        id_calle = objeto_nuevo.get('id')
        id_ciudad = objeto_nuevo.get('id_ciudad')
        nombre = objeto_nuevo.get('nombre')

        # Llamo a la función grabar de la clase calle_postgres
        rta = self.calle_postgres.grabar(
            id_calle=id_calle, nombre=nombre, id_ciudad=id_ciudad
        )

        rta['operacion'] = operacion

        return rta

    def agregar_calle(self, operacion):
        '''
        Esta función crea una nueva calle.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo los datos.
        id_calle = objeto_nuevo.get('id')
        id_ciudad = objeto_nuevo.get('id_ciudad')
        nombre = objeto_nuevo.get('nombre')

        # Llamo a la función grabar de la clase calle_postgres
        rta = self.calle_postgres.grabar(
            id_calle=id_calle, nombre=nombre, id_ciudad=id_ciudad,
            agregar_con_id_especifico=True
        )

        rta['operacion'] = operacion

        return rta

    def eliminar_segmento(self, operacion):
        '''
        Esta función elimina un segmento.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo el id_segmento
        id_segmento = objeto_original['gid']

        # Trato de eliminar el segmento.
        rta_borrar = self.calle_postgres.borrar_segmento(id_segmento)

        rta_borrar['operacion'] = operacion

        return rta_borrar

    def modificar_segmento(self, operacion):

        '''
        Esta función modifica un segmento.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo los atributos.
        id_segmento = objeto_nuevo.get('gid')
        sentido = objeto_nuevo.get('sentido')
        desde_der = objeto_nuevo.get('desde_der')
        hasta_der = objeto_nuevo.get('hasta_der')
        desde_izq = objeto_nuevo.get('desde_izq')
        hasta_izq = objeto_nuevo.get('hasta_izq')
        id_tipo = objeto_nuevo.get('id_tipo')
        letra = objeto_nuevo.get('letra')
        id_calle_nombre = objeto_nuevo.get('id_calle_nombre')
        puntos = objeto_nuevo.get('puntos')

        # Grabo.
        rta_grabar_segmento = self.calle_postgres.grabar_segmento(
            id_segmento=id_segmento, sentido=sentido, desde_der=desde_der,
            hasta_der=hasta_der, desde_izq=desde_izq, hasta_izq=hasta_izq,
            id_tipo=id_tipo, letra=letra, id_calle_nombre=id_calle_nombre,
            puntos=puntos
        )

        rta = rta_grabar_segmento

        rta['operacion'] = operacion

        return rta

    def agregar_segmento(self, operacion):
        '''
        Esta función agrega un segmento
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        id_segmento = objeto_nuevo.get('gid')
        sentido = objeto_nuevo.get('sentido')
        desde_der = objeto_nuevo.get('desde_der')
        hasta_der = objeto_nuevo.get('hasta_der')
        desde_izq = objeto_nuevo.get('desde_izq')
        hasta_izq = objeto_nuevo.get('hasta_izq')
        id_tipo = objeto_nuevo.get('id_tipo')
        letra = objeto_nuevo.get('letra')
        id_calle_nombre = objeto_nuevo.get('id_calle_nombre')
        puntos = objeto_nuevo.get('puntos')

        # Grabo.
        rta_grabar_segmento = self.calle_postgres.grabar_segmento(
            id_segmento=id_segmento, sentido=sentido, desde_der=desde_der,
            hasta_der=hasta_der, desde_izq=desde_izq, hasta_izq=hasta_izq,
            id_tipo=id_tipo, letra=letra, id_calle_nombre=id_calle_nombre,
            puntos=puntos, agregar_con_id_especifico=True
        )

        rta = rta_grabar_segmento

        rta['operacion'] = operacion

        return rta

    def eliminar_ciudad(self, operacion):

        '''
        Esta función eliminá una ciudad.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto original.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo el id_ciudad
        id_ciudad = objeto_original['id']

        # Trato de eliminar la calle.
        rta_eliminar = self.ciudad_postgres.eliminar(id_ciudad)

        rta_eliminar['operacion'] = operacion

        return rta_eliminar

    def modificar_ciudad(self, operacion):
        '''
        Esta función modifica una ciudad.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo los datos.
        id_ciudad = objeto_nuevo.get('id')
        id_provincia = objeto_nuevo.get('id_provincia')
        nombre = objeto_nuevo.get('nombre')
        id_tipo = objeto_nuevo.get('id_tipo')

        # Llamo a la función grabar de la clase calle_postgres
        rta = self.ciudad_postgres.grabar(
            id_provincia, nombre, id_ciudad=id_ciudad, id_tipo=id_tipo
        )

        rta['operacion'] = operacion

        return rta

    def agregar_ciudad(self, operacion):
        '''
        Esta función agrega una ciudad.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        id_ciudad = objeto_nuevo.get('id')
        id_provincia = objeto_nuevo.get('id_provincia')
        nombre = objeto_nuevo.get('nombre')
        id_tipo = objeto_nuevo.get('id_tipo')

        # Grabo.
        rta_grabar_ciudad = self.ciudad_postgres.grabar(
            id_provincia, nombre, id_ciudad=id_ciudad, id_tipo=id_tipo
        )

        rta = rta_grabar_ciudad

        rta['operacion'] = operacion

        return rta

    def eliminar_provincia(self, operacion):

        '''
        Esta función eliminá una ciudad.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto original.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo el id_provincia
        id_provincia = objeto_original['id']

        # Trato de eliminar la provincia.
        rta_eliminar = self.provincia_postgres.eliminar(id_provincia)

        rta_eliminar['operacion'] = operacion

        return rta_eliminar

    def modificar_provincia(self, operacion):
        '''
        Esta función modifica una provincia.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        id_pais, nombre, id_provincia = None


        # Obtengo los datos.
        id_pais = objeto_nuevo.get('id_pais')
        nombre = objeto_nuevo.get('nombre')
        id_provincia = objeto_nuevo.get('id')

        # Llamo a la función grabar de la clase calle_postgres
        rta = self.provincia_postgres.grabar(
            id_pais, nombre, id_provincia=id_provincia
        )

        rta['operacion'] = operacion

        return rta

    def agregar_provincia(self, operacion):
        '''
        Esta función agrega una provincia.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        id_pais = objeto_nuevo.get('id_pais')
        nombre = objeto_nuevo.get('nombre')
        id_provincia = objeto_nuevo.get('id')

        # Grabo.
        rta_grabar_ciudad = self.provincia_postgres.grabar(
            id_pais, nombre, id_provincia=id_provincia
        )

        rta = rta_grabar_ciudad

        rta['operacion'] = operacion

        return rta

    def deshacer_agregar_calle(self, operacion):

        '''
        Esta función eliminá una calle que se agregó.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo el id_calle
        id_calle = objeto_nuevo['id']

        # Trato de eliminar la calle.
        rta_borrar = self.calle_postgres.borrar(id_calle)

        rta_borrar['operacion'] = operacion

        return rta_borrar

    def deshacer_modificar_calle(self, operacion):
        '''
        Esta función vuelve al estado original una calle que fué modificada.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo los datos.
        id = objeto_original.get('id')
        id_ciudad = objeto_original.get('id_ciudad')
        nombre = objeto_original.get('nombre')

        # Llamo a la función grabar de la clase calle_postgres
        rta = self.calle_postgres.grabar(
            id_calle=id, nombre=nombre, id_ciudad=id_ciudad
        )

        rta['operacion'] = operacion

        return rta

    def deshacer_eliminar_calle(self, operacion):
        '''
        Esta función vuelve a agregar una calle que fué eliminada.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        id_calle = objeto_original.get('id')
        id_ciudad = objeto_original.get('id_ciudad')
        nombre = objeto_original.get('nombre')
        segmentos = objeto_original.get('segmentos')

        # Llamo a la función grabar de la clase calle_postgres
        rta_grabar = self.calle_postgres.grabar(
            id_calle=id_calle,
            nombre=nombre,
            id_ciudad=id_ciudad,
            agregar_con_id_especifico=True
        )

        # Verifico si se grabó.
        if rta_grabar['Estado'] is not True:
            return rta_grabar

        # Verifico si tiene segmentos.
        if segmentos is None:
            return rta_grabar

        # Guardo la respuesta por si tiene una lista vacia.
        rta = rta_grabar

        # Recorro los segmentos.
        for segmento in segmentos:

            # Obtengo los atributos.
            id_segmento = segmento.get('Id')
            sentido = segmento.get('Sentido')
            desde_der = segmento.get('DesdeDer')
            hasta_der = segmento.get('HastaDer')
            desde_izq = segmento.get('DesdeIzq')
            hasta_izq = segmento.get('HastaIzq')
            id_tipo = segmento.get('IdTipo')
            letra = segmento.get('Letra')
            id_calle_nombre = id_calle
            puntos = segmento.get('Puntos')

            # Grabo.
            rta_grabar_segmento = self.calle_postgres.grabar_segmento(
                id_segmento=id_segmento, sentido=sentido, desde_der=desde_der,
                hasta_der=hasta_der, desde_izq=desde_izq, hasta_izq=hasta_izq,
                id_tipo=id_tipo, letra=letra, id_calle_nombre=id_calle_nombre,
                puntos=puntos, agregar_con_id_especifico=True
            )

            # Guardo la respuesta.
            rta = rta_grabar_segmento

        rta['operacion'] = operacion

        # Retorno la respuesta.
        return rta

    def deshacer_agregar_segmento(self, operacion):
        '''
        Esta función elimina un segmento que fué creado previamente.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo el id_segmento
        id_segmento = objeto_nuevo['gid']

        # Trato de eliminar el segmento.
        rta_borrar = self.calle_postgres.borrar_segmento(id_segmento)

        rta_borrar['operacion'] = operacion

        return rta_borrar

    def deshacer_modificar_segmento(self, operacion):

        '''
        Esta función vuelve al estado original un segmento que fué modificado.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo los atributos.
        id_segmento = objeto_original.get('gid')
        sentido = objeto_original.get('sentido')
        desde_der = objeto_original.get('desde_der')
        hasta_der = objeto_original.get('hasta_der')
        desde_izq = objeto_original.get('desde_izq')
        hasta_izq = objeto_original.get('hasta_izq')
        id_tipo = objeto_original.get('id_tipo')
        letra = objeto_original.get('letra')
        id_calle_nombre = objeto_original.get('id_calle_nombre')
        puntos = objeto_original.get('puntos')

        # Grabo.
        rta_grabar_segmento = self.calle_postgres.grabar_segmento(
            id_segmento=id_segmento, sentido=sentido, desde_der=desde_der,
            hasta_der=hasta_der, desde_izq=desde_izq, hasta_izq=hasta_izq,
            id_tipo=id_tipo, letra=letra, id_calle_nombre=id_calle_nombre,
            puntos=puntos
        )

        rta = rta_grabar_segmento

        rta['operacion'] = operacion

        return rta

    def deshacer_eliminar_segmento(self, operacion):
        '''
        Esta función vuelve a crear un segmento que fué creado.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        id_segmento = objeto_original.get('gid')
        sentido = objeto_original.get('sentido')
        desde_der = objeto_original.get('desde_der')
        hasta_der = objeto_original.get('hasta_der')
        desde_izq = objeto_original.get('desde_izq')
        hasta_izq = objeto_original.get('hasta_izq')
        id_tipo = objeto_original.get('id_tipo')
        letra = objeto_original.get('letra')
        id_calle_nombre = objeto_original.get('id_calle_nombre')
        puntos = objeto_original.get('puntos')

        # Grabo.
        rta_grabar_segmento = self.calle_postgres.grabar_segmento(
            id_segmento=id_segmento, sentido=sentido, desde_der=desde_der,
            hasta_der=hasta_der, desde_izq=desde_izq, hasta_izq=hasta_izq,
            id_tipo=id_tipo, letra=letra, id_calle_nombre=id_calle_nombre,
            puntos=puntos, agregar_con_id_especifico=True
        )

        rta = rta_grabar_segmento

        rta['operacion'] = operacion

        return rta

    def deshacer_agregar_ciudad(self, operacion):

        '''
        Esta función eliminá una ciudad que se agregó.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo el id_ciudad
        id_ciudad = objeto_nuevo['id']

        # Trato de eliminar la ciudad.
        rta_eliminar = self.ciudad_postgres.eliminar(id_ciudad)

        rta_eliminar['operacion'] = operacion

        return rta_eliminar

    def deshacer_modificar_ciudad(self, operacion):
        '''
        Esta función vuelve al estado original una ciudad que fué modificada.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo los datos.
        id_ciudad = objeto_original.get('id')
        id_provincia = objeto_original.get('id_provincia')
        nombre = objeto_original.get('nombre')
        id_tipo = objeto_original.get('id_tipo')

        # Llamo a la función grabar de la clase calle_postgres
        rta = self.ciudad_postgres.grabar(
            id_provincia, nombre, id_ciudad=id_ciudad, id_tipo=id_tipo
        )

        rta['operacion'] = operacion

        return rta

    def deshacer_eliminar_ciudad(self, operacion):
        '''
        Esta función vuelve a crear una ciudad que fué eliminada.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo los datos.
        id_ciudad = objeto_original.get('id')
        id_provincia = objeto_original.get('id_provincia')
        nombre = objeto_original.get('nombre')
        id_tipo = objeto_original.get('id_tipo')

        # Grabo.
        # Llamo a la función grabar de la clase ciudad_postgres
        rta_grabar_ciudad = self.ciudad_postgres.grabar(
            id_provincia, nombre, id_ciudad=id_ciudad, id_tipo=id_tipo
        )

        rta = rta_grabar_ciudad

        rta['operacion'] = operacion

        return rta

    def deshacer_agregar_provincia(self, operacion):

        '''
        Esta función eliminá una provincia que se agregó.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_nuevo = cjson.decode(operacion['objeto_nuevo'])

        # Obtengo el id_provincia
        id_provincia = objeto_nuevo['id']

        # Trato de eliminar la provincia.
        rta_eliminar = self.provincia_postgres.eliminar(id_provincia)

        rta_eliminar['operacion'] = operacion

        return rta_eliminar

    def deshacer_modificar_provincia(self, operacion):
        '''
        Esta función vuelve al estado original una provincia que fué modificada.
        :param operacion:
        :return:
        '''

        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo los datos.
        id_pais = objeto_original.get('id_pais')
        nombre = objeto_original.get('nombre')
        id_provincia = objeto_original.get('id')

        # Llamo a la función grabar de la clase calle_postgres
        rta = self.provincia_postgres.grabar(
            id_pais, nombre, id_provincia=id_provincia
        )

        rta['operacion'] = operacion

        return rta

    def deshacer_eliminar_provincia(self, operacion):
        '''
        Esta función vuelve a crear una provincia que fué eliminada.
        :param operacion:
        :return:
        '''
        # Obtengo el objeto nuevo.
        objeto_original = cjson.decode(operacion['objeto_original'])

        # Obtengo los datos.
        id_pais = objeto_original.get('id_pais')
        nombre = objeto_original.get('nombre')
        id_provincia = objeto_original.get('id')

        # Grabo.
        # Llamo a la función grabar de la clase provincia_postgres
        rta = self.provincia_postgres.grabar(
            id_pais, nombre, id_provincia=id_provincia
        )

        rta['operacion'] = operacion

        return rta

    def actualizar(self, version, operaciones):

        # Valido el obejto
        v_version = validar.dic_con_datos(version)
        if v_version['Estado'] is not True:
            return v_version

        # Valido la lista de operaciones.
        if len(operaciones) < 1:
            return respuesta(False, _(u'Error al procesar las operaciones.'))

        # Defino la respuesta por defecto.
        rta_operar = {}

        # Defino una lista vacio donde guardaré los errores.
        errores = []

        # Recorro las operaciones.
        for operacion in operaciones:

            # Obtengo el id_accion.
            id_accion = operacion['id_accion']

            # 101 es la accion de agregar segmento.
            if id_accion == 101:
                rta_operar = self.agregar_segmento(operacion)

            # 102 es la accion de modificar segmento.
            if id_accion == 102:
                rta_operar = self.modificar_segmento(operacion)

            # 103 es la accion de eliminar segmento.
            if id_accion == 103:
                rta_operar = self.eliminar_segmento(operacion)

            # 111 es la accion de deshacer agregar segmento.
            if id_accion == 111:
                rta_operar = self.deshacer_agregar_segmento(operacion)

            # 112 es la accion de deshacer modificar segmento.
            if id_accion == 112:
                rta_operar = self.deshacer_modificar_segmento(operacion)

            # 113 es la accion de deshacer eliminar segmento.
            if id_accion == 113:
                rta_operar = self.deshacer_eliminar_segmento(operacion)

            # 201 es la accion de agregar una calle.
            if id_accion == 201:
                rta_operar = self.agregar_calle(operacion)

            # 202 es la accion de modificar una calle.
            if id_accion == 202:
                rta_operar = self.modificar_calle(operacion)

            # 203 es la accion de eliminar una calle.
            if id_accion == 203:
                rta_operar = self.eliminar_calle(operacion)

            # 211 es la accion de deshacer agregar calle.
            if id_accion == 211:
                rta_operar = self.deshacer_agregar_calle(operacion)

            # 212 es la accion de deshacer modificar calle.
            if id_accion == 212:
                rta_operar = self.deshacer_modificar_calle(operacion)

            # 213 es la accion de deshacer eliminar calle.
            if id_accion == 213:
                rta_operar = self.deshacer_eliminar_calle(operacion)

            # 301 es la accion de agregar una ciudad.
            if id_accion == 301:
                rta_operar = self.agregar_ciudad(operacion)

            # 302 es la accion de modificar una ciudad.
            if id_accion == 302:
                rta_operar = self.modificar_ciudad(operacion)

            # 303 es la accion de eliminar una ciudad.
            if id_accion == 303:
                rta_operar = self.eliminar_ciudad(operacion)

            # 311 es la accion de deshacer agregar ciudad.
            if id_accion == 311:
                rta_operar = self.deshacer_agregar_ciudad(operacion)

            # 312 es la accion de deshacer modificar ciudad.
            if id_accion == 312:
                rta_operar = self.deshacer_modificar_ciudad(operacion)

            # 313 es la accion de deshacer eliminar ciudad.
            if id_accion == 313:
                rta_operar = self.deshacer_eliminar_ciudad(operacion)

            # 401 es la accion de agregar una provincia.
            if id_accion == 401:
                rta_operar = self.agregar_provincia(operacion)

            # 402 es la accion de modificar una provincia.
            if id_accion == 402:
                rta_operar = self.modificar_provincia(operacion)

            # 403 es la accion de eliminar una provincia.
            if id_accion == 403:
                rta_operar = self.eliminar_provincia(operacion)

            # 411 es la accion de deshacer agregar provincia.
            if id_accion == 411:
                rta_operar = self.deshacer_agregar_provincia(operacion)

            # 412 es la accion de deshacer modificar provincia.
            if id_accion == 412:
                rta_operar = self.deshacer_modificar_provincia(operacion)

            # 413 es la accion de deshacer eliminar provincia.
            if id_accion == 413:
                rta_operar = self.deshacer_eliminar_provincia(operacion)

        if rta_operar['Estado'] is not True:
            errores.append(rta_operar)

        # Obtengo la fecha_hora del servidor.
        fecha_hora_actual = self.dth.get_now()

        # Declaro el dic con los datos.
        datos_version = {
            'id': version['id'],
            'fecha_hora': fecha_hora_actual
        }

        # agrego la operaciones.
        agregar_version = self.gis2.insertar(
            'versiones', **datos_version
        )

        # Verifico el resultado.
        if agregar_version['Estado'] is not True:
            return agregar_version

        # Verifico si se actualizó correctamente.
        if len(errores) > 0:

            informe = {
                'version': version,
                'errores': errores
            }

            rta = respuesta(
                False, _(u'Se actualizó pero con errores.'), informe
            )
        else:
            rta = respuesta(
                True, _(u'Se agregó correctamente la operación.'), version
            )

        # Ejecuto para que se generen las imágenes de la versión de forma líneal
        generar_area_version(operaciones, False)

        # Retorno.
        return rta

