# -*- coding: utf-8 -*-

import math
from math import sqrt
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import func
from sqlalchemy.orm import aliased

from at.map import _
from at.map.exc import CalleNoEncontrada
from at.map.ciudad import CiudadPostgres
from at.map.utiles.validaciones_especificas import ValidacionesEspecificas
from at.map.utiles.funciones_geo import FuncionesGeo
from at.utiles.conversiones import respuesta
from at.map.utiles.punto import Punto
from at.map.utiles.topologia import Topologia
from at.orm.decorators import DBSwitch


# En longitud, con latitud = 0
ANCHO_CALLE = 0.00015


@DBSwitch('gis')
class Calle(ValidacionesEspecificas):
    '''
       Esta clase contiene todos lo métodos relacionados con las calles.
    '''

    def obtener_calle(self, id_ciudad, id_calle):
        '''
            Dado el ID de una ciudad y el ID de una calle obtengo su nombre.

            Args:
                id_ciudad (int)
                id_calle (int)

            Returns:
                calle (str)

            Raises:
                'No se ha encontrado la calle'

            Functions:
                validar_id_ciudad(id_ciudad)
                validar_id_calle(id_calle)
                binded_session()
                tabla('calles')
        '''

        if not self.validar_id_ciudad(id_ciudad):
            return self.validar_id_ciudad(id_ciudad)

        if not self.validar_id_calle(id_calle):
            return self.validar_id_calle(id_calle)

        session = self.gis.binded_session()
        tabla_calles = self.gis.tabla('calles')

        q_obj = session.query(tabla_calles.Nombre)
        q_obj = q_obj.filter(and_(tabla_calles.idCiudad == id_ciudad,
                                  tabla_calles.idCalle == id_calle))

        calle = q_obj.first()

        if calle is None:
            session.close()
            raise (CalleNoEncontrada, _('No se ha encontrado la calle.'))

        session.close()
        return calle.Nombre

    def listar_calles(self, id_ciudad):
        '''
            Dado el ID de una ciudad, esta funcion devuelve un listado con todas
            las calles de la misma. Estos datos corresponden al id y a l nombre
            de la calle.

            Args:
                id_ciudad (int)

            Returns:
                calles (list [  list [(int),(str)] ])


            Functions:
                validar_id_ciudad(id_ciudad)
        '''

        # Validamos el id_ciudad
        if not self.validar_id_ciudad(id_ciudad):
            return self.validar_id_ciudad(id_ciudad)

        # Iniciamos la sesion con la BD gis
        session = self.gis.binded_session()
        # Nos conectamos a la tabla "calles"
        tabla_calles = self.gis.tabla('calles')

        # Hacemos la query pidiendo todas las calles de esa ciudad
        q_obj = session.query(tabla_calles.idCalle, tabla_calles.Nombre)
        q_obj = q_obj.filter(tabla_calles.idCiudad == id_ciudad)
        q_obj = q_obj.order_by(tabla_calles.Nombre.asc())

        calles_tmp = q_obj.all()

        # Generamos la lista de elementos que devolveremos
        calles = []
        for c in calles_tmp:
            c_tmp = []
            c_tmp.append(int(c.idCalle))
            c_tmp.append(c.Nombre)

            calles.append(c_tmp)

        # Cerramos la sesion
        session.close()

        return calles

    def obtener_tipo_de_numeracion(self, id_tipo_numeracion):
        '''
            Dado el ID de un tipo de numeracion obtengo su nombre.

            Args:
                id_tipo_numeracion (int)

            Returns:
                tipo_numeracion (str)

        '''
        # Validamos el id con el validador de id_ciudad
        if not self.validar_id_ciudad(id_tipo_numeracion):
            return self.validar_id_ciudad(id_tipo_numeracion)

        # Iniciamos la sesion con la BD gis
        session = self.gis.binded_session()
        # Nos conectamos a la tabla "tipodenumeracion"
        tabla_tipodenumeracion = self.gis.tabla('tipodenumeracion')

        # Hacemos la query pidiendo todas los tipos de numeracion
        q_obj = session.query(tabla_tipodenumeracion)
        q_obj = q_obj.filter(
            tabla_tipodenumeracion.idTipoNumeracion == id_tipo_numeracion)

        tipo_numeracion = q_obj.first()

        if tipo_numeracion is None:
            session.close()
            raise (Exception, _(u'No se ha encontrado el tipo de numeración.'))

        session.close()
        return tipo_numeracion.TipoNumeracion

    def listar_tipos_de_numeracion(self):
        '''
            Funcion devuelve un listado con todos los tipos de numeracion.
            Estos datos corresponden al id y al nombre.

            Returns:
                tipodenumeracion (list [  list [(int),(str)] ])

        '''

        # Iniciamos la sesion con la BD gis
        session = self.gis.binded_session()
        # Nos conectamos a la tabla "tipodenumeracion"
        tabla_tipodenumeracion = self.gis.tabla('tipodenumeracion')

        # Hacemos la query pidiendo todas los tipos de numeracion
        q_obj = session.query(tabla_tipodenumeracion)
        q_obj = q_obj.order_by(tabla_tipodenumeracion.idTipoNumeracion.asc())

        tipodenumeracion_tmp = q_obj.all()

        # Generamos la lista de elementos que devolveremos
        tipodenumeracion = []
        for t in tipodenumeracion_tmp:
            t_tmp = []
            t_tmp.append(int(t.idTipoNumeracion))
            t_tmp.append(t.TipoNumeracion)

            tipodenumeracion.append(t_tmp)

        # Cerramos la sesion
        session.close()

        return tipodenumeracion

    def sugerir_calle(self, id_ciudad, parte_calle):
        '''
            Dado un id de una ciudad y parte del nombre de una calle,
            sugiere posibles nombres de calles que pertencen a esa ciudad.

            Args:
                id_ciudad (int)
                parte_calle (str)

            Returns:
                lista_calles[i]['id_calle']
                lista_calles[i]['nombre_calle']
                None

            Functions:
                validar_string(parte_calle)
                validar_id_ciudad(id_ciudad)
                binded_session()
                tabla('calles')
        '''

        if not self.validar_string(parte_calle):
            return self.validar_string(parte_calle)

        if len(parte_calle) < 3:
            return None

        if not self.validar_id_ciudad(id_ciudad):
            return self.validar_id_ciudad(id_ciudad)

        session = self.gis.binded_session()
        tabla_calles = self.gis.tabla('calles')

        q_obj = session.query(tabla_calles.idCalle, tabla_calles.Nombre)
        q_obj = q_obj.filter(and_(
                      tabla_calles.idCiudad == id_ciudad,
                      tabla_calles.Nombre.like('%' + parte_calle + '%')))

        calles = q_obj.all()

        if int(q_obj.count()) == 0:
            session.close()
            return None

        lista_calles = []

        for registro in calles:
            c = {}
            c['id_calle'] = int(registro.idCalle)
            c['nombre_calle'] = registro.Nombre
            lista_calles.append(c)

        session.close()
        return lista_calles

    def generar_direccion(self, calle1=None, calle2=None, altura_nro=None,
            altura_bis=None, altura_letra=None, departamento=None, sitio=None):

        '''
        Esta función se encarga de generar un String a partir de los datos que
        se le pasan como argumentos. Dependiendo de si se agrego o no una calle
        principal, una altura o un sitio (a rasgos generales) arma el String
        resultante.

        Al utilizar la misma podemos mantener una coherencia en las direcciones
        almacenadas en la base de datos

         Args:
               calle1 (str)
               calle2 (str)
               altura_nro (str)
               altura_bis (str)
               altura_letra (str)
               departamento (str)
               sitio (str)

        Returns:
            direccion (str)
            None

        Functions:
            validar_string()

        '''
        direccion = ''
        if calle1 is not None:

            if not self.validar_string(calle1):
                return self.validar_string(calle1)

            direccion = calle1

            if altura_nro is not None:

                if not self.validar_string(altura_nro):
                    return self.validar_string(altura_nro)

                if altura_nro.isdigit():
                    direccion += ' ' + altura_nro
                    if altura_letra is not None:

                        if not self.validar_string(altura_letra):
                            return self.validar_string(altura_letra)

                        direccion += ' ' + altura_letra
                    if altura_bis is not None:

                        if not self.validar_string(altura_bis):
                            return self.validar_string(altura_bis)

                        if altura_bis == 'BIS':
                            direccion += ' ' + altura_bis

                    if departamento is not None:

                        if not self.validar_string(departamento):
                            return self.validar_string(departamento)

                        direccion += '  ' + departamento
                else:
                    if calle2 is not None:

                        if not self.validar_string(calle2):
                            return self.validar_string(calle2)

                        direccion += ' y ' + calle2

            else:
                if calle2 is not None:

                    if not self.validar_string(calle2):
                        return self.validar_string(calle2)

                    direccion += ' y ' + calle2
            direccion.strip()
        else:
            if sitio is not None:

                if not self.validar_string(sitio):
                    return self.validar_string(sitio)

                direccion = sitio
                direccion.strip()
            else:
                direccion = None

        return direccion

    def direccion_xy(self, id_ciudad=None, id_calle1=None, id_calle2=None,
                               altura=None, letra_altura=None, id_sitio=None):

        '''
            Dado el id_ciudad, una calle y su altura obtengo X e Y de la tabla
            Nodos.
            Dado el id_ciudad, y dos calles obtengo X e Y de la tabla Nodos.
            Dado el id_ciudad y el id_sitio obtengo X e Y de la tabla Nodos.

            Args:
                id_ciudad (int)
                id_calle1 (int)
                id_calle2 (int)
                altura (int)
                letra_altura (str)
                id_sitio (int)

            Returns:
                nodos.X, nodos.Y

            Functions:
                binded_session()
                tabla('lineas')
                tabla('nodos')
        '''

        if (id_ciudad is not None and id_calle1 is not None and
           altura is not None):
            # Hago el cálculo de X e Y con la calle y la altura

            session = self.gis.binded_session()

            tabla_lineas = self.gis.tabla('lineas')

            q_obj = session.query(tabla_lineas.idLinea,
                                  tabla_lineas.AlturaDesde,
                                  tabla_lineas.AlturaHasta)

            if letra_altura is not None:
                # Hago el cálculo de X e Y con la calle y la altura con letra
                q_obj = q_obj.filter(and_(
                      tabla_lineas.idMapa == id_ciudad,
                      tabla_lineas.idCiudad == id_ciudad,
                      tabla_lineas.idCalle == id_calle1,
                      tabla_lineas.LetraAltura == letra_altura,
                      or_(
                          and_(
                              tabla_lineas.AlturaDesde <= altura,
                              tabla_lineas.AlturaHasta >= altura,
                          ),
                          and_(
                              tabla_lineas.AlturaDesde >= altura,
                              tabla_lineas.AlturaHasta <= altura,
                          )
                      )
                  ))
            else:
                # Hago el cálculo de X e Y con la calle y la altura sin letra
                q_obj = q_obj.filter(and_(
                      tabla_lineas.idMapa == id_ciudad,
                      tabla_lineas.idCiudad == id_ciudad,
                      tabla_lineas.idCalle == id_calle1,
                      or_(
                          and_(
                              tabla_lineas.AlturaDesde <= altura,
                              tabla_lineas.AlturaHasta >= altura,
                          ),
                          and_(
                              tabla_lineas.AlturaDesde >= altura,
                              tabla_lineas.AlturaHasta <= altura,
                          )
                      )
                  ))

            try:
                linea = q_obj.first()

                altura_desde = linea.AlturaDesde
                altura_hasta = linea.AlturaHasta
                id_linea = linea.idLinea

                r = self.calcular_xy(id_ciudad, altura, altura_desde,
                                                     altura_hasta, id_linea)
                session.close()
                return Punto(x=int(r['x']), y=int(r['y']), id_mapa=id_ciudad)
            except:
                session.close()
                raise (Exception, _('No se pudieron hayar los puntos X e Y.'))

        elif (id_ciudad is not None and id_calle1 is not None and
            id_calle2 is not None):
            # Hago el cálculo de X e Y con las dos calles, busco la esquina

            session = self.gis.binded_session()

            tabla_lineas = self.gis.tabla('lineas')
            tabla_nodos = self.gis.tabla('nodos')

            lineas1 = aliased(tabla_lineas)
            lineas2 = aliased(tabla_lineas)

            q_obj = session.query(tabla_nodos.X, tabla_nodos.Y)

            q_obj = q_obj.outerjoin((lineas1, and_(
                tabla_nodos.idMapa == lineas1.idMapa,
                tabla_nodos.idNodo == lineas1.idNodoDesde,
                )
            )).outerjoin((lineas2, and_(
                tabla_nodos.idMapa == lineas2.idMapa,
                tabla_nodos.idNodo == lineas2.idNodoDesde,
                )
            )).filter(and_(lineas1.idMapa == id_ciudad,
                           lineas1.idCiudad == id_ciudad,
                           lineas1.idCalle == id_calle1,
                           lineas2.idMapa == id_ciudad,
                           lineas2.idCiudad == id_ciudad,
                           lineas2.idCalle == id_calle2,
                           tabla_nodos.idMapa == id_ciudad)
                       )

            if int(q_obj.count()) == 0:
                q_obj = q_obj.outerjoin((lineas1, and_(
                tabla_nodos.idMapa == lineas1.idMapa,
                tabla_nodos.idNodo == lineas1.idNodoHasta,
                )
            )).outerjoin((lineas2, and_(
                tabla_nodos.idMapa == lineas2.idMapa,
                tabla_nodos.idNodo == lineas2.idNodoHasta,
                )
            )).filter(and_(lineas1.idMapa == id_ciudad,
                           lineas1.idCiudad == id_ciudad,
                           lineas1.idCalle == id_calle1,
                           lineas2.idMapa == id_ciudad,
                           lineas2.idCiudad == id_ciudad,
                           lineas2.idCalle == id_calle2,
                           tabla_nodos.idMapa == id_ciudad)
                       )

            if int(q_obj.count()) == 0:
                q_obj = q_obj.outerjoin((lineas1, and_(
                    tabla_nodos.idMapa == lineas1.idMapa,
                    tabla_nodos.idNodo == lineas1.idNodoDesde,
                    )
                )).outerjoin((lineas2, and_(
                    tabla_nodos.idMapa == lineas2.idMapa,
                    tabla_nodos.idNodo == lineas2.idNodoHasta,
                    )
                )).filter(and_(lineas1.idMapa == id_ciudad,
                               lineas1.idCiudad == id_ciudad,
                               lineas1.idCalle == id_calle1,
                               lineas2.idMapa == id_ciudad,
                               lineas2.idCiudad == id_ciudad,
                               lineas2.idCalle == id_calle2,
                               tabla_nodos.idMapa == id_ciudad)
                           )

            if int(q_obj.count()) == 0:
                q_obj = q_obj.outerjoin((lineas1, and_(
                    tabla_nodos.idMapa == lineas1.idMapa,
                    tabla_nodos.idNodo == lineas1.idNodoHasta,
                    )
                )).outerjoin((lineas2, and_(
                    tabla_nodos.idMapa == lineas2.idMapa,
                    tabla_nodos.idNodo == lineas2.idNodoDesde,
                    )
                )).filter(and_(lineas1.idMapa == id_ciudad,
                               lineas1.idCiudad == id_ciudad,
                               lineas1.idCalle == id_calle1,
                               lineas2.idMapa == id_ciudad,
                               lineas2.idCiudad == id_ciudad,
                               lineas2.idCalle == id_calle2,
                               tabla_nodos.idMapa == id_ciudad)
                           )

            try:
                xy = q_obj.first()
                session.close()
                return Punto(x=int(xy.X), y=int(xy.Y), id_mapa=id_ciudad)
            except:
                session.close()
                raise (Exception, _('No se pudieron hayar los puntos X e Y.'))

        elif id_ciudad is not None and id_sitio is not None:
            # Hago el cálculo con el sitio

            session = self.gis.binded_session()

            tabla_nodos = self.gis.tabla('nodos')

            q_obj = session.query(tabla_nodos.X, tabla_nodos.Y)

            q_obj = q_obj.filter(and_(
                                    tabla_nodos.idMapa == id_ciudad,
                                    tabla_nodos.idNodo == id_sitio,
                                    )
                                )

            xy = q_obj.first()

            try:
                xy = q_obj.first()
                session.close()
                return Punto(x=int(xy.X), y=int(xy.Y), id_mapa=id_ciudad)
            except:
                session.close()
                raise (Exception, _('No se pudieron hayar los puntos X e Y.'))
        else:
            return None

    def calcular_xy(self, id_ciudad, altura, altura_desde, altura_hasta,
                                                                     id_linea):
        '''
            Hago el cálculo de X e Y con la calle y la altura,
            una vez obtenido su id_linea.
        '''

        session = self.gis.binded_session()

        tabla_lineas = self.gis.tabla('lineas')
        tabla_nodos = self.gis.tabla('nodos')

        nodos2 = aliased(tabla_nodos)

        q_obj = session.query(
            tabla_nodos.X.label('XD'),
            tabla_nodos.Y.label('YD'),
            nodos2.X.label('XH'),
            nodos2.Y.label('YH')
        )

        q_obj = q_obj.outerjoin((tabla_lineas, and_(
            tabla_nodos.idMapa == tabla_lineas.idMapa,
            tabla_nodos.idNodo == tabla_lineas.idNodoDesde,
            )
        )).outerjoin((nodos2, and_(
            nodos2.idMapa == tabla_lineas.idMapa,
            nodos2.idNodo == tabla_lineas.idNodoHasta,
            )
        )).filter(and_(tabla_lineas.idMapa == id_ciudad,
                       tabla_lineas.idCiudad == id_ciudad,
                       tabla_lineas.idLinea == id_linea,
                       )
                   )

        try:
            xy = q_obj.first()

            XH = xy.XH
            YH = xy.YH
            XD = xy.XD
            YD = xy.YD

            if altura_desde < altura_hasta:
                AlturaMenor = altura_desde
                AlturaMayor = altura_hasta

                AlturaTotalRelativa = AlturaMayor - AlturaMenor

                ModuloX = XH - XD
                ModuloY = YH - YD

                AlturaX = float(ModuloX) / float(AlturaTotalRelativa) *\
                                                         (altura - AlturaMenor)
                AlturaX = XD + AlturaX

                AlturaY = float(ModuloY) / float(AlturaTotalRelativa) *\
                                                         (altura - AlturaMenor)
                AlturaY = YD + AlturaY
            else:
                AlturaMenor = altura_hasta
                AlturaMayor = altura_desde
                AlturaTotalRelativa = AlturaMayor - AlturaMenor

                ModuloX = XD - XH
                ModuloY = YD - YH

                AlturaX = float(ModuloX) / float(AlturaTotalRelativa) *\
                                                         (altura - AlturaMenor)
                AlturaX = XH + AlturaX

                AlturaY = float(ModuloY) / float(AlturaTotalRelativa) *\
                                                         (altura - AlturaMenor)
                AlturaY = YH + AlturaY

            xy = {}
            xy['x'] = int(AlturaX)
            xy['y'] = int(AlturaY)

            session.close()
            return xy
        except:
            session.close()
            raise (Exception, _('No se pudieron hayar los puntos X e Y.'))


@DBSwitch('gis2')
class CallePostgres(ValidacionesEspecificas):
    '''
        Esta clase contiene todos lo métodos relacionados con las calles
        trabajando con Postgres y postgis.
    '''

    def __init__(self):
        self.f_geo = FuncionesGeo()
        self.ciudad = CiudadPostgres()
        self.topologia = Topologia()

    def listar_tipos(self):
        '''
            Dado el ID de una ciudad, esta funcion devuelve un listado con todas
            las calles de la misma. Estos datos corresponden al id y a l nombre
            de la calle.

            Args:
                id_ciudad (int)

            Returns:
                calles (list [  list [(int),(str)] ])


            Functions:
                validar_id_ciudad(id_ciudad)
        '''
        # Declaramos respuesta
        rta = {'Estado': False}
        # Declaramos hack para compilado
        str_vacio = ''
        # Iniciamos la sesion con la BD gis
        session = self.gis2.binded_session()
        # Nos conectamos a la tabla "tipos"
        tabla_calles_tipos = self.gis2.db['calles_tipos']
        # Hacemos la consulta
        try:
            q_obj = session.query(
                tabla_calles_tipos.id,
                tabla_calles_tipos.nombre
            )
            q_obj = q_obj.filter(and_(
                tabla_calles_tipos.nombre is not None,
                tabla_calles_tipos.nombre != str_vacio
            ))
            q_obj = q_obj.order_by(tabla_calles_tipos.nombre.asc())
            calles_tipos_tmp = q_obj.all()
        except:
            # Cerramos y salimos si Falló
            session.close()
            rta['Respuesta'] = _('Falló al momento de realizar la consutla.')
            return rta
        # Cerramops la session
        session.close()
        # Generamos la lista de elementos que devolveremos
        calles_tipos = []
        for t in calles_tipos_tmp:
            t_tmp = []
            t_tmp.append(int(t.id))
            t_tmp.append(t.nombre.encode('latin1'))
            calles_tipos.append(t_tmp)
        # Devolvemos bien
        rta['Estado'] = True
        rta['Respuesta'] = calles_tipos
        return rta

    def listar_calles(self, id_ciudad, real=False):
        '''
            Dado el ID de una ciudad, esta funcion devuelve un listado con todas
            las calles de la misma. Estos datos corresponden al id y a l nombre
            de la calle.

            Args:
                id_ciudad (int)

            Returns:
                calles (list [  list [(int),(str)] ])


            Functions:
                validar_id_ciudad(id_ciudad)
        '''
        # Declaramos respuesta
        rta = {'Estado': False}
        # Validamos el id_ciudad
        try:
            self.validar_id_ciudad(id_ciudad)
        except:
            rta['Respuesta'] = _(u'Debe ingresar datos válidos.')
            return rta
        # Declaramos hack para compilado
        str_vacio = ''
        # Iniciamos la sesion con la BD gis
        session = self.gis2.binded_session()
        # Nos conectamos a la tabla "calles"
        tabla_calles_nombres = self.gis2.db['calles_nombres']
        tabla_calles = self.gis2.db['calles']
        if not real:
            tabla_ciudades_aplicaciones = self.gis2.db['ciudades_aplicaciones']
            # Hacemos la consulta
            try:
                q_obj = session.query(tabla_ciudades_aplicaciones.id_ciudad)
                q_obj = q_obj.filter(
                    tabla_ciudades_aplicaciones.id == id_ciudad
                )
                ciudad_real = q_obj.first()
            except:
                # Cerramos y salimos si Falló
                session.close()
                rta['Respuesta'] = _(u'Falló al momento de hacer la consutla.')
                return rta
            # Salimos si no esta la ciudad
            if ciudad_real is None:
                session.close()
                rta['Respuesta'] = _('No se encontró la ciudad buscada.')
                return rta
            id_ciudad_real = ciudad_real.id_ciudad
        else:
            # Sino, hacemos la query pidiendo todas las calles de esa ciudad
            id_ciudad_real = id_ciudad
        # Hacemos la consulta
        try:
            q_obj = session.query(
                tabla_calles_nombres.id,
                tabla_calles_nombres.nombre,
                func.count(tabla_calles.gid).label('cantidad_segmentos')
            ).outerjoin((tabla_calles,
                tabla_calles.id_calle_nombre == tabla_calles_nombres.id
            )).filter(and_(
                tabla_calles_nombres.id_ciudad == id_ciudad_real,
                tabla_calles_nombres.nombre is not None,
                tabla_calles_nombres.nombre != str_vacio
            ))
            q_obj = q_obj.order_by(func.lower(tabla_calles_nombres.nombre).asc()).group_by(
                tabla_calles_nombres.id, tabla_calles_nombres.nombre
            )
            calles_tmp = q_obj.all()
        except Exception, e:
            # Cerramos y salimos si Falló
            session.close()
            rta['Respuesta'] = _('Falló al momento de realizar la consutla.')
            return rta
        # Cerramops la session
        session.close()
        # Generamos la lista de elementos que devolveremos
        calles = []
        for c in calles_tmp:
            c_tmp = []
            c_tmp.append(int(c.id))
            c_tmp.append(c.nombre.encode('latin1'))
            c_tmp.append(int(c.cantidad_segmentos))
            calles.append(c_tmp)
        # Devolvemos bien
        rta['Estado'] = True
        rta['Respuesta'] = calles
        return rta

    def sugerir_calle(self, id_ciudad, parte_calle,
                      real=False, encode='latin1'):
        '''
            Dado un id de una ciudad y parte del nombre de una calle,
            sugiere posibles nombres de calles que pertencen a esa ciudad.

            Args:
                id_ciudad (int)
                parte_calle (str)
                real (bool)
                encode (str)

            Returns:
                lista_calles[i]['id_calle']
                lista_calles[i]['nombre_calle']
                None

            Functions:
                validar_string(parte_calle)
                validar_id_ciudad(id_ciudad)
                binded_session()
                tabla('calles')
        '''

        # Declaramos respuesta
        rta = {'Estado': False}

        # Validamos
        try:
            self.validar_string(parte_calle)
            self.validar_id_ciudad(id_ciudad)
        except:
            rta['Respuesta'] = _(u'Debe ingresar datos válidos.')
            return rta
        # Si no tenemos ni dos caracteres, salimos
        if len(parte_calle) < 3:
            rta['Estado'] = True
            rta['Respuesta'] = []
            return rta

        # Validamos si tenemos que buscar la ciudad real
        if real:
            id_ciudad_real = id_ciudad
        else:
            # Hacemos la query
            # Abrimos la session
            session = self.gis2.binded_session()
            tabla_ciudades_aplicaciones = self.gis2.db['ciudades_aplicaciones']
            q_obj = session.query(tabla_ciudades_aplicaciones.id_ciudad)
            q_obj = q_obj.filter(
                tabla_ciudades_aplicaciones.id == id_ciudad
            )
            ciudad_real = q_obj.first()
            session.close()
            # Si no hay ciudad, salimos
            if ciudad_real is None:
                rta['Respuesta'] = _(u'No se encontró la ciudad buscada.')
                return rta
            # Sino, la guardamos
            id_ciudad_real = ciudad_real.id_ciudad

        # FIX por la BD en AWS, trato de usar la función to_ascii para no tener
        # en cuenta los acentos, si no la puedo usar hago la misma query pero
        # sin usar la función ascii
        # Abro nuevamente la sesión porque sino me muestra un error
        # sin importar el try
        try:
            session = self.gis2.binded_session()
            tabla_calles_nombres = self.gis2.db['calles_nombres']
            q_obj_calles = session.query(
                tabla_calles_nombres.id,
                tabla_calles_nombres.nombre
            )
            q_obj_calles = q_obj_calles.filter(
                and_(
                    tabla_calles_nombres.id_ciudad == id_ciudad_real,
                    func.to_ascii(
                        tabla_calles_nombres.nombre
                    ).ilike(func.to_ascii('%' + parte_calle + '%'))
                )
            )
            calles = q_obj_calles.all()
            session.close()
        except:
            session = self.gis2.binded_session()
            tabla_calles_nombres = self.gis2.db['calles_nombres']
            q_obj_calles = session.query(
                tabla_calles_nombres.id,
                tabla_calles_nombres.nombre
            )
            q_obj_calles = q_obj_calles.filter(
                and_(
                    tabla_calles_nombres.id_ciudad == id_ciudad_real,
                    tabla_calles_nombres.nombre.ilike('%' + parte_calle + '%')
                )
            )
            calles = q_obj_calles.all()
            session.close()

        # Declaramos el listado de calles
        lista_calles = []
        for registro in calles:
            c = {}
            c['id_calle'] = int(registro.id)
            if encode == 'latin1':
                c['nombre_calle'] = registro.nombre.encode('latin1')
            else:
                c['nombre_calle'] = registro.nombre
            lista_calles.append(c)

        # Salimos bien
        rta['Estado'] = True
        rta['Respuesta'] = lista_calles
        return rta

    def direccion_xy(self, id_ciudad=None, id_calle1=None, id_calle2=None,
                               altura=None, letra_altura=None, id_sitio=None):

        '''
            Dado el id_ciudad, una calle y su altura obtengo la latitud y
            la longitud

            Args:
                id_ciudad (int)
                id_calle1 (int)
                id_calle2 (int)
                altura (int)
                letra_altura (str)
                id_sitio (int)

            Returns:
                nodos.X, nodos.Y

            Functions:
                binded_session()
                tabla('lineas')
                tabla('nodos')
        '''
        # Declaramos la respuesta
        rta = {'Estado': False}
        # Vamos a buscar el xy con la calle y altura
        if id_calle1 is not None and altura is not None:
            # Validamos que ambos sean enteros
            try:
                self.validar_entero(id_calle1)
                self.validar_entero(altura)
            except:
                rta['Respuesta'] = _(u'Los datos ingresados no son válidos')
                return rta
            # Abrimos la session
            session = self.gis2.binded_session()
            # Vamos a consultar la tabla de calles
            tabla_calles = self.gis2.db['calles']
            # Pedimos las alturas y la geometria a postgres
            try:
                q_obj = session.query(
                    tabla_calles.desde_izq,
                    tabla_calles.hasta_izq,
                    tabla_calles.desde_der,
                    tabla_calles.hasta_der,
                    func.st_astext(tabla_calles.geom).label('geom'))
                # Filtramos, buscando que la altura se encuentre en algun tramo
                q_obj = q_obj.filter(and_(
                    tabla_calles.id_calle_nombre == id_calle1,
                    or_(
                        and_(
                            tabla_calles.desde_izq <= altura,
                            tabla_calles.hasta_izq >= altura
                        ),
                        and_(
                            tabla_calles.desde_izq >= altura,
                            tabla_calles.hasta_izq <= altura
                        ),
                        and_(
                            tabla_calles.desde_der >= altura,
                            tabla_calles.hasta_der <= altura
                        ),
                        and_(
                            tabla_calles.desde_der <= altura,
                            tabla_calles.hasta_der >= altura
                        )
                    )
                ))
                # Obtenemos todos los tramos que coincidan
                tramos = q_obj.all()
            except:
                # Salimos si fallamos
                session.close()
                rta['Respuesta'] = _(u'Falló al momento de hacer la consulta.')
                return rta
            # Cerramos la session
            session.close()
            # Si no encontramos tramo que coincida, devolvemos None
            if len(tramos) == 0:
                rta['Respuesta'] = _(u'No se pudo encontrar la dirección.')
                return rta

            # Chequeamos si la altura es par o impar
            altura_par = False
            if altura % 2 == 0:
                altura_par = True
            # Vamos a definir el tramo que se va a procesar
            tramo = None
            if len(tramos) > 1:

                # Si tenemos mas de un tramo, los recorremos
                for tramo_tmp in tramos:
                    # No puede haber un tramo comn mas de 200 de difrencia
                    # ente alturas. Sino hay algo mal
                    if (abs(tramo_tmp.desde_der - tramo_tmp.hasta_der) > 300 or
                        abs(tramo_tmp.desde_izq - tramo_tmp.hasta_izq) > 300):
                        continue
                    # Validamos si tiene alturas derechas. Si es asi, chequeamos
                    # si coinciden con la altura ingresada,
                    if not (tramo_tmp.desde_der == 0 and
                       tramo_tmp.hasta_der == 0):
                        if (tramo_tmp.desde_der % 2 == 0 and altura_par) or \
                           (tramo_tmp.desde_der % 2 != 0 and
                            not altura_par):
                            # El tramo coincide, asi que vamos a usar este
                            tramo = tramo_tmp
                            break
                    # Si no hay alturas derechas, validamos las izquierdas
                    if not (tramo_tmp.desde_izq == 0 and
                       tramo_tmp.hasta_izq == 0):
                        if ((tramo_tmp.desde_izq % 2 == 0 and altura_par) or
                           (tramo_tmp.desde_izq % 2 != 0 and not altura_par)):
                            # El tramo coincide, asi que vamos a usar este
                            tramo = tramo_tmp
                            break
                ##########################################################
                # Este caso es para cuando buscamos una calle impar y
                # solo tenemos pares o viceversa
                # Es un ultimo recurso, si no encontramos nada
                ##########################################################
                if tramo is None:
                    for tramo_tmp in tramos:
                        # No puede haber un tramo comn mas de 200 de difrencia
                        # ente alturas. Sino hay algo mal
                        if (
                        abs(tramo_tmp.desde_der - tramo_tmp.hasta_der) > 300 or
                        abs(tramo_tmp.desde_izq - tramo_tmp.hasta_izq) > 300):
                            continue
                        abs_altura = abs(altura)
                        # Validamos si tiene alturas correctas
                        if (abs(tramo_tmp.desde_der) >= abs_altura and
                            abs(tramo_tmp.hasta_der) <= abs_altura) or (
                            abs(tramo_tmp.desde_der) <= abs_altura and
                            abs(tramo_tmp.hasta_der) >= abs_altura) or (
                            abs(tramo_tmp.desde_izq) >= abs_altura and
                            abs(tramo_tmp.hasta_izq) <= abs_altura) or (
                            abs(tramo_tmp.desde_izq) <= abs_altura and
                            abs(tramo_tmp.hasta_izq) >= abs_altura):
                                # El tramo coincide, asi que vamos a usar este
                                tramo = tramo_tmp
                                break
            else:
                # Si solo tenemos uno solo, usamos ese
                tramo = tramos[0]

            # Si no encontramos nada válido, salimos
            if tramo is None:
                rta['Respuesta'] = _(
            u'Hay tramos pero no sirven. Las alturas de los mismos están mal.'
                )
                return rta
            # Nos aseguramos que sea un tramos válido y no vacío
            try:
                tramo.geom = str(tramo.geom)
            except:
                rta['Respuesta'] = _(
            u'Hay tramos pero no sirven. Las alturas de los mismos están mal.'
                )
                return rta

            # Como la geometria esta en formato de string, tenemos que obtener
            # los puntos parseando la cadena
            geometria = tramo.geom
            aux = geometria.find('(')
            geometria = geometria[aux:]
            geometria = geometria.replace('((', '')
            geometria = geometria.replace('))', '')
            aux_puntos = geometria.split(',')
            # Punto1 es una lista con la longitud y latitud del punto
            # inicial del tramo
            punto1 = aux_puntos[0].split(' ')
            punto1[0] = float(punto1[0])
            punto1[1] = float(punto1[1])
            # Punto2 es una lista con la longitud y latitud del punto
            # final del tramo
            punto2 = aux_puntos[1].split(' ')
            punto2[0] = float(punto2[0])
            punto2[1] = float(punto2[1])
            # Segmento_x y segmento_y son la representacion ortogonal del tramo
            # ubicando su origen en el origen de coordenadas
            segmento_x = punto2[0] - punto1[0]
            segmento_y = punto2[1] - punto1[1]
            segmento = sqrt((segmento_x ** 2) + (segmento_y ** 2))
            # Obtenemos M, que es la pendiente del tramo (inclinacion) #
            # en relacion al eje cartesiano
            if segmento_x != 0:
                m = segmento_y / segmento_x
            else:
                if segmento_y > 0:
                    m = 1
                else:
                    m = -1

            # Vamos a guardar las alturas en una lista, para que sea mas facil
            # obtener las mayores y menores
            alturas = []
            # Validamos que tenga alturas derechas
            alturas_derechas_pares = None
            if not (tramo.desde_der == 0 and tramo.hasta_der == 0):
                # Agregamos las alturas
                alturas.append(abs(tramo.desde_der))
                alturas.append(abs(tramo.hasta_der))
                # Verificamos si las alturas derechas son pares o impares
                alturas_derechas_pares = False
                if (tramo.desde_der - tramo.hasta_der) % 2 == 0:
                    alturas_derechas_pares = True

            # Validamos que tenga alturas izquierdas
            alturas_izquierdas_pares = None
            if not (tramo.desde_izq == 0 and tramo.hasta_izq == 0):
                # Agregamos las alturas
                alturas.append(abs(tramo.desde_izq))
                alturas.append(abs(tramo.hasta_izq))
                # Verificamos si las alturas izquierdas son pares o impares
                alturas_izquierdas_pares = False
                if (tramo.desde_izq - tramo.hasta_izq) % 2 == 0:
                    alturas_izquierdas_pares = True

            # Obtenemos la proporcion en la cual se encuentra la altura buscada
            # en nuestro tramo
            proporcion = 0
            if len(alturas) > 1:
                proporcion = (
                    float(abs(altura) - min(alturas)) /
                    float(max(alturas) - min(alturas))
                )
            # Calculamos la desviacion de la calle, por que la altura 0 no
            # esta en la mitad de la calle. Que existen las veredas, joder!!!
            delta_vereda_x = sqrt((ANCHO_CALLE ** 2) / (1 + (m ** 2)))
            delta_vereda_y = delta_vereda_x * m
            proporcion_calle = float(ANCHO_CALLE) / float(segmento)
            calle_x = segmento_x * proporcion_calle
            calle_y = segmento_y * proporcion_calle

            if abs(tramo.desde_der) < abs(tramo.hasta_der) or \
               abs(tramo.desde_izq) < abs(tramo.hasta_izq):
                # Si el sentido de la altura corresponde al sentido de la calle,
                # sumamos la proporcion de la altura buscada al punto incial
                # del tramo
                longitud_final = punto1[0] + calle_x + (
                    (segmento_x - calle_x * 2) * proporcion
                )
                latitud_final = punto1[1] + calle_y + (
                    (segmento_y - calle_y * 2) * proporcion
                )
            else:
                # Si el sentido es contrario, restamos la proporcion de la
                # altura buscada al punto final del tramo
                longitud_final = punto2[0] - calle_x - (
                    (segmento_x - calle_x * 2) * proporcion
                )
                latitud_final = punto2[1] - calle_y - (
                    (segmento_y - calle_y * 2) * proporcion
                )

            # Obtenemos la desviacion de la vereda. Esto modificara el punto
            # obtenido recien, ubicandolo sobre la vereda par o impar de la
            # calle en la derecha o izquierda, segun corresponda
            delta_x = 0
            delta_y = 0
            # Primero vemos si habia datos de alturas derechas
            if alturas_derechas_pares is not None:
                # Si la altura es par y las alturas derechas tambien, o
                # ambas son impares, desviamos perpendicularmente a la derecha
                if alturas_derechas_pares == altura_par:
                    delta_x = delta_vereda_y
                    delta_y = -1 * delta_vereda_x
                # En caso contraro, desviamos perpendicularmente a la izquierda
                else:
                    delta_x = -1 * delta_vereda_y
                    delta_y = delta_vereda_x
            # En caso de que no hubiese datos de alturas derechas, intentamos
            # obtener la desviacion segun las alturas izquierdas
            elif alturas_izquierdas_pares is not None:
                # Si tanto la altura como las alturas izquierdas coinciden,
                # desviamos a la izquierda
                if alturas_izquierdas_pares == altura_par:
                    delta_x = -1 * delta_vereda_y
                    delta_y = delta_vereda_x
                # Sino, desviamos a la derecha
                else:
                    delta_x = delta_vereda_y
                    delta_y = -1 * delta_vereda_x
            # Sumamos los delta correspondientes a las desviaciones, sean hacia
            # la izquierda o a la derecha
            longitud_final += delta_x
            latitud_final += delta_y
            # Devolvemos el punto final
            rta['Estado'] = True
            rta['Respuesta'] = Punto(
                longitud=float(longitud_final),
                latitud=float(latitud_final),
                id_mapa=id_ciudad
            )
            return rta

        # Hago el cálculo de X e Y con las dos calles, busco la esquina
        elif id_calle1 is not None and id_calle2 is not None:
            session = self.gis2.binded_session()
            tabla_calles = self.gis2.db['calles']
            try:
                sub_q_obj1 = session.query(tabla_calles.geom)
                sub_q_obj1 = sub_q_obj1.filter(
                    tabla_calles.id_calle_nombre == id_calle1
                )
                sub_q_obj2 = session.query(tabla_calles.geom)
                sub_q_obj2 = sub_q_obj2.filter(
                    tabla_calles.id_calle_nombre == id_calle2
                )
                q_obj = session.query(
                    func.st_astext(
                        func.st_intersection(
                            sub_q_obj2.subquery().columns.geom,
                            sub_q_obj1.subquery().columns.geom
                        ).label('interseccion')
                    ).label('interseccion')
                )
                q_obj = q_obj.distinct()
                intersecciones_tmp = q_obj.all()
            except Exception, ex:
                # Salimos si fallamos
                session.close()
                rta['Respuesta'] = _(
                    u'Falló al momento de hacer la consulta por la esquina. ERROR: %s'
                ) % ex
                return rta
            # Cerramos la session
            session.close()
            # Buscamos las intersecciones
            intersecciones = []
            for i in intersecciones_tmp:
                if i.interseccion != 'GEOMETRYCOLLECTION EMPTY':
                    i_temp = i.interseccion.replace('POINT(', '')
                    i_temp = i_temp.replace(')', '')
                    lon, lat = i_temp.split(' ')
                    intersecciones.append(
                        Punto(
                            longitud=float(lon),
                            latitud=float(lat),
                            id_mapa=id_ciudad
                        )
                    )
            # Armamos la respuesta
            if len(intersecciones) == 0:
                rta['Respuesta'] = _(u'No se pudo encontrar la dirección por esquina.')
            else:
                rta['Estado'] = True
                rta['Respuesta'] = intersecciones[0]
            # Devolvemos la respuesta
            return rta

        elif id_ciudad is not None and id_sitio is not None:
            try:
                from at.map.sitio import Sitio
                # Hago el cálculo con el sitio
                rta_obtener_sitio = Sitio().obtener_sitio(id_ciudad, id_sitio)

                rta['Estado'] = True
                rta['Respuesta'] = Punto(
                    longitud=float(rta_obtener_sitio['longitud_sitio']) / 3600,
                    latitud=float(rta_obtener_sitio['latitud_sitio']) / 3600,
                    id_mapa=id_ciudad
                )
                return rta
            except:
                rta['Estado'] = False
                rta['Respuesta'] = _('No se encontró la posición del sitio')
                return rta

        rta['Estado'] = False
        rta['Respuesta'] = _(u'Los datos ingresados son incorrectos.')
        return rta

    def obtener_puntos_calles_por_zona(self, zona):
        ''' Dada una zona, se obtendran todos los puntos que pertenezcan
        a calles en la misma. Estos puntos permitiran hacer una cuadricula
        de las calles de forma mas precisa

            Args:
                zona: list[
                    puntos : dict {
                        latitud: float,
                        longitud: float
                    }
                ]

            Returns:
                rta['Estado']: (bool)
                rta['Respuesta']: puntos: list[
                    puntos : list [
                        latitud: float,
                        longitud: float
                    ]
                ]

        '''

        # Definimos la respuesta
        rta = {'Estado': False}

        # Validamos que sea un diccionario
        if not isinstance(zona, list):
            rta['Respuesta'] = _('La zona debe ser una lista.')
            return rta

        # Si es un poligono abierto, lo cerramos nosotros
        if zona[0] != zona[len(zona) - 1]:
            zona.append(zona[0])

        # Armamos el string del poligono con los puntos que tenemos
        str_zona = 'POLYGON(('
        for punto in zona:
            if not isinstance(punto, dict):
                rta['Respuesta'] = _(
                    'La zona debe contener unicamente diccionarios.'
                )
                return rta

            if punto.get('Longitud') is None or punto.get('Latitud') is None:
                rta['Respuesta'] = _(
                    u'La zona debe contener diccionarios con "Latitud" y "Longitud".'
                )
                return rta

            str_zona += str(punto['Longitud']) + ' ' + \
                                                 str(punto['Latitud']) + ','
        str_zona = str_zona[:-1] + '))'

        # Hacemos la consulta, buscando las calles que intersecten
        # nuestro poligono
        session = self.gis2.binded_session()
        tabla_calles = self.gis2.db['calles']
        q_obj = session.query(func.st_astext(tabla_calles.geom).label('geom'))
        q_obj = q_obj.filter(
            func.st_dwithin(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_zona,
                    4326
                ).label('punto'),
                0.0001
            ).label('poligono')
        )

        # Obtenemos las calles
        try:
            calles_tmp = q_obj.all()
        except:
            # Cerramos la session
            session.close()
            # Salimos
            rta['Respuesta'] = _(
                u'Fallo al momento de realizar la consulta. Chequeé el estado y codificación de la base de datos'
            )
            return rta
        session.close()
        # Recorremos las calles y obtenemos todos sus puntos
        puntos = []
        geometria = ''
        for c in calles_tmp:
            # Si no es una geometria, que siga
            if c is None:
                continue
            # Sino, procesamos
            geometria += c.geom.replace('MULTILINESTRING((', '').replace(
                'LINESTRING((', '').replace(' ', ',')[:-2] + ','
        aux = geometria[:-1].split(',')
        for i in range(0, len(aux) - 1, 2):
            punto = [float(aux[i + 1]), float(aux[i])]
            if punto not in puntos:
                puntos.append(punto)
        # Salimos bien
        rta['Estado'] = True
        rta['Respuesta'] = puntos
        return rta

    def obtener_nombre_calle(self, id, latin1=True):
        ''' Esta funcion toma como argumento un id y devuleve el nombre de
        la misma, si es que la encuentra '''

        # Declaramos respuesta
        rta = {'Estado': False}
        # Iniciamos la session
        session = self.gis2.binded_session()
        tabla_calles_nombres = self.gis2.db['calles_nombres']
        # Hacemos la consulta
        try:
            q_obj = session.query(
                tabla_calles_nombres.nombre,
            )
            q_obj = q_obj.filter(
                tabla_calles_nombres.id == id,
            )
            calle_nombre = q_obj.first()
        except:
            # Cerramos la session
            session.close()
            rta['Respuesta'] = _('Falló al momento de realizar la consulta.')
            return rta
        # Cerramos la session
        session.close()
        # Si esta vacio, salimos con error
        if calle_nombre is None:
            rta['Respuesta'] = _(u'No se encontró una calle con ese id.')
            return rta
        # Salimos correctamente
        rta['Estado'] = True
        rta['Respuesta'] = ''
        if calle_nombre.nombre is not None:

            # if latin1:
            #     nombre_calle = calle_nombre.nombre.encode("latin1")
            # else:
            #     nombre_calle = calle_nombre.nombre
            try:
                nombre_calle = calle_nombre.nombre.encode("latin1")
            except:
                nombre_calle = calle_nombre.nombre

            rta['Respuesta'] = nombre_calle
        return rta

    def obtener_altura_por_latlon(self, latitud, longitud):
        ''' Esta funcion devuelve la altura, si la hay, de la calle, si la hay
        que este cercana a la posicion ingresada

        Args:
            latitud (numero)
            longitud (numero)

        Returns:
            rta['Estado'] (bool)
            rta['Respuesta'] (str) or
                altura: {
                    'Calle': (int),
                    'DesdeDer': (int),
                    'HastaDer': (int),
                    'DesdeIzq': (int),
                    'HastaIzq': (int),
                }

        Functions:
            validar_numero()

        '''

        # Definimos la respuesta
        rta = {'Estado': False}
        # Validamos los datos
        try:
            self.validar_numero(latitud)
            self.validar_numero(longitud)
        except:
            rta['Respuesta'] = _(u'Los datos deben ser válidos.')
            return rta
        # Hacemos la consulta, buscando las calles que intersecten
        # nuestro poligono
        str_punto = 'POINT(%s %s)' % (str(longitud), str(latitud))
        session = self.gis2.binded_session()
        tabla_calles = self.gis2.db['calles']
        q_obj = session.query(
            tabla_calles.desde_der,
            tabla_calles.hasta_der,
            tabla_calles.desde_izq,
            tabla_calles.hasta_izq,
            tabla_calles.id_calle_nombre,
            func.st_distance(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_punto, 4326
                )
            ).label('distancia'),
        )
        q_obj = q_obj.filter(
            func.st_dwithin(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_punto,
                    4326
                ).label('punto'),
                0.0001
            )
        ).order_by('distancia')

        # Obtenemos el primero, es decir, el mas cercano
        try:
            calle = q_obj.first()
        except:
            # Cerramos la session
            session.close()
            # Salimos
            rta['Respuesta'] = _('''Fallo al momento de realizar la
 consulta. Chequeé el estado y codificación de la base de datos''')
            return rta

        # Cerramos la session
        session.close()

        # Si esta vacio, salimos
        if calle is None:
            rta['Respuesta'] = _(u'No se encontró una calle cercana.')
            return rta

        # Buscamos el nombre de la calle
        rta_calle = self.obtener_nombre_calle(calle.id_calle_nombre)
        # Si fallo, salimos
        if not rta_calle['Estado']:
            return rta_calle
        # Armamos la respuesta
        rta['Estado'] = True
        rta['Respuesta'] = {
            'Calle': rta_calle['Respuesta'],
            'DesdeDer': calle.desde_der,
            'HastaDer': calle.hasta_der,
            'DesdeIzq': calle.desde_izq,
            'HastaIzq': calle.hasta_izq,
        }

        # Retornamos
        return rta

    def _obtener_segmentos_por_latlon(self, latitud, longitud, vacios=False):
        ''' Esta funcion devuelve el segmento mas cercano a la posicion y el
        listado total de segmentos mas cercanos. El mas
        cercano se cataloga como Principal, aquel que cumple con el minimo.
        Ademas, devolvemos el listado ordenado de calles mas cercanas aparte
        del principal

        Args:
            latitud (numero)
            longitud (numero)
            vacio (bool) # Este argumento permite obtener un segmento de una
                           calle vacia, si es que se encuentra en True

        Returns:
            Caso incorrecto
                rta['Estado'] = False
                rta['Respuesta'] = 'No se encontraron datos cercanos a
                                    la posicion'
            Caso correcto
                rta['Estado'] = True
                rta['Respuesta'] = {
                    Principal: segmento que cumple estar lo mas cerca del punto
                    Cercanos: [
                        segmento mas cercanos, que no cumplieron el minimo
                    ]
                }

        Functions:
            validar_numero()

        '''

        # Iniciamos nuestra respuesta
        rta = {'Estado': False}

        try:
            self.validar_numero(latitud)
            self.validar_numero(longitud)
        except:
            rta['Respuesta'] = _(u'La latitud y longitud deben ser válidos.')
            return rta

        # Hacemos la consulta, buscando las calles que esten cerca de nuestro
        # punto. Luego filtraremos quienes estan lo suficientemente cerca
        str_punto = 'POINT(%s %s)' % (str(longitud), str(latitud))
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles = self.gis2.db['calles']
            tabla_calles_nombres = self.gis2.db['calles_nombres']
        except:
            rta['Respuesta'] = _(u'Surgió un error con la base de datos.')
            return rta

        # Declaramos hack para compilado
        str_vacio = ''

        # Buscamos la calle donde esta parado
        q_obj = session.query(
            tabla_calles.gid,
            tabla_calles.desde_der,
            tabla_calles.hasta_der,
            tabla_calles.desde_izq,
            tabla_calles.hasta_izq,
            tabla_calles.id_calle_nombre,
            tabla_calles_nombres.nombre,
            tabla_calles_nombres.etiqueta,
            tabla_calles_nombres.id_ciudad,
            func.st_distance(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_punto, 4326
                )
            ).label('distancia'),
            func.st_astext(
                func.st_line_interpolate_point(
                    func.st_linemerge(tabla_calles.geom),
                    func.st_line_locate_point(
                        func.st_linemerge(tabla_calles.geom),
                        func.st_geomfromtext(
                            str_punto, 4326
                        )
                    )
                )
            ).label('punto_cercano'),
            func.st_astext(tabla_calles.geom).label('geom'),
        ).outerjoin((tabla_calles_nombres, and_(
                tabla_calles_nombres.id == tabla_calles.id_calle_nombre,
          ))
        )
        q_obj = q_obj.filter(
            func.st_dwithin(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_punto,
                    4326
                ).label('punto'),
                0.001
            )
        )

        if not vacios:
            q_obj = q_obj.filter(
                and_(
                    tabla_calles_nombres.nombre != str_vacio,
                    tabla_calles_nombres.nombre is not None,
                )
            )
        # Ordenamos por distancia
        q_obj = q_obj.order_by('distancia')

        # Obtenemos todas las calles cercanas al punto que tenemos
        try:
            segmentos = q_obj.all()
        except:
            # Cerramos la session
            session.close()
            # Salimos
            rta['Respuesta'] = _('''Fallo al momento de realizar la
 consulta. Chequeé el estado y codificación de la base de datos''')
            return rta

        # Vamos a limpiar las calles repetidas
        segmentos_validos = []
        segmentos_a_borrar = []
        for segmento in segmentos:
            if segmento.nombre in segmentos_validos:
                segmentos_a_borrar.append(segmento)
            else:
                segmentos_validos.append(segmento.nombre)

        # Limpiamos las calles repetidas
        for segmento in segmentos_a_borrar:
            segmentos.remove(segmento)

        # Buscamos la calle principal
        segmento_principal = None
        if len(segmentos) > 0:
            # Si la calle mas cercana esta dentro del rango permitido, la
            # marcamos como la principal
            if segmentos[0].distancia < 0.0003:
                segmento_principal = segmentos[0]
                segmentos.remove(segmento_principal)

        # Cerramos la session antes de salir
        session.close()

        # Devolvemos la respuesta
        rta['Estado'] = True
        rta['Respuesta'] = {
            'Principal': segmento_principal,
            'Cercanos': segmentos
        }
        return rta

    def obtener_posicion_por_latlon(self, latitud, longitud, esquina=False,
                                                            paralelas=False):
        ''' Esta funcion devuelve una calle y la altura, si la hay. Sino,
        dependiendo de si esta cerca de una esquina y no tenga altura, devovlera
        la Calle2 y si esta en una mitad de cuadra, lejos de la esquina,
        devolvera una tercera calle. De esta forma la posicion puede estae en:
            Calle Altura
            Calle y Calle2
            Calle, entre Calle2 y Calle3

        Args:
            latitud (numero)
            longitud (numero)
            esquina (bool)
                Si el campo esquina esta activado, siempre devolvera la misma,
                mas alla de que tenga la altura.
            paralelas (bool)
                Si el campo paralelas esta activado devuelve las dos calles
                paralelas de la principal.

        Returns:
            Caso incorrecto
                rta['Estado'] = False
                rta['Respuesta'] = 'No se encontraron datos cercanos a
                                    la posicion'
            Caso correcto
                rta['Estado'] = True
                rta['Respuesta'] = {
                    'Calle': (str)
                    'Altura': (int)
                    'Calle2': (str)
                    'Calle3': (str)
                    'CalleCercana': (str)
                    'AlturaCercana': (int)
                    'Localidad': (str)
                    'Distancia': (int)
                    'Orientacion': (int)
                    'IdLocalidad': (int)
                    'IdCalle': (int)
                    'IdCalle2': (int)
                    'IdCalle3': (int)
                    'IdCalleCercana': (int)
                    'IdCalleParalela1': (int)
                    'NombreCalleParalela1': (str)
                    'IdCalleParalela2': (int)
                    'NombreCalleParalela2': (str)
                    'AlturaCalle2': (int)
                    'AlturaCalle3': (int)
                }

        Functions:
            validar_numero()
            obtener_calles_paralelas()
        '''

        # Obtenemos los segmentos mas cercanos
        rta_segmentos = self._obtener_segmentos_por_latlon(latitud, longitud)
        # Si fallo, salimos
        if not rta_segmentos['Estado']:
            return rta_segmentos

        # Guardamos lo que nos devolvio la funcion
        calle_principal = rta_segmentos['Respuesta'].get('Principal')
        calles = rta_segmentos['Respuesta'].get('Cercanos')

        # Definimos la respuesta
        rta = {'Estado': False}

        # Hacemos la consulta, buscando las calles que esten cerca de nuestro
        # punto. Luego filtraremos quienes estan lo suficientemente cerca
        str_punto = 'POINT(%s %s)' % (str(longitud), str(latitud))
        # Hack para compilado
        str_vacio = ''
        # Definimos los datos
        datos = {
            'Calle': None,
            'Altura': None,
            'Calle2': None,
            'Calle3': None,
            'CalleCercana': None,
            'AlturaCercana': None,
            'Localidad': None,
            'Distancia': None,
            'Orientacion': None,
            'IdLocalidad': None,
            'IdCalle': None,
            'IdCalle2': None,
            'IdCalle3': None,
            'IdCalleCercana': None,
            'IdCalleParalela1': None,
            'NombreCalleParalela1': None,
            'IdCalleParalela2': None,
            'NombreCalleParalela2': None,
            'AlturaCalle2': None,
            'AlturaCalle3': None,
        }
        # Iniciamos la session
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles = self.gis2.db['calles']
            tabla_calles_nombres = self.gis2.db['calles_nombres']
        except:
            rta['Respuesta'] = _(u'Surgio un error con la base de datos.')
            return rta

        # Si no hay calle principal
        if calle_principal is None:
            # Si hay alguna cerca, usamos esa
            if len(calles) > 0:
                calle_cercana = calles[0]
            else:
                # Como no encontramos nada Vamos a buscar la calle mas cercana
                # al punto, a como maximo 1000 metros
                q_obj = session.query(
                    tabla_calles.gid,
                    tabla_calles.id_calle_nombre,
                    tabla_calles.desde_der,
                    tabla_calles.desde_izq,
                    tabla_calles.hasta_der,
                    tabla_calles.hasta_izq,
                    tabla_calles_nombres.nombre,
                    tabla_calles_nombres.id_ciudad,
                    func.st_distance(
                        func.st_geomfromtext(
                            str_punto, 4326
                        ),
                        tabla_calles.geom
                    ).label('distancia'),
                    func.st_astext(tabla_calles.geom).label('geom'),
                    func.st_astext(
                        func.st_line_interpolate_point(
                            func.st_linemerge(tabla_calles.geom),
                            func.st_line_locate_point(
                                func.st_linemerge(tabla_calles.geom),
                                func.st_geomfromtext(
                                    str_punto, 4326
                                )
                            )
                        )
                    ).label('punto_cercano'),
                ).outerjoin((tabla_calles_nombres, and_(
                        tabla_calles_nombres.id == tabla_calles.id_calle_nombre,
                    ))
                )
                q_obj = q_obj.filter(
                    and_(
                        func.st_distance(
                            func.st_geomfromtext(
                                str_punto, 4326
                            ),
                            tabla_calles.geom
                        ) <= 0.01,
                        tabla_calles_nombres.nombre != str_vacio,
                        tabla_calles_nombres.nombre is not None,
                    )
                )
                q_obj = q_obj.order_by('distancia')
                # Pedimos la calle mas cercana
                try:
                    calle_cercana = q_obj.first()
                except Exception:
                    # Cerramos la session
                    session.close()
                    # Salimos
                    rta['Respuesta'] = _('''Fallo al momento de realizar la
 consulta. Chequeé el estado y codificación de la base de datos''')
                    return rta
            # Cerramos session
            session.close()

            # Si no encontramos siquiera una calle cercana, salimos
            if calle_cercana is None:
                # Salimos
                rta['Estado'] = False
                rta['Respuesta'] = _(u'No se pudo encontrar ninguna calle \
cercana a esa posición')
                return rta

            # En cambio, si encontramos, salimos indicando la calle cercana
            # y la distancia, en metros:
            # Salimos
            # Guardamos el nombre de la calle en la respuesta
            datos['CalleCercana'] = calle_cercana.nombre.encode("latin1")

            # Guardamos el id de la calle cercana
            datos['IdCalleCercana'] = calle_cercana.id_calle_nombre

            # Guardamos el id de la localidad
            datos['IdLocalidad'] = calle_cercana.id_ciudad

            # Analizamos donde esta el punto en nuestra calle encontrada
            # Procesamos los puntos obtenidos
            puntos = self.f_geo.procesar_geometria(calle_cercana.geom)
            # Para obtener la orientacion, obtenemos primero los datos del
            # punto mas cercano de la calle mas cercana
            punto_cercano = self.f_geo.procesar_geometria(
                calle_cercana.punto_cercano
            )
            # Calculamos la distancia absoluta desde nuestro punto a ambos
            # vertices del segmentos
            dist_a_punto1 = self.f_geo.mts_entre_puntos(
                puntos[0][1],
                puntos[0][0],
                punto_cercano[0][1],
                punto_cercano[0][0],
            )
            dist_a_punto2 = self.f_geo.mts_entre_puntos(
                puntos[1][1],
                puntos[1][0],
                punto_cercano[0][1],
                punto_cercano[0][0],
            )
            # Obtenemos la proporcion desde el inicio en relacion a todo
            # el tramo
            proporcion = dist_a_punto1 / (dist_a_punto1 + dist_a_punto2)
            # Declaramos la posible altura
            altura = ''
            # Vamos a tomar en cuenta primero la altura de la derecha
            if calle_cercana.desde_der != 0 and \
               calle_cercana.desde_der is not None and \
               calle_cercana.hasta_der != 0 and \
               calle_cercana.hasta_der is not None:
                # Vamos a obtenr la proporcion del a altura
                delta = abs(calle_cercana.desde_der - calle_cercana.hasta_der)
                delta = int(float(delta) * proporcion)
                # Sumamos el delta de la altura, para indicar la mitad
                # de la cuadra
                if calle_cercana.desde_der < calle_cercana.hasta_der:
                    altura = str(calle_cercana.desde_der + delta)
                else:
                    altura = str(calle_cercana.hasta_der + delta)
            # Sino, tomamos la izquierda
            elif calle_cercana.desde_izq != 0 and \
                 calle_cercana.desde_izq is not None and \
                 calle_cercana.hasta_izq != 0 and \
                 calle_cercana.hasta_izq is not None:
                # Obtenemos la proporcion de la altura
                delta = abs(calle_cercana.desde_izq - calle_cercana.hasta_izq)
                delta = int(float(delta) * proporcion)
                # Sumamos el delta de la altura, para indicar la mitad
                # de la cuadra
                if calle_cercana.desde_izq < calle_cercana.hasta_izq:
                    altura = str(calle_cercana.desde_izq + delta)
                else:
                    altura = str(calle_cercana.hasta_izq + delta)
            # Gurdamos la altura
            datos['AlturaCercana'] = altura
            # Guardamos la localidad
            rta_localidad = self.ciudad.obtener_ciudad(calle_cercana.id_ciudad)
            localidad = '(localidad no encontrada)'
            if rta_localidad['Estado']:
                localidad = rta_localidad['Respuesta']['Nombre']
            datos['Localidad'] = localidad
            # Guardamos la distancia a la calle
            datos['Distancia'] = self.f_geo.mts_entre_puntos(
                latitud, longitud,
                punto_cercano[0][1], punto_cercano[0][0]
            )
            # Guardamos la orientacion donde esta la calle cercana
            datos['Orientacion'] = self.f_geo.obtener_orientacion(
                punto_cercano[0][1], punto_cercano[0][0],
                latitud, longitud,
            )

            # Si paralelas es True busco las calles paralelas
            # de la calle cercana obtenida
            if paralelas:
                # Obtengo las calles paralelas del segmento principal
                id_segmento = calle_cercana.gid
                rta_obtener_calles_paralelas = self.obtener_calles_paralelas(
                                                                id_segmento)
                if rta_obtener_calles_paralelas['Estado']:
                    calles_paralelas = rta_obtener_calles_paralelas['Respuesta']

                    datos['IdCalleParalela1'] = \
                                    calles_paralelas['id_calle_paralela_1']
                    datos['NombreCalleParalela1'] = \
                                    calles_paralelas['nombre_calle_paralela_1']
                    datos['IdCalleParalela2'] = \
                                        calles_paralelas['id_calle_paralela_2']
                    datos['NombreCalleParalela2'] = \
                                    calles_paralelas['nombre_calle_paralela_2']

            # Generamos la respuesta
            rta['Estado'] = True
            rta['Respuesta'] = datos
            return rta

        # Si no encontramos siquiera una calle cercana, salimos
        if calle_principal is None:
            # Salimos
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo encontrar ninguna calle \
cercana a esa posición')
            return rta

        # Si paralelas es True busco las calles paralelas de la calle principal
        if paralelas:
            # Obtengo las calles paralelas del segmento principal
            id_segmento = calle_principal.gid
            rta_obtener_calles_paralelas = self.obtener_calles_paralelas(
                                                                id_segmento)
            if rta_obtener_calles_paralelas['Estado']:
                calles_paralelas = rta_obtener_calles_paralelas['Respuesta']

                datos['IdCalleParalela1'] = \
                                    calles_paralelas['id_calle_paralela_1']
                datos['NombreCalleParalela1'] = \
                                    calles_paralelas['nombre_calle_paralela_1']
                datos['IdCalleParalela2'] = \
                                        calles_paralelas['id_calle_paralela_2']
                datos['NombreCalleParalela2'] = \
                                    calles_paralelas['nombre_calle_paralela_2']

        # En cambio, si encontramos una calle principal donde este
        # nuestro punto, buscamos los datos de esta
        # Guardamos el nombre de la calle en la respuesta
        datos['Calle'] = calle_principal.nombre.encode("latin1")

        # Guardo el id de la localidad
        datos['IdLocalidad'] = calle_principal.id_ciudad
        # Guardo el id de la calle
        datos['IdCalle'] = calle_principal.id_calle_nombre

        # Guardamos la localidad
        localidad = '(localidad no encontrada)'
        rta_localidad = self.ciudad.obtener_ciudad(
            calle_principal.id_ciudad
        )
        if rta_localidad['Estado']:
            localidad = rta_localidad['Respuesta']['Nombre']
        datos['Localidad'] = localidad
        # Analizamos donde esta el punto en nuestra calle encontrada
        # Procesamos los puntos obtenidos
        puntos = self.f_geo.procesar_geometria(calle_principal.geom)

        # Calculamos la distancia absoluta desde nuestro punto a ambos
        # vertices del segmentos
        dist_a_punto1 = self.f_geo.mts_entre_puntos(
            puntos[0][1],
            puntos[0][0],
            latitud,
            longitud
        )
        dist_a_punto2 = self.f_geo.mts_entre_puntos(
            puntos[1][1],
            puntos[1][0],
            latitud,
            longitud
        )

        # Obtenemos la proporcion desde el inicio en relacion a todo
        # el tramo
        proporcion = dist_a_punto1 / (dist_a_punto1 + dist_a_punto2)
        # Declaramos la posible altura
        altura = None
        # Si tenemos altura izquierda, calculamos la altura aproximada
        if calle_principal.desde_izq is not None and \
           calle_principal.hasta_izq is not None and \
           calle_principal.desde_izq != 0 and calle_principal.hasta_izq != 0:
            # Solo si hay altura
            prop_altura = calle_principal.hasta_izq - calle_principal.desde_izq
            prop_altura *= proporcion
            altura = int(calle_principal.desde_izq + prop_altura)

        # Sino, calculamos con la altura derecha
        elif calle_principal.desde_der is not None and \
             calle_principal.hasta_der is not None and \
             calle_principal.desde_der != 0 and calle_principal.hasta_der != 0:
            # Solo si hay altura
            prop_altura = calle_principal.hasta_der - calle_principal.desde_der
            prop_altura *= proporcion
            altura = int(calle_principal.desde_der + prop_altura)

        # Si encontramos alguna altura, salimos
        if altura is not None:
            # Cerramos la session
            session.close()
            # Guardamos la altura
            datos['Altura'] = altura

            if not esquina:
                # Como no vamos a buscar la esquina, salimos
                rta['Estado'] = True
                rta['Respuesta'] = datos
                return rta

        # Si no tenemos altura, buscamos las calles entre las que está
        if len(calles) >= 2:
            # Usamos la segunda calle encontrada por la bd, es la mas
            # cercana y si corresponde al margen permitido, a
            # una esquina.
            # Buscamos el nombres de esta calle
            if calles[1].distancia < 0.0005:
                datos['Calle2'] = calles[1].nombre.encode("latin1")
                datos['IdCalle2'] = calles[1].id_calle_nombre

                # Cerramos la session
                session.close()
                # Salimos
                rta['Estado'] = True
                rta['Respuesta'] = datos
                return rta

        # Si no tenemos nada con que trabajar, volvemos a consultar
        # Buscamos las lineas que pasen por los extremos y que
        # no sea la qe ya tenemos
        q_obj = session.query(
            tabla_calles.id_calle_nombre,
            tabla_calles_nombres.nombre,
            func.st_distance(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_punto,
                    4326
                ).label('punto'),
            ).label('distancia'),
            tabla_calles.desde_izq,
        ).outerjoin((tabla_calles_nombres, and_(
                tabla_calles_nombres.id == tabla_calles.id_calle_nombre,
          ))
        )
        q_obj = q_obj.filter(
            and_(
                func.st_intersects(
                    tabla_calles.geom,
                    func.st_geomfromtext(
                        calle_principal.geom,
                        4326
                    )
                ),
            tabla_calles_nombres.nombre != calle_principal.nombre,
            tabla_calles_nombres.nombre != str_vacio,
            tabla_calles_nombres.nombre is not None,
        )).order_by('distancia')

        # Obtenemos las calles
        try:
            calles_entre = q_obj.all()
        except:
            # Cerramos la session
            session.close()
            # Salimos
            rta['Respuesta'] = _(
                u'Fallo al momento de realizar la consulta. Chequeé el estado y codificación de la base de datos'
            )
            return rta

        # Vamos a limpiar las calles repetidas
        calles_validas = []
        calles_a_borrar = []
        for c in calles_entre:
            if c.nombre in calles_validas:
                calles_a_borrar.append(c)
            else:
                calles_validas.append(c.nombre)

        # Limpiamos las calles repetidas
        for c in calles_a_borrar:
            calles_entre.remove(c)

        # Vemos si guardamos la esquina
        if len(calles_entre) > 0:
            datos['Calle2'] = calles_entre[0].nombre.encode("latin1")
            datos['IdCalle2'] = calles_entre[0].id_calle_nombre
            datos['AlturaCalle2'] = calles_entre[0].desde_izq

        # Si esta, guardamos la otra calle entre la que esta la principal
        if len(calles_entre) > 1:
            datos['Calle3'] = calles_entre[1].nombre.encode("latin1")
            datos['IdCalle3'] = calles_entre[1].id_calle_nombre
            datos['AlturaCalle3'] = calles_entre[1].desde_izq

        # Cerramos la session
        session.close()
        # Salimos
        rta['Estado'] = True
        rta['Respuesta'] = datos
        return rta

    def obtener_segmentos(self, id_calle_nombre, id_segmento=None,
                                    latitud=None, longitud=None, limite=500):
        ''' Funcion para obtener los datos de varios o un segmento '''

        # Declaro str_punto como un str vacío
        str_punto = ''
        # Si tengo la latitud y la longitud
        if latitud is not None and longitud is not None:
            # Armo el PUNTO
            str_punto = 'POINT(%s %s)' % (str(longitud), str(latitud))

        # Definimos respuesta
        rta = {'Estado': False}
        # Iniciamos la session
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles = self.gis2.db['calles']
        except:
            rta['Respuesta'] = _(u'Surgio un error con la base de datos.')
            return rta
        # Buscamos los segmentos
        q_obj = session.query(
            tabla_calles.gid,
            tabla_calles.desde_der,
            tabla_calles.desde_izq,
            tabla_calles.hasta_der,
            tabla_calles.hasta_izq,
            tabla_calles.id_tipo,
            tabla_calles.sentido,
            tabla_calles.letra,
            func.st_astext(tabla_calles.geom).label('geom')
        )

        q_obj = q_obj.filter(
            tabla_calles.id_calle_nombre == id_calle_nombre
        )

        # Si nos especificaron un segmento, filtramos
        if id_segmento is not None:
            q_obj = q_obj.filter(tabla_calles.gid == id_segmento)

        # Si tengo el PUNTO
        if str_punto != '':
            # Ordeno por la cercanía al punto dado
            q_obj = q_obj.order_by(
                func.st_distance(
                    tabla_calles.geom,
                    func.st_geomfromtext(
                        str_punto, 4326
                    )
                )
            )
        else:
            # Si no, ordeno por gid
            q_obj = q_obj.order_by('gid').distinct()

        # Verifico que el limite sea valido.
        if isinstance(limite, int):
            # Le agrego un limit para que si tiene muchas calles no se tilde
            q_obj = q_obj.limit(limite)
        else:
            rta['Respuesta'] = _(u'Debe ingresar un límite válido.')
            return rta

        # Pedimos la calle mas cercana
        try:
            segmentos_tmp = q_obj.all()
        except:
            # Cerramos la session
            session.close()
            # Salimos
            rta['Respuesta'] = _(
                u'Fallo al momento de realizar la consulta. Chequeé el estado y codificación de la base de datos'
            )
            return rta
        # Cerramos la session
        session.close()
        # Si estan vacios, salimos
        if len(segmentos_tmp) == 0:
            if id_segmento is None:
                rta['Respuesta'] = _(u'La calle no tiene segmentos cargados.')
            else:
                rta['Respuesta'] = _(u'No se encontró el segmento buscado.')
            return rta
        # Recorremos los segmentos y los guardamos
        segmentos = []
        for segmento_tmp in segmentos_tmp:
            # Si esta vacio, salimos
            if segmento_tmp is None:
                continue
            puntos = self.f_geo.procesar_geometria(segmento_tmp.geom)
            for i in range(len(puntos)):
                puntos[i] = [puntos[i][1], puntos[i][0]]
            # Agregamos el punto
            segmentos.append({
                'Id': int(segmento_tmp.gid),
                'Puntos': puntos,
                'IdTipo': segmento_tmp.id_tipo,
                'Sentido': segmento_tmp.sentido,
                'Letra': segmento_tmp.letra,
                'DesdeIzq': segmento_tmp.desde_izq,
                'HastaIzq': segmento_tmp.hasta_izq,
                'DesdeDer': segmento_tmp.desde_der,
                'HastaDer': segmento_tmp.hasta_der
            })

        # Salimos correctamente
        rta['Estado'] = True
        rta['Respuesta'] = segmentos
        return rta

    def grabar_segmento(self, id_segmento=None, sentido=False, desde_der=None,
            hasta_der=None, desde_izq=None, hasta_izq=None, id_tipo=None,
            letra=None, id_calle_nombre=None, puntos=None, agregar_con_id_especifico=False):

        ''' Funcion para modificar los datos de un segmento '''

        # Declaramos la respuesta
        rta = {'Estado': False}

        # Validamos los datos
        try:
            # Si tenemos id, vamos a validarlo
            if id_segmento is not None:
                self.validar_entero(id_segmento)
            # Validamos las alturas, si nos las pasan como datos
            if desde_der is not None and desde_der != '':
                self.validar_entero(desde_der)
            if hasta_der is not None and hasta_der != '':
                self.validar_entero(hasta_der)
            if desde_izq is not None and desde_izq != '':
                self.validar_entero(desde_izq)
            if hasta_izq is not None and hasta_izq != '':
                self.validar_entero(hasta_izq)
            self.validar_entero(sentido)
            # Si nos pasan el id_calle_nombre lo validamos
            if id_tipo is not None:
                self.validar_entero(id_tipo)
            # Si nos pasan el id_calle_nombre lo validamos
            if id_calle_nombre is not None:
                self.validar_entero(id_calle_nombre)
            # Si nos pasan puntos, validamos
            if puntos is not None:
                self.f_geo.validar_puntos_geometria(puntos)
        except Exception, e:
            rta['Exception'] = e
            rta['Respuesta'] = _(u'Los datos ingresados no son válidos.')
            return rta

        # Creamos el diccionario para acutalizar
        segmento = {'sentido': sentido}

        if segmento is not False:

            # Declaro la variable one_way en doble sentido(B).
            one_way = 'B'

            # Si es de un solo sentido.
            if sentido == 1:
                one_way = 'FT'

            segmento['one_way'] = one_way

        # Si hay alguna altura, la cambiamos
        if desde_der is not None:
            # Si nos pasaron un string vacio, ponemos a none la alutra
            if desde_der == '':
                desde_der = None
            segmento['desde_der'] = desde_der
        if hasta_der is not None:
            # Si nos pasaron un string vacio, ponemos a none la alutra
            if hasta_der == '':
                hasta_der = None
            segmento['hasta_der'] = hasta_der
        if desde_izq is not None:
            # Si nos pasaron un string vacio, ponemos a none la alutra
            if desde_izq == '':
                desde_izq = None
            segmento['desde_izq'] = desde_izq
        if hasta_izq is not None:
            # Si nos pasaron un string vacio, ponemos a none la alutra
            if hasta_izq == '':
                hasta_izq = None
            segmento['hasta_izq'] = hasta_izq
        # Si hay id_tipo, lo cambiamos
        if id_tipo is not None:
            segmento['id_tipo'] = id_tipo
            segmento['max_speed_km'] = self.topologia.obtener_velocidad_maxima(
                id_tipo
            )
        # Si hay letra, la cambiamos
        if letra is not None:
            segmento['letra'] = letra
        # Si hay id de calle_nombre, lo agregamos
        if id_calle_nombre is not None:
            # Validamos que exista la calle
            rta_calle = self.obtener_nombre_calle(id_calle_nombre)
            if not rta_calle['Estado']:
                return rta_calle
            # Si no salimos, vamos a modificar el id de la calle
            segmento['id_calle_nombre'] = id_calle_nombre
        # Si hay puntos, los agregamos
        if puntos is not None:
            segmento['geom'] = func.st_geomfromtext(
                self.f_geo.generar_geometria_linea(puntos),
                4326
            )

            # Trato de encontrar los nodos correspondientes.
            rta_nodos = self.reordenar_nodos_del_segmento(segmento['geom'])
            if rta_nodos['Estado'] is not True:
                return rta_nodos

            segmento['source'] = rta_nodos['Dato']['source']
            segmento['target'] = rta_nodos['Dato']['target']

            # Obtengo los origenes y destinos.
            length = func.st_length(segmento['geom'], True)

            # length = round(length, 6)

            # Calculo la longitud en kilometros.
            len_km = length / 1000

            # len_km = round(len_km, 6)

            # Obtengo la velidad máxima permitida.
            max_speed_km = self.topologia.obtener_velocidad_maxima(id_tipo)

            segmento['cost_len'] = length
            segmento['rcost_len'] = length
            segmento['len_km'] = len_km
            segmento['max_speed_km'] = max_speed_km

            if max_speed_km != 0:
                # Calculo el tiempo.
                cost_time = (len_km / max_speed_km) * 3600

            else:
                cost_time = 0

            segmento['cost_time'] = cost_time

        # Iniciamos la session
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles = self.gis2.db['calles']
        except:
            rta['Respuesta'] = _(u'Surgio un error con la base de datos.')
            return rta
        # Si no tenemos id vamos a agregar
        if id_segmento is None or agregar_con_id_especifico is True:
            # Se obtine el ultimo id segmento de las calles
            session_gis2 = self.gis2.binded_session()
            calles = self.gis2.tabla('calles')
            q_obj = session_gis2.query(calles.gid)
            q_obj = q_obj.order_by(calles.gid.desc())
            registro = q_obj.first()
            session_gis2.close()
            if registro is None:
                rta['Estado'] = False
                rta['Respuesta'] = \
                    _(u'No se pudo obtener el último id de calles nombres')
                return rta

            if agregar_con_id_especifico is True and id_segmento is not None:
                id_nueva_calle = id_segmento
            else:
                id_nueva_calle = registro.gid + 1
            segmento['gid'] = id_nueva_calle

            rta_agregar = self.gis2.insertar(
                'calles',
                'gid',
                **segmento
            )

            # Si no se agrego correctamente, salimos
            if rta_agregar['Estado'] is not True:
                rta['Respuesta'] = _(
                    u'No se pudo agregar el segmento.'
                )
                return rta
            # Sino, salimos bien
            rta['Estado'] = True
            rta['DatosExtra'] = {
                'gid': rta_agregar['ValorCampo']
            }
            rta['Respuesta'] = _(u'Se agregó correctamente el segmento.')
            return rta
        # Sino, actualizamos
        else:
            # Buscamos el segmento
            q_obj = session.query(
                tabla_calles.gid,
                tabla_calles.source,
                tabla_calles.target
            ).filter(
                tabla_calles.gid == id_segmento
            )
            if int(q_obj.count()) == 0:
                session.close()
                rta['Respuesta'] = _(u'No se encontro el segmento buscado.')
                return rta
            # Intentamos actualizar
            try:

                # Trato de obtener el registro.
                registro = q_obj.first()

                q_obj.update(segmento, synchronize_session=False)
                session.commit()
                session.close()

                # Verifico si hay que eliminar el nodo anterior.
                rta_verificar = self.verificar_eliminar_nodo(registro.source)
                if rta_verificar['Estado'] is not True:
                    return rta_verificar

                # Verifico si hay que eliminar el nodo anterior.
                rta_verificar = self.verificar_eliminar_nodo(registro.target)
                if rta_verificar['Estado'] is not True:
                    return rta_verificar

            except Exception:
                session.rollback()
                session.close()
                rta['Respuesta'] = _(u'No se pudo actualizar el segmento.')
                return rta
            # Enviamos la respuesta
            rta['Estado'] = True
            rta['Respuesta'] = _(u'Se actualizó correctamente el segmento.')
            return rta

    def obtener_calle(self, id, latitud=None, longitud=None,
                      obtener_segmentos=True):
        ''' Funcion para obtener todos los datos de una calle a partir del
        id de la misma

        Argumentos:
            id (int)
            latitud (float)
            longitud (float)

        Retorna:
            Caso incorrecto
                rta['Estado'] = False
                rta['Respuesta'] = 'No se encontraron datos cercanos a
                                    la posicion'
            Caso correcto
                rta['Estado'] = True
                rta['Respuesta'] = [{
                    'IdCalle': (int) id del nombre de la calle
                    'Nombre': (str)
                    'Etiqueta': (str)
                    'Segmentos': {
                        'Id': (int) el id de la calle
                        'Tipo': (int) el tipo de la calle
                        'Puntos':[{
                            'Latitud': (long)
                            'Longitud': (long)
                        }]
                        'DesdeIzq':  (int) altura desde izquierda
                        'HastaIzq':  (int) altura hasta izquierda
                        'DesdeDer':  (int) altura desde derecha
                        'HastaDer':  (int) altura hasta derecha
                    }
                }]
        '''

        # Definimos respuesta
        rta = {'Estado': False}
        # Iniciamos la session
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles_nombres = self.gis2.db['calles_nombres']
        except:
            rta['Respuesta'] = _(u'Surgio un error con la base de datos.')
            return rta
        # Buscamos la calle
        q_obj = session.query(tabla_calles_nombres).filter(
            tabla_calles_nombres.id == id
        )
        # Pedimos la calle mas cercana
        try:
            calle_tmp = q_obj.first()
        except:
            # Cerramos la session
            session.close()
            # Salimos
            rta['Respuesta'] = _(
                u'Fallo al momento de realizar la consulta. Chequeé el estado y codificación de la base de datos'
            )
            return rta
        # Cerramos la session
        session.close()
        # Salimos si no existe la calle
        if calle_tmp is None:
            rta['Respuesta'] = _('No existe una calle con ese id.')
            return rta

        # Obtenemos lo segmentos
        segmentos = []

        if obtener_segmentos is True:
            rta_segmentos = self.obtener_segmentos(
                id, latitud=latitud, longitud=longitud
            )
            if rta_segmentos['Estado']:
                segmentos = rta_segmentos['Respuesta']

        # Obtenemos la localidad
        rta_ciudad = self.ciudad.obtener_ciudad(calle_tmp.id_ciudad)
        ciudad = '(localidad no encontrada)'
        if rta_ciudad['Estado']:
            ciudad = rta_ciudad['Respuesta']
        # Armamos los datos de la calle
        calle = {
            'Nombre': calle_tmp.nombre,
            'Etiqueta': calle_tmp.etiqueta,
            'Ciudad': ciudad,
            'IdCalleNombre': int(calle_tmp.id),
            'Segmentos': segmentos
        }
        # Salimos correctamente
        rta['Estado'] = True
        rta['Respuesta'] = calle
        return rta

    def obtener_calle_por_latlon(self, latitud, longitud):
        ''' Esta funcion devuelve una todos los puntos de una calle, junto
        con los id de cada segmento y las alturas correspondientes. Si no
        encuentra nada cercano, devuelve un error.

        Argumentos:
            latitud (numero)
            longitud (numero)

        Retorna:
            Caso incorrecto
                rta['Estado'] = False
                rta['Respuesta'] = 'No se encontraron datos cercanos a
                                    la posicion'
            Caso correcto
                rta['Estado'] = True
                rta['Respuesta'] = [{
                    'IdCalle': (int) id del nombre de la calle
                    'Nombre': (str)
                    'Etiqueta': (str)
                    'Segmentos': {
                        'Id': (int) el id de la calle
                        'Tipo': (int) el tipo de la calle
                        'Puntos':[{
                            'Latitud': (long)
                            'Longitud': (long)
                        }]
                        'DesdeIzq':  (int) altura desde izquierda
                        'HastaIzq':  (int) altura hasta izquierda
                        'DesdeDer':  (int) altura desde derecha
                        'HastaDer':  (int) altura hasta derecha
                    }
                }]

        Functions:
            validar_numero()

        '''

        # Obtenemos los segmentos mas cercanos
        rta_segmentos = self._obtener_segmentos_por_latlon(
            latitud, longitud, vacios=True
        )
        # Si fallo, salimos
        if not rta_segmentos['Estado']:
            return rta_segmentos

        # Definimos la respuesta
        rta = {'Estado': False}
        # Vemos si encontramos una calle realmente cercana
        segmento_principal = rta_segmentos['Respuesta'].get('Principal')
        if segmento_principal is None:
            # Salimos indicando que no se encontro la calle buscada
            rta['Respuesta'] = _(u'No se encontro la calle.')
            return rta
        # Sino, obtenemos la calle
        return self.obtener_calle(
            segmento_principal.id_calle_nombre, latitud, longitud
        )

    def grabar(self, id_calle=None, nombre='', etiqueta=None, id_ciudad=None,
               agregar_con_id_especifico=False):

        ''' Funcion para modificar los datos de una calle '''

        # Declaramos la respuesta
        rta = {'Estado': False}
        # Validamos los datos
        try:
            # Si tenemos id lo validamos
            if id_calle is not None:
                self.validar_entero(id_calle)
            self.validar_string(nombre)
            # Validamos la etiqueta
            if etiqueta is not None:
                self.validar_string(etiqueta)
            else:
                etiqueta = ''
            # Validamos el id de ciudad
            if id_ciudad is not None:
                self.validar_entero(id_ciudad)
        except:
            rta['Respuesta'] = _(u'Los datos ingresados no son válidos.')
            return rta
        # Creamos el diccionario para acutalizar
        calle = {
            'nombre': nombre,
            'etiqueta': etiqueta
        }
        # Si hay id de calle_nombre, lo agregamos
        if id_ciudad is not None:
            # Validamos que la ciudad exista
            rta_ciudad = self.ciudad.obtener_ciudad(id_ciudad)
            if not rta_ciudad['Estado']:
                return rta_ciudad
            # Si no fallo, agregamos la ciudad
            calle['id_ciudad'] = id_ciudad
        # Iniciamos la session
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles_nombres = self.gis2.db['calles_nombres']
        except:
            rta['Respuesta'] = _(u'Surgió un error con la base de datos.')
            return rta

        # Si no tenemos id vamos a agregar
        if id_calle is None or agregar_con_id_especifico is True:
            # Llamo a la función obtener_siguiente_id_calles_nombres
            rta_obtener_siguiente_id_calles_nombres = \
                self.obtener_siguiente_id_calles_nombres()
            # Si no se pudo obtener el sigueinte id para agregar la calle
            if rta_obtener_siguiente_id_calles_nombres['Estado'] is False:
                # Retorno el error
                return rta_obtener_siguiente_id_calles_nombres

            if agregar_con_id_especifico is True and id_calle is not None:
                calle['id'] = id_calle
            else:
                # Guardo el id que le corresponde a la calle a agregar
                calle['id'] = rta_obtener_siguiente_id_calles_nombres['Respuesta']

            # Agrego la calle
            rta_agregar = self.gis2.insertar(
                'calles_nombres',
                'id',
                **calle
            )
            # Si no se agrego correctamente, salimos
            if rta_agregar['Estado'] is not True:
                rta['Respuesta'] = _(
                    u'No se pudo agregar la calle.'
                )
                return rta

            # Sino, salimos bien
            rta['Estado'] = True
            rta['DatosExtra'] = {
                'id': rta_agregar['ValorCampo']
            }
            rta['Respuesta'] = _(u'Se agregó correctamente la calle.')
            return rta
        else:
            # Buscamos la calle
            q_obj = session.query(tabla_calles_nombres.id).filter(
                tabla_calles_nombres.id == id_calle
            )
            if int(q_obj.count()) == 0:
                session.close()
                rta['Respuesta'] = _(u'No se encontró la calle buscada.')
                return rta
            # Intentamos actualizar
            try:
                q_obj.update(calle, synchronize_session=False)
                session.commit()
                session.close()
            except:
                session.rollback()
                session.close()
                rta['Respuesta'] = _(u'No se pudo actualizar la calle.')
                return rta
            # Enviamos la respuesta
            rta['Estado'] = True
            rta['Respuesta'] = _(u'Se actualizó correctamente la calle.')
            return rta

    def borrar(self, id_calle):
        '''' Funcion que toma como parametro el id de una calle nombre
        y borra la misma del sistema, asi como tambien todos los segmentos
        de la misma

        Argumentos:
            id_calle: (int)
        Return:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Declaramos la respuesta
        rta = {'Estado': False}
        # Validamos los datos
        try:
            # Si tenemos id lo validamos
            self.validar_entero(id_calle)
        except:
            rta['Respuesta'] = _(u'Los datos ingresados no son válidos.')
            return rta
        # Iniciamos la session
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles = self.gis2.db['calles']
            tabla_calles_nombres = self.gis2.db['calles_nombres']
        except:
            rta['Respuesta'] = _(u'Surgió un error con la base de datos.')
            return rta
        # Buscamos la calle
        q_obj = session.query(tabla_calles_nombres.id).filter(
            tabla_calles_nombres.id == id_calle
        )
        if int(q_obj.count()) == 0:
            session.close()
            rta['Respuesta'] = _(u'No se encontró la calle buscada.')
            return rta
        # Intentamos borrar los segmentos de la calle
        q_obj2 = session.query(tabla_calles.gid).filter(
            tabla_calles.id_calle_nombre == id_calle
        )
        # Intentamos borrar los segmentos de la calle
        try:
            q_obj2.delete(synchronize_session=False)
        except:
            session.rollback()
            session.close()
            rta['Respuesta'] = _(
                u'No se pudo borrar los segmentos de la calle.'
            )
            return rta
        # Intentamos borrar la calle
        try:
            q_obj.delete(synchronize_session=False)
            session.commit()
            session.close()
        except:
            session.rollback()
            session.close()
            rta['Respuesta'] = _(u'No se pudo borrar la calle.')
            return rta
        # Enviamos la respuesta
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se borró correctamente la calle.')
        return rta

    def borrar_segmento(self, id_segmento):
        '''' Funcion que toma como parametro el id de una calle nombre
        y borra la misma del sistema, asi como tambien todos los segmentos
        de la misma

        Argumentos:
            id_calle: (int)
        Return:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Declaramos la respuesta
        rta = {'Estado': False}
        # Validamos los datos
        try:
            # Si tenemos id lo validamos
            self.validar_entero(id_segmento)
        except:
            rta['Respuesta'] = _(u'Los datos ingresados no son válidos.')
            return rta
        # Iniciamos la session
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles = self.gis2.db['calles']
        except:
            rta['Respuesta'] = _(u'Surgió un error con la base de datos.')
            return rta
        # Buscamos la calle
        q_obj = session.query(
            tabla_calles.gid,
            tabla_calles.source,
            tabla_calles.target
        ).filter(
            tabla_calles.gid == id_segmento
        )
        if int(q_obj.count()) == 0:
            session.close()
            rta['Respuesta'] = _(u'No se encontró el segmento buscado.')
            return rta
        # Intentamos borrar la calle
        try:

            # Trato de obtener el registro.
            registro = q_obj.first()

            q_obj.delete(synchronize_session=False)
            session.commit()
            session.close()

            # Verifico si hay que eliminar el nodo anterior.
            rta_verificar = self.verificar_eliminar_nodo(registro.source)
            if rta_verificar['Estado'] is not True:
                return rta_verificar

            # Verifico si hay que eliminar el nodo anterior.
            rta_verificar = self.verificar_eliminar_nodo(registro.target)
            if rta_verificar['Estado'] is not True:
                return rta_verificar

        except Exception, e:
            session.rollback()
            session.close()
            rta['Exception'] = e
            rta['Respuesta'] = _(u'No se pudo borrar el segmento.')
            return rta
        # Enviamos la respuesta
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se borró correctamente el segmento.')
        return rta

    def generar_direccion(self, calle1=None, calle2=None, altura_nro=None,
            altura_bis=None, altura_letra=None, departamento=None, sitio=None):

        '''
        Esta función se encarga de generar un String a partir de los datos que
        se le pasan como argumentos. Dependiendo de si se agrego o no una calle
        principal, una altura o un sitio (a rasgos generales) arma el String
        resultante.

        Al utilizar la misma podemos mantener una coherencia en las direcciones
        almacenadas en la base de datos

         Args:
               calle1 (str)
               calle2 (str)
               altura_nro (str)
               altura_bis (str)
               altura_letra (str)
               departamento (str)
               sitio (str)

        Returns:
            direccion (str)
            None

        Functions:
            validar_string()

        '''

        agregar_esquina = True

        try:
            calle1 = calle1.decode('latin1')
        except:
            pass
        try:
            calle2 = calle2.decode('latin1')
        except:
            pass

        direccion = u''
        if calle1 is not None:

            if not self.validar_string(calle1):
                return self.validar_string(calle1)

            direccion = calle1

            if altura_nro is not None:

                if not self.validar_string(altura_nro):
                    return self.validar_string(altura_nro)

                if altura_nro.isdigit():
                    direccion += u' ' + altura_nro
                    if altura_letra is not None:

                        if not self.validar_string(altura_letra):
                            return self.validar_string(altura_letra)

                        direccion += u' ' + altura_letra
                    if altura_bis is not None:

                        if not self.validar_string(altura_bis):
                            return self.validar_string(altura_bis)

                        if altura_bis == u'BIS':
                            direccion += u' ' + altura_bis

                    if departamento is not None:

                        if not self.validar_string(departamento):
                            return self.validar_string(departamento)

                        direccion += u'  ' + departamento
                else:
                    if calle2 is not None:

                        if not self.validar_string(calle2):
                            return self.validar_string(calle2)

                        direccion += u' y ' + calle2

                        agregar_esquina = False

            else:
                if calle2 is not None:

                    if not self.validar_string(calle2):
                        return self.validar_string(calle2)

                    direccion += u' y ' + calle2

                    agregar_esquina = False

            direccion.strip()
        else:
            if sitio is not None:

                if not self.validar_string(sitio):
                    return self.validar_string(sitio)

                direccion = sitio
                direccion.strip()
            else:
                direccion = None

        if agregar_esquina is True and calle2 is not None and calle2 != '':
            direccion += u' Esq. ' + calle2

        return direccion

    def obtener_calles_paralelas(self, id_segmento):
        '''
            Esta función se encarga de obtener las dos calles paralelas de
            una calle dada.

            Argumento:
                id_segmento (int)

            Funciones:
                CallePostgres().obtener_nombre_calle

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['id_calle_paralela_1'] (int)
                                        ['nombre_calle_paralela_1'] (str)
                                        ['id_calle_paralela_2'] (int)
                                        ['nombre_calle_paralela_2'] (str)
        '''

        rta = {}

        # Fix para SqlAlchemy
        none = None

        # Abro la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')

        # Obtengo la tabla calles_nombres
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Obtengo datos del segmento dado
        q_obj = session.query(
            calles.id_calle_nombre,
            calles.desde_izq,
            func.st_astext(calles.geom).label('geom'),
            calles_nombres.id_ciudad,
        )
        q_obj = q_obj.join((
            calles_nombres,
            calles.id_calle_nombre == calles_nombres.id
        ))

        q_obj = q_obj.filter(calles.gid == id_segmento)

        # Guardo las geometrías obtenidas de la calle dada
        calle_principal = q_obj.first()

        # Si no se encontró la geometría de la calle dada retorno un mensaje
        if calle_principal is None:
            # Cierro la sesión de la base de datos
            session.close
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró la calle dada')
            return rta

        # Si la calle principal no tiene altura retorno un error
        if calle_principal.desde_izq is None:
            # Cierro la sesión de la base de datos
            session.close
            rta['Estado'] = False
            rta['Respuesta'] = _(u'La calle no tiene altura')
            return rta

        # Armo una subquery para obtener todas las calles nombres de la ciudad
        q_obj3 = session.query(calles_nombres.id)
        q_obj3 = q_obj3.filter(
            calles_nombres.id_ciudad == calle_principal.id_ciudad
        )
        q_obj3 = q_obj3.subquery()

        # Obtengo las calles paralelas a la geometría dada
        q_obj2 = session.query(
            calles.id_calle_nombre,
        )

        # Filtro solo por las calles de la ciudad dada
        q_obj2 = q_obj2.filter(calles.id_calle_nombre.in_(q_obj3))

        # Filtro por la cercanía de las calles y la altura del segmento
        q_obj2 = q_obj2.filter(
            and_(
                func.st_distance(
                    calles.geom,
                    func.st_geomfromtext(calle_principal.geom, 4326),
                ) < 0.0015,
                and_(
                    calles.desde_izq != none,
                    calles.desde_izq != none,
                ),
                and_(
                    calles.desde_izq <= calle_principal.desde_izq + 99,
                    calles.desde_izq >= calle_principal.desde_izq - 99,
                ),
                calles.id_calle_nombre != calle_principal.id_calle_nombre,
            )
        )
        # Ordeno por las más cercanas
        q_obj2 = q_obj2.order_by(
            func.st_distance(
                calles.geom,
                func.st_geomfromtext(calle_principal.geom, 4326),
            ).asc()
        )

        # Guardo todos los registros encontrados
        registros = q_obj2.all()

        # Cierro la sesión de la base de datos
        session.close()

        # Si no se encontraron calles paralelas retorno un mensaje
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron calles paralelas')
            return rta

        # Saco las calles repetidas
        paralelas_no_repetidas = []
        for registro in registros:
            if registro.id_calle_nombre not in paralelas_no_repetidas:
                paralelas_no_repetidas.append(registro.id_calle_nombre)

        # Obtengo el nombre de la primer calle paralela
        id_calle_paralela_1 = paralelas_no_repetidas[0]
        rta_obtener_nombre_calle = self.obtener_nombre_calle(
                                                        id_calle_paralela_1)
        if rta_obtener_nombre_calle['Estado']:
            nombre_calle_paralela_1 = rta_obtener_nombre_calle['Respuesta']
        else:
            id_calle_paralela_1 = None
            nombre_calle_paralela_1 = ''

        # Si tengo una segunda calle paralela obtengo su nombre también
        if len(paralelas_no_repetidas) > 1:
            id_calle_paralela_2 = paralelas_no_repetidas[1]
            rta_obtener_nombre_calle = self.obtener_nombre_calle(
                                                            id_calle_paralela_2)
            if rta_obtener_nombre_calle['Estado']:
                nombre_calle_paralela_2 = rta_obtener_nombre_calle['Respuesta']
            else:
                id_calle_paralela_2 = None
                nombre_calle_paralela_2 = ''
        else:
            id_calle_paralela_2 = None
            nombre_calle_paralela_2 = ''

        # Retorno los datos obtenidos
        rta['Estado'] = True
        dic = {}
        dic['id_calle_paralela_1'] = id_calle_paralela_1
        dic['nombre_calle_paralela_1'] = nombre_calle_paralela_1
        dic['id_calle_paralela_2'] = id_calle_paralela_2
        dic['nombre_calle_paralela_2'] = nombre_calle_paralela_2
        rta['Respuesta'] = dic
        return rta

    def trazar_alturas(self, segmentos_a_editar, id_calle_nombre, id_tipo,
                                            altura_inicial, salto, bis, letra):
        '''
            Esta función se encarga de modificar los datos de cada segmento
            recibido con los datos correspondientes.

            Argumentos:
                id_calle_nombre_original (int)
                segmentos_a_editar (list de int)
                id_calle_nombre (int)
                id_tipo (int)
                altura_inicial (int)
                salto (int)
                bis (bool)
                letra (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Guardo la tabla calles
        calles = self.gis2.tabla('calles')

        # Guardo la altura inicial como la altura inicial del primer segmento
        altura_inicial_segmento = altura_inicial

        # Recorro la lista de segmentos a editar
        for segmento_a_editar in segmentos_a_editar:
            # Obtengo la calle (segmento) que quiero modificar
            q_obj = session.query(calles.gid)

            # Filtro por el gid recibido
            q_obj = q_obj.filter(calles.gid == segmento_a_editar)

            # Guardo los datos a modificar
            dic = {}
            dic['id_calle_nombre'] = id_calle_nombre
            dic['id_tipo'] = id_tipo
            dic['letra'] = letra
            dic['modificada'] = 1

            # Obtengo la altura final del segmento
            altura_final_segmento = altura_inicial_segmento + salto

            # Guardo las alturas de la calle
            dic['desde_izq'] = altura_inicial_segmento + 1
            dic['hasta_izq'] = altura_final_segmento - 1
            dic['desde_der'] = altura_inicial_segmento
            dic['hasta_der'] = altura_final_segmento - 2

            # Si es bis
            if bis is True:
                # Pongo las alturas en negativo
                dic['desde_izq'] = -dic['desde_izq']
                dic['hasta_izq'] = -dic['hasta_izq']
                dic['desde_der'] = -dic['desde_der']
                dic['hasta_der'] = -dic['hasta_der']

            # La altura inicial del siguiente segmento va a ser la altura final
            # del el segmento que se está editando en este momento
            altura_inicial_segmento = altura_final_segmento

            # Actualizo el registro
            q_obj.update(dic, synchronize_session=False)

        # Realizo el commit con la base de datos gis2
        session.commit()

        # Cierro la sesión con la base de datos gis2
        session.close()

        # Si todo salió bien retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Los segmentos se modificaron de forma correcta')
        return rta

    def segm_prox_altmod(self, punto_cercano, geom_pto_1, geom_pto_2, id_calle):
        '''
            Esta función se encarga de traer el segmento mas cercano al punto
            solicitado
            ingresada.
            Argumentos:
                punto_cercano (int)
                geom_pto1(geometry)
                geom_pto2(geometry)
                id_calle(int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (obj)
        '''

        rta = {}

        session = self.gis2.binded_session()

        calles = self.gis2.tabla('calles')
        q_obj = session.query(calles.gid, calles.desde_izq, calles.hasta_izq,
                                calles.desde_der, calles.hasta_der,
                                calles.id_tipo,
                                func.st_astext(calles.geom).label('geom'),
                                func.st_distance(calles.geom,
                                   func.st_geomfromtext(geom_pto_1,
                                   4326),
                                    ).label('distancia_punto1'),
                                func.st_distance(calles.geom,
                                   func.st_geomfromtext(geom_pto_2,
                                   4326),
                                    ).label('distancia_punto2'))

        q_obj = q_obj.filter(
                calles.id_calle_nombre == id_calle)

        #q_obj = q_obj.order_by(calles.geom.desc())

        resultado = q_obj.all()
        session.close()
        if len(resultado) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el segmento dado')
            return rta
        segmento_min_dist = 1000
        for segmento in resultado:
            if punto_cercano == 1 and\
                        segmento.distancia_punto1 < segmento.distancia_punto2:
                if  segmento.distancia_punto1 < segmento_min_dist:
                    segmento_min_dist = segmento.distancia_punto1
                    segm_a_mod = segmento

            elif punto_cercano == 2 and\
                         segmento.distancia_punto2 < segmento.distancia_punto1:
                if  segmento.distancia_punto2 < segmento_min_dist:
                    segmento_min_dist = segmento.distancia_punto2
                    segm_a_mod = segmento

        rta['Estado'] = True
        rta['Respuesta'] = segm_a_mod
        return rta

    def NorthAzimuth(self, punto1, punto2):
        degBearing = math.degrees(
            math.atan2(
                (punto2[0] - punto1[0]),
                (punto2[1] - punto1[1])
            )
        )

        if(degBearing < 0):
            #degBearing += 360.0
            degBearing += 180.0

        #degBearing = abs(degBearing)

        return degBearing

    def altura_final(self, id_calle, id_segm_ini, id_segm_fin):
        '''
            Esta función se encarga de mostrarme la cantidad de segmentos
            a modificar.
            Argumentos:
                id_calle (int)
                id_segm_ini (int)
                id_segm_fin (int)
            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Recuperamos los segmentos de una calle dada
        session = self.gis2.binded_session()

        calles = self.gis2.tabla('calles')
        # Obtengo el geom del segmento inicial y su id_tipo

        q_obj_geom = session.query(calles.id_tipo,
                                    func.st_astext(calles.geom).label('geom'))
        q_obj_geom = q_obj_geom.filter(
                calles.gid == id_segm_ini)
        resul_geom_ini = q_obj_geom.first()

        if resul_geom_ini is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el segmento dado')
            return rta
        geom_segm_ini = resul_geom_ini.geom
        id_tipo_segm_ini = resul_geom_ini.id_tipo

        # obtenemos los dos puntos que forman el segmento inicial
        linea = self.f_geo.procesar_geometria(geom_segm_ini)

        longitud_pto1 = linea[0][0]
        latitud_pto1 = linea[0][1]
        longitud_pto2 = linea[1][0]
        latitud_pto2 = linea[1][1]

        # Obtenemos el angulo del segmento inicial
        angulo = self.NorthAzimuth(linea[0], linea[1])

        # Verificamos si es horizontal o vertical para asignar los puntos 1 y 2
        if angulo < 30 or angulo > 150:
            calle_orientacion = 'vertical'
        else:
            calle_orientacion = 'horizontal'

        if calle_orientacion == 'vertical':
            if latitud_pto1 > latitud_pto2:
                segm_pto_1 = linea[0]
                segm_pto_2 = linea[1]
            else:
                segm_pto_1 = linea[1]
                segm_pto_2 = linea[0]
        else:
            if longitud_pto1 > longitud_pto2:
                segm_pto_1 = linea[0]
                segm_pto_2 = linea[1]
            else:
                segm_pto_1 = linea[1]
                segm_pto_2 = linea[0]

        # Convertimos el punto 1 y 2 a geom
        # Convertimos el punto 1
        geom_pto_1 = self.f_geo.punto_geometria(segm_pto_1[0], segm_pto_1[1])

        # Ahora convertimos el punto 2
        geom_pto_2 = self.f_geo.punto_geometria(segm_pto_2[0], segm_pto_2[1])

        # Obtengo distancia geom del segmento final al punto 1
        q_obj_fin_pto1 = session.query(
            func.st_astext(calles.geom).label('geom'),
            func.st_distance(calles.geom,
            func.st_geomfromtext(geom_pto_1, 4326),
        ).label('distancia'))
        q_obj_fin_pto1 = q_obj_fin_pto1.filter(calles.gid == id_segm_fin)
        resul_geom_fin = q_obj_fin_pto1.first()

        if resul_geom_fin is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el segmento dado')
            return rta
        #geom_segm_fin = resul_geom_fin.geom
        dist_pto1_final = resul_geom_fin.distancia

        # Obtengo distancia geom del segmento final al punto 2
        q_obj_fin_pto2 = session.query(
            func.st_astext(calles.geom).label('geom'),
            func.st_distance(calles.geom,
            func.st_geomfromtext(geom_pto_2, 4326),
        ).label('distancia'))

        q_obj_fin_pto2 = q_obj_fin_pto2.filter(calles.gid == id_segm_fin)
        resul_geom_fin = q_obj_fin_pto2.first()

        if resul_geom_fin is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el segmento dado')
            return rta
        #geom_segm_fin = resul_geom_fin.geom
        dist_pto2_final = resul_geom_fin.distancia

        # Comparamos cual distancia esta mas cerca del segemento final
        punto_cerca_fin = 0
        if dist_pto1_final < dist_pto2_final:
            punto_cerca_fin = 1
        elif dist_pto1_final > dist_pto2_final:
            punto_cerca_fin = 2

        # Guardamos el segmento inicial en el primer lugar de la lista
        dic = {}
        dic['id_segmento'] = id_segm_ini
        dic['id_tipo_segm'] = id_tipo_segm_ini
        resul_alt = []
        resul_alt.append(dic)
        existe_segm_sig = True
        while existe_segm_sig:
            if dist_pto1_final == dist_pto2_final:
                break
            # Buscamos el segmento que esta mas cerca del punto 1 o 2
            rta_prox_segm = self.segm_prox_altmod(punto_cerca_fin, geom_pto_1,
                                                        geom_pto_2, id_calle)
            if rta_prox_segm['Estado'] is False:
                return rta_prox_segm
            else:
                segmento_act = rta_prox_segm['Respuesta']

            # Guardamos el segmento recuperado en la lista  actualizar si es
            #el segemento es mayor a 15 mts
            linea_seg_act = self.f_geo.procesar_geometria(segmento_act.geom)
            longitud_pto1_act = linea_seg_act[0][0]
            latitud_pto1_act = linea_seg_act[0][1]
            longitud_pto2_act = linea_seg_act[1][0]
            latitud_pto2_act = linea_seg_act[1][1]

            mts_dist = self.f_geo.mts_entre_puntos(
                latitud_pto1_act,
                longitud_pto1_act,
                latitud_pto2_act,
                longitud_pto2_act,
            )
            if mts_dist > 15:
                # cargo segmetno a actualizar
                dic = {}
                dic['id_segmento'] = segmento_act.gid
                dic['id_tipo_segm'] = segmento_act.id_tipo
                resul_alt.append(dic)
            # Busco el punto 1 y 2 del nuevo segmento solo si es mayor a
            linea = self.f_geo.procesar_geometria(segmento_act.geom)

            longitud_pto1 = linea[0][0]
            latitud_pto1 = linea[0][1]
            longitud_pto2 = linea[1][0]
            latitud_pto2 = linea[1][1]
            angulo = self.NorthAzimuth(linea[0], linea[1])

            # Verificamos si es horizontal o vertical para asignar los
            #puntos 1 y 2
            if angulo < 30 or angulo > 150:
                calle_orientacion = 'vertical'
            else:
                calle_orientacion = 'horizontal'

            if calle_orientacion == 'vertical':
                if latitud_pto1 > latitud_pto2:
                    segm_pto_1 = linea[0]
                    segm_pto_2 = linea[1]
                else:
                    segm_pto_1 = linea[1]
                    segm_pto_2 = linea[0]
            else:
                if longitud_pto1 > longitud_pto2:
                    segm_pto_1 = linea[0]
                    segm_pto_2 = linea[1]
                else:
                    segm_pto_1 = linea[1]
                    segm_pto_2 = linea[0]

            #Convertimos el punto 1 y 2 a geom
            #Convertimos el punto 1
            geom_pto_1 = self.f_geo.punto_geometria(segm_pto_1[0],
                                                            segm_pto_1[1])
            #Ahora convertimos el punto 2
            geom_pto_2 = self.f_geo.punto_geometria(segm_pto_2[0],
                                                            segm_pto_2[1])
            if segmento_act.gid == id_segm_fin:
                existe_segm_sig = False

        for segmento in resul_alt:
            if segmento['id_tipo_segm'] == 10:
                continue

            # Se actualiza modificada y id_tipo de calles correctas (temporal)
            q_objup = session.query(calles.gid)
            q_objup = q_objup.filter(calles.gid == segmento['id_segmento'])
            v = {}
            v['dudosa'] = segmento['id_tipo_segm']
            v['modificada'] = 2
            v['id_tipo'] = 10
            q_objup.update(v, synchronize_session=False)

        session.commit()
        # Guardamos la cantidad de segmentos a modificar
        cant_segm = len(resul_alt)

        session.close()
        rta['Estado'] = True
        rta['Respuesta'] = cant_segm
        return rta

    def datos_segmentos_calle(self, id_calle):
        '''
            Esta función se encarga de devolver los datos que pueden tener
            los segmentos de una calle dada, como por ejemplo, que algún
            segmento sea biz o que tenga letra altura.

            Argumentos:
                id_calle (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['id_calle'] (int)
                                        ['tiene_biz'] (bool)
                                        ['tiene_letra'] (bool)
        '''

        # Declaro la respuesta a retornar que va a ser un dict
        rta = {}

        # Me conecto con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')

        # Busco el id de la calle nombre, su altura y su letra de cada segmento
        q_obj = session.query(
            calles.id_calle_nombre, calles.desde_izq, calles.letra
        )

        # FIX SqlAlchemy
        none = None

        # Filtro los segmento de la calle dada
        q_obj = q_obj.filter(
            and_(
                calles.id_calle_nombre == id_calle,
                or_(
                    calles.desde_izq < 0,
                    and_(
                        calles.letra != '',
                        calles.letra != none,
                    ),
                ),
            )
        )

        # Guardo todos los segmentos encontrados
        registros = q_obj.all()

        # Cierro la conexión con gis2
        session.close()

        # En esta función siempre voy a retornar True en el estado
        rta['Estado'] = True

        # Creo un diccionario donde se van a guardar los datos
        # de los segmentos de la calle
        dic = {}
        # Guardo el id de la calle
        dic['id_calle'] = id_calle
        # Seteo tiene_biz en False
        dic['tiene_biz'] = False
        # Seteo tiene_letra en False
        dic['tiene_letra'] = False

        # Si no hay registros quiere decir que la calle no tiene ningún
        # segmento biz ni con letra
        if len(registros) == 0:
            # Retorno que la calle no tiene biz ni letra
            rta['Respuesta'] = dic
            return rta

        # Recorro los registros obtenidos
        for registro in registros:
            # Si el segmento tiene la altura negativa quiere decir que tiene biz
            if registro.desde_izq < 0:
                dic['tiene_biz'] = True

            # Si el segmento tiene letra guardo el dato en el diccionario
            if registro.letra != '' and registro.letra is not None:
                dic['tiene_letra'] = True

        # Retorno los datos que tienen los segmentos de la calle
        rta['Respuesta'] = dic
        return rta

    def obtener_entre_calles(self, id_ciudad, id_calle, altura):
        '''
            Dado el id de una calle y su altura se deben obtener sus 2 esquinas.

            Argumentos:
                id_ciudad (int)
                id_calle (int)
                altura (int)

            Funciones:
                CallePostgres().NorthAzimuth
                CallePostgres().obtener_nombre_calle
                FuncionesGeo().procesar_geometria

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['entre_calles'] (str)
                                        ['id_esquina1'] (int)
                                        ['nombre_esquina1'] (str)
                                        ['id_esquina2'] (int)
                                        ['nombre_esquina2'] (str)
        '''

        # Declaro la respuesta a retornar que va a ser un dict
        rta = {}

        # Si no tengo los datos retorno un mensaje de error
        if id_ciudad is None or id_calle is None or altura is None:
            # Retorno que no se pudo obtener las esquinas
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'Para obtener las entre calles se requiere la ciudad, la calle principal y su altura')
            return rta

        # Me conecto con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')

        # Busco el id del segmento
        q_obj = session.query(
            func.st_astext(calles.geom).label('geom'),
        )

        # Filtro los segmento de la calle dada según su altura para obtener un
        # único segmento
        q_obj = q_obj.filter(
            and_(
                calles.id_calle_nombre == id_calle,
                or_(
                    and_(
                        calles.desde_izq <= altura,
                        calles.hasta_izq >= altura,
                    ),
                    and_(
                        calles.desde_der <= altura,
                        calles.hasta_der >= altura,
                    ),
                    and_(
                        calles.desde_izq >= altura,
                        calles.hasta_izq <= altura,
                    ),
                    and_(
                        calles.desde_der >= altura,
                        calles.hasta_der <= altura,
                    ),
                ),
            )
        )

        # Guardo los datos de la calle principal
        calle_principal = q_obj.first()

        # Si no encontré ninguna calle
        if calle_principal is None:
            # Cierro la conexión con gis2
            session.close()

            # Retorno que no se pudo obtener las esquinas
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo encontrar la calle principal en la altura dada')
            return rta

        # Obtengo el ángulo de la calle principal
        linea_calle_principal = self.f_geo.procesar_geometria(
            calle_principal.geom
        )
        angulo_calle_principal = self.NorthAzimuth(
            linea_calle_principal[0], linea_calle_principal[1]
        )

        # Obtengo la tabla calles_nombres
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Armo una subquery para obtener todas las calles nombres de la ciudad
        sub_q_obj = session.query(calles_nombres.id)
        sub_q_obj = sub_q_obj.filter(
            calles_nombres.id_ciudad == id_ciudad
        )
        sub_q_obj = sub_q_obj.subquery()

        # Obtengo las esquinas a la geometría dada
        q_obj2 = session.query(
            calles.id_calle_nombre,
            func.st_astext(calles.geom).label('geom'),
        )

        # Filtro solo por las calles de la ciudad dada
        q_obj2 = q_obj2.filter(calles.id_calle_nombre.in_(sub_q_obj))

        # Busco los segmentos cercanos a la calle principal
        q_obj2 = q_obj2.filter(
            and_(
                func.st_distance(
                    calles.geom,
                    func.st_geomfromtext(calle_principal.geom, 4326),
                ) < 0.003,  # 0.0015
                calles.id_calle_nombre != id_calle,
            )
        )

        # Ordeno por las más cercanas
        q_obj2 = q_obj2.order_by(
            func.st_distance(
                calles.geom,
                func.st_geomfromtext(calle_principal.geom, 4326),
            ).asc()
        )

        # Guardo todos los registros encontrados
        registros = q_obj2.all()

        # Cierro la conexión con gis2
        session.close()

        # Si no se encontraron esquinas
        if len(registros) == 0:
            # Retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudieron obtener las esquinas')
            return rta

        # Creo el diccionario donde se van a guardar los datos de las esquinas
        dic = {}
        dic['entre_calles'] = ''
        dic['id_esquina1'] = None
        dic['nombre_esquina1'] = ''
        dic['id_esquina2'] = None
        dic['nombre_esquina2'] = ''

        # Recorra la calles obtenidas
        for registro in registros:
            # Si ya tengo guardada la esquina dada
            if dic['id_esquina1'] == registro.id_calle_nombre:
                # Paso a la siguiente
                continue

            # Declaro la variable punto_cercano en False
            punto_cercano = False

            ### CASO 1 ###
            # Comparo el ángulo de la calle con el de la principal
            linea = self.f_geo.procesar_geometria(registro.geom)
            angulo = self.NorthAzimuth(linea[0], linea[1])
            dif_angulo = abs(angulo_calle_principal - angulo)

            # Si la diferencia entre el ángulo de la calle principal
            # y el ángulo de la esquina es menor a 100 y mayor a 80
            if dif_angulo < 110 and dif_angulo > 80:
                # La considero como esquina
                punto_cercano = True

            ### CASO 2 ###
            # Si el punto no es cercano
            if punto_cercano is False:
                # Obtengo la distancia en metros entre dos puntos de las calles
                dist_puntos = self.f_geo.mts_entre_puntos(
                    linea_calle_principal[0][1],
                    linea_calle_principal[0][0],
                    linea[0][1],
                    linea[0][0],
                )
                # Redondeo para que me quede un entero
                dist_puntos = int(round(dist_puntos, 0))

                # Si el punto está a menos de 10 metros
                if dist_puntos < 10:
                    # Lo concidero como un punto cercano
                    punto_cercano = True

            ### CASO 3 ###
            # Si el punto no es cercano
            if punto_cercano is False:
                # Intento con el otro extremo de la calle principal
                # Obtengo la distancia en metros entre dos puntos de las calles
                dist_puntos = self.f_geo.mts_entre_puntos(
                    linea_calle_principal[1][1],
                    linea_calle_principal[1][0],
                    linea[0][1],
                    linea[0][0],
                )
                # Redondeo para que me quede un entero
                dist_puntos = int(round(dist_puntos, 0))

                # Si el punto está a menos de 10 metros
                if dist_puntos < 10:
                    # Lo concidero como un punto cercano
                    punto_cercano = True

            ### CASO 4 ###
            # Si el punto no es cercano
            if punto_cercano is False:
                # Intento con el otro extremo de la posible esquina
                # Obtengo la distancia en metros entre dos puntos de las calles
                dist_puntos = self.f_geo.mts_entre_puntos(
                    linea_calle_principal[0][1],
                    linea_calle_principal[0][0],
                    linea[1][1],
                    linea[1][0],
                )
                # Redondeo para que me quede un entero
                dist_puntos = int(round(dist_puntos, 0))

                # Si el punto está a menos de 10 metros
                if dist_puntos < 10:
                    # Lo concidero como un punto cercano
                    punto_cercano = True

            ### CASO 5 ###
            # Si el punto no es cercano
            if punto_cercano is False:
                # Intento con el otro extremo de la posible esquina y el otro
                # extremo de la calle principal
                # Obtengo la distancia en metros entre dos puntos de las calles
                dist_puntos = self.f_geo.mts_entre_puntos(
                    linea_calle_principal[1][1],
                    linea_calle_principal[1][0],
                    linea[1][1],
                    linea[1][0],
                )
                # Redondeo para que me quede un entero
                dist_puntos = int(round(dist_puntos, 0))

                # Si el punto está a menos de 10 metros
                if dist_puntos < 10:
                    # Lo concidero como un punto cercano
                    punto_cercano = True

            # Si ningún caso indica que la calle puede ser una esquina
            if punto_cercano is False:
                # Paso a la siguiente calle
                continue

            # Obtengo el nombre de la calle obtenida
            rta_obtener_nombre_calle = self.obtener_nombre_calle(
                registro.id_calle_nombre
            )

            # Si no se pudo obtener el nombre de la calle
            if rta_obtener_nombre_calle['Estado'] is False:
                # Paso a la siguiente calle
                continue

            # Guardo el nombre de la esquina
            nombre_esquina = rta_obtener_nombre_calle['Respuesta']

            # Si no tengo la primer esquina la guardo
            if dic['id_esquina1'] is None:
                dic['id_esquina1'] = registro.id_calle_nombre
                dic['nombre_esquina1'] = nombre_esquina
                continue

            # Guardo la segunda esquina
            dic['id_esquina2'] = registro.id_calle_nombre
            dic['nombre_esquina2'] = nombre_esquina
            break

        # Si no encuentro ninguna esquina
        if dic['nombre_esquina1'] == '' and dic['nombre_esquina2'] == '':
            # Retorno un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No hay ninguna esquina cercana')
            return rta

        # Si no tengo la segunda esquina
        if dic['nombre_esquina2'] == '':
            # Muestro solo una
            dic['entre_calles'] = 'Entre ' + dic['nombre_esquina1']
        else:
            # Armo el string que se va a aguardar en 'entre_calles'
            # con las dos esquinas
            dic['entre_calles'] = 'Entre ' + dic['nombre_esquina1'] + \
                                    ' y ' + dic['nombre_esquina2']

        # Si todo salió bien retorno las esquinas obtenidas
        rta['Estado'] = True
        rta['Respuesta'] = dic
        return rta

    def seleccionar_segmentos(self, id_calle_nombre,
                    id_segmento_ini, id_segmento_fin, distancia_segmento=False):
        '''
            Esta función se encarga de ordenar los segmentos desde uno inicial
            hasta uno final.
            Retorna una lista de segmentos ordenados.

            Argumentos:
                id_calle_nombre (int)
                id_segmento_ini (int)
                id_segmento_fin (int)

            Funciones:
                CallePostgres().obtener_segmentos
                CallePostgres().obtener_segmento_siguiente

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de int)
        '''

        # Si el segmento inicial es igual al final
        if id_segmento_ini == id_segmento_fin:
            # Retorno un mensaje de error
            rta = {}
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'Debe seleccionar un segmento final distinto al inicial')
            return rta

        # Obtengo los datos del segmento inicial
        rta_obtener_segmento_ini = self.obtener_segmentos(
            id_calle_nombre=id_calle_nombre, id_segmento=id_segmento_ini
        )

        # Si no se pudo obtener el segmento inicial
        if rta_obtener_segmento_ini['Estado'] is False:
            # Retorno el error
            return rta_obtener_segmento_ini

        # Guardo los datos del segmento inicial, es el único dato de la lista
        segmento_ini = rta_obtener_segmento_ini['Respuesta'][0]

        # Obtengo los datos del segmento final
        rta_obtener_segmento_fin = self.obtener_segmentos(
            id_calle_nombre=id_calle_nombre, id_segmento=id_segmento_fin
        )

        # Si no se pudo obtener el segmento final
        if rta_obtener_segmento_fin['Estado'] is False:
            # Retorno el error
            return rta_obtener_segmento_fin

        # Guardo los datos del segmento final, es el único dato de la lista
        segmento_fin = rta_obtener_segmento_fin['Respuesta'][0]

        # Obtengo todos los segmentos cercanos al segmento inicial seleccionado
        # Utilizo la latitud y la longitud del segmento inicial
        rta_obtener_lista_segmentos = self.obtener_segmentos(
            id_calle_nombre=id_calle_nombre,
            latitud=segmento_ini['Puntos'][0][0],
            longitud=segmento_ini['Puntos'][0][1],
        )

        # Si no se pudieron obtener los segmentos cercanos
        if rta_obtener_lista_segmentos['Estado'] is False:
            # Retorno el error
            return rta_obtener_lista_segmentos

        # Guardo la lista de segmentos obtenidos
        lista_segmentos = rta_obtener_lista_segmentos['Respuesta']

        # Declaro vacía la lista ordenada a retornar
        lista_ordenada = []

        # Defino la distancia del punto inicial al punto final en 0.
        distancia = self.f_geo.mts_entre_puntos(
                segmento_ini['Puntos'][0][1],
                segmento_ini['Puntos'][0][0],
                segmento_fin['Puntos'][0][1],
                segmento_fin['Puntos'][0][0],
            )

        # Defino la distancia del punto inicial.
        distancia_inicial = self.f_geo.mts_entre_puntos(
                segmento_ini['Puntos'][0][1],
                segmento_ini['Puntos'][0][0],
                segmento_ini['Puntos'][1][1],
                segmento_ini['Puntos'][1][0],
            )

        # Guardo el punto inicial.
        if distancia_segmento is not False:
            lista_ordenada.append(
                {'Id': segmento_ini['Id'],
                 'Distancia': distancia_inicial
                 }
            )
        else:
            lista_ordenada.append(segmento_ini['Id'])

        # Recorro la lista de segmentos cercanos de la misma calle.
        for segmento_t in lista_segmentos:

            # Obtengo la distancia del segmento temporal al segmento final.
            distancia_tmp = self.f_geo.mts_entre_puntos(
                segmento_t['Puntos'][0][1],
                segmento_t['Puntos'][0][0],
                segmento_fin['Puntos'][0][1],
                segmento_fin['Puntos'][0][0],
            )

            # Obtengo la distancia del segmento.
            distancia_s = self.f_geo.mts_entre_puntos(
                segmento_t['Puntos'][0][1],
                segmento_t['Puntos'][0][0],
                segmento_t['Puntos'][1][1],
                segmento_t['Puntos'][1][0],
            )

            # Si el segmento temporal es el inicial, paso al siguiente segmento,
            # ya que el primero ya lo tengo.
            if segmento_t['Id'] == segmento_ini['Id']:
                continue

            # Si el segmento temporal es el segmento final, lo agrego
            # directamente a la lista y salgo del bucle.
            if segmento_t['Id'] == segmento_fin['Id']:

                if distancia_segmento is not False:
                    lista_ordenada.append(
                        {'Id': segmento_t['Id'],
                         'Distancia': distancia_s
                         }
                    )
                else:
                    lista_ordenada.append(segmento_t['Id'])

                break

            # Si la distancia del segmento temporal es mayor a la distancia
            # estudiada, continuo con el próximo segmento.
            if distancia_tmp > distancia:
                continue

            # Si la distancia del segmento temporal es menor a la distancia
            # estudiada, agrego el segmento a la lista y seteo la
            # nueva distancia
            if distancia_tmp < distancia:

                if distancia_segmento is not False:
                    lista_ordenada.append(
                        {'Id': segmento_t['Id'],
                         'Distancia': distancia_s
                         }
                    )
                else:
                    lista_ordenada.append(segmento_t['Id'])

                distancia = distancia_tmp

        # # Si todo salió bien, retorno la lista ordenada
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = lista_ordenada
        return rta

    def obtener_segmento_siguiente(self, segmento_actual, lista_segmentos,
                                                                segmento_fin):
        '''
            Esta función se encarga de obtener el segmento siguiente a un
            segmento dado.

            Argumentos:
                segmento_actual (dict)
                lista_segmentos (list de dict)
                segmento_fin (dict)

            Funciones:
                FuncionesGeo().mts_entre_puntos

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o int)
        '''

        # Declaro un diccionario a retornar
        rta = {}

        # Si recibo el segmento actual igual al segmento fin
        if segmento_actual['Id'] == segmento_fin['Id']:
            # Retorno el segmento final
            rta['Estado'] = True
            rta['Respuesta'] = segmento_fin
            return rta

        # Creo una lista para guardar los segmentos que están pegados
        posibles_segmentos = []

        # Recorro la lista de segmentos
        for segmento in lista_segmentos:
            # No comparo al segmento actual con si mismo
            if segmento_actual['Id'] == segmento['Id']:
                continue

            # Si coincide el primer punto
            if segmento['Puntos'][1] == segmento_actual['Puntos'][0]:
                # Agrego el posible segmento a la lista
                posibles_segmentos.append(segmento)
                # Paso al siguiente segmento
                continue

            # Si coincide el segundo punto
            if segmento['Puntos'][0] == segmento_actual['Puntos'][1]:
                # Agrego el posible segmento a la lista
                posibles_segmentos.append(segmento)
                # Paso al siguiente segmento
                continue

        # Si no tengo elementos en la lista
        if len(posibles_segmentos) == 0:
            # Retorno que False y un error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el segmento siguiente')
            return rta

        # Declaro el segmento siguiente en None hasta que lo obtenga
        segmento_siguiente = None

        # Recorro los posibles segmentos
        for posible_segmento in posibles_segmentos:
            # Si el posible segmento es igual al final
            if posible_segmento['Id'] == segmento_fin['Id']:
                # Retorno el segmento final
                rta['Estado'] = True
                rta['Respuesta'] = segmento_fin
                return rta

            # Obtengo la distancia entre el posible segmento y el segmento final
            # primero del primer punto del posible segmento hasta el primer
            # punto del punto final
            distancia_punto1 = self.f_geo.mts_entre_puntos(
                posible_segmento['Puntos'][0][1],
                posible_segmento['Puntos'][0][0],
                segmento_fin['Puntos'][0][1],
                segmento_fin['Puntos'][0][0],
            )

            # Obtengo la distancia entre el posible segmento y el segmento final
            # después del segundo punto del posible segmento hasta el primer
            # punto del punto final
            distancia_punto2 = self.f_geo.mts_entre_puntos(
                posible_segmento['Puntos'][1][1],
                posible_segmento['Puntos'][1][0],
                segmento_fin['Puntos'][0][1],
                segmento_fin['Puntos'][0][0],
            )

            # Me quedo con la distancia más corta para usar y comparar con los
            # demás segmentos
            if distancia_punto1 < distancia_punto2:
                distancia = distancia_punto1
            else:
                distancia = distancia_punto2

            # Agrego la distancia al posible segmento
            posible_segmento['dist_segm_fin'] = distancia

            # Si no tengo un segmento siguiente
            if segmento_siguiente is None:
                # Guardo el primero
                segmento_siguiente = posible_segmento

                # Paso al siguiente posible segmento
                continue

            # Si ya tengo un posible segmento
            # Compara la distancia con el segmento final
            # El posible segmento que este más cerca va a ser el siguiente
            if posible_segmento['dist_segm_fin'] < segmento_siguiente['dist_segm_fin']:
                # Guardo el segmento con la menor distancia al segmento final
                segmento_siguiente = posible_segmento

                # Paso al siguiente posible segmento
                continue

        # Si encontré el segmento siguiente lo retorno
        rta['Estado'] = True
        rta['Respuesta'] = segmento_siguiente
        return rta

    def actualizar_segmento(self, gid, id_calle_nombre=None, id_tipo=None,
            sentido=None, geom=None, desde_izq=None, hasta_izq=None,
            desde_der=None, hasta_der=None, letra=None, dudosa=None):
        '''
            Esta función se encarga de modificar cualquier dato de un segmento
            dado según su gid.

            Argumentos:
                gid (int)
                id_calle_nombre (int)
                id_tipo (int)
                sentido (int)
                geom (geometry)
                desde_izq (int)
                hasta_izq (int)
                desde_der (int)
                hasta_der (int)
                letra (str)
                dudosa (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Armo los datos a modificar
        dic = {}

        # Si tengo un id_calle_nombre
        if id_calle_nombre is not None:
            # Lo modifico
            dic['id_calle_nombre'] = id_calle_nombre

        # Si tengo un id_tipo
        if id_tipo is not None:
            # Lo modifico
            dic['id_tipo'] = id_tipo

        # Si tengo un sentido
        if sentido is not None:
            # Lo modifico
            dic['sentido'] = sentido

        # Si tengo un geom
        if geom is not None:
            # Lo modifico
            dic['geom'] = geom

        # Si tengo el dato desde_izq
        if desde_izq is not None:
            # Lo modifico
            dic['desde_izq'] = desde_izq

        # Si tengo el dato hasta_izq
        if hasta_izq is not None:
            # Lo modifico
            dic['hasta_izq'] = hasta_izq

        # Si tengo el dato desde_der
        if desde_der is not None:
            # Lo modifico
            dic['desde_der'] = desde_der

        # Si tengo el dato hasta_der
        if hasta_der is not None:
            # Lo modifico
            dic['hasta_der'] = hasta_der

        # Si tengo el dato letra
        if letra is not None:
            # Lo modifico
            dic['letra'] = letra

        # Si tengo el dato dudosa
        if dudosa is not None:
            # Lo modifico
            dic['dudosa'] = dudosa

        # Marco el campo modificada en 1
        dic['modificada'] = 1

        # Me conecto con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles que es donde se guardan los segmentos
        calles = self.gis2.tabla('calles')

        # Busco el segmento por el gid dado
        q_obj = session.query(calles.gid)
        q_obj = q_obj.filter(calles.gid == gid)

        # Si no se encontró el segmento
        if q_obj.count() <= 0:
            # Cierro la sesión con la base de datos gis2
            session.close()

            # Retorno el error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el segmento %s' % gid)
            return rta

        # Si se encontró el segmento actualizo sus datos
        q_obj.update(dic, synchronize_session=False)

        # Realizo el commit con la base de datos gis2
        session.commit()

        # Cierro la sesión con la base de datos gis2
        session.close()

        # Si todo salió bien retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'El segmento %s se actualizó correctamente' % gid)
        return rta

    def obtener_segmento(self, gid):
        '''
            Dado el gid de un segmento esta función se encarga de retornar
            todos sus datos.

            Argumentos:
                gid (int)

            Funciones:
                FuncionesGeo().procesar_geometria
                CallePostgres().obtener_nombre_calle

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['gid'] (int)
                                        ['id_calle_nombre'] (int)
                                        ['id_tipo'] (int)
                                        ['sentido'] (int)
                                        ['puntos'] (list de list de float)
                                        ['desde_izq'] (int)
                                        ['hasta_izq'] (int)
                                        ['desde_der'] (int)
                                        ['hasta_der'] (int)
                                        ['letra'] (str)
                                        ['modificada'] (int)
                                        ['dudosa'] (int)
                                        ['nombre_calle'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Me conecto con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles que es donde se guardan los segmentos
        calles = self.gis2.tabla('calles')

        # Busco el segmento por el gid dado
        q_obj = session.query(
            calles.gid,
            calles.id_calle_nombre,
            calles.id_tipo,
            calles.sentido,
            func.st_astext(calles.geom).label('geom'),
            calles.desde_izq,
            calles.hasta_izq,
            calles.desde_der,
            calles.hasta_der,
            calles.letra,
            calles.modificada,
            calles.dudosa,
        )
        q_obj = q_obj.filter(calles.gid == gid)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesión con la base de datos gis2
        session.close()

        # Si no se encontró el registro
        if registro is None:
            # Retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el segmento %s' % gid)
            return rta

        # Guardo los datos del segmento
        dic = {}
        dic['gid'] = registro.gid
        dic['id_calle_nombre'] = registro.id_calle_nombre
        dic['id_tipo'] = registro.id_tipo
        dic['sentido'] = registro.sentido
        dic['puntos'] = registro.geom
        dic['desde_izq'] = registro.desde_izq
        dic['hasta_izq'] = registro.hasta_izq
        dic['desde_der'] = registro.desde_der
        dic['hasta_der'] = registro.hasta_der
        dic['letra'] = registro.letra
        dic['modificada'] = registro.modificada
        dic['dudosa'] = registro.dudosa

        # Si tengo una geometría
        if dic['puntos'] is not None:
            # La proceso
            geom = self.f_geo.procesar_geometria(dic['puntos'])
            # Ordeno los datos de la geometría
            dic['puntos'] = [[geom[0][1], geom[0][0]], [geom[1][1], geom[1][0]]]

        # Obtengo el nombre de la calle del segmento
        rta_obtener_nombre_calle = self.obtener_nombre_calle(
            dic['id_calle_nombre']
        )
        # Si no puedo obtener el nombre de la calle
        if rta_obtener_nombre_calle['Estado'] is False:
            # Retorno el error
            return rta_obtener_nombre_calle

        # Guardo el nombre de la calle
        dic['nombre_calle'] = rta_obtener_nombre_calle['Respuesta']

        # Si se encontró el segmento retorno True y sus datos
        rta['Estado'] = True
        rta['Respuesta'] = dic
        return rta

    def obtener_siguiente_id_calles_nombres(self):
        '''
            Esta función se encarga de obtener el siguiente id de la tabla
            "calles_nombres" para usar para agregar un nuevo registro al
            final de la tabla.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o int)
        '''

        rta = {}

        session = self.gis2.binded_session()

        calles_nombres = self.gis2.tabla('calles_nombres')

        q_obj = session.query(calles_nombres.id)
        q_obj = q_obj.order_by(calles_nombres.id.desc())
        registro = q_obj.first()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo obtener el último id de calles nombres')
            return rta

        id_nueva_calle = registro.id + 1

        rta['Estado'] = True
        rta['Respuesta'] = id_nueva_calle
        return rta

    def listar_tipos_de_numeracion(self):
        '''
            Esta funcion devuelve un listado con todos los tipos de numeración.
            Estos datos corresponden al id y al nombre.

            Returns:
                list de list
        '''

        rta = {}

        lista = []
        lista.append([1, _(u'Todos')])
        lista.append([2, _(u'Pares')])
        lista.append([3, _(u'Impares')])

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def obtener_tipo_de_numeracion(self, id_tipo_numeracion):
        '''
            Esta función se encarga de obtener que tipo de numeración es
            según el id.

            Argumentos:
                id_tipo_numeracion (int)

            Funciones:
                CallePostgres -> listar_tipos_de_numeracion

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list)
        '''

        rta = {}

        rta_listar_tipos_de_numeracion = self.listar_tipos_de_numeracion()
        if rta_listar_tipos_de_numeracion['Estado'] is False:
            return rta_listar_tipos_de_numeracion

        for tipo_de_numeracion in rta_listar_tipos_de_numeracion['Respuesta']:
            if tipo_de_numeracion[0] == id_tipo_numeracion:
                rta['Estado'] = True
                rta['Respuesta'] = tipo_de_numeracion
                return rta

        rta['Estado'] = False
        rta['Respuesta'] = _(
            u'No se encontró el tipo de numeración con id %s'
        ) % id_tipo_numeracion
        return rta

    def verificar_calle_de_zona(
            self, id_calle_nombre, zona=None, geometria_zona=None
    ):
        '''
        Esta función verifica si los segmentos de una calle están
        :param id_calle_nombre:
        :param zona:
        :return:
        '''
        # Valido el id_calle_nombre
        v_id_calle_nombre = self.obtener_nombre_calle(id_calle_nombre)
        if v_id_calle_nombre['Estado'] is not True:
            return v_id_calle_nombre

        if zona is not None:

            # Valido la zona.
            try:
                if zona is not None:
                    self.f_geo.validar_puntos_geometria(zona, True)

            except:
                return respuesta(
                    False, _(u'Los datos de la geometria no son validos.')
                )

            # No es un poligono.
            if len(zona) < 3:
                return respuesta(
                    False, _(u'La zona debe tener al menos 3 puntos.')
                )

            # Antes de armar los datos vamos a cerrar el poligono de la zona.
            if zona[0] != zona[-1]:
                zona.append(zona[0])

            # Intento obtener la geometría con los puntos obtenidos.
            try:
                geometria_zona = func.st_geomfromtext(
                    self.f_geo.generar_geometria_poligono(zona),
                    4326
                )
            except:
                return respuesta(
                    False, _(u'No se pudo crear la geometría de la zona dada')
                )



        # Abro  la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla calles.
        calles = self.gis2.tabla('calles')

        # Determino los campos que quiero obtener.
        q_obj = session.query(
            calles.gid,
            calles.id_calle_nombre,
        )

        # Filtro por geometría.
        q_obj = q_obj.filter(
            and_(
                func.st_disjoint(
                    geometria_zona,
                    calles.geom
                ),
                calles.id_calle_nombre == id_calle_nombre
            )
        )

        # Verfico si se obtuvieron registros.
        if int(q_obj.count()) == 0:
            rta = respuesta(
                True,
                _(u'La totalidad de los segmentos de la calle están la zona.')
            )
        else:

            # Determino los campos que quiero obtener.
            q_obj_2 = session.query(
                calles.gid,
                calles.id_calle_nombre,
            )

            # Filtro por geometría.
            q_obj_2 = q_obj_2.filter(
                and_(
                    func.st_intersects(
                        geometria_zona,
                        calles.geom
                    ),
                    calles.id_calle_nombre == id_calle_nombre
                )
            )

            # Obtengo los registros.
            registros = q_obj_2.all()

            segmentos = []

            for registro in registros:
                segmentos.append(registro.gid)

            rta = respuesta(
                False,
                segmentos
            )

        # Cierro sesión.
        session.close()

        # Retorno.
        return rta

    def obtener_calles_por_zona(self, zona):
        '''
        Esta función obtiene las calles que están en la zona dada.
        :param zona: [[]]
        :return:
        '''
        # Valido la zona.
        try:
            if zona is not None:
                self.f_geo.validar_puntos_geometria(zona, True)

        except:
            return respuesta(
                False, _(u'Los datos de la geometria no son validos.')
            )

        # No es un poligono.
        if len(zona) < 3:
            return respuesta(
                False, _(u'La zona debe tener al menos 3 puntos.')
            )

        # Antes de armar los datos vamos a cerrar el poligono de la zona.
        if zona[0] != zona[-1]:
            zona.append(zona[0])

        # Intento obtener la geometría con los puntos obtenidos.
        try:
            geometria_zona = func.st_geomfromtext(
                self.f_geo.generar_geometria_poligono(zona),
                4326
            )
        except:
            return respuesta(
                False, _(u'No se pudo crear la geometría de la zona dada')
            )

        # Abro  la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla calles.
        calles = self.gis2.tabla('calles')
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Determino los campos que quiero obtener(que no se repitan).
        q_obj = session.query(
            calles.id_calle_nombre,
            calles_nombres.nombre,
            calles_nombres.id_ciudad
        ).distinct()

        # Cruzo las tablas.
        q_obj = q_obj.join((
            calles_nombres,
            calles.id_calle_nombre == calles_nombres.id,
        ))

        # Filtro por geometría.
        q_obj = q_obj.filter(
            and_(
                func.st_intersects(
                    calles.geom,
                    geometria_zona
                )
            )
        )

        # Obtengo los registros.
        registros = q_obj.all()

        # Cierro la sesión.
        session.close()

        # Verfico si se obtuvieron registros.
        if int(q_obj.count()) == 0:
            return respuesta(False, _(u'No se encontro el segmento buscado.'))

        # Inicializo la lista.
        calles = []

        # Recorro los registros.
        for registro in registros:

            segmentos = []

            rta_verificar = self.verificar_calle_de_zona(
                registro.id_calle_nombre, geometria_zona=geometria_zona
            )

            en_zona = rta_verificar['Estado']

            if en_zona is False:
                segmentos = rta_verificar['Respuesta']

            # Uso un diccionario temportal.
            dic_tmp = {
                'id': registro.id_calle_nombre,
                'nombre': registro.nombre,
                'id_ciudad': registro.id_ciudad,
                'en_zona': en_zona,
                'segmentos': segmentos
            }

            # Lo agrego a la lista.
            calles.append(dic_tmp)

        # Retorno correctamente.
        return respuesta(True, calles)

    def obtener_extremos_segmento(self, id_segmento):
        '''
        Esta función obtiene los puntos extremos de un segmento.
        1 - Punto inferior izquierdo.
        2 - Punto superior derecho.
        :param id_segmento: int
        :return:
        '''

        # Verifico si existe el segmento.
        rta_segmento = self.obtener_segmento(id_segmento)

        if rta_segmento['Estado'] is not True:
            return rta_segmento

        # Abro  la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla calles.
        calles = self.gis2.tabla('calles')

        # Determino los campos que quiero obtener.
        q_obj = session.query(
            func.st_xmin(calles.geom),
            func.st_ymin(calles.geom),
            func.st_xmax(calles.geom),
            func.st_ymax(calles.geom)
        )

        # Si recibo parametro filtro por id_segmento.
        if id_segmento is not None:
            # Filtro por id_segmento.
            q_obj = q_obj.filter(
                calles.gid == id_segmento
            )

        # Verfico si se obtuvieron registros.
        if int(q_obj.count()) == 0:

            # Ciero sesión.
            session.close()

            # Retorno.
            return respuesta(False, _(u'No se encontró la geometría.'))

        # Obtengo el registro.
        registro = q_obj.first()

        # Ciero sesión.
        session.close()

        # Ordeno los puntos.
        puntos = {
            'inferior_izquierdo': [registro[1], registro[0]],
            'superior_derecho': [registro[3], registro[2]]
        }

        # Retorno.
        return respuesta(True, puntos)

    def reordenar_nodos_del_segmento(self, geom):

        '''
        Esta función reordeno los nodos del segmento.
        :param geom:
        :return:
        '''

        # Acomodo la geom.
        campo_source = func.ST_StartPoint(
            func.ST_LineMerge(geom)
        )

        # Acomodo la geom.
        campo_target = func.ST_EndPoint(
            func.ST_LineMerge(geom)
        )

        dic = {}

        # Trato de encontrar el nodo.
        rta_source = self.topologia.encontrar_nodo(campo_source)
        if rta_source['Estado'] is True:

            # Obtengo el id_source.
            id_source = rta_source['Dato']

        else:

            # Declaro el dic con los datos.
            datos_source = {
                'the_geom': campo_source
            }

            # agrego los nodos
            agregar_nodo = self.gis2.insertar(
                'calles_vertices_pgr', 'id', **datos_source
            )

            if agregar_nodo['Estado'] is not True:
                return agregar_nodo

            # Obtengo el número de id.
            id_source = agregar_nodo['ValorCampo']

        dic['source'] = id_source

        # Trato de encontrar el nodo.
        rta_target = self.topologia.encontrar_nodo(campo_target)
        if rta_target['Estado'] is True:

            # Obtengo el id_target.
            id_target = rta_target['Dato']

        else:

            # Declaro el dic con los datos.
            datos_target = {
                'the_geom': campo_target
            }

            # agrego los nodos
            agregar_nodo = self.gis2.insertar(
                'calles_vertices_pgr', 'id', **datos_target
            )

            if agregar_nodo['Estado'] is not True:
                return agregar_nodo

            # Obtengo el número de id.
            id_target = agregar_nodo['ValorCampo']

        # Guardo el id.
        dic['target'] = id_target

        # Retorno.
        return respuesta(True, _(u'Se obtuvieron los nodos.'), dic)

    def verificar_eliminar_nodo(self, id_nodo):

        # Abro  la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla operaciones.
        tabla_calles = self.gis2.tabla('calles')

        # Buscamos el segmento
        q_obj = session.query(
            tabla_calles.gid,
            tabla_calles.source,
            tabla_calles.target
        ).filter(
            or_(
                tabla_calles.source == id_nodo,
                tabla_calles.target == id_nodo
            )
        )

        registro = q_obj.first()

        session.close()

        if registro is None:

            # Si supera todos los filtros borro el usuario.
            datos = {}
            datos['id'] = ('==', id_nodo)
            borrar = self.gis2.borrar('calles_vertices_pgr', **datos)
            # En el caso que falle la función retorno el error.
            if borrar is not True:
                return respuesta(False, _(u'Error en la operación.'))

        return respuesta(
            True,
            _(u'No hay que eliminarel nodo porque hay segmentos que lo están usando.')
        )

    def sugerir_calle_cercana(self, parte_calle, punto):
        '''
            Dado un punto geográfico y parte del nombre de una calle,
            sugiere posibles nombres de calles que están cerca del punto y
            el nombre es parecido al dado.

            Parámetros:
                parte_calle (str)
                punto (list de float) // [lat, lon]

            Respuesta:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de dict)
                                                ['id_calle'] (int)
                                                ['nombre_calle'] (str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # Si no tenemos ni dos caracteres, salimos
        if len(parte_calle) < 3:
            rta['Estado'] = True
            rta['Respuesta'] = []
            return rta

        # Creo el punto como una string para postgis
        str_punto = 'POINT(%s %s)' % (punto[1], punto[0])

        # Abro la sesión con la base de datos
        session = self.gis2.binded_session()

        # Obtengo la tabla "calles_nombres"
        tabla_calles_nombres = self.gis2.tabla('calles_nombres')

        # Obtengo la tabla "calles"
        tabla_calles = self.gis2.tabla('calles')

        # Obtengo la tabla "ciudades"
        tabla_ciudades = self.gis2.tabla('ciudades')

        # Obtengo el id y el nombre de la calle
        q_obj = session.query(
            tabla_calles_nombres.id,
            tabla_calles_nombres.nombre,
            tabla_calles_nombres.id_ciudad,
            tabla_ciudades.nombre.label('nombre_ciudad')
        )

        # Hago un join con la tabla calles para poder filtrar por la calles
        # cercanas
        q_obj = q_obj.join((
            tabla_calles,
            tabla_calles.id_calle_nombre == tabla_calles_nombres.id
        ))

        # Hago un join con la tabla ciudades para tener el nombre de la ciudad
        q_obj = q_obj.join((
            tabla_ciudades,
            tabla_ciudades.id == tabla_calles_nombres.id_ciudad
        ))

        # Filtro por el nombre recibido
        q_obj = q_obj.filter(
            and_(
                tabla_calles_nombres.nombre.ilike('%' + parte_calle + '%'),
                func.st_distance(
                    tabla_calles.geom, func.st_geomfromtext(str_punto, 4326)
                ) <= 0.2
            )
        )

        # Puede que me devulva muchos segmento de la misma calle
        # así que voy a querer solo un registro por calle
        q_obj = q_obj.distinct()

        # Guardo los registros obtenidos
        registros = q_obj.all()

        # Cierro la sesión con la base de datos
        session.close()

        # Creo una lista para ir guardando las calles obtenidas
        lista_calles = []

        # Recorro los registros obtenidos
        for registro in registros:
            # Creo un diccionario para guardar los datos de cada calle
            dic = {}
            dic['id_calle'] = int(registro.id)
            dic['nombre_calle'] = registro.nombre
            dic['id_ciudad'] = registro.id_ciudad
            dic['nombre_ciudad'] = registro.nombre_ciudad
            # Se agrega cada dato a la lista de calles
            lista_calles.append(dic)

        # Retorno True y el listado de calles obtenidas
        rta['Estado'] = True
        rta['Respuesta'] = lista_calles
        return rta
