# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import func

from at.map import _
from at.map.utiles.validaciones_especificas import ValidacionesEspecificas
from at.map.utiles.funciones_geo import FuncionesGeo
from at.map.provincia import ProvinciaPostgres
from at.orm.decorators import DBSwitch
from at.utiles.validar import numero_entero_positivo
from at.utiles.validar import string
from at.utiles.conversiones import respuesta


@DBSwitch('gis')
class Ciudad(ValidacionesEspecificas):
    '''
       Esta clase contiene todos lo métodos relacionados con la ciudades.
    '''

    def obtener_ciudad(self, id_ciudad):
        '''
            Dado el ID de una ciudad obtengo su nombre.

            Args:
                id_ciudad (int)

            Returns:
                ciudad (str)

            Raises:
                'No se ha encontrado ninguna ciudad con el ID ingresado'

            Functions:
                validar_id_ciudad(id_ciudad)
                binded_session()
                tabla('ciudades')
        '''

        if not self.validar_id_ciudad(id_ciudad):
            return self.validar_id_ciudad(id_ciudad)

        session = self.gis.binded_session()
        tabla_ciudades = self.gis.tabla('ciudades')

        q_obj = session.query(tabla_ciudades.Ciudad)
        q_obj = q_obj.filter(tabla_ciudades.IdCiudad == id_ciudad)

        ciudad = q_obj.first()

        if ciudad is None:
            session.close()
            raise (Exception,
                _('No se ha encontrado ninguna ciudad con el id: %s.' %
                                                                    id_ciudad)
            )
        session.close()
        return ciudad.Ciudad

    def listar_ciudades(self):
        '''
            Esta funcion devuelve un listado con todas las ciudades disponibles
            en el sistema. Estos datos corresponden al id y al nombre de la
            misma.

            Returns:
                ciudades (list [  list [(int),(str)] ])
        '''

        # Iniciamos la sesion con la BD gis
        session = self.gis.binded_session()
        # Nos conectamos a la tabla "ciudades"
        tabla_ciudades = self.gis.tabla('ciudades')

        q_obj = session.query(tabla_ciudades.IdCiudad, tabla_ciudades.Ciudad)
        q_obj = q_obj.order_by(tabla_ciudades.Ciudad.asc())
        ciudades_tmp = q_obj.all()

        # Generamos la lista de elementos que devolveremos
        ciudades = []
        for c in ciudades_tmp:
            c_tmp = []
            c_tmp.append(int(c.IdCiudad))
            c_tmp.append(c.Ciudad)

            ciudades.append(c_tmp)

        # Cerramos la sesion
        session.close()

        return ciudades

    def sugerir_ciudad(self, parte_ciudad):
        '''
            Dado parte del nombre de una ciudad,
            sugiere posibles nombres de ciudades.

            Args:
                parte_ciudad (str)

            Returns:
                lista_ciudades[i]['id_ciudad']
                lista_ciudades[i]['nombre_ciudad']
                None

            Functions:
                validar_string(parte_ciudad)
                binded_session()
                tabla('ciudades')
        '''

        if not self.validar_string(parte_ciudad):
            return self.validar_string(parte_ciudad)

        if len(parte_ciudad) < 2:
            return None

        session = self.gis.binded_session()
        tabla_ciudades = self.gis.tabla('ciudades')

        q_obj = session.query(tabla_ciudades.IdCiudad, tabla_ciudades.Ciudad)
        q_obj = q_obj.filter(tabla_ciudades.Ciudad.like(
                                                    '%' + parte_ciudad + '%'))

        ciudades = q_obj.all()

        if int(q_obj.count()) == 0:
            session.close()
            return None

        lista_ciudades = []

        for registro in ciudades:
            c = {}
            c['id_ciudad'] = int(registro.IdCiudad)
            c['nombre_ciudad'] = registro.Ciudad
            lista_ciudades.append(c)

        session.close()
        return lista_ciudades


@DBSwitch('gis2')
class CiudadPostgres(ValidacionesEspecificas):
    '''
       Esta clase contiene todos lo métodos relacionados con las calles
       trabajando con Postgres y postgis.
    '''

    def __init__(self):
        self.f_geo = FuncionesGeo()
        self.provincia_postgres = ProvinciaPostgres()

    def obtener_ciudad(self, id_ciudad):
        '''
            Dado el ID de una ciudad obtengo su nombre.

            Argumentos:
                id_ciudad (int)

            Retorna:
                rta['Estado'] = (bool)
                rta['Respuesta'] = ciudad (str)

            Funciones:
                validar_id_ciudad(id_ciudad)
                binded_session()
                tabla('ciudades')
        '''

        # Definimos la respuesta
        rta = {'Estado': False}

        # Validamos
        try:
            self.validar_id_ciudad(id_ciudad)
        except:
            rta['Respuesta'] = _(u'Debe ingresar un id de ciudad válido.')
            return rta

        # Abrimos la session
        session = self.gis2.binded_session()
        tabla_ciudades = self.gis2.db['ciudades']
        tabla_provincias = self.gis2.db['provincias']
        # Hacemos la consulta
        try:
            q_obj = session.query(
                tabla_ciudades.id,
                tabla_ciudades.nombre,
                tabla_ciudades.id_provincia,
                tabla_ciudades.id_tipo,
                tabla_ciudades.geom,
                func.st_astext(tabla_ciudades.geom).label('punto'),
                tabla_provincias.id_pais,
            ).join((
                tabla_provincias,
                tabla_provincias.id == tabla_ciudades.id_provincia
            ))
            q_obj = q_obj.filter(tabla_ciudades.id == id_ciudad)
            ciudad = q_obj.first()
        except:
            session.close()
            rta['Respuesta'] = _(u'Falló al momento de realizar la consulta')
            return rta
        # Cerramos la session
        session.close()
        # Si esta vacio, salimos
        if ciudad is None:
            rta['Respuesta'] = _('''No se ha encontrado ninguna ciudad con
 el id: %s.''' % id_ciudad)
            return rta

        # Obtengo el punto de la ciudad (latitud y longitud)
        punto = None
        if ciudad.punto is not None:
            punto = self.f_geo.procesar_geometria(ciudad.punto)
            punto = [punto[0][1], punto[0][0]]

        # Sino, salimos bien
        rta['Estado'] = True
        rta['Respuesta'] = {
            'Nombre': ciudad.nombre.encode('latin1'),
            'Id': int(ciudad.id),
            'IdTipo': int(ciudad.id_tipo),
            'IdProvincia': int(ciudad.id_provincia),
            'IdPais': int(ciudad.id_pais),
            'Punto': punto,
        }
        return rta

    def listar_ciudades(self, id_provincia):
        '''
            Esta funcion devuelve un listado con todas las ciudades disponibles
            en el sistema. Estos datos corresponden al id y al nombre de la
            misma.

            Argumentos
                id_provincia (int)

            Retorna:
                rta['Estado'] (bool)
                rta['Respuesta'] ciudades (list [  list [(int),(str)] ])
        '''

        # Definimos respuesta
        rta = {'Estado': False}

        # Validamos
        try:
            self.validar_id_mapa(id_provincia)
        except:
            rta['Respuesta'] = _(u'Debe ingresar un id de provincia válido.')
            return rta

        # Abrimos la session
        session = self.gis2.binded_session()
        tabla_ciudades = self.gis2.db['ciudades']

        # Hacemos la consulta
        try:
            q_obj = session.query(tabla_ciudades).order_by(
                tabla_ciudades.nombre
            )
            q_obj = q_obj.filter(tabla_ciudades.id_provincia == id_provincia)
            ciudades_tmp = q_obj.all()
        except:
            session.close()
            rta['Respuesta'] = _(u'Falló al momento de realizar la consulta')
            return rta

        # Cerramos la sesion
        session.close()
        # Generamos la lista de elementos que devolveremos
        ciudades = []
        for ciudad in ciudades_tmp:
            c_tmp = []
            c_tmp.append(int(ciudad.id))
            c_tmp.append(ciudad.nombre.encode('latin1'))
            ciudades.append(c_tmp)

        # Sino, salimos bien
        rta['Estado'] = True
        rta['Respuesta'] = ciudades
        return rta

    def agregar_ciudad_aplicacion(self, id, id_ciudad):
        '''
            Esta funcion agrega una ciudad aplicacion, validando que no exista
            previamente una ciudad con este dato.

            Args:
                id (int)
                id_ciudad (int)

            Return:
                True
                False

            Funciones:
                validar_id_map
        '''

        # Definimos respuesta
        rta = {'Estado': False}

        # Validamos los datos de entrada
        try:
            self.validar_id_ciudad(id)
            self.validar_id_ciudad(id_ciudad)
        except:
            rta['Respuesta'] = _('Los datos de entrada no son validos')
            return rta

        # Validamos que exista la ciudad
        rta_ciudad = self.obtener_ciudad(id_ciudad)
        if not rta_ciudad['Estado']:
            return rta_ciudad

        # Validamos que no exista la ciudad aplicacion
        session = self.gis2.binded_session()
        tabla_ciudades_aplicaciones = self.gis2.db['ciudades_aplicaciones']

        try:
            q_obj = session.query(tabla_ciudades_aplicaciones)
            q_obj = q_obj.filter(or_(
                tabla_ciudades_aplicaciones.id == id,
                tabla_ciudades_aplicaciones.id_ciudad == id_ciudad,
            ))
            ciudad_aplicacion_tmp = q_obj.first()
        except:
            # Si fallo, cerramos y salimos
            session.close()
            rta['Respuesta'] = _(u'Falló al momento de realizar la consulta')
            return rta

        # Cerramos la session
        session.close()

        # Si encontramos una ciudad, salimos
        if ciudad_aplicacion_tmp is not None:
            rta['Respuesta'] = _(u'Ya existe la ciudad ingresada.')
            return rta

        # Agregamos la ciudad
        datos_ciudad_aplicacion = {
            'id': id,
            'id_ciudad': id_ciudad,
        }
        rta_agregar = self.gis2.agregar(
            'ciudades_aplicaciones',
            **datos_ciudad_aplicacion
        )

        # Si no se agrego correctamente, salimos
        if rta_agregar is not True:
            rta['Respuesta'] = _(u'No se pudo agregar la ciudad aplicación.')
            return rta

        # Sino, salimos correctamente
        rta['Estado'] = True
        rta['Respuesta'] = _('Se agregag� correctamente la ciudad aplicaci�n.')
        return rta

    def eliminar_ciudad_aplicacion(self, id):
        '''
            Dado el id de la ciudad aplicacion, borramos el campo en la tabla
            de ciudades aplicaciones

            Args:
                id (int)

            Retorna:
                rta['Estado'] (bool)
                rta['Respuesta'] (str)
        '''

        # Declaramos respuesta
        rta = {'Estado': False}
        # Validamos
        try:
            self.validar_id_ciudad(id)
        except:
            rta['Respuesta'] = _(u'Debe ingresar un id de ciudad válido.')
            return rta
        # Abrimos la session
        session = self.gis2.binded_session()
        tabla_ciudades_aplicaciones = self.gis2.db['ciudades_aplicaciones']
        # Intentamos realizar la consulta
        q_obj = session.query(tabla_ciudades_aplicaciones)
        q_obj = q_obj.filter(tabla_ciudades_aplicaciones.id == id)
        try:
            # Borramos el usuario
            q_obj.delete(synchronize_session=False)
        except:
            session.rollback()
            session.close()
            rta['Respuesta'] = _(u'No se pudo eliminar la ciudad aplicación.')
            return rta
        # SIno, comiteamos
        session.commit()
        session.close()
        # Salimos correctamente
        rta['Estado'] = True
        rta['Respuesta'] = _('Se elimin� correctamente la ciudad aplicaci�n')
        return rta

    def obtener_id_ciudad_aplicacion(self, id_ciudad):
        '''
            Dado el ID REAL de una ciudad obtengo su ID DE LA APLICACION.

            Args:
                id_ciudad (int)

            Returns:
                ciudad (str)

            Raises:
                'No se ha encontrado ninguna ciudad con el ID ingresado'

            Functions:
                validar_id_ciudad(id_ciudad)
                binded_session()
                tabla('ciudades')
        '''
        # Declaramos la respuesta
        rta = {'Estado': False}
        # Validamos
        try:
            self.validar_id_ciudad(id_ciudad)
        except:
            rta['Respuesta'] = _(u'Debe ingresar un id de ciudad válido.')
            return rta
        # Abrimos la session
        session = self.gis2.binded_session()
        tabla_ciudades_aplicaciones = self.gis2.db['ciudades_aplicaciones']
        # Hacemos la consulta
        try:
            q_obj = session.query(tabla_ciudades_aplicaciones.id)
            q_obj = q_obj.filter(
                tabla_ciudades_aplicaciones.id_ciudad == id_ciudad
            )
            ciudad_aplicacion = q_obj.first()
        except:
            session.close()
            rta['Respuesta'] = _(u'Falló al momento de realizar la consulta.')
            return rta
        # Cerramos la session
        session.close()
        # Si esta vacia, salimos
        if ciudad_aplicacion is None:
            rta['Respuesta'] = _('''No se ha encontrado ninguna ciudad con
 el id: %s.''' % id_ciudad)
            return rta
        # Sino, salimos bien
        rta['Estado'] = True
        rta['Respuesta'] = ciudad_aplicacion.id
        return rta

    def obtener_id_ciudad_real(self, id):
        '''
            Dado el ID DE LA APLICACION de una ciudad obtengo su ID REAL.

            Args:
                id (int)

            Returns:
                id_ciudad (int)

            Raises:
                'No se ha encontrado ninguna ciudad con el ID ingresado'

            Functions:
                validar_id_ciudad(id_ciudad)
                binded_session()
                tabla('ciudades')
        '''
        # Definimos la respuesta
        rta = {'Estado': False}
        # Validamos
        try:
            self.validar_id_ciudad(id)
        except:
            rta['Respuesta'] = _('Debe ingresar un id de ciudad valido.')
            return rta
        # Iniciamos session
        session = self.gis2.binded_session()
        tabla_ciudades_aplicaciones = self.gis2.db['ciudades_aplicaciones']
        # Hacemos la consulta
        try:
            q_obj = session.query(tabla_ciudades_aplicaciones.id_ciudad)
            q_obj = q_obj.filter(tabla_ciudades_aplicaciones.id == id)
            ciudad_real = q_obj.first()
        except:
            # Sino, salimos con error
            session.close()
            rta['Respuesta'] = _(u'Falló al momento de hacer la consulta.')
            return rta
        # Cerramos la session
        session.close()
        # Si esta vacia, salimos
        if ciudad_real is None:
            rta['Respuesta'] = _('''No se ha encontrado ninguna ciudad con
 el id: %s.''' % id)
            return rta
        # Sino, salimos bien
        rta['Estado'] = True
        rta['Respuesta'] = ciudad_real.id_ciudad
        return rta

    def obtener_ciudad_aplicacion(self, id_ciudad):
        '''
            Dado el ID de una ciudad obtengo su nombre.

            Args:
                id_ciudad (int)

            Returns:
                rta['Estado'] = (bool)
                rta['Respuesta'] id (int) � error (str)

            Funciones:
                obtener_id_ciudad_real
                obtener_ciudad
        '''

        # Obtenemos el id real
        rta_id_ciudad = self.obtener_id_ciudad_real(id_ciudad)
        if not rta_id_ciudad['Estado']:
            return rta_id_ciudad
        # Devolvemos la ciudad
        return self.obtener_ciudad(rta_id_ciudad['Respuesta'])

    def listar_ciudades_aplicacion(self, ordenar_por_nombre=False):
        '''
            Esta funcion devuelve un listado con todas las ciudades disponibles
            en el sistema. Estos datos corresponden al id y al nombre de la
            misma.

            Argumentos:
                ordenar_por_nombre (bool)

            Returns:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (list[list[(int),(str)]])
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Instancio la tabla "ciudades_aplicaciones"
        ciudades_aplicaciones = self.gis2.db['ciudades_aplicaciones']

        # Instancio la tabla "ciudades"
        ciudades = self.gis2.db['ciudades']

        # Busco el id de la ciudad aplicación y su nombre
        q_obj = session.query(
            ciudades_aplicaciones.id,
            ciudades.nombre,
        )

        # Hago un join con la tabla "ciudades" para obtener el nombre
        q_obj = q_obj.outerjoin((
            ciudades,
            ciudades_aplicaciones.id_ciudad == ciudades.id
        ))

        # Si recibo "ordenar_por_nombre" en True
        if ordenar_por_nombre is True:
            # Ordeno los registro por el nombre de la ciudad de forma ascendente
            q_obj = q_obj.order_by(ciudades.nombre.asc())

        # Guardo todos los registros obtenidos
        registros = q_obj.all()

        # Declaro una lista vacía
        lista = []

        # Recorro los registros
        for registro in registros:
            # Agrego cada registro a la lista
            lista.append(
                [
                    registro.id,
                    registro.nombre.encode('latin1'),
                ]
            )

        # Cierro la sesión con la base de datos gis2
        session.close()

        # Retorno True y la lista de ciudades
        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def obtener_id_ciudad_por_latlon(self, latitud, longitud):
        ''' Esta funcion devuelve el id de una ciudad buscando el segmento
        mas cercano a la posicion.

        Args:
            latitud (numero)
            longitud (numero)

        Returns:
            Caso incorrecto
                rta['Estado'] = False
                rta['Respuesta'] = 'No se encontraron datos cercanos a
                                    la posicion'
            Caso correcto
                rta['Estado'] = True
                rta['Respuesta'] = id_ciudad
        Functions:
            validar_numero()

        '''

        # Iniciamos nuestra respuesta
        rta = {'Estado': False}

        try:
            self.validar_numero(latitud)
            self.validar_numero(longitud)
        except:
            rta['Respuesta'] = _(u'La latitud y longitud deben ser válidos.')
            return rta

        # Hacemos la consulta, buscando las calles que esten cerca de nuestro
        # punto. Luego filtraremos quienes estan lo suficientemente cerca
        str_punto = 'POINT(%s %s)' % (str(longitud), str(latitud))
        session = self.gis2.binded_session()
        try:
            # Declaramos las tablas
            tabla_calles = self.gis2.db['calles']
            tabla_calles_nombres = self.gis2.db['calles_nombres']
        except:
            rta['Respuesta'] = _(u'Surgio un error con la base de datos.')
            return rta

        # Buscamos la calle donde esta parado
        q_obj = session.query(
            tabla_calles.gid,
            tabla_calles_nombres.id_ciudad,
            func.st_distance(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_punto, 4326
                )
            ).label('distancia'),
            func.st_astext(
                func.st_line_interpolate_point(
                    func.st_linemerge(tabla_calles.geom),
                    func.st_line_locate_point(
                        func.st_linemerge(tabla_calles.geom),
                        func.st_geomfromtext(
                            str_punto, 4326
                        )
                    )
                )
            ).label('punto_cercano'),
            func.st_astext(tabla_calles.geom).label('geom'),
        ).outerjoin((tabla_calles_nombres, and_(
                tabla_calles_nombres.id == tabla_calles.id_calle_nombre,
          ))
        )
        q_obj = q_obj.filter(
            func.st_dwithin(
                tabla_calles.geom,
                func.st_geomfromtext(
                    str_punto,
                    4326
                ).label('punto'),
                0.001
            )
        )
        # Ordenamos por distancia
        q_obj = q_obj.order_by('distancia')

        # Obtenemos todas las calles cercanas al punto que tenemos
        try:
            segmento = q_obj.first()
        except:
            # Cerramos la session
            session.close()
            # Salimos
            rta['Respuesta'] = _(u'''Fallo al momento de realizar la
 consulta. Chequeé el estado y codificación de la base de datos''')
            return rta

        # Si no se encontró una calle, buscamos en un radio mas grande.
        if segmento is None:
            # Como no encontramos nada Vamos a buscar la calle mas cercana
            # al punto, a como maximo 1000 metros
            q_obj = session.query(
                tabla_calles.gid,
                tabla_calles_nombres.id_ciudad,
                func.st_distance(
                    func.st_geomfromtext(
                        str_punto, 4326
                    ),
                    tabla_calles.geom
                ).label('distancia'),
                func.st_astext(tabla_calles.geom).label('geom'),
                func.st_astext(
                    func.st_line_interpolate_point(
                        func.st_linemerge(tabla_calles.geom),
                        func.st_line_locate_point(
                            func.st_linemerge(tabla_calles.geom),
                            func.st_geomfromtext(
                                str_punto, 4326
                            )
                        )
                    )
                ).label('punto_cercano'),
            ).outerjoin((tabla_calles_nombres, and_(
                    tabla_calles_nombres.id == tabla_calles.id_calle_nombre,
                ))
            )
            q_obj = q_obj.filter(
                    and_(
                        func.st_distance(
                            func.st_geomfromtext(
                                str_punto, 4326
                            ),
                            tabla_calles.geom
                        ) <= 0.01,
                    )
                )
            q_obj = q_obj.order_by('distancia')
            # Pedimos la calle mas cercana
            try:
                segmento = q_obj.first()
            except Exception:
                # Cerramos la session
                session.close()
                # Salimos
                rta['Respuesta'] = _(u'''Fallo al momento de realizar la
 consulta. Chequeé el estado y codificación de la base de datos''')
                return rta

        # Cerramos la session antes de salir
        session.close()
        if segmento is None:
            rta['Respuesta'] = _(
                u'No se encontró ningún segmento cercano al punto.'
            )
            return rta

        # Devolvemos la respuesta
        rta['Estado'] = True
        rta['Respuesta'] = segmento.id_ciudad
        return rta

    def sugerir_ciudad(self, parte_nombre):
        '''
            Esta función se encarga de obtener el nombre de la ciudad según
            una parte de su nombre.
            Solo tiene en cuenta las ciudades que están en la tabla
            "ciudades_aplicaciones" que son las que utiliza el cliente.

            Argumentos:
                parte_nombre (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Si la parte del nombre son menos de 3 letras
        if len(parte_nombre) < 3:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'Para buscar la ciudad debe ingresar un mínimo de 3 letras'
            )
            return rta

        # Me conecto a la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla ciudades
        ciudades = self.gis2.tabla('ciudades')

        # Obtengo la tabla ciudades_aplicaciones
        ciudades_aplicaciones = self.gis2.tabla('ciudades_aplicaciones')

        # Realizo una subquery para obtener las ciudades que usa la aplicación
        subquery = session.query(ciudades_aplicaciones.id_ciudad)
        subquery = subquery.subquery()

        # Pido los datos que me interesan de la tabla ciudades
        q_obj = session.query(ciudades.id, ciudades.nombre)

        # Filtro por las ciudades que usa la aplicación
        # y por la parte del nombre recibido
        q_obj = q_obj.filter(
            and_(
                ciudades.id.in_(subquery),
                ciudades.nombre.ilike('%' + parte_nombre + '%')
            )
        )

        # Guardo todos los registros obtenidos
        registros = q_obj.all()

        # Cierro la sesión con la base de datos gis2
        session.close()

        # Si no se encontró ninguna ciudad
        if len(registros) == 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró ninguna ciudad')
            return rta

        # Creo una lista para ir guardando las ciudades
        lista = []

        # Recorro las ciudades obtenidas
        for registro in registros:
            # Agrego cada ciudad a la lista
            lista.append(
                [
                    int(registro.id),
                    registro.nombre.encode('latin1')
                 ]
            )

        # Retorno True y la lista de ciudades
        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def __agregar(self, datos):

        # Abro la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla provincias.
        tabla_ciudades = self.gis2.tabla('ciudades')

        q_obj = session.query(tabla_ciudades)
        q_obj = q_obj.filter(tabla_ciudades.nombre == datos['nombre'])
        q_obj = q_obj.filter(
            tabla_ciudades.id_provincia == datos['id_provincia']
        )

        # Obtengo el registro.
        registro = q_obj.first()

        session.close()

        if registro is not None:
            return respuesta(
                False, _(u'Ya existe una ciudad con ese nombre en la provincia.')
            )

        # Agrego la ciudad.
        agregar_ciudad = self.gis2.insertar(
            'ciudades', 'id', **datos
        )

        # Retorno la respuesta.
        if agregar_ciudad['Estado'] is not True:
            return respuesta(False, _(u'No se pudo agregar la ciudad.'))

        # Obtengo el id.
        id = agregar_ciudad['ValorCampo']

        # Guardo el id.
        datos['id'] = id

        return respuesta(
            True, _(u'Se agregó correctamente la ciudad.'), datos
        )

    def grabar(self, id_provincia, nombre, id_ciudad=None, id_tipo=3):

        '''
        Esta función crea o modifica una ciudad
        :param id_provincia:
        :param nombre:
        :param id_ciudad:
        :param id_tipo:
        :return:
        '''

        # Validaciones

        # Declaro un dic para guardar los datos.
        datos = {}

        # Valido el id_ciudad si no es None.
        if id_ciudad is not None:
            # Valido el id_provincia.
            v_id_ciudad = numero_entero_positivo(id_ciudad)
            if v_id_ciudad['Estado'] is not True:
                return v_id_ciudad

        # Valido que exista la provincia.
        v_id_provincia = self.provincia_postgres.obtener_provincia(id_provincia)
        if v_id_provincia['Estado'] is not True:
            return v_id_provincia

        # Valido el id_pais.
        v_nombre = string(nombre)
        if v_nombre['Estado'] is not True:
            return v_nombre

        # Valido el id_tipo.
        v_id_tipo = numero_entero_positivo(id_tipo)
        if v_id_tipo['Estado'] is not True:
            return v_id_tipo

        # Guardo los datos.
        datos['id_provincia'] = id_provincia
        datos['id_tipo'] = id_tipo
        datos['nombre'] = nombre

        # Si no tengo id_ciudad, trato de agregar.
        if id_ciudad is None:

            ### VERIFICO SI YA EXISTE UNA CIUDAD CON ESE NOMBRE.

            # Abro la sesión.
            session = self.gis2.binded_session()

            # Obtengo la tabla ciudades.
            tabla_ciudades = self.gis2.tabla('ciudades')

            q_obj = session.query(tabla_ciudades)
            q_obj = q_obj.filter(tabla_ciudades.nombre == nombre)
            q_obj = q_obj.filter(tabla_ciudades.id_provincia == id_provincia)

            # Obtengo el registro.
            registro = q_obj.first()

            # Cierro sesión.
            session.close()

            if registro is not None:
                return respuesta(
                    False, _(u'Ya existe una ciudad con ese nombre')
                )

            ###

            # Retorno.
            return self.__agregar(datos)

        # Si recibo el id, modifico la provincia.

        # Abro la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla provincias.
        tabla_ciudades = self.gis2.tabla('ciudades')

        q_obj = session.query(tabla_ciudades)
        q_obj = q_obj.filter(tabla_ciudades.id == id_ciudad)

        # Obtengo el registro.
        registro = q_obj.first()

        # Si no existe lo agrego.
        if registro is None:

            datos['id'] = id_ciudad

            return self.__agregar(datos)

        # Actualizo.
        update = q_obj.update(datos)

        # Aplico los cambios
        session.commit()

        # Cierro sesión.
        session.close()

        datos['id'] = id_ciudad

        # Sino se pudo modificar, retorno la respuesta.
        if update == 0:
            return respuesta(False, _(u'No se puedo modificar la ciudad.'))

        # Retorno.
        return respuesta(
            True, _(u'Se actualizó correctamente la ciudad.'), datos
        )

    def eliminar(self, id_ciudad):
        '''
        Esta función elimina una ciudad.
        :param id_ciudad:
        :return:
        '''
        # Valido el id_provincia.
        v_id_ciudad = numero_entero_positivo(id_ciudad)
        if v_id_ciudad['Estado'] is not True:
            return v_id_ciudad


        # Abro la sesión.
        session = self.gis2.binded_session()

        # Obtengo la tabla ciudades.
        tabla_ciudades = self.gis2.tabla('ciudades')

        # Determino los campos que quiero obtener
        q_obj = session.query(
            tabla_ciudades.id
        )

        # Filtro por id.
        q_obj = q_obj.filter(tabla_ciudades.id == id_ciudad)

        # Verifico si ya existe un usuario.
        if int(q_obj.count()) == 0:
            # Cierro sesión.
            session.close()
            return respuesta(False, _(u'No existe la ciudad.'))

        # Cierro sesión.
        session.close()

        # Si supera todos los filtros borro la ciudad.
        datos = {}
        datos['id'] = ('==', id_ciudad)
        borrar = self.gis2.borrar('ciudades', **datos)
        # En el caso que falle la función retorno el error.
        if borrar is not True:
            return respuesta(False, _(u'Error al eliminar la ciudad.'))

        # Retorno.
        return respuesta(True, _(u'Se eliminó correctamente la ciudad.'))

