# -*- coding: utf-8 -*-
from twisted.python.logfile import LogFile
from twisted.python import log

import logging.handlers
import logging
import os
import sys

# Logging levels
DEBUG = logging.DEBUG
INFO = logging.INFO
WARNING = logging.WARNING
ERROR = logging.ERROR
CRITICAL = logging.CRITICAL
SILENT = CRITICAL + 1


LOG_PATH = None
level_names = {
    None: 'MSG',
    logging.DEBUG: "DEBUG",
    logging.INFO: "INFO",
    logging.WARNING: "WARNING",
    logging.ERROR: "ERROR",
    logging.CRITICAL: "CRITICAL",
    SILENT: "SILENT",
}

started = False


class CustomFileLogObserver(log.FileLogObserver):

    def __init__(self, f, level=INFO, encoding='utf-8', mode=None):
        self.level = level
        self.encoding = encoding
        self.mode = mode
        log.FileLogObserver.__init__(self, f)

    def emit(self, eventDict):
        # Evitamos los logs del servicio de SSH
        if (str(eventDict.get('system')).find('SSHService') != -1 and
           eventDict.get('mode') is None):
            return

        # Enviamos los logs al trash
        if ((str(eventDict.get('system')).strip() == '-' or
           str(eventDict.get('system')).strip() == '') and
           eventDict.get('mode') is None):
            # Lo mandamos al trash
            self.mode = 'trash'
            eventDict['mode'] = 'trash'

        # Si no es todo None, salimos
        if self.mode is None and eventDict.get('mode') is not None:
            return

        # Si no conicide el modo, salimos
        if (eventDict.get('mode') != self.mode and
           eventDict.get('mode') is not None):
            return

        ev = _adapt_eventdict(eventDict, self.level, self.encoding)
        if ev is not None:
            log.FileLogObserver.emit(self, ev)


def _adapt_eventdict(ev, log_level=INFO, encoding='utf-8',
                                                        prepend_level=True):
    """ Adaptamos el diccionario de errores de twisted, arreglando cualquier
    error de encoding
    """
    #ev = eventDict.copy()
    if ev['isError']:
        ev.setdefault('logLevel', ERROR)

    level = ev.get('logLevel')
    spider = ev.get('spider')
    if spider:
        ev['system'] = spider.name
    message = ev.get('message')
    lvlname = level_names.get(level, 'NOLEVEL')
    if message:
        message = [unicode_to_str(x, encoding) for x in message]
        if prepend_level:
            message[0] = "%s: %s" % (lvlname, message[0])
    ev['message'] = message
    why = ev.get('why')
    if why:
        why = unicode_to_str(why, encoding)
        if prepend_level:
            why = "%s: %s" % (lvlname, why)
    ev['why'] = why
    return ev


def unicode_to_str(text, encoding=None):
    """ Le pasamos un texto en unicode y lo pasa a string para que no se rompa
    todo.
    """

    if encoding is None:
        encoding = 'utf-8'
    if isinstance(text, unicode):
        return text.encode(encoding)
    elif isinstance(text, str):
        return text
    else:
        raise TypeError(
            'Se debe ingresar una cadena, obtuve %s' % str(
                type(text).__name__
            )
        )


def _get_log_level(level_name_or_id=None):
    if level_name_or_id is None:
        return 'DEBUG'
    elif isinstance(level_name_or_id, int):
        return level_name_or_id
    elif isinstance(level_name_or_id, basestring):
        return globals()[level_name_or_id]
    else:
        raise ValueError("Unknown log level: %r" % level_name_or_id)


class CustomStdout:

    def __init__(self):
        pass

    def wirite(self, *args, **kwargs):
        sys.stdout.write(*args, **kwargs)

    def flush(self, *args, **kwargs):
        sys.stdout.flush()


def start(logfile, debug=False):
    # check twisted log not already started
    global started

    if started:
        return

    started = True
    log_level = _get_log_level(None)
    if not debug:
        sflo = CustomFileLogObserver(
            LogFile.fromFullPath(
                logfile,
                # 5 Mb para cada log
                rotateLength=5242880,
                # Hasta 5 archivos
                maxRotatedFiles=5,
            ),
            log_level,
            'utf-8'
        )
        # Iniciamos
        log.startLoggingWithObserver(sflo.emit, setStdout=False)
        log.msg("######################################")
        log.msg("## INICIANDO <%s> " % str(
            os.path.basename(logfile).upper()
        ))
        log.msg("######################################")
    else:
        # Iniciamos
        log.startLogging(CustomStdout())


def start_custom_log(logfile, mode, loglevel=None):
    if not os.path.isdir(os.path.dirname(logfile)):
        os.mkdir(os.path.dirname(logfile))

    loglevel = _get_log_level(loglevel)
    flo = CustomFileLogObserver(
        LogFile.fromFullPath(
            '%s.%s' % (logfile, mode),
            # 5 Mb para cada log
            rotateLength=5242880,
            # Hasta 5 archivos
            maxRotatedFiles=5
        ),
        loglevel,
        'utf-8',
        mode=mode
    )
    log.addObserver(flo.emit)
