# -*- coding: utf-8 -*-

# Librerias para AWS.
import os
import shutil
try:
    import boto
    import boto.s3
    import boto.s3.connection
    import mimetypes
    from boto.utils import compute_md5
    aws = True
except:
    aws = False

from PIL import Image
from math import sqrt
from math import floor
from cStringIO import StringIO

from at.map.utiles.funciones_geo import FuncionesGeo
from at.map.utiles.layer import Layer
from at.map.utiles.tile import Tile
from at.map.tile_generator import generate_tile
from at.utiles.validaciones_generales import ValidacionesGenerales


class Mapas(ValidacionesGenerales):

    def __init__(self):
        self.layer = Layer()
        self.fg = FuncionesGeo()

    def existe_imagen_tile(self, x, y, z, basico=False):
        ''' Funcion para acceder a a la funcion de si existe una de las
        imagenes o no '''

        return Tile(self.layer, x, y, z).existe_imagen(basico)

    def obtener_xy_por_latlon(self, latitud, longitud, z):
        ''' Esta funcion devuelve el X e Y en pixeles, en la altura Z dada,
        la latitud y la longitud dada.

        Argumentos:
            latitud (float) Grados
            longitud (flota) Grados
            z (int)

        Retorna:
            (x, y)
            None
        '''
        # Validamos los datos
        self.validar_numero(latitud)
        self.validar_no_vacio(latitud)
        self.validar_numero(longitud)
        self.validar_no_vacio(longitud)
        self.validar_entero(z)

        # Obtenemos la cantidad de tiles que hay en la capa:
        cant_tiles_horizontales = 2 ** (z + 1)
        cant_tiles_verticales = 2 ** z

        # Ahora calculamos el tile correspondiente a la longitud
        x = int(
            float(longitud + 180) * float(cant_tiles_horizontales * 256) / 360.0
        )

        # Calculamos el y en relacion a la latitud
        y = int(
            float(latitud + 90) * float(cant_tiles_verticales * 256) / 180.0
        )

        return (x, y)

    def obtener_tile_por_latlon(self, latitud, longitud, z):
        ''' Esta funcion devuelve el X e Y del tile al que pertenece, en la
        altura Z dada, la latitud y la longitud dada.

        Argumentos:
            latitud (float) Grados
            longitud (flota) Grados
            z (int)

        Retorna:
            (x, y)
            None
        '''

        # Validamos los datos
        self.validar_numero(latitud)
        self.validar_no_vacio(latitud)
        self.validar_numero(longitud)
        self.validar_no_vacio(longitud)
        self.validar_entero(z)

        # Obtenemos la cantidad de tiles que hay en la capa:
        cant_tiles_horizontales = 2 ** (z + 1)
        cant_tiles_verticales = 2 ** z

        # Ahora calculamos el tile correspondiente a la longitud
        x = int(float(longitud + 180) * float(cant_tiles_horizontales) / 360.0)
        # Calculamos el y en relacion a la latitud
        y = int(float(latitud + 90) * float(cant_tiles_verticales) / 180.0)

        return (x, y)

    def obtener_imagen_cuadrante(self, alto, ancho, lat1, lon1, lat2, lon2):
        ''' Esta funcion toma como parametro un alto y un ancho en pixeles,
        un par de puntos geograficos y devuelve una imagen de un cuadrante
        con el tamaño máximo, centrado en ambos puntos

        Argumentos:
            alto: (int)
            ancho: (int)
            lat1: (float) Grados
            lon1: (float) Grados
            lat2: (float) Grados
            lon2: (flota) Grados

        Retorna:
            rta['Estado']
            rta['Respuesta']
                ['Data'] = data (binary)
                ['Vertice1']
                    ['Latitud']
                    ['Longitud']
                ['Vertice2']
                    ['Latitud']
                    ['Longitud']
        '''

        # Definimos la respuesta
        rta = {'Estado': False}

        # Validamos los datos
        try:
            self.validar_entero(alto)
            self.validar_entero(ancho)
        except:
            rta['Respuesta'] = 'El alto y el ancho deben ser enteros.'
            return rta

        try:
            self.validar_no_vacio(lat1)
            self.validar_numero(lat1)
            self.validar_no_vacio(lon1)
            self.validar_numero(lon1)
            self.validar_no_vacio(lat2)
            self.validar_numero(lat2)
            self.validar_no_vacio(lon2)
            self.validar_numero(lon2)
        except:
            rta['Respuesta'] = 'Las posiciones deben ser del tipo "float".'
            return rta

        try:
            # Definimos la altura maxima de capa
            z_maximo = 16
            # Definimos un margen de pixeles para que no esten al borde
            # de la pantalla
            margen = 40
            # Calculamos el maximo de la pantalla
            maximo = ancho
            minimo = alto
            if alto > ancho:
                maximo = alto
                minimo = ancho
            # Obtenemos la distancia
            distancia = self.fg.kms_entre_puntos(lat1, lon1, lat2, lon2)
            # Calculamos la capa
            z = None
            for i in range(z_maximo + 1):
                x1, y1 = self.obtener_xy_por_latlon(lat1, lon1, i)
                x2, y2 = self.obtener_xy_por_latlon(lat2, lon2, i)
                distancia = sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2)
                if distancia > minimo - margen:
                    # La altura valida era la anterior
                    z = i - 1
                    break
            if z is None:
                z = z_maximo
            # Calculamos el x e y del punto medio
            lat_punto_medio = lat1 + (lat2 - lat1) / 2
            lon_punto_medio = lon1 + (lon2 - lon1) / 2
            x_tile_p_medio, y_tile_p_medio = self.obtener_tile_por_latlon(
                lat_punto_medio, lon_punto_medio, z
            )
            x_p_medio, y_p_medio = self.obtener_xy_por_latlon(
                lat_punto_medio, lon_punto_medio, z
            )
            # Calculamos la posicion del punto medio DENTRO DEL TILE
            # en pixeles. Ej: p(20, 45)
            delta_x_p_medio = x_p_medio - (x_tile_p_medio * 256)
            delta_y_p_medio = y_p_medio - (y_tile_p_medio * 256)

            # Calculamos los datos de la imagen a crear
            cant_tiles = 5
            lado = cant_tiles * 256
            # Generamos la imagen donde vamos a trabajar
            imagen = Image.new("RGB", (lado, lado))

            # Obtenemos los x e y de los tiles de su alrededor
            for i in range(cant_tiles):
                x = int(x_tile_p_medio - floor(cant_tiles / 2) + i)
                for j in range(cant_tiles):
                    y = int(y_tile_p_medio - floor(cant_tiles / 2) + j)
                    # Obtenemos la imagen del tile
                    data = self.obtener_imagen_tile(x, y, z)
                    # Si no obtuvimos imagen, intentamos generarla
                    if data is None:
                        rta_tile = generate_tile(int(x), int(y), int(z))
                        # Si falla, salimos
                        if not rta_tile:
                            # Mostramos el mensaje y salimos
                            rta['Respuesta'] = '''No se pudo crear la imagen
 %s %s %s''' % (str(x), str(y), str(z))
                            return rta
                        # Si salio bien y lo genero, lo obtenemos
                        data = self.obtener_imagen_tile(x, y, z)
                    # Lo agregamos a la imagen
                    imagen.paste(
                        Image.open(StringIO(data)),
                        (i * 256, (cant_tiles - 1 - j) * 256)
                    )
            # Calculamos el delta de inicio para el cuadrado a recortar
            delta_x = int((lado - maximo) / 2) + (delta_x_p_medio - 128)
            delta_y = int((lado - maximo) / 2) - (delta_y_p_medio - 128)
            # Recortamos
            imagen = imagen.crop((
                delta_x, delta_y,
                maximo + delta_x, maximo + delta_y
            ))

            # Calculamos los vertices superior izquierdo y el vertice
            # inferior derecho
            # Latitud superior izquierda
            lat_vertice1 = (y_p_medio - (maximo - delta_y_p_medio))
            lat_vertice1 *= lat_punto_medio / y_p_medio
            # Longitud superior izquierda
            lon_vertice1 = (x_p_medio + (maximo)) * lon_punto_medio / x_p_medio

            # Latitud inferior derecha
            lat_vertice2 = (y_p_medio + (maximo - delta_y_p_medio))
            lat_vertice2 *= lat_punto_medio / y_p_medio
            # Longitud inferior derecha
            lon_vertice2 = (x_p_medio - (maximo)) * lon_punto_medio / x_p_medio

            # Verifico si ya estoy en la última capa
            ultima_capa = False
            if z == z_maximo:
                ultima_capa = True

            # Salvamos la imagen a una variable, con calidad 30
            str_buffer = StringIO()
            imagen.save(str_buffer, 'jpeg', quality=30)
            str_buffer.seek(0)
            data = str_buffer.read()

            # Generamos la respuesta
            rta['Estado'] = True
            rta['Respuesta'] = {
                'Data': data,
                'Vertice1': {
                    'Latitud': lat_vertice1,
                    'Longitud': lon_vertice1
                },
                'Vertice2': {
                    'Latitud': lat_vertice2,
                    'Longitud': lon_vertice2
                },
                'UltimaCapa': ultima_capa
            }
            # Devolvemos la respuesta final
            return rta
        except:
            # Si algo falló retorno un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = 'No se pudo obtener el mapa'
            return rta

    def obtener_imagen_tile(self, x, y, z):
        ''' Funcion paar obtener la imagen de un tile segun x y z '''

        # Creamos y devolvemos
        return Tile(self.layer, x, y, z).obtener_imagen()

    def obtener_tile_padre_xy(self, x, y, z=0):
        ''' Funcion para obtener el padre de un tile segun un x y z'''

        # Creamos y devolvemos
        tile = Tile(self.layer, x, y, z).obtener_padre()

        # Devolvemos el x y el y:
        return tile.x, tile.y

    def sincronizar_mapas(self, sub_directorio, z1=10, z2=17):
        '''
        Esta función sincroniza las imagenes que están en /var/lib/at/mapas y
         las sincroniza con la S3 de AWS.
        :param sub_directorio:
        :param z1:
        :param z2:
        :return:
        '''
        # Si no estoy en AWS retorno.
        if aws is False:
            return

        # AWS ACCESS DETAILS
        AWS_ACCESS_KEY_ID = 'AKIAIWIEGKJTKVCJOUAA'
        AWS_SECRET_ACCESS_KEY = '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'

        # Defino el nombre del bucket.
        bucket_name = 'map.axsisweb'

        # El destino de las imagenes.
        dest_folder = 'map/1.0.0/basic'

        # Genero la conexión.
        conn = boto.connect_s3(
            AWS_ACCESS_KEY_ID,
            AWS_SECRET_ACCESS_KEY,
            calling_format=boto.s3.connection.OrdinaryCallingFormat()
        )

        # Abro el bucket.
        bucket = conn.get_bucket(bucket_name)

        # Defino de donde tengo que buscar las imagenes.
        src_folder = '/var/lib/at/mapas'

        # Aca voy a guardar los zooms.
        z_folders = []

        # Parseo.
        z1 = int(z1)
        z2 = int(z2)

        # Creo los zoom.
        for i in range(z1, z2+1):
            z = str(i)
            z = 'z' + z
            z_folders.append(z)

        # Obtengo la ruta de las imagenes.
        ruta_img = os.path.join(src_folder, sub_directorio)

        # Aca voy a guardar las rutas del las imagenes que tengo invalidar para
        # la s3.
        img_list = []

        # Recorro los zoom.
        for z_folder in z_folders:

            # Recorro las subcarpetas.
            for root, dirs, files in os.walk(os.path.join(ruta_img, z_folder)):
                if files and not '.svn' in root:
                    path = os.path.relpath(root, ruta_img)
                    for f in files:
                        name = os.path.normpath(os.path.join(path, f))

                        # Obtengo el nombre del objeto.
                        filename = os.path.join(
                            dest_folder, name.replace('.jpeg', '.jpg')
                        )

                        # Obtengo el objeto a actualizar
                        content = open(os.path.join(ruta_img, name))

                        # Obtengo el tipo de archivo
                        filetype, encoding = mimetypes.guess_type(name)
                        filetype = filetype or 'application/octet-stream'

                        # Seteo la cabecera.
                        headers = {'Content-Type': filetype, 'x-amz-acl': 'public-read'}

                        # Si existe el objeto, lo obtengo.
                        if bucket.get_key(filename):

                            key = bucket.get_key(filename)

                            # Armo la ruta de la imagen a actualizar.
                            ruta_bucket = '/' + key.name

                            # La agrego a la lista.
                            img_list.append(ruta_bucket)

                        # Sino, lo creamos.
                        else:

                            key = bucket.new_key(filename)

                        # Guardo el contenido en el objeto.
                        key.set_contents_from_file(
                            content, headers, md5=compute_md5(content)
                        )
                        key.make_public()
                        content.close()

        # Cuando termino de sincronizar, borro la carpetade las imagenes.
        shutil.rmtree(ruta_img, ignore_errors=True)

        # if len(img_list) > 0:
        #     # Invalido las imagenes que voy actualizar.
        #     c = boto.connect_cloudfront(
        #         AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY
        #     )
        #     c.create_invalidation_request(u'EGWA3C9LVVAEJ', img_list)

        return {
            'Estado': True,
            'Respuesta': u'Se sincronizó el mapa correctamente.'
        }

