# -*- coding: utf-8 -*-

from at.map import _
from at.map.exc import *
from at.map.utiles.validaciones_especificas import ValidacionesEspecificas
from at.orm.decorators import DBSwitch
from at.map.calle import CallePostgres

#from at.utiles.validaciones_generales import ValidacionesGenerales


@DBSwitch('cp')
class Sitio(ValidacionesEspecificas):
    '''
       Esta clase contiene todos lo métodos relacionados con los sitios.
    '''

    def __init__(self):
        self.calles_postgres = CallePostgres()

    def obtener_sitio(self, id_ciudad, id_sitio):
        '''
            Dado el ID de una ciudad y el ID de un sitio obtengo su nombre.

            Args:
                id_ciudad (int)
                id_sitio (int)

            Returns:
                dict

            Raises:
                'No se pudo encontrar el sitio. IdSitio = %s'

            Functions:
                validar_id_ciudad(id_ciudad)
                validar_id_sitio(id_calle)
        '''

        if not self.validar_id_ciudad(id_ciudad):
            return self.validar_id_ciudad(id_ciudad)

        if not self.validar_id_calle(id_sitio):
            return self.validar_id_calle(id_sitio)

        session = self.cp.binded_session()

        sitios = self.cp.tabla('sitios')
        q_obj = session.query(sitios)
        q_obj = q_obj.filter(sitios.IdSitio == id_sitio)
        registro = q_obj.first()

        session.close()

        if registro is None:
            raise (
                Exception,
                'No se pudo encontrar el sitio. IdSitio = %s' % id_sitio
            )

        rta = {}
        rta['id_sitio'] = registro.IdSitio
        rta['id_sitio_tipo'] = registro.IdSitioTipo
        rta['nombre_sitio'] = registro.Nombre
        rta['latitud_sitio'] = float(registro.Latitud)
        rta['longitud_sitio'] = float(registro.Longitud)
        rta['id_ciudad_sitio'] = registro.IdCiudad
        rta['mostrar_sitio_en_mapa'] = registro.MostrarEnMapa
        return rta

    def sugerir_sitio(self, id_ciudad, parte_sitio):
        '''
            Dado un id de una ciudad y parte del nombre de un sitio,
            sugiere posibles nombres de sitios que pertencen a esa ciudad.
            Si id_ciudad es None no se filtra por la ciudad

            Args:
                id_ciudad (int) // puede ser None
                parte_sitio (str)

            Returns:
                lista_sitios[i]['id_sitio']
                lista_sitios[i]['nombre_sitio']
                o
                None

            Functions:
                validar_string(parte_sitio)
                validar_id_ciudad(id_ciudad)
        '''

        if not self.validar_string(parte_sitio):
            return self.validar_string(parte_sitio)

        if len(parte_sitio) < 3:
            return None

        if id_ciudad is not None and not self.validar_id_ciudad(id_ciudad):
            return self.validar_id_ciudad(id_ciudad)

        session = self.cp.binded_session()

        sitios = self.cp.tabla('sitios')

        q_obj = session.query(sitios.IdSitio, sitios.Nombre)

        if id_ciudad is not None:
            q_obj = q_obj.filter(sitios.IdCiudad == id_ciudad)

        q_obj = q_obj.filter(sitios.Nombre.like('%' + parte_sitio + '%'))

        registros = q_obj.all()

        session.close()

        lista_sitios = []

        if int(q_obj.count()) == 0:
            return lista_sitios

        for registro in registros:
            c = {}
            c['id_sitio'] = int(registro.IdSitio)
            c['nombre_sitio'] = registro.Nombre
            lista_sitios.append(c)

        return lista_sitios

    #se agrego para sitios
    def listar_tipos_sitios(self):
        ''' Funcion para listar los tipos de sitios.

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: list[[int, str]]
        '''

        rta = {}
        lista_sitios_tipos = []
        # Me conecto a la base de datos cp2000
        session = self.cp.binded_session()

        # Traigo la tabla sitios_tipos
        sitios_tipos = self.cp.tabla('sitios_tipos')

        # Traigo todos los registros para obtener todos los tipos de sitios
        q_obj = session.query(
            sitios_tipos.IdSitioTipo,
            sitios_tipos.Descripcion
        )
        # Ordeno registros por la descripción en forma ascendente
        q_obj = q_obj.order_by(sitios_tipos.Descripcion.asc())
        registros = q_obj.all()
        session.close()

        for registro in registros:
            dic = {}
            dic['IdSitioTipo'] = registro.IdSitioTipo
            dic['Descripcion'] = registro.Descripcion
            lista_sitios_tipos.append(dic)
        rta['Estado'] = True
        rta['Respuesta'] = lista_sitios_tipos
        return rta

    def listar_sitios(self, id_tipo=None):
        ''' Funcion para listar los sitios de un tipo determinado.

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: list[[int, str]]
        '''

        rta = {}
        lista_sitios = []
        # Me conecto a la base de datos cp2000
        session = self.cp.binded_session()

        # Traigo la tabla sitios_tipos
        sitios = self.cp.tabla('sitios')

        # Traigo todos los registros para obtener todos los tipos de sitios
        q_obj = session.query(
            sitios.IdSitio,
            sitios.IdSitioTipo,
            sitios.Nombre
        )
        # Si nos pasan el tipo, filtramos por el mismo
        if id_tipo is not None:
            # Validamos el id_tipo de la zona
            rta_tipo = self.obtener_tipo(id_tipo)
            # Si no existe, salimos
            if not rta_tipo['Estado']:
                session.close()
                return rta_tipo
            # Filtro por tipo de sitio
            q_obj = q_obj.filter(sitios.IdSitioTipo == id_tipo)
        # Ordeno registros por la descripción en forma ascendente
        q_obj = q_obj.order_by(sitios.Nombre.asc())
        registros = q_obj.all()
        session.close()

        for registro in registros:
            dic = {}
            dic['IdSitio'] = registro.IdSitio
            dic['Nombre'] = registro.Nombre
            dic['IdSitioTipo'] = registro.IdSitioTipo
            lista_sitios.append(dic)

        rta['Estado'] = True
        rta['Respuesta'] = lista_sitios
        return rta

    def obtener(self, id_sitio):
        ''' Funcion para obtener un sitio del sistema.

        Argumentos:
            id_sitio: int
        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        rta = {}
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_sitio)
        except:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'El id ingresado no es valido.')
            return rta

        # Generamos la session y la tabla
        session = self.cp.binded_session()
        tabla_sitios = self.cp.tabla('sitios')
        # Pedimos el interno que coincida
        q_obj = session.query(
            tabla_sitios.IdSitio,
            tabla_sitios.IdSitioTipo,
            tabla_sitios.Nombre,
            tabla_sitios.Latitud,
            tabla_sitios.Longitud,
            tabla_sitios.IdCiudad,
            tabla_sitios.MostrarEnMapa
        ).filter(
            tabla_sitios.IdSitio == id_sitio
        )
        # Buscamos
        sitio_tmp = q_obj.first()
        # Cerramos la session
        session.close()
        # Si fallamos, salimos
        if sitio_tmp is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el sitio buscado')
            return rta
        # Sino, parseamos
        sitio = {}
        sitio['IdSitio'] = int(sitio_tmp.IdSitio)
        sitio['IdSitioTipo'] = int(sitio_tmp.IdSitioTipo)
        sitio['Nombre'] = sitio_tmp.Nombre.decode('iso-8859-1')
        # Procesamos el marcador
        sitio['Ubicacion'] = []
        puntos = []
        # Pasamos a grados
        sitio_tmp.Latitud = float(sitio_tmp.Latitud / 3600)
        sitio_tmp.Longitud = float(sitio_tmp.Longitud / 3600)
        puntos.append(sitio_tmp.Latitud)
        puntos.append(sitio_tmp.Longitud)
        sitio['Ubicacion'] = puntos
        sitio['IdCiudad'] = int(sitio_tmp.IdCiudad)
        sitio['MostrarEnMapa'] = int(sitio_tmp.MostrarEnMapa)
        rta['Estado'] = True
        rta['Respuesta'] = sitio
        return rta

    def grabar(self, id_tipo, nombre, puntos=None, id_ciudad=None,
               id_sitio=None):
        ''' Funcion para grabar un sitio en la base de datos

        Argumentos:
            id_tipo: int
            nombre: str
            puntos: list[list[float, float]] (lat, lon)
            id_ciudad: int
            id_sitio: int

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        rta = {}
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_tipo)
            self.validar_string_no_vacio(nombre)
            if id_sitio is not None:
                self.validar_entero(id_sitio)
        except:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'Los datos ingresados no son validos.')
            return rta

        # # Validamos el id_tipo de sitio
        # rta_tipo = self.obtener_tipo(id_tipo)
        # # Si no existe, salimos
        # if not rta_tipo['Estado']:
        #     return rta_tipo

        # Generamos la session y la tabla
        session = self.cp.binded_session()
        tabla_sitios = self.cp.tabla('sitios')
        if puntos is not None:
            # No es un punto, asi que vamos a vaciar
            if len(puntos) < 2:
                puntos = None
        if puntos is not None and puntos != [0.0, 0.0]:
            # data_punto = self.calles_postgres.obtener_posicion_por_latlon(
            #     puntos[0], puntos[1]
            # )
            # if not data_punto['Estado']:
            #     return data_punto
            # id_ciudad = data_punto['Respuesta']['IdLocalidad']
            # FIX, no obtengo la ciudad porque se está poniendo lenta la función
            # pensar como ahcer esto de una forma más óptima
            id_ciudad = 0
            # Armamos diccionario de datos
            datos_sitio = {
                'IdSitioTipo': id_tipo,
                'Nombre': nombre,
                'Latitud': puntos[0] * 3600,
                'Longitud': puntos[1] * 3600,
                'IdCiudad': id_ciudad,
                'MostrarEnMapa': 1
            }
        else:
            datos_sitio = {
                'IdSitioTipo': id_tipo,
                'Nombre': nombre,
                'Latitud': 0,
                'Longitud': 0,
                'IdCiudad': 0,
                'MostrarEnMapa': 0
            }
        #Se agregan sitios nuevos
        #Como no hay id, agregamos el sitio
        if id_sitio is None:
            r = self.cp.agregar('sitios', **datos_sitio)
            # Vemos si salio algo mal
            if r is not True:
                session.rollback()
                session.close()
                rta['Estado'] = False
                rta['Respuesta'] = _(u'No se pudo agregar el la zona.')
                return rta
            # Sino, guardamos y cerramos el session
            session.commit()
            session.close()
        ## Si nos pasan el id, vemos si existe
        else:
            # Pedimos el interno que conicida
            q_obj = session.query(tabla_sitios)
            q_obj = q_obj.filter(tabla_sitios.IdSitio == id_sitio)
            # Obtenemos
            sitio_tmp = q_obj.first()
            if sitio_tmp is None:
                session.close()
                rta['Estado'] = False
                rta['Respuesta'] = _(u'No existe el sitio buscado')
                return rta
            # Como encontramos, vamos a actualizar
            try:
                q_obj.update(datos_sitio, synchronize_session=False)
                session.commit()
                session.close()
            except:
                session.rollback()
                session.close()
                # Salimos, indicando que fallo!!!
                rta['Estado'] = False
                rta['Respuesta'] = _(u'No se pudo actualizar el sitio.')
                return rta
        # Como llegamos hasta aca, salimos bien!!!
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se grabó correctamente el sitio')
        return rta

    def obtener_tipo(self, id_tipo):
        ''' Funcion para obtener un tipo de sitio especifico

        Argumentos:
            id_tipo: int
        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        rta = {}
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_tipo)
        except:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'El id ingresado no es valido')
            return rta
        # Generamos la session y la tabla
        session = self.cp.binded_session()
        tabla_sitios_tipos = self.cp.tabla('sitios_tipos')
        # Intentamos obtener el tipo
        q_obj = session.query(tabla_sitios_tipos)
        q_obj = q_obj.filter(tabla_sitios_tipos.IdSitioTipo == id_tipo)
        # Obtenemos
        sitio_tipo_tmp = q_obj.first()
        # Cerramos sesion
        session.close()
        # Si es vacio, salimos
        if sitio_tipo_tmp is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No existe el tipo de sitio buscado')
            return rta
        # Sino, devolvemos correctamente
        rta['Estado'] = True
        rta['Respuesta'] = sitio_tipo_tmp.Descripcion.decode('iso-8859-1')
        return rta

    def borrar(self, id_sitio):
        ''' Funcion para borrar una zona especifica del sistema
        Argumentos:
            id_zona: int
        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        rta = {}
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_sitio)
        except:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'Los datos ingresados no son validos.')
            return rta
        # Generamos la session y la tabla
        session = self.cp.binded_session()
        tabla_sitios = self.cp.tabla('sitios')
        # Pedimos el interno que conicida
        q_obj = session.query(tabla_sitios)
        q_obj = q_obj.filter(tabla_sitios.IdSitio == id_sitio)
        # Obtenemos
        sitio_tmp = q_obj.first()
        if sitio_tmp is None:
            session.close()
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No existe el sitio a borrar')
            return rta
        # Como encontramos, vamos a borrar
        try:
            q_obj.delete(synchronize_session=False)
            session.commit()
            session.close()
        except:
            session.rollback()
            session.close()
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se pudo borrar la zona de id %s' % str(id_sitio))
            return rta
        # Sino, salimos bien
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se borro correctamente el sitio')
        return rta