# -*- coding: utf-8 -*-

import unittest
import time

from at.map import _
from at.map.calle import Calle
from at.map.calle import CallePostgres
from at.map.ciudad import CiudadPostgres
from at.map.utiles.referencia import Referencia
from at.orm.abm import ABM


GIS2 = {
    'user': 'postgres',
    'passwd': 'TrackAir',
    'dbname': 'gis-test',
    'name': 'gis2',
    'host': '192.168.0.68',
    'engine': 'postgres',
    'port': '5432',
}


class CallePostgresTesting(CallePostgres):

    def __init__(self):
        CallePostgres.__init__(self)
        self.gis2 = ABM(GIS2)
        self.orm = self.gis2


class CiudadPostgresTesting(CiudadPostgres):

    def __init__(self):
        CiudadPostgres.__init__(self)
        self.gis2 = ABM(GIS2)
        self.orm = self.gis2


class TestCalle(unittest.TestCase):

    def setUp(self):
        self.calle = Calle()
        self.calle_postgres = CallePostgresTesting()
        self.ciudad_postgres = CiudadPostgresTesting()
        self.referencia = Referencia()
        self.ciudad_postgres.agregar_ciudad_aplicacion(1, 324)

    def tearDown(self):
        self.ciudad_postgres.eliminar_ciudad_aplicacion(1)

    def test_obtener_calle(self):
        # caso correcto
        id_ciudad = 1
        id_calle = 1
        r = self.calle.obtener_calle(id_ciudad, id_calle)
        self.assertTrue(r)

        # intento pasarle None
        id_ciudad = None
        id_calle = None
        self.assertRaises(Exception, self.calle.obtener_calle, id_ciudad,
                                                               id_calle)

        # intento pasarle un ID que no existe
        id_ciudad = 123
        id_calle = 123
        self.assertRaises(Exception, self.calle.obtener_calle, id_ciudad,
                                                               id_calle)

    def test_listar_calles(self):
        # caso correcto
        id_ciudad = 1
        r = self.calle.listar_calles(id_ciudad)
        self.assertTrue(len(r) > 0)

        # caso vacio
        id_ciudad = 15698
        r = self.calle.listar_calles(id_ciudad)
        self.assertTrue(len(r) == 0)

        # intento pasarle None
        id_ciudad = None
        self.assertRaises(Exception, self.calle.listar_calles, id_ciudad)

    def test_listar_calles_postgres(self):
        # caso correcto
        id_ciudad = 1
        r = self.calle_postgres.listar_calles(id_ciudad)
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r['Respuesta']) > 0)

        # caso vacio
        id_ciudad = 15698
        r = self.calle_postgres.listar_calles(id_ciudad)
        self.assertFalse(r['Estado'])

        # intento pasarle None
        id_ciudad = None
        r = self.calle_postgres.listar_calles(id_ciudad)
        self.assertFalse(r['Estado'])

    def test_obtener_tipo_de_numeracion(self):

        # caso correcto
        id_tipo_numeracion = 1
        r = self.calle.obtener_tipo_de_numeracion(id_tipo_numeracion)
        self.assertTrue(r)

        # intento pasarle None
        id_tipo_numeracion = None
        self.assertRaises(Exception, self.calle.obtener_tipo_de_numeracion,
                                                         id_tipo_numeracion)

        # intento pasarle un ID que no existe
        id_tipo_numeracion = 123
        self.assertRaises(Exception, self.calle.obtener_tipo_de_numeracion,
                                                         id_tipo_numeracion)

    def test_listar_tipos_de_numeracion(self):
        # caso correcto
        r = self.calle.listar_tipos_de_numeracion()
        self.assertTrue(len(r) > 0)

    def test_sugerir_calle(self):
        # caso correcto, le paso un id de una ciudad existente y parte del
        # nombre de una calle
        id_ciudad = 1
        parte_calle = 'men'
        r = self.calle.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r)

        # caso correcto 2, le paso dos palabras
        id_ciudad = 1
        parte_calle = 'san j'
        r = self.calle.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r)

        # caso correcto 3
        id_ciudad = 1
        parte_calle = 'juan'
        r = self.calle.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r)
        # caso correcto 3, le paso una palabra en mayásculas
        id_ciudad = 1
        parte_calle = 'MEN'
        r = self.calle.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r)

        # esto no debe devolver ningún registro, devulve None
        id_ciudad = 1
        parte_calle = 'sadiwduwdkk'
        r = self.calle.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(None == r)

        # no le paso el id_ciudad
        id_ciudad = None
        parte_calle = 'men'
        self.assertRaises(Exception, self.calle.sugerir_calle, id_ciudad,
                                                               parte_calle)

        # le paso un string vacio como parte_calle
        # devulve None, porque sería el mismo caso que un string
        # de menos de 3 caracteres
        id_ciudad = 1
        parte_calle = ''
        r = self.calle.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(None == r)

        # le paso un id_ciudad que no existe, debe devolver None
        id_ciudad = 999
        parte_calle = 'men'
        r = self.calle.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(None == r)

    def test_sugerir_calle_postgres(self):
        # caso correcto, le paso un id de una ciudad existente y parte del
        # nombre de una calle
        id_ciudad = 1
        parte_calle = 'men'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r['Respuesta']) > 0)

        # caso correcto 2, le paso dos palabras
        id_ciudad = 1
        parte_calle = 'san j'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r['Respuesta']) > 0)

        # caso correcto 3
        id_ciudad = 1
        parte_calle = 'juan'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r['Respuesta']) > 0)

        # caso correcto 3, le paso una palabra en mayúsculas
        id_ciudad = 1
        parte_calle = 'MEN'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(r['Estado'])
        self.assertTrue(len(r['Respuesta']) > 0)

        # esto no debe devolver ningún registro, devulve None
        id_ciudad = 1
        parte_calle = 'sadiwduwdkk'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(len(r['Respuesta']) == 0)

        # no le paso el id_ciudad
        id_ciudad = None
        parte_calle = 'men'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertFalse(r['Estado'])

        # le paso un string vacio como parte_calle
        # devulve None, porque sería el mismo caso que un string
        # de menos de 3 caracteres
        id_ciudad = 1
        parte_calle = ''
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertTrue(len(r['Respuesta']) == 0)

        # le paso un id_ciudad que no existe, debe devolver un error
        id_ciudad = 99965656
        parte_calle = 'men'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle)
        self.assertFalse(r['Estado'])

        # Pruebo con una calle con acento en las primeras 3 letras
        id_ciudad = 1975
        parte_calle = 'vel'
        r = self.calle_postgres.sugerir_calle(id_ciudad, parte_calle, True)
        self.assertTrue(r['Estado'] is True)
        self.assertTrue(len(r['Respuesta']) == 7)

    def test_generar_direccion(self):
        # caso correcto, le pasamos datos para armar una direccion
        calle = 'Pellegrini'
        nro = '250'
        bis = 'BIS'
        r = self.calle.generar_direccion(calle1=calle,
                                         altura_nro=nro,
                                         altura_bis=bis)
        self.assertTrue(r)

        #Caso de exito utilizando pasando todos los argumentos
        calle = 'Pellegrini'
        nro = '250'
        bis = 'BIS'
        letra = 'A'
        depto = '2 B'

        r = self.calle.generar_direccion(calle1=calle,
                                         altura_nro=nro,
                                         altura_bis=bis,
                                         altura_letra=letra,
                                         departamento=depto
                                     )
        self.assertTrue(r)

        #Caso de exito utilizando una segunda calle
        calle = 'Pellegrini'
        calle2 = 'Ayacucho'

        r = self.calle.generar_direccion(
            calle1=calle,
            calle2=calle2,
        )
        self.assertTrue(r)

        #Caso de exito utilizando un sitio
        sitio = 'Alto Rosario'

        r = self.calle.generar_direccion(sitio=sitio)
        self.assertTrue(r)

        # caso de fallo, le pasamos una altura en forma de Int
        calle = 'Pellegrini'
        nro = 250
        bis = 'BIS'
        self.assertRaises(
            ValueError,
            self.calle.generar_direccion,
            calle1=calle,
            altura_nro=nro,
            altura_bis=bis
        )

    def test_generar_direccion_postgres(self):
        # caso correcto, le pasamos datos para armar una direccion
        calle = 'Pellegrini'
        nro = '250'
        bis = 'BIS'
        r = self.calle_postgres.generar_direccion(calle1=calle,
                                         altura_nro=nro,
                                         altura_bis=bis)
        self.assertTrue(r)

        #Caso de exito utilizando pasando todos los argumentos
        calle = 'Pellegrini'
        nro = '250'
        bis = 'BIS'
        letra = 'A'
        depto = '2 B'
        r = self.calle_postgres.generar_direccion(calle1=calle,
                                         altura_nro=nro,
                                         altura_bis=bis,
                                         altura_letra=letra,
                                         departamento=depto
                                     )
        self.assertTrue(r)

        #Caso de exito utilizando una segunda calle
        calle = 'Pellegrini'
        calle2 = 'Ayacucho'

        r = self.calle_postgres.generar_direccion(
            calle1=calle,
            calle2=calle2,
        )
        self.assertTrue(r)

        #Caso de exito utilizando un sitio
        sitio = 'Alto Rosario'

        r = self.calle_postgres.generar_direccion(sitio=sitio)
        self.assertTrue(r)

        # caso de fallo, le pasamos una altura en forma de Int
        calle = 'Pellegrini'
        nro = 250
        bis = 'BIS'
        self.assertRaises(
            ValueError,
            self.calle_postgres.generar_direccion,
            calle1=calle,
            altura_nro=nro,
            altura_bis=bis
        )

        # Le paso una calle, una altura y una esquina
        calle = 'Urquiza'
        nro = '7164'
        calle2 = u'México'
        r = self.calle_postgres.generar_direccion(
            calle1=calle, altura_nro=nro, calle2=calle2,
        )
        self.assertTrue(r == u'Urquiza 7164 Esq. México')

        # Le paso todos los datos
        calle = 'Pellegrini'
        nro = '250'
        bis = 'BIS'
        letra = 'A'
        depto = '2 B'
        calle2 = 'Dorrego'
        r = self.calle_postgres.generar_direccion(
            calle1=calle, altura_nro=nro, altura_bis=bis, altura_letra=letra,
            departamento=depto, calle2=calle2
        )
        self.assertTrue(r == u'Pellegrini 250 A BIS  2 B Esq. Dorrego')

    def test_direccion_xy(self):
        # caso correcto, paso una calle y la altura
        # Rosario
        id_ciudad = 1
        # Rioja
        id_calle1 = 934
        id_calle2 = None
        altura = 3600
        letra_altura = None
        id_sitio = None
        r = self.calle.direccion_xy(id_ciudad, id_calle1, id_calle2, altura,
                                    letra_altura, id_sitio)
        self.assertTrue(r)

        # le paso mal la calle y la altura
        # Rosario
        id_ciudad = 1
        # Rioja
        id_calle1 = 79
        id_calle2 = None
        altura = 360011
        letra_altura = None
        id_sitio = None
        self.assertRaises(Exception, self.calle.direccion_xy, id_ciudad,
                                    id_calle1, id_calle2, altura, letra_altura,
                                    id_sitio)

        # caso correcto 2, paso una calle y la altura con letra
        # Rosario
        id_ciudad = 1
        # Callao
        id_calle1 = 1044
        id_calle2 = None
        altura = 4550
        letra_altura = 'F'
        id_sitio = None
        r = self.calle.direccion_xy(id_ciudad, id_calle1, id_calle2, altura,
                                    letra_altura, id_sitio)
        self.assertTrue(r)

        # le paso mal los parámetros para el caso de paso una calle y la altura
        # con letra
        id_ciudad = 1111
        id_calle1 = 1044
        id_calle2 = None
        altura = 4550
        letra_altura = 'F'
        id_sitio = None
        self.assertRaises(
            Exception, self.calle.direccion_xy, id_ciudad,
            id_calle1, id_calle2, altura, letra_altura,
            id_sitio
        )

        # caso correcto 3, le paso dos calles
        id_ciudad = 1
        id_calle1 = 747
        id_calle2 = 1052
        altura = None
        letra_altura = None
        id_sitio = None
        r = self.calle.direccion_xy(id_ciudad, id_calle1, id_calle2, altura,
                                    letra_altura, id_sitio)
        self.assertTrue(r)

        # le paso mal los valores
        id_ciudad = 1
        id_calle1 = 7471
        id_calle2 = 1052
        altura = None
        letra_altura = None
        id_sitio = None
        self.assertRaises(
            Exception, self.calle.direccion_xy, id_ciudad,
            id_calle1, id_calle2, altura, letra_altura,
            id_sitio
        )

        # caso correcto 4, le paso la ciudad y el sitio
        id_ciudad = 1
        id_calle1 = None
        id_calle2 = None
        altura = None
        letra_altura = None
        id_sitio = 11
        r = self.calle.direccion_xy(id_ciudad, id_calle1, id_calle2, altura,
                                    letra_altura, id_sitio)
        self.assertTrue(r)

        # le paso un sitio que no existe
        id_ciudad = 1
        id_calle1 = None
        id_calle2 = None
        altura = None
        letra_altura = None
        id_sitio = 1111
        self.assertRaises(
            Exception, self.calle.direccion_xy, id_ciudad,
            id_calle1, id_calle2, altura, letra_altura,
            id_sitio
        )

        # no le paso ningún valor
        id_ciudad = None
        id_calle1 = None
        id_calle2 = None
        altura = None
        letra_altura = None
        id_sitio = None
        r = self.calle.direccion_xy(id_ciudad, id_calle1, id_calle2, altura,
                                    letra_altura, id_sitio)
        self.assertTrue(r is None)

        # prueba para debugear, Ituzaingo 1330
        id_ciudad = 1
        id_calle1 = 828
        id_calle2 = None
        altura = 1330
        letra_altura = None
        id_sitio = None
        r = self.calle.direccion_xy(id_ciudad, id_calle1, id_calle2, altura,
                                    letra_altura, id_sitio)
        self.assertTrue(r)

        # prueba para debugear
        id_ciudad = 1
        id_calle1 = 775
        id_calle2 = 1054
        altura = None
        letra_altura = None
        id_sitio = None
        r = self.calle.direccion_xy(id_ciudad, id_calle1, id_calle2, altura,
                                    letra_altura, id_sitio)

        l = self.referencia.de_xy_a_segundos(r)
        self.assertTrue(l)

    def test_direccion_xy_postgres(self):

        # caso correcto , le paso calle y altura
        id_ciudad = None
        id_calle1 = 1333417
        id_calle2 = None
        altura = 3797
        letra_altura = None
        id_sitio = None
        r = self.calle_postgres.direccion_xy(
            id_ciudad, id_calle1, id_calle2, altura,
            letra_altura, id_sitio
        )
        self.assertTrue(r['Estado'])

        # caso correcto , le paso dos calles
        id_ciudad = None
        id_calle1 = 1334188
        id_calle2 = 1333417
        altura = None
        letra_altura = None
        id_sitio = None
        r = self.calle_postgres.direccion_xy(
            id_ciudad, id_calle1, id_calle2, altura,
            letra_altura, id_sitio
        )

        self.assertTrue(r['Estado'])

        # le paso mal los valores
        id_ciudad = None
        id_calle1 = 7471
        id_calle2 = 1052
        altura = None
        letra_altura = None
        id_sitio = None

        r = self.calle_postgres.direccion_xy(
            id_ciudad, id_calle1, id_calle2, altura, letra_altura, id_sitio
        )
        self.assertFalse(r['Estado'])

    def test_obtener_puntos_calles_por_zona(self):

        # Caso incorrecto, ragion
        rta = self.calle_postgres.obtener_puntos_calles_por_zona({})
        self.assertFalse(rta['Estado'])

        # Caso incorrecto, puntos incorrectos
        rta = self.calle_postgres.obtener_puntos_calles_por_zona([1])
        self.assertFalse(rta['Estado'])

        # Caso incorrecto, puntos sin latitud
        rta = self.calle_postgres.obtener_puntos_calles_por_zona(
            [{'Longitud':12}]
        )
        self.assertFalse(rta['Estado'])

        # Caso correcto con un poligono cerrado
        zona = [
            {'Longitud': -60.647612, 'Latitud': -32.949224},
            {'Longitud': -60.647650, 'Latitud': -32.949290},
            {'Longitud': -60.647500, 'Latitud': -32.949290},
            {'Longitud': -60.647612, 'Latitud': -32.949224}
        ]

        r = self.calle_postgres.obtener_puntos_calles_por_zona(zona)

        self.assertTrue(len(r) > 0)

        # Caso correcto con un poligono abierto
        zona = [
            {'Longitud': -60.647612, 'Latitud': -32.949224},
            {'Longitud': -60.647650, 'Latitud': -32.949290},
            {'Longitud': -60.647500, 'Latitud': -32.949290}
        ]

        rta = self.calle_postgres.obtener_puntos_calles_por_zona(zona)

        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) > 0)

    def test_obtener_altura_por_latlon(self):
        # Caso incorrecto, por datos
        rta = self.calle_postgres.obtener_altura_por_latlon(1213, '')
        self.assertFalse(rta['Estado'])
        rta = self.calle_postgres.obtener_altura_por_latlon('', '')
        self.assertFalse(rta['Estado'])

        # Caso incorrecto, no encuentra nada
        r = self.calle_postgres.obtener_altura_por_latlon(123, 123)
        self.assertFalse(r['Estado'])

        # Caso correcto, encuentra la calle y la altura
        r = self.calle_postgres.obtener_altura_por_latlon(
                                                        -26.815894, -65.222366)
        self.assertTrue(r['Estado'])
        self.assertTrue(isinstance(r['Respuesta']['DesdeDer'], int))

    def test_obtener_posicion_por_latlon(self):
        # Caso incorrecto, por datos
        rta = self.calle_postgres.obtener_posicion_por_latlon(1213, '')
        self.assertFalse(rta['Estado'])
        rta = self.calle_postgres.obtener_posicion_por_latlon('', '')
        self.assertFalse(rta['Estado'])

        # Caso incorrecto, no encuentra nada
        rta = self.calle_postgres.obtener_posicion_por_latlon(0, 0)
        self.assertTrue(not rta['Estado'])

        # Caso correcto, no encuentra nada preciso, pero si una calle cercana
        # a menos de un kilometro
        rta = self.calle_postgres.obtener_posicion_por_latlon(
            -26.824808, -65.305043
        )
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['Calle'] is None)
        self.assertTrue(rta['Respuesta']['CalleCercana'] == 'Av  Vera Solano')
        self.assertTrue(rta['Respuesta']['Localidad'] == 'Gob Piedrabuena')

        # Caso correcto de una calle y altura cercana
        rta = self.calle_postgres.obtener_posicion_por_latlon(
            -26.82268, -65.259862
        )
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['Calle'] is None)
        self.assertTrue(rta['Respuesta']['CalleCercana'] == 'San Lorenzo')
        self.assertTrue(rta['Respuesta']['AlturaCercana'] == '4385')
        self.assertTrue(
                    rta['Respuesta']['Localidad'] == 'San Miguel De Tucuman')

        # Caso correcto, no encuentra nada preciso, pero si una calle cercana
        # a un kilometro (hace mas consultas este test)
        rta = self.calle_postgres.obtener_posicion_por_latlon(
            -32.951243, -60.804269
        )

        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['Calle'] is None)
        self.assertTrue(rta['Respuesta']['CalleCercana'] is not None)
        self.assertTrue(rta['Respuesta']['Orientacion'] == 'S')
        self.assertTrue(rta['Respuesta']['Distancia'] >= 1000)
        self.assertTrue(rta['Respuesta']['Localidad'] == 'Funes')

        # Caso correcto, encuentra la calle y la altura
        rta = self.calle_postgres.obtener_posicion_por_latlon(
            -26.815894, -65.222366
        )

        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['Calle'] == 'Santa Fe')
        self.assertTrue(rta['Respuesta']['Altura'] == 1843)
        self.assertTrue(
            rta['Respuesta']['Localidad'] == 'San Miguel De Tucuman'
        )

        # Esto seria: "Santa Fe al 1800"

#        # Buscamos una calle sin altura, en la esquina
#        rta = self.calle_postgres.obtener_posicion_por_latlon(
#            -26.809373, -65.305526
#        )
#
#        self.assertTrue(rta['Estado'])
#        self.assertTrue(rta['Respuesta']['Calle'] == 'Quito')
#        self.assertTrue(rta['Respuesta']['Calle2'] == 'Imbaud M')
#        self.assertTrue(rta['Respuesta']['Localidad'] == 'Yerba Buena')

        # Esto seria: "Quito e Imbaud M"

#        # Buscamos una calle sin altura, pero a mitad de cuadra
#        rta = self.calle_postgres.obtener_posicion_por_latlon(
#            -26.808952, -65.307575
#        )
#        print rta

#        self.assertTrue(rta['Estado'])
#        self.assertTrue(rta['Respuesta']['Calle'] == 'Quito')
#        self.assertTrue(rta['Respuesta']['Calle2'] == 'Guatemala')
#        self.assertTrue(rta['Respuesta']['Calle3'] == 'Uruguay')
#        self.assertTrue(rta['Respuesta']['Localidad'] == 'Yerba Buena')
#
        # Esto seria: "Quito, entre Uruguay y Guatemala"

        # Caso correcto, dirección de Airtrack
        rta = self.calle_postgres.obtener_posicion_por_latlon(
            -32.93356, -60.718326, False, True
        )

        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['Calle3'] is None)
        self.assertTrue(rta['Respuesta']['Calle2'] is None)
        self.assertTrue(rta['Respuesta']['Calle'] == 'Urquiza')
        self.assertTrue(rta['Respuesta']['Localidad'] == 'Rosario')
        self.assertTrue(rta['Respuesta']['IdLocalidad'] == 324)
        self.assertTrue(rta['Respuesta']['Altura'] == 7157)
        self.assertTrue(rta['Respuesta']['IdCalle2'] is None)
        self.assertTrue(rta['Respuesta']['Distancia'] is None)
        self.assertTrue(rta['Respuesta']['CalleCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalle3'] is None)
        self.assertTrue(rta['Respuesta']['AlturaCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalle'] == 1333579)
        self.assertTrue(rta['Respuesta']['Orientacion'] is None)
        self.assertTrue(rta['Respuesta']['IdCalleCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalleParalela1'] == 1333482)
        self.assertTrue(rta['Respuesta']['NombreCalleParalela1'] == 'Tucuman')
        self.assertTrue(rta['Respuesta']['IdCalleParalela2'] == 1333436)
        self.assertTrue(
            rta['Respuesta']['NombreCalleParalela2'] == 'San Lorenzo'
        )

        rta = self.calle_postgres.obtener_posicion_por_latlon(
                                            -32.93356, -60.718326, True, False)

        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['Calle3'] == 'Guatemala')
        self.assertTrue(rta['Respuesta']['Calle2'] == 'Mexico')
        self.assertTrue(rta['Respuesta']['Calle'] == 'Urquiza')
        self.assertTrue(rta['Respuesta']['Localidad'] == 'Rosario')
        self.assertTrue(rta['Respuesta']['IdLocalidad'] == 324)
        self.assertTrue(rta['Respuesta']['Altura'] == 7157)
        self.assertTrue(rta['Respuesta']['IdCalle2'] == 1333407)
        self.assertTrue(rta['Respuesta']['Distancia'] is None)
        self.assertTrue(rta['Respuesta']['CalleCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalle3'] == 1333408,)
        self.assertTrue(rta['Respuesta']['AlturaCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalle'] == 1333579)
        self.assertTrue(rta['Respuesta']['Orientacion'] is None)
        self.assertTrue(rta['Respuesta']['IdCalleCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalleParalela1'] is None)
        self.assertTrue(rta['Respuesta']['NombreCalleParalela1'] is None)
        self.assertTrue(rta['Respuesta']['IdCalleParalela2'] is None)
        self.assertTrue(rta['Respuesta']['NombreCalleParalela2'] is None)
        self.assertTrue(rta['Respuesta']['AlturaCalle2'] == 400)
        self.assertTrue(rta['Respuesta']['AlturaCalle3'] == 400)

        rta = self.calle_postgres.obtener_posicion_por_latlon(
            -32.9491, -60.647586, True, True
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta']['Calle3'] == 'Av Mendoza'
        )
        self.assertTrue(
            rta['Respuesta']['Calle2'] == 'San Juan'
        )
        self.assertTrue(
            rta['Respuesta']['Calle'] == 'Espa\xf1A'
        )
        self.assertTrue(
            rta['Respuesta']['Localidad'] == 'Rosario'
        )
        self.assertTrue(rta['Respuesta']['IdLocalidad'] == 324)
        self.assertTrue(rta['Respuesta']['Altura'] == 1129)
        self.assertTrue(rta['Respuesta']['IdCalle2'] == 1333662)
        self.assertTrue(rta['Respuesta']['Distancia'] is None)
        self.assertTrue(rta['Respuesta']['CalleCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalle3'] == 1333416)
        self.assertTrue(rta['Respuesta']['AlturaCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalle'] == 1334209)
        self.assertTrue(rta['Respuesta']['Orientacion'] is None)
        self.assertTrue(rta['Respuesta']['IdCalleCercana'] is None)
        self.assertTrue(rta['Respuesta']['IdCalleParalela1'] == 1334224)
        self.assertTrue(
            rta['Respuesta']['NombreCalleParalela1'] == 'Italia'
        )
        self.assertTrue(rta['Respuesta']['IdCalleParalela2'] == 1334228)
        self.assertTrue(
            rta['Respuesta']['NombreCalleParalela2'] == 'Pte Roca'
        )
        self.assertTrue(rta['Respuesta']['AlturaCalle2'] == 1701)
        self.assertTrue(rta['Respuesta']['AlturaCalle3'] == 1698)

    def test_obtener_calle_por_latlon(self):
        # Caso incorrecto, por datos
        rta = self.calle_postgres.obtener_calle_por_latlon(1213, '')
        self.assertFalse(rta['Estado'])
        rta = self.calle_postgres.obtener_calle_por_latlon('', '')
        self.assertFalse(rta['Estado'])

        rta = self.calle_postgres.obtener_calle_por_latlon(
            -32.973172, -60.685592
        )
        self.assertTrue(rta['Estado'])
        self.assertTrue(
            rta['Respuesta']['Nombre'].lower() == '24 de septiembre'
        )
        self.assertTrue(
            rta['Respuesta']['Ciudad']['Nombre'].lower() == 'rosario'
        )
        self.assertTrue(len(rta['Respuesta']['Segmentos']) > 0)

    def test_grabar_calle(self):
        # Validamos el fallo
        rta = self.calle_postgres.grabar('asd', True)
        self.assertFalse(rta['Estado'])
        # Caso correcto de cambio
        id_calle = 1334178
        # Obtenemos el nombre original
        rta = self.calle_postgres.obtener_calle(id_calle)
        self.assertTrue(rta['Estado'])
        nombre_original = rta['Respuesta']['Nombre']
        ciudad_original = rta['Respuesta']['Ciudad']['Id']
        # Declaramos los datos nuevos
        nuevo_nombre = 'Prueba de calle'
        nueva_etiqueta = 'P. D. C.'
        nueva_ciudad = 320
        # Modificamos
        rta = self.calle_postgres.grabar(
            id_calle, nuevo_nombre, nueva_etiqueta, id_ciudad=nueva_ciudad
        )
        self.assertTrue(rta['Estado'])
        # Validamos que se halla cambiado
        rta = self.calle_postgres.obtener_calle(id_calle)
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['Nombre'] == nuevo_nombre)
        self.assertTrue(rta['Respuesta']['Etiqueta'] == nueva_etiqueta)
        self.assertTrue(rta['Respuesta']['Ciudad']['Id'] == nueva_ciudad)
        # Volvemos al estado original
        rta = self.calle_postgres.grabar(
            id_calle, nombre_original, '', id_ciudad=ciudad_original
        )
        self.assertTrue(rta['Estado'])
        # Validamos que volvimos al estado original
        rta = self.calle_postgres.obtener_calle(id_calle)
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['Nombre'] == nombre_original)
        self.assertTrue(rta['Respuesta']['Etiqueta'] == '')
        self.assertTrue(rta['Respuesta']['Ciudad']['Id'] == ciudad_original)

    def test_grabar_segmento(self):
        # Validamos el fallo
        rta = self.calle_postgres.grabar_segmento(
            'asd', True, '0asd', 'ad.', 'asd', object, True
        )
        self.assertFalse(rta['Estado'])
        # Obtenemos los datos del segmento
        return
        id_calle_nombre = 158272
        id_segmento = 198773
        rta = self.calle_postgres.obtener_segmentos(
            id_calle_nombre, id_segmento
        )
        self.assertTrue(rta['Estado'])
        d_d_original = rta['Respuesta'][0]['DesdeDer']
        h_d_original = rta['Respuesta'][0]['HastaDer']
        d_i_original = rta['Respuesta'][0]['DesdeIzq']
        h_i_original = rta['Respuesta'][0]['HastaIzq']
        sentido_original = rta['Respuesta'][0]['Sentido']
        puntos_originales = rta['Respuesta'][0]['Puntos']
        datos_originales = rta['Respuesta'][0]
        # Modificamos
        desde_der = 80
        hasta_der = 150
        desde_izq = 345
        hasta_izq = 3402
        sentido = 0
        puntos_nuevos = [] + puntos_originales
        puntos_nuevos.reverse()
        id_calle_nombre_nuevo = 404076
        rta = self.calle_postgres.grabar_segmento(id_segmento, desde_der,
           hasta_der,
         desde_izq, hasta_izq, sentido, id_calle_nombre=id_calle_nombre_nuevo,
            puntos=puntos_nuevos
         )
        self.assertTrue(rta['Estado'])
        # Validamos que se modifico todo bien
        rta = self.calle_postgres.obtener_segmentos(
            id_calle_nombre_nuevo, id_segmento
        )
        self.assertTrue(rta['Estado'])
        self.assertTrue(desde_der == rta['Respuesta'][0]['DesdeDer'])
        self.assertTrue(hasta_der == rta['Respuesta'][0]['HastaDer'])
        self.assertTrue(desde_izq == rta['Respuesta'][0]['DesdeIzq'])
        self.assertTrue(hasta_izq == rta['Respuesta'][0]['HastaIzq'])
        self.assertTrue(sentido == rta['Respuesta'][0]['Sentido'])
        self.assertTrue(puntos_nuevos == rta['Respuesta'][0]['Puntos'])
        # Volvemos al estado original
        rta = self.calle_postgres.grabar_segmento(id_segmento, d_d_original,
            h_d_original, d_i_original, h_i_original, sentido_original,
            id_calle_nombre=id_calle_nombre, puntos=puntos_originales)
        self.assertTrue(rta['Estado'])
        # Validamos que volvio todo bien
        rta = self.calle_postgres.obtener_segmentos(
            id_calle_nombre, id_segmento
        )
        self.assertTrue(rta['Estado'])
        self.assertTrue(datos_originales == rta['Respuesta'][0])

    def test_obtener_calles_paralelas(self):
        # Obtengo las calles paralelas del segmento de la calle Urquiza al 7100
        id_segmento = 2844337
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle_paralela_1'] == 1333482)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_1'] == 'Tucuman')
        self.assertTrue(rta['Respuesta']['id_calle_paralela_2'] == 1333436)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_2'] == 'San Lorenzo')

        # Obtengo las calles paralelas del segmento de la calle Guatemala al 700
        id_segmento = 2846856
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle_paralela_1'] == 1333407)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_1'] == 'Mexico')
        self.assertTrue(rta['Respuesta']['id_calle_paralela_2'] == 1333409)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_2'] == 'Colombia')

        # Le paso un segmento que no existe
        id_segmento = 1234
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(rta['Respuesta'] == _(u'No se encontró la calle dada'))

        # Caso con cortadas
        # Av Provincias Unidas 333 bis
        id_segmento = 2844234
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle_paralela_1'] == 1333556)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_1'] == 'Cayasta')
        self.assertTrue(rta['Respuesta']['id_calle_paralela_2'] == 1333460)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_2'] == 'Isaacs')

        # San Lorenzo 7445
        id_segmento = 2843649
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle_paralela_1'] == 1333475)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_1'] == 'Santa Fe')
        self.assertTrue(rta['Respuesta']['id_calle_paralela_2'] == 1333579)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_2'] == 'Urquiza')

        # Santa Fe 6400
        id_segmento = 2847460
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle_paralela_1'] == 1333436)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_1'] == 'San Lorenzo')
        self.assertTrue(rta['Respuesta']['id_calle_paralela_2'] == 1333584)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_2'] == 'Zuviria')

        # Marcos Paz 5541
        id_segmento = 2846798
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle_paralela_1'] == 1333831)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_1'] == 'Rioja')
        self.assertTrue(rta['Respuesta']['id_calle_paralela_2'] == 1333861)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_2'] == 'San Luis')

        # San Luis 1861
        id_segmento = 2856438
        rta = self.calle_postgres.obtener_calles_paralelas(id_segmento)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle_paralela_1'] == 1333662)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_1'] == 'San Juan')
        self.assertTrue(rta['Respuesta']['id_calle_paralela_2'] == 1333831)
        self.assertTrue(
                rta['Respuesta']['nombre_calle_paralela_2'] == 'Rioja')

    def test_trazar_alturas(self):
        # Caso correcto, modifico la altura de segmentos de la calle
        # Zeballos de Rosario
        # Consulta de prueba
        # select * from calles where id_calle_nombre = 1333914 and gid >= 2856341 and gid <= 2856350
        # Al terminar dejar los datos como estaban
        # Estos segmentos van del 1000 al 1999
        # Del tipo 2 (calles)

        segmentos_a_editar = [2856341, 2856342, 2856343, 2856344, 2856345,
                                2856346, 2856347, 2856348, 2856349, 2856350]
        id_calle_nombre = 1333914
        id_tipo = 2
        altura_inicial = 0
        salto = 100
        bis = False
        letra = ''
        rta = self.calle_postgres.trazar_alturas(
            segmentos_a_editar, id_calle_nombre, id_tipo, altura_inicial,
            salto, bis, letra
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Los segmentos se modificaron de forma correcta')
        )

    def test_altura_final(self):
        # Comento este test porque tarda demasiado
        pass
        # # Le paso una calle que en gis2 test no existe
        # id_calle = 1968580
        # id_segm_ini = 4269365
        # id_segm_fin = 4269378
        # rta = self.calle_postgres.altura_final(id_calle, id_segm_ini,
        #                                                  id_segm_fin)
        # #self.assertTrue(rta['Estado'] is False)
        # #self.assertTrue(rta['Respuesta'] == _(u'No se encontró la calle dada'))
        #
        # gis2 = {
        #     'user': 'postgres',
        #     'passwd': 'TrackAir',
        #     'dbname': 'gis2',
        #     'name': 'gis2',
        #     'host': '192.168.0.51',
        #     'engine': 'postgresql',
        #     'port': '5432',
        # }
        # self.calle_postgres.gis2 = ABM(gis2)
        # # Caso correcto
        # id_calle = 1969663
        # id_segm_ini = 4285602
        # id_segm_fin = 4285607
        # rta = self.calle_postgres.altura_final(id_calle, id_segm_ini,
        #                                                  id_segm_fin)
        # self.assertTrue(rta['Estado'] is True)
        # self.assertTrue(rta['Respuesta'] == 6)

    def test_2_obtener_posicion_por_latlon(self):
        # Test para probar el tiempo de ejecución de la función
        tiempo_actual = time.time()

        rta = self.calle_postgres.obtener_posicion_por_latlon(
            -32.93356, -60.718326, True, True
        )
        print rta

        print 'TIME - fin test_2_obtener_posicion_por_latlon'
        print round(time.time() - tiempo_actual, 4)

    def test_datos_segmentos_calle(self):
        # Le paso una calle que no tiene biz ni letra
        id_calle = 1333579
        rta = self.calle_postgres.datos_segmentos_calle(id_calle)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle'] == 1333579)
        self.assertTrue(rta['Respuesta']['tiene_biz'] is False)
        self.assertTrue(rta['Respuesta']['tiene_letra'] is False)

        # Le paso una calle que tiene biz
        id_calle = 1334400
        rta = self.calle_postgres.datos_segmentos_calle(id_calle)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle'] == 1334400)
        self.assertTrue(rta['Respuesta']['tiene_biz'] is True)
        self.assertTrue(rta['Respuesta']['tiene_letra'] is False)

        # Le paso una calle que tiene letra
        id_calle = 1334208
        rta = self.calle_postgres.datos_segmentos_calle(id_calle)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle'] == 1334208)
        self.assertTrue(rta['Respuesta']['tiene_biz'] is False)
        self.assertTrue(rta['Respuesta']['tiene_letra'] is True)

        # Le paso una calle que tiene biz y letra
        id_calle = 1333754
        rta = self.calle_postgres.datos_segmentos_calle(id_calle)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_calle'] == 1333754)
        self.assertTrue(rta['Respuesta']['tiene_biz'] is True)
        self.assertTrue(rta['Respuesta']['tiene_letra'] is True)

    def test_obtener_entre_calles(self):
        # Caso correcto
        id_ciudad = 324
        id_calle = 1333579  # Urquiza
        altura = 7164
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 1333408)
        self.assertTrue(rta['Respuesta']['nombre_esquina1'] == 'Guatemala')
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 1333407)
        self.assertTrue(rta['Respuesta']['nombre_esquina2'] == 'Mexico')
        self.assertTrue(
            rta['Respuesta']['entre_calles'] == 'Entre Guatemala y Mexico'
        )

        # Le paso mal la calle
        id_ciudad = 324
        id_calle = 1234
        altura = 7164
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se pudo encontrar la calle principal en la altura dada')
        )

        # Le paso mal la ciudad
        id_ciudad = 1234
        id_calle = 1333579  # Urquiza
        altura = 7164
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se pudieron obtener las esquinas')
        )

        # Le paso mal la altura
        id_ciudad = 324
        id_calle = 1333579  # Urquiza
        altura = -5
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'No se pudo encontrar la calle principal en la altura dada')
        )

        # Caso correcto
        id_ciudad = 324
        id_calle = 1333682  # Bv. Rondeau
        altura = 971
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 1333722)
        self.assertTrue(rta['Respuesta']['nombre_esquina1'] == 'Gurruchaga')
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 1333977)
        self.assertTrue(rta['Respuesta']['nombre_esquina2'] == 'Marull')
        self.assertTrue(
            rta['Respuesta']['entre_calles'] == 'Entre Gurruchaga y Marull'
        )

        # Caso correcto
        id_ciudad = 324
        id_calle = 1333772  # Gallo
        altura = 345
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 1333980)
        self.assertTrue(rta['Respuesta']['nombre_esquina1'] == 'Maza')
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 1334105)
        self.assertTrue(rta['Respuesta']['nombre_esquina2'] == 'Alvarez Thomas')
        self.assertTrue(
            rta['Respuesta']['entre_calles'] == 'Entre Maza y Alvarez Thomas'
        )

        # Caso correcto, con BIZ
        id_ciudad = 324
        id_calle = 1334400  # Balcarce
        altura = -54
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 1334307)
        self.assertTrue(rta['Respuesta']['nombre_esquina1'] == 'Brown')
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 1334374)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina2'] == u'Güemes'.encode("latin1")
        )
        self.assertTrue(
            rta['Respuesta']['entre_calles'] ==
            u'Entre Brown y Güemes'.encode("latin1")
        )

        # Caso correcto
        id_ciudad = 324
        id_calle = 1333861  # San Luis
        altura = 931
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 1334401)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina1'] == 'Av San Martin'
        )
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 1334437)
        self.assertTrue(rta['Respuesta']['nombre_esquina2'] == 'Maipu')
        self.assertTrue(
            rta['Respuesta']['entre_calles'] == 'Entre Av San Martin y Maipu')

        # Calle que cruza
        id_ciudad = 2238
        id_calle = 411854  # Av Dr. Ricardo Balbin
        altura = 3668
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 411890)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina1'] == 'Jaramillo'
        )
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 411907)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina2'] == 'Dr. Romulo Naon'
        )
        self.assertTrue(
            rta['Respuesta']['entre_calles'] ==
            'Entre Jaramillo y Dr. Romulo Naon'
        )

        # Caso de Bs As
        id_ciudad = 2238
        id_calle = 412089  # Roque Perez
        altura = 2689
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 411920)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina1'] == 'Dr. Pedro Ignacio Rivera'
        )
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 411927)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina2'] == 'Franklin D. Roosevelt'
        )
        self.assertTrue(
            rta['Respuesta']['entre_calles'] ==
            'Entre Dr. Pedro Ignacio Rivera y Franklin D. Roosevelt'
        )

        # No le paso una altura
        id_ciudad = 2238
        id_calle = 412538  # Av Cordoba
        altura = None
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Para obtener las entre calles se requiere la ciudad, la calle principal y su altura')
        )

        # No le paso una calle
        id_ciudad = 2238
        id_calle = None
        altura = 2689
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Para obtener las entre calles se requiere la ciudad, la calle principal y su altura')
        )

        # No le paso la ciudad
        id_ciudad = None
        id_calle = 412089
        altura = 2689
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Para obtener las entre calles se requiere la ciudad, la calle principal y su altura')
        )

        # Caso de Bs As
        id_ciudad = 2238
        id_calle = 413057  # Juncal
        altura = 3678
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 425348)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina1'] ==
            'Av  Ra\xfal Scalabrini Ortiz'
        )
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 413250)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina2'] == 'Ugarteche'
        )
        self.assertTrue(
            rta['Respuesta']['entre_calles'] ==
            'Entre Av  Ra\xfal Scalabrini Ortiz y Ugarteche'
        )

        # Caso de Bs As
        id_ciudad = 2238
        id_calle = 411195  # Federico Lacroze
        altura = 2478
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 422967)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina1'] == 'Av  Gral  Paz Provincia'
        )
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 411188)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina2'] == 'Jose Ingenieros'
        )
        self.assertTrue(
            rta['Respuesta']['entre_calles'] ==
            'Entre Av  Gral  Paz Provincia y Jose Ingenieros'
        )

        # Caso de Bs As
        id_ciudad = 2238
        id_calle = 412687  # Arribeños
        altura = 2740
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 412194)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina1'] == 'Manuel Ugarte'
        )
        self.assertTrue(rta['Respuesta']['id_esquina2'] == 411853)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina2'] == 'Av Congreso'
        )
        self.assertTrue(
            rta['Respuesta']['entre_calles'] ==
            'Entre Manuel Ugarte y Av Congreso'
        )

        # Caso de Bs As
        id_ciudad = 2238
        id_calle = 413239  # Av Casares
        altura = 3997
        rta = self.calle_postgres.obtener_entre_calles(
            id_ciudad, id_calle, altura
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['id_esquina1'] == 412724)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina1'] == 'Av Pres. Figueroa Alcorta'
        )
        self.assertTrue(rta['Respuesta']['id_esquina2'] is None)
        self.assertTrue(
            rta['Respuesta']['nombre_esquina2'] == ''
        )
        self.assertTrue(
            rta['Respuesta']['entre_calles'] ==
            'Entre Av Pres. Figueroa Alcorta'
        )

    def test_seleccionar_segmentos(self):
        # Caso correcto, utilizando el punto 1 a comparar
        id_calle_nombre = 1333914  # Calle Zeballos de Rosario
        id_segmento_ini = 2856341  # Zeballos al 1099
        id_segmento_fin = 2856349  # Zeballos al 1899
        rta = self.calle_postgres.seleccionar_segmentos(
            id_calle_nombre, id_segmento_ini, id_segmento_fin
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            [2856341, 2856342, 2856343, 2856344,
            2856345, 2856346, 2856347, 2856348, 2856349]
        )

        # Caso correcto, utilizando el punto 2 a comparar
        id_calle_nombre = 1333914  # Calle Zeballos de Rosario
        id_segmento_ini = 2852979  # Zeballos al 3050
        id_segmento_fin = 2856347  # Zeballos al 1601
        rta = self.calle_postgres.seleccionar_segmentos(
            id_calle_nombre, id_segmento_ini, id_segmento_fin
        )
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            [2852979, 2852978, 2852977, 2852976, 2852975, 2852974, 2852973,
            2852972, 2852971, 2852970, 2852969, 2856352, 2856351, 2856350,
            2856349, 2856348, 2856347]
        )

        # Le paso una calle que no existe
        id_calle_nombre = 1234  # No existe
        id_segmento_ini = 2852979  # Zeballos al 3050
        id_segmento_fin = 2856347  # Zeballos al 1601
        rta = self.calle_postgres.seleccionar_segmentos(
            id_calle_nombre, id_segmento_ini, id_segmento_fin
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el segmento buscado.')
        )

        # Le paso un segmento que no exsite
        id_calle_nombre = 1333914  # Calle Zeballos de Rosario
        id_segmento_ini = 1234  # No existe
        id_segmento_fin = 2856347  # Zeballos al 1601
        rta = self.calle_postgres.seleccionar_segmentos(
            id_calle_nombre, id_segmento_ini, id_segmento_fin
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el segmento buscado.')
        )

        # Le paso el mismo segmento inicial y final
        id_calle_nombre = 1333914  # Calle Zeballos de Rosario
        id_segmento_ini = 2856347  # Zeballos al 1601
        id_segmento_fin = 2856347  # Zeballos al 1601
        rta = self.calle_postgres.seleccionar_segmentos(
            id_calle_nombre, id_segmento_ini, id_segmento_fin
        )
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'Debe seleccionar un segmento final distinto al inicial')
        )

    def test_actualizar_segmento(self):
        # Le paso un segmento cualquiera
        ### NO MODIFICO NADA PORQUE ES DE PRUEBA ###
        gid = 471352
        rta = self.calle_postgres.actualizar_segmento(gid)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(
            rta['Respuesta'] ==
            _(u'El segmento 471352 se actualizó correctamente')
        )

        # Le paso un segmento que no existe
        gid = 1234
        rta = self.calle_postgres.actualizar_segmento(gid)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el segmento 1234')
        )

    def test_obtener_segmento(self):
        # Le paso un segmento cualquiera de la calle Zeballos de Rosario
        gid = 2856348
        rta = self.calle_postgres.obtener_segmento(gid)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta']['gid'] == 2856348)
        self.assertTrue(rta['Respuesta']['id_calle_nombre'] == 1333914)
        self.assertTrue(rta['Respuesta']['id_tipo'] == 2)
        self.assertTrue(rta['Respuesta']['desde_izq'] == 701)
        self.assertTrue(rta['Respuesta']['hasta_izq'] == 799)
        self.assertTrue(rta['Respuesta']['desde_der'] == 700)
        self.assertTrue(rta['Respuesta']['hasta_der'] == 798)
        self.assertTrue(rta['Respuesta']['letra'] == u'  ')
        self.assertTrue(rta['Respuesta']['modificada'] == 1)
        self.assertTrue(rta['Respuesta']['dudosa'] == 0)
        self.assertTrue(rta['Respuesta']['sentido'] == 1)
        self.assertTrue(
            rta['Respuesta']['puntos'] ==
            [[-32.95351, -60.64861], [-32.95328, -60.65]]
        )
        self.assertTrue(rta['Respuesta']['nombre_calle'] == 'E. Zeballos')

        # Le paso un segmento que no existe
        gid = 1234
        rta = self.calle_postgres.obtener_segmento(gid)
        self.assertTrue(rta['Estado'] is False)
        self.assertTrue(
            rta['Respuesta'] == _(u'No se encontró el segmento 1234')
        )

    def test_obtener_calles_por_zona(self):

        zona = [
            [-32.936290, -60.721105],
            [-32.932769, -60.718112]
        ]

        rta = self.calle_postgres.obtener_calles_por_zona(zona)
        self.assertTrue(rta['Estado'] is False)

        rta = self.calle_postgres.obtener_calles_por_zona('')
        self.assertTrue(rta['Estado'] is False)

        zona = [
            [-32.936290, -60.721105],
            [-32.932769, -60.718112],
            [-32.936200, -60.715140]
        ]

        rta = self.calle_postgres.obtener_calles_por_zona(zona)
        self.assertTrue(rta['Estado'] is True)

        self.assertTrue(rta['Respuesta'][0]['nombre'] == u'Ecuador')

        zona = [
            [-32.935725, -60.733321],
            [-32.933063, -60.715668],
            [-32.936215, -60.710733]
        ]

        rta = self.calle_postgres.obtener_calles_por_zona(zona)
        self.assertTrue(rta['Estado'] is True)

    def test_verificar_calle_en_zona(self):

        # Urquiza
        id_calle = 1333579

        zona = [
            [-32.936290, -60.721105],
            [-32.932769, -60.718112],
            [-32.936200, -60.715140]
        ]
        rta = self.calle_postgres.verificar_calle_de_zona(id_calle, zona=zona)
        self.assertTrue(rta['Estado'] is False)

        # Pellegrini.
        id_calle = 1333403

        rta = self.calle_postgres.verificar_calle_de_zona(id_calle, zona=zona)
        self.assertTrue(rta['Estado'] is False)

        # Prusia
        id_calle = 1333469

        zona = [
            [-32.935725, -60.733321],
            [-32.933063, -60.715668],
            [-32.936215, -60.710733]
        ]

        rta = self.calle_postgres.verificar_calle_de_zona(id_calle, zona=zona)
        self.assertTrue(rta['Estado'] is True)

    def test_obtener_extremos_segmento(self):

        # Segmento diagonal 1.
        id_segmento = 2894126
        rta = self.calle_postgres.obtener_extremos_segmento(
            id_segmento=id_segmento
        )
        self.assertTrue(rta['Estado'] is True)

        # Segmento diagonal 2.
        id_segmento = 2893956
        rta = self.calle_postgres.obtener_extremos_segmento(
            id_segmento=id_segmento
        )
        self.assertTrue(rta['Estado'] is True)

        # Segmento horizontal.
        id_segmento = 2894125
        rta = self.calle_postgres.obtener_extremos_segmento(
            id_segmento=id_segmento
        )
        self.assertTrue(rta['Estado'] is True)

        # Segmento vertical.
        id_segmento = 2893951
        rta = self.calle_postgres.obtener_extremos_segmento(
            id_segmento=id_segmento
        )
        self.assertTrue(rta['Estado'] is True)

    def test_sugerir_calle_cercana(self):
        # Caso correcto
        parte_calle = 'urq'
        punto = [-32.933592, -60.718202]
        rta = self.calle_postgres.sugerir_calle_cercana(parte_calle, punto)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 18)

        # Caso correcto
        parte_calle = 'sar'
        punto = [-32.933592, -60.718202]
        rta = self.calle_postgres.sugerir_calle_cercana(parte_calle, punto)
        self.assertTrue(len(rta['Respuesta']) == 56)

        # Caso correcto
        parte_calle = 'Sarmiento'
        punto = [-32.933592, -60.718202]
        rta = self.calle_postgres.sugerir_calle_cercana(parte_calle, punto)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 16)

        # Le paso una calle que no existe
        # NOTA:
        # Va a retornar True y una lista vacía para hacer la función compatible
        # con la anterior que buscaba con la ciudad dada
        parte_calle = 'asdqwe'
        punto = [-32.933592, -60.718202]
        rta = self.calle_postgres.sugerir_calle_cercana(parte_calle, punto)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == [])

        # Paso solo 2 caracteres para buscar la calle,
        # empieza a buscar con 3 o más caracteres
        parte_calle = 'sa'
        punto = [-32.933592, -60.718202]
        rta = self.calle_postgres.sugerir_calle_cercana(parte_calle, punto)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(rta['Respuesta'] == [])


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCalle))
    return suite
