# -*- coding: utf-8 *-*
import unittest
from at.orm.abm import ABM
from at.map.zonas import Zonas

geodata = {
    'user': 'postgres',
    'passwd': 'TrackAir',
    'dbname': 'geodata-test',
    'name': 'geodata-test',
    'host': '192.168.0.68',
    'engine': 'postgres',
    'port': '5432',
}


class ZonasTesting(Zonas):

    def __init__(self):
        Zonas.__init__(self)
        self.geodata = ABM(geodata)
        self.orm = self.geodata


class TestZonas(unittest.TestCase):

    def setUp(self):
        self.zt = ZonasTesting()
        # Borramos
        self.zt.geodata.borrar('zonas')
        self.zt.geodata.borrar('zonas_cp')
        self.zt.geodata.borrar('zonas_tipos')
        # Agregamos
        data = {'id': 1, 'descripcion': 'CallProcessor'}
        self.zt.geodata.agregar('zonas_tipos', **data)
        data = {'id': 2, 'descripcion': 'Alarmas'}
        self.zt.geodata.agregar('zonas_tipos', **data)
        data = {'id': 3, 'descripcion': 'Descpacho'}
        self.zt.geodata.agregar('zonas_tipos', **data)

    def tearDown(self):
        self.zt.geodata.borrar('zonas')
        self.zt.geodata.borrar('zonas_cp')
        self.zt.geodata.borrar('zonas_tipos')

    def test_obtener_tipo(self):
        # Fallamos cuando le pasamos un string
        rta = self.zt.obtener_tipo('asd')
        self.assertFalse(rta['Estado'])

        # Fallamos al pasarle un id inexistente
        rta = self.zt.obtener_tipo(568)
        self.assertFalse(rta['Estado'])

        # Obtenemos correctamente un tipo
        rta = self.zt.obtener_tipo(2)
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta'] == 'Alarmas')

    def test_listar_tipos(self):
        # Obtenemos los tipos del sistema
        rta = self.zt.listar_tipos()
        self.assertTrue(rta['Estado'])
        # Obtenemos el primer tipo listado
        rta = self.zt.obtener_tipo(rta['Respuesta'][0]['Id'])
        self.assertTrue(rta['Estado'])

    def test_grabar(self):
        # Fallamos cuando le pasamos datos incorrectos
        rta = self.zt.grabar('asd', 234, True, '234', 324)
        self.assertFalse(rta['Estado'])

        # Fallamos cuando le pasamos tipos incorrectos
        rta = self.zt.grabar(256, 'asdasd', 'asd')
        self.assertFalse(rta['Estado'])

        # Agregamos correctamente una zona sin marcaodr
        rta = self.zt.grabar(1, 'Prueba', [
            [34.5, 60.3], [34.56, 60.145], [34.55889, 60.658]
        ])
        self.assertTrue(rta['Estado'])
        # Obtenemos el id agregado
        rta = self.zt.listar()
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        self.assertTrue(rta['Respuesta'][0]['Descripcion'] == 'Prueba')

        # Modificamos
        id_zona = rta['Respuesta'][0]['Id']
        marcador = [20.3, 30.5]
        rta = self.zt.grabar(1, 'Prueba2', marcador=marcador, id_zona=id_zona)
        self.assertTrue(rta['Estado'])

        rta = self.zt.listar()
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        self.assertTrue(rta['Respuesta'][0]['Id'] == id_zona)
        self.assertTrue(rta['Respuesta'][0]['Descripcion'] == 'Prueba2')

    def test_borrar(self):
        # Fallamos cuando le pasamos datos incorrectos
        rta = self.zt.borrar('asdasd')
        self.assertFalse(rta['Estado'])

        # Fallamos cuando le pasamos datos inexistentes
        rta = self.zt.borrar(-1)
        self.assertFalse(rta['Estado'])

        # Agregamos y borramos
        datos = {'id': 20, 'id_tipo': 1, 'descripcion': '234',
            'marcador': None,
            'geom': None}
        rta = self.zt.geodata.agregar('zonas', **datos)
        self.assertTrue(rta is True)
        rta = self.zt.borrar(20)
        self.assertTrue(rta['Estado'])

    def test_listar(self):
        # Fallamos al querer listar con tipos incorrectos e inexistentes
        rta = self.zt.listar('asdasd')
        self.assertFalse(rta['Estado'])
        rta = self.zt.listar(598)
        self.assertFalse(rta['Estado'])

        # Agregamos correctamente una zona sin marcaodr
        rta = self.zt.grabar(1, 'Prueba3', [
            [34.5, 60.3], [34.56, 60.145], [34.55889, 60.658]
        ])
        self.assertTrue(rta['Estado'])
        # Agregamos correctamente una zona con marcador
        rta = self.zt.grabar(3, 'Prueba5', [
            [34.5, 60.3], [34.56, 60.145], [34.55889, 60.658]
        ], marcador=[34.5, 60.3])
        self.assertTrue(rta['Estado'])
        # Agregamos correctamente una zona sin marcaodr
        rta = self.zt.grabar(3, 'Prueba4', [
            [34.5, 60.3], [34.56, 60.145], [34.55889, 60.658]
        ])
        self.assertTrue(rta['Estado'])

        # Listamos todo correctamente
        rta = self.zt.listar()
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 3)

        # Listamos todo correctamente
        rta = self.zt.listar(1)
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)

        # Agrego una zona con su marcador
        dic = {}
        dic['id_tipo'] = 1
        dic['descripcion'] = 'Zona con marcador 1'
        dic['marcador'] = '0101000020E61000008D0600B0D9574EC0E3FCFF4F747640C0'
        agregar = self.zt.geodata.agregar('zonas', **dic)
        self.assertTrue(agregar is True)

        # Agrego una zona con su marcador
        dic = {}
        dic['id_tipo'] = 1
        dic['descripcion'] = 'Zona con marcador 2'
        dic['marcador'] = '0101000020E610000013090078865B4EC0B91400707F7840C0'
        agregar = self.zt.geodata.agregar('zonas', **dic)
        self.assertTrue(agregar is True)

        # Agrego una zona sin marcador
        dic = {}
        dic['id_tipo'] = 1
        dic['descripcion'] = 'Zona sin marcador 1'
        agregar = self.zt.geodata.agregar('zonas', **dic)
        self.assertTrue(agregar is True)

        # Listo zonas del CP con paradas
        rta = self.zt.listar(1)
        self.assertTrue(rta['Estado'] is True)
        self.assertTrue(len(rta['Respuesta']) == 4)
        self.assertTrue(rta['Respuesta'][0]['Descripcion'] == 'Prueba3')
        self.assertTrue(rta['Respuesta'][0]['Marcador'] is None)
        self.assertTrue(rta['Respuesta'][0]['IdTipo'] == 1)
        self.assertTrue(isinstance(rta['Respuesta'][0]['Id'], int))
        self.assertTrue(rta['Respuesta'][1]['Descripcion'] ==
                                                        'Zona con marcador 1')
        self.assertTrue(rta['Respuesta'][1]['Marcador'] ==
                                            [-32.9254245758, -60.6863307953])
        self.assertTrue(rta['Respuesta'][1]['IdTipo'] == 1)
        self.assertTrue(isinstance(rta['Respuesta'][1]['Id'], int))
        self.assertTrue(rta['Respuesta'][2]['Descripcion'] ==
                                                        'Zona con marcador 2')
        self.assertTrue(rta['Respuesta'][2]['Marcador'] ==
                                            [-32.9413890839, -60.7150411606])
        self.assertTrue(rta['Respuesta'][2]['IdTipo'] == 1)
        self.assertTrue(isinstance(rta['Respuesta'][2]['Id'], int))
        self.assertTrue(rta['Respuesta'][3]['Descripcion'] ==
                                                        'Zona sin marcador 1')
        self.assertTrue(rta['Respuesta'][3]['Marcador'] is None)
        self.assertTrue(rta['Respuesta'][3]['IdTipo'] == 1)
        self.assertTrue(isinstance(rta['Respuesta'][3]['Id'], int))

    def test_obtener_ubicacion_zona(self):
        # Fallamos al querer obtener con un id incorrecto e inexistente
        rta = self.zt.obtener('asdasd')
        self.assertFalse(rta['Estado'])
        rta = self.zt.obtener(-598)
        self.assertFalse(rta['Estado'])

        # Agregamos correctamente una zona sin marcaodr
        rta = self.zt.grabar(3, 'Prueba2', [
            [34.5, 60.3], [34.56, 60.145], [34.55889, 60.658]
        ])
        # Listamos
        rta = self.zt.listar(3)
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        id_zona = rta['Respuesta'][0]['Id']

        # Obtenemos
        rta = self.zt.obtener(id_zona)
        print 'rta de ubicacion de zona'
        print rta
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['Descripcion'] == 'Prueba2')

    def test_obtener_por_punto(self):
        # Agregamos una zona
        rta = self.zt.grabar(1, 'Prueba', puntos=[
            [0, 0], [1, 0], [1, 1], [0, 1]
        ])
        self.assertTrue(rta['Estado'])
        # Buscamos el punto
        rta = self.zt.obtener_por_punto('[0.5, 0.5]asd')
        self.assertFalse(rta['Estado'])
        # Buscamos el punto
        rta = self.zt.obtener_por_punto([1.0005, 0.5])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 0)
        # Buscamos el punto
        rta = self.zt.obtener_por_punto([0.5, 0.5])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        # Buscamos cerquita
        rta = self.zt.obtener_por_punto([0.0001, 0.0005])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        # Agregamos una zona mas complicada
        rta = self.zt.grabar(1, 'Prueba', puntos=[
            [0, 0], [0, 4], [5, 4], [5, 3], [1, 3], [1, 1], [5, 1], [5, 0]
        ])
        self.assertTrue(rta['Estado'])
        # Buscamos el punto
        rta = self.zt.obtener_por_punto('[0.5, 0.5]asd')
        self.assertFalse(rta['Estado'])
        # Buscamos el punto
        rta = self.zt.obtener_por_punto([1.0005, 1.00005])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 0)
        # Buscamos el punto
        rta = self.zt.obtener_por_punto([3, 2])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 0)
        # Buscamos el punto
        rta = self.zt.obtener_por_punto([5, 1.1])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 0)
        # Buscamos cerquita
        rta = self.zt.obtener_por_punto([5, 0.999])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        rta = self.zt.obtener_por_punto([4.9999, 0.999])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        # Buscamos cerquita
        rta = self.zt.obtener_por_punto([4.0001, 3.0005])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        # Agregamos una zona aun mas complicada
        rta = self.zt.grabar(1, 'Prueba', puntos=[
            [0, 0], [0, 4], [5, 4], [5, 3], [-3, 2], [5, 1], [5, 0]
        ])
        self.assertTrue(rta['Estado'])
        rta = self.zt.obtener_por_punto([1.11, 2])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 0)
        rta = self.zt.obtener_por_punto([0.5, 2])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        # Buscamos en el triangulo que quedo
        rta = self.zt.obtener_por_punto([-2, 1.9])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        # Ahora la interseccion de las tres zonas
        rta = self.zt.obtener_por_punto([0.5, 0.5])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 3)

    def test_grabar_zona_cp(self):
        # Fallamos con datos incorrectos
        rta = self.zt.grabar_zona_cp('asd', False, [])
        self.assertFalse(rta['Estado'])
        # Buscamos zona inexistente
        rta = self.zt.grabar_zona_cp(4566, 1, 3)
        self.assertFalse(rta['Estado'])

        # Agregamos una zona
        rta = self.zt.grabar(1, 'Prueba', puntos=[
            [0, 0], [1, 0], [1, 1], [0, 1]
        ])
        self.assertTrue(rta['Estado'])
        # Listamos
        rta = self.zt.listar()
        self.assertTrue(rta['Estado'])
        # Obtenemos el id
        id_zona = rta['Respuesta'][0]['Id']
        # Grabamos su zona cp
        rta = self.zt.grabar_zona_cp(id_zona, 1, 2)
        self.assertTrue(rta['Estado'])

    def test_obtener_zona_cp(self):
        # Fallamos con datos incorrectos
        rta = self.zt.obtener_zona_cp('asd', False)
        self.assertFalse(rta['Estado'])
        # Buscamos zona inexistente
        rta = self.zt.obtener_zona_cp(4566, 1)
        self.assertFalse(rta['Estado'])

        # Agregamos una zona
        rta = self.zt.grabar(1, 'Prueba2', puntos=[
            [0, 0], [1, 0], [1, 1], [0, 1]
        ])
        self.assertTrue(rta['Estado'])
        # Listamos
        rta = self.zt.listar()
        self.assertTrue(rta['Estado'])
        # Obtenemos el id
        id_zona = rta['Respuesta'][0]['Id']
        # Grabamos su zona cp
        id_zona_cp = 1
        id_sucursal_cp = 2
        rta = self.zt.grabar_zona_cp(id_zona, id_sucursal_cp, id_zona_cp)
        self.assertTrue(rta['Estado'])
        # Obtenemos la zona cp, pero falalamos por la sucursal
        rta = self.zt.obtener_zona_cp(id_zona, id_sucursal=23423)
        self.assertFalse(rta['Estado'])
        # Obtenemos la zona cp
        rta = self.zt.obtener_zona_cp(id_zona)
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['IdZona'] == id_zona)
        self.assertTrue(rta['Respuesta']['IdZonaCp'] == id_zona_cp)
        self.assertTrue(rta['Respuesta']['IdSucursalCp'] == id_sucursal_cp)

        # Obtenemos la zona cp, pero con la sucursal como argumetno
        rta = self.zt.obtener_zona_cp(id_zona, id_sucursal_cp)
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['IdZona'] == id_zona)
        self.assertTrue(rta['Respuesta']['IdZonaCp'] == id_zona_cp)
        self.assertTrue(rta['Respuesta']['IdSucursalCp'] == id_sucursal_cp)

    def test_borrar_zona_cp(self):
        # Fallamos con datos incorrectos
        rta = self.zt.borrar_zona_cp('asd')
        self.assertFalse(rta['Estado'])
        # Buscamos zona inexistente
        rta = self.zt.borrar_zona_cp(4566)
        self.assertFalse(rta['Estado'])

        # Agregamos una zona
        rta = self.zt.grabar(1, 'Prueba', puntos=[
            [0, 0], [1, 0], [1, 1], [0, 1]
        ])
        self.assertTrue(rta['Estado'])
        # Listamos
        rta = self.zt.listar()
        self.assertTrue(rta['Estado'])
        # Obtenemos el id
        id_zona = rta['Respuesta'][0]['Id']
        # Grabamos su zona cp
        rta = self.zt.grabar_zona_cp(id_zona, 1, 2)
        self.assertTrue(rta['Estado'])
        rta = self.zt.obtener_zona_cp(id_zona)
        self.assertTrue(rta['Estado'])
        self.assertTrue(rta['Respuesta']['IdZonaCp'] == 2)
        self.assertTrue(rta['Respuesta']['IdSucursalCp'] == 1)
        # Borrarmos correctamente
        rta = self.zt.borrar_zona_cp(id_zona)
        self.assertTrue(rta['Estado'])
        # Vemos si esta la zona cp
        rta = self.zt.obtener_zona_cp(id_zona)
        self.assertFalse(rta['Estado'])

    def test_obtener_zona_cp_por_punto(self):
        # Fallamos con datos incorrectos
        rta = self.zt.obtener_zonas_cp_por_punto('asd', False)
        self.assertFalse(rta['Estado'])
        # Agregamos una zona
        rta = self.zt.grabar(1, 'Prueba2', puntos=[
            [0, 0], [1, 0], [1, 1], [0, 1]
        ])
        self.assertTrue(rta['Estado'])
        # Listamos
        rta = self.zt.listar()
        self.assertTrue(rta['Estado'])
        # Obtenemos el id
        id_zona = rta['Respuesta'][0]['Id']
        # Grabamos su zona cp
        id_zona_cp = 1
        id_sucursal_cp = 2
        # Buscamos por punto, pero fallamos por que no hay zona cp
        rta = self.zt.obtener_zonas_cp_por_punto([0.5, 0.5])
        self.assertFalse(rta['Estado'])
        # Ahora si agregamos la zona cp
        rta = self.zt.grabar_zona_cp(id_zona, id_sucursal_cp, id_zona_cp)
        self.assertTrue(rta['Estado'])
        # Obtenemos la zona cp, pero fallamos por la sucursal
        rta = self.zt.obtener_zonas_cp_por_punto([0.5, 0.5], id_sucursal=23423)
        self.assertFalse(rta['Estado'])
        # Obtenemos la zona cp
        rta = self.zt.obtener_zonas_cp_por_punto([0.5, 0.5])
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        self.assertTrue(rta['Respuesta'][0]['IdZona'] == id_zona)
        self.assertTrue(rta['Respuesta'][0]['IdZonaCp'] == id_zona_cp)
        self.assertTrue(rta['Respuesta'][0]['IdSucursalCp'] == id_sucursal_cp)

        # Obtenemos la zona cp, pero con la sucursal como argumetno
        rta = self.zt.obtener_zonas_cp_por_punto([0.5, 0.5], id_sucursal_cp)
        self.assertTrue(rta['Estado'])
        self.assertTrue(len(rta['Respuesta']) == 1)
        self.assertTrue(rta['Respuesta'][0]['IdZona'] == id_zona)
        self.assertTrue(rta['Respuesta'][0]['IdZonaCp'] == id_zona_cp)
        self.assertTrue(rta['Respuesta'][0]['IdSucursalCp'] == id_sucursal_cp)

    def test_listar_con_geom(self):
        # Agregamos una zona
        rta = self.zt.grabar(1, 'Prueba2', puntos=[
            [0, 0], [1, 0], [1, 1], [0, 1]
        ])
        self.assertTrue(rta['Estado'] is True)
        # Listamos las zonas.
        rta_listar = self.zt.listar_con_geom(True)
        self.assertTrue(rta_listar['Estado'] is True)
        self.assertTrue(isinstance(rta_listar['Respuesta'][0][0], int))
        self.assertTrue(rta_listar['Respuesta'][0][1] == 'Prueba2')
        self.assertTrue(rta_listar['Respuesta'][0][2] == [])
        # Agregamos otra zona
        rta = self.zt.grabar(1, 'Prueba3', puntos=[
            [1, 2], [2, 3], [3, 4], [1, 2]
        ])
        self.assertTrue(rta['Estado'] is True)
        # Listamos las zonas.
        rta_listar = self.zt.listar_con_geom(True)
        self.assertTrue(rta_listar['Estado'] is True)
        self.assertTrue(isinstance(rta_listar['Respuesta'][1][0], int))
        self.assertTrue(rta_listar['Respuesta'][1][1] == 'Prueba3')
        self.assertTrue(rta_listar['Respuesta'][1][2] == [])


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestZonas))
    return suite
