# -*- coding: utf-8 -*-

import math
from operator import itemgetter
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import func
from sqlalchemy.sql.expression import not_

from at.map import _
from at.map.utiles.funciones_geo import FuncionesGeo
from at.map.calle import CallePostgres
from at.orm.decorators import DBSwitch
from at.orm.utiles import obtener_bd

v_mapadmin = True

try:
    from at.mapadmin.operaciones import Operaciones
except:
    print '>> warn: No hay at.mapadmin'
    v_mapadmin = False


@DBSwitch('gis2', 'cp', 'gis', 'geodata', 'muni')
class Migraciones(object):

    def __init__(self):
        self.proyeccion = 4326
        self.funciones_geo = FuncionesGeo()
        self.calle_postgres = CallePostgres()
        self.operaciones = Operaciones()

    def migrar_nombres_calles(self, lat1, lon1, lat2, lon2, bd_osm,
                                                    ids_ciudades, bd_muni=None):
        '''
            Esta función se encarga de reemplazar nombres de una ciudad de OSM
            a gis2.

            Argumento:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                bd_osm (str)
                ids_ciudades (dict)
                bd_muni (str)

            Funciones:
                Migraciones().calles_ciudad
                Migraciones().comparar_calles

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Llamo a la función calles_ciudad
        rta_calles_ciudad = self.calles_ciudad(lat1, lon1, lat2, lon2)
        # Si no se encuentra ningún segmento retorno el mensaje
        if rta_calles_ciudad['Estado'] is False:
            return rta_calles_ciudad

        # Guardo los segmentos obtenidos de la ciudad
        lista_calles = rta_calles_ciudad['Respuesta']

        # Llamo a la función comparar_calles pasandole la lista de calles
        rta_comparar_calle = self.comparar_calles(
            lista_calles, lat1, lon1, lat2, lon2,
            bd_osm, ids_ciudades, bd_muni
        )
        return rta_comparar_calle

    def calles_ciudad(self, lat1, lon1, lat2, lon2):
        '''
            Esta función se encarga de obtener todos los segmentos de una
            zona dada.

            Argumento:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de tuplas)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Abro la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')

        # Obtengo todos los segmentos de la zona dada
        q_obj = session.query(
            calles.gid,
            func.astext(calles.geom).label('geom'),
            calles.desde_izq,
        )

        q_obj = q_obj.filter(
            func.st_intersects(
                calles.geom,
                func.st_geomfromtext(
                    'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                        str(lon1), str(lat1),
                        str(lon2), str(lat1),
                        str(lon2), str(lat2),
                        str(lon1), str(lat2),
                        str(lon1), str(lat1),
                    ),
                    self.proyeccion
                )
            ),
        )

        # Guardamos todos los registros encontrados
        registros = q_obj.all()

        # Cierro la conexión con la base de datos
        session.close()

        # Si no encuentro registros retorno el error
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron segmentos en la zona dada')
            return rta

        # Si encontré registro retorno una lista de tuplas con los datos
        rta['Estado'] = True
        rta['Respuesta'] = registros
        return rta

    def comparar_calles(self, lista_calles, lat1, lon1, lat2, lon2,
                                            bd_osm, ids_ciudades, bd_muni=None):
        '''
            Esta función se encarga de comparar una lista de calles con las
            que están guardadas en la base de datos OSM.

            Argumento:
                lista_calles (list de tuple)
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                bd_osm (str)
                ids_ciudades (dict)
                bd_muni (str)

            Funciones:
                at.orm.utiles.obtener_bd
                FuncionesGeo().procesar_geometria
                Migraciones().NorthAzimuth
                Migraciones().actualizar_calle

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # 898
        # 84
        # 656
        # 107
        # 724
        # 0
        # 20
        # 23
        # 24
        # 26 (no está en OSM)
        # 27
        # 28 (se cambiaría Colon por Laplace)
        # 30
        # 32 (VER)
        # 35
        # 37
        # 25
        # 66
        # 76

        # Uso la BD de OSM que se pasó como parámetro
        rta_obtener_bd = obtener_bd(bd_osm)
        if rta_obtener_bd['Estado'] is False:
            return rta_obtener_bd
        self.osm = rta_obtener_bd['Respuesta']

        # Si se pasó una BD de la municipalidad de alguna ciudad
        if bd_muni is not None:
            # Utilizo la BD de la municipalidad en vez de la OSM
            rta_obtener_bd = obtener_bd(bd_muni)
            if rta_obtener_bd['Estado'] is False:
                return rta_obtener_bd
            self.osm = rta_obtener_bd['Respuesta']

        # Abro la sesión con la base de datos osm
        session = self.osm.binded_session()

        # Obtengo la tabla lines
        lines = self.osm.tabla('lines')

        #pos = 0

        for calle in lista_calles:
            ## Calle de prueba
            #calle = lista_calles[1]
            #print '\n-----------------------------------------------------'
            #print 'pos de la lista: ' + str(pos)
            #pos += 1
            #print '* CALLE DE GIS2'
            #print calle

            calle_gis2 = calle[1]
            linea = self.funciones_geo.procesar_geometria(calle_gis2)
            angulo = self.NorthAzimuth(linea[0], linea[1])

            q_obj = session.query(
                lines.name,
                lines.gid,
                func.astext(lines.the_geom).label('the_geom'),
                func.ST_HausdorffDistance(
                    lines.the_geom,
                    func.st_geomfromtext(calle_gis2, self.proyeccion),
                    0.5,
                ).label('hausdorf'),
                func.st_distance(
                    lines.the_geom,
                    func.st_geomfromtext(calle_gis2, self.proyeccion),
                ).label('distancia')
            )

            q_obj = q_obj.filter(
                and_(
                    func.st_intersects(
                        lines.the_geom,
                        func.st_geomfromtext(
                            'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                                str(lon1), str(lat1),
                                str(lon2), str(lat1),
                                str(lon2), str(lat2),
                                str(lon1), str(lat2),
                                str(lon1), str(lat1),
                            ),
                            self.proyeccion
                        )
                    ),
                    func.st_distance(
                        lines.the_geom,
                        func.st_geomfromtext(calle_gis2, self.proyeccion),
                    ) < 0.0005,  # Estaba 0.0005
                )
            )

            # Guardo todos los registros obtenidos
            registros = q_obj.all()

            calles_correctas = []

            for registro in registros:
                calle_correcta = None
                menor_dif_angulo = 360

                linea = self.funciones_geo.procesar_geometria(registro[2])

                for i in range(len(linea) - 1):
                    angulo_aux = self.NorthAzimuth(linea[i], linea[i + 1])

                    dif_angulo = abs(angulo_aux - angulo)

                    if dif_angulo < menor_dif_angulo and dif_angulo < 10:
                        menor_dif_angulo = dif_angulo
                        datos = {}
                        datos['nombre'] = registro[0]
                        datos['id'] = registro[1]
                        datos['linea'] = registro[2]
                        datos['hausdorff'] = registro[3]
                        datos['distancia'] = registro[4]
                        datos['dif_angulo'] = dif_angulo
                        datos['puntos'] = 0
                        calle_correcta = datos

                if calle_correcta is not None:
                    calles_correctas.append(calle_correcta)

            dudosa = 0

            # Si no tengo ninguna posible calle
            if len(calles_correctas) == 0:
                # Paso a la siguiente
                continue
                #print '* No hay calle cercana en OSM, la guardo como desconocida'
                ## Creo una calle desconocida
                #calle_correcta = {}
                #calle_correcta['nombre'] = None
                #calle_correcta['puntos'] = 0
                #calle_menor_dist = {}
                #calle_menor_dist['puntos'] = 0
                #calle_menor_hausdorff = {}
                #calle_menor_hausdorff['puntos'] = 0
                #calle_menor_angulo = {}
                #calle_menor_angulo['puntos'] = 0

            calle_menor_dist = {}
            calle_menor_dist['puntos'] = 0

            calle_menor_hausdorff = {}
            calle_menor_hausdorff['puntos'] = 0

            calle_menor_angulo = {}
            calle_menor_angulo['puntos'] = 0

            menor_dist = 1
            menor_hausdorff = 2
            menor_dif_angulo = 360
            for c in calles_correctas:
                # Si el segmento de OSM no tiene nombre
                if c['nombre'] is None:
                    # No le sumo puntos
                    continue

                if c['distancia'] < menor_dist:
                    menor_dist = c['distancia']
                    calle_menor_dist = c

                if c['hausdorff'] < menor_hausdorff:
                    menor_hausdorff = c['hausdorff']
                    calle_menor_hausdorff = c

                if c['dif_angulo'] < menor_dif_angulo:
                    menor_dif_angulo = c['dif_angulo']
                    calle_menor_angulo = c

            calle_menor_dist['puntos'] += 1
            calle_menor_hausdorff['puntos'] += 1
            calle_menor_angulo['puntos'] += 1

            menor_puntos = 0
            for c in calles_correctas:
                if c['puntos'] > menor_puntos:
                    menor_puntos = c['puntos']
                    calle_correcta = c

            # Si no tengo ninguna posible calle
            if calle_correcta is None:
                # Paso al siguiente segmento
                continue

            if calle_correcta['puntos'] < 3:
                dudosa = 1

            # Si la posible calle no tiene ningún punto
            if calle_correcta['puntos'] == 0:
                # Paso al siguiente segmento
                continue

            #es_desconocida = False

            # Si la calle de OSM no tiene nombre
            if calle_correcta['nombre'] is None:
                # Paso a la siguiente
                continue
                #print '* La calle en OSM no tiene nombre, la guardo como desconocida'
                #calle_correcta['nombre'] = 'Desconocida'
                #dudosa = 1
                #es_desconocida = True

            #nombre_calle_gis2 = calle[3]
            #if nombre_calle_gis2 is None:
                #nombre_calle_gis2 = 'None'

            #if nombre_calle_gis2.lower() != \
                                    #calle_correcta['nombre'].strip().lower():

            #if calle_correcta['nombre'] is None:
                #dudosa = 1

            # Llamo a la función actualizar_calle
            self.actualizar_calle(
                calle[0], calle_correcta['nombre'], dudosa, bd_osm, ids_ciudades
            )

            print '* Seteo el nombre: ' + calle_correcta['nombre']

        # Cierro la conexión con la base de datos
        session.close()

    def NorthAzimuth(self, punto1, punto2):
        degBearing = math.degrees(
            math.atan2(
                (punto2[0] - punto1[0]),
                (punto2[1] - punto1[1])
            )
        )

        if(degBearing < 0):
            #degBearing += 360.0
            degBearing += 180.0

        #degBearing = abs(degBearing)

        return degBearing

    def actualizar_calle(self, id_calle_gis2, nombre_calle_osm, dudosa,
                                                        bd_osm, ids_ciudades):
        rta = {}

        # Llamo a la función obtener_ciudad_del_segmento
        rta_obtener_ciudad_del_segmento = self.obtener_ciudad_del_segmento(
            id_calle_gis2, bd_osm, ids_ciudades
        )
        # Si no se pudo obtener la ciudad del segmento
        if rta_obtener_ciudad_del_segmento['Estado'] is False:
            # Retorno el error
            return rta_obtener_ciudad_del_segmento

        # Si pude obtener la ciudad la guardo para actualizar el segmento
        id_ciudad = rta_obtener_ciudad_del_segmento['Respuesta']

        print 'id_ciudad obtenida'
        print id_ciudad

        session = self.gis2.binded_session()

        calles_nombres = self.gis2.tabla('calles_nombres')
        q_obj = session.query(calles_nombres.id)
        q_obj = q_obj.filter(
            and_(
                calles_nombres.id_ciudad == id_ciudad,
                calles_nombres.nombre == nombre_calle_osm,
            )
        )
        registro = q_obj.first()

        # Defino id_nueva_calle
        id_nueva_calle = None

        # El usuario 1 es el admin.
        id_usuario = 1

        if registro is None:

            # Trato de agregar la calle.
            rta_grabar_calle = self.operaciones.grabar_calle(
                id_usuario, nombre_calle_osm, id_ciudad, id_nueva_calle
            )

            # Verifico el resultado.
            if rta_grabar_calle['Estado'] is not True:
                return rta_grabar_calle

            # Guardo el id_calle.
            id_nueva_calle = rta_grabar_calle['DatosExtra']['id']
        else:
            id_nueva_calle = registro.id

        segmento = {
            'Id': id_calle_gis2,
            'IdCalleNombre': id_nueva_calle
        }

        # Trato de modificar
        rta = self.operaciones.grabar_segmento(id_usuario, segmento)

        return rta

    def limpiar_calles(self, id_ciudad):
        '''
            Esta función se encarga de eliminar las calles que no tienen
            segmentos asignados en una ciudad dada.

            Argumentos:
                id_ciudad (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Abro la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')
        # Obtengo la tabla calles_nombres
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Armo una subquery para obtener todas las calles nombres de la ciudad
        q_obj1 = session.query(calles_nombres.id)
        q_obj1 = q_obj1.filter(calles_nombres.id_ciudad == id_ciudad)
        q_obj1 = q_obj1.subquery()

        # Armo una subquery para obtener todos los segmentos de la ciudad dada
        q_obj2 = session.query(
            calles.id_calle_nombre,
        )
        q_obj2 = q_obj2.filter(
            calles.id_calle_nombre.in_(q_obj1)
        )
        q_obj2 = q_obj2.subquery()

        # Obtengo las calles que no tienen segmentos
        q_obj3 = session.query(calles_nombres.id)
        q_obj3 = q_obj3.filter(
            and_(
                calles_nombres.id_ciudad == id_ciudad,
                not_(calles_nombres.id.in_(q_obj2)),
            )
        )

        if q_obj3.count() == 0:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No hay calles sin segmentos en la ciudad %s'
                  % id_ciudad)
            return rta

        registros = q_obj3.all()

        # Cierro la sesión con la base de datos
        session.close()

        # Usuario admin.
        id_usuario = 1

        # Defino un estado de respuesta.
        rta_func = {'Estado': True}

        # Recorro las calles.
        for registro in registros:

            # Trato de eliminar.
            rta_func = self.operaciones.eliminar_calle(id_usuario, registro.id)

        # Verifico el estado de la funcion.
        if rta_func['Estado'] is True:
            rta['Estado'] = True
            rta['Respuesta'] = \
                _(u'Se eliminaron las calles sin segmentos de la ciudad %s'
                % id_ciudad)
        else:
            rta = rta_func

        return rta

    def borrar_ciudad(self, lat1, lon1, lat2, lon2):
        '''
            Esta función se encarga de borrar una ciudad de gis2.
            Borra todas las calles que están en un rectángulo dado.

            Argumentos:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Abro la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')
        # Obtengo la tabla calles_nombres

        # Obtengo todos los segmentos que están en el rectángulo dado
        q_obj = session.query(calles.gid)
        q_obj = q_obj.filter(
            func.st_intersects(
                calles.geom,
                func.st_geomfromtext(
                    'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                        str(lon1), str(lat1),
                        str(lon2), str(lat1),
                        str(lon2), str(lat2),
                        str(lon1), str(lat2),
                        str(lon1), str(lat1),
                    ),
                    4326
                )
            ),
        )

        registros = q_obj.all()
        print 'Segmentos a borrar'
        print len(registros)

        for registro in registros:
            q_obj = session.query(calles.gid)
            q_obj = q_obj.filter(calles.gid == registro.gid)
            q_obj.delete()

        # Commiteo a la base de datos
        session.commit()
        # Cierro la sesión con la base de datos
        session.close()

        rta['Estado'] = True
        rta['Respuesta'] = _(u'La ciudad se borró correctamente')
        return rta

    def pegar_ciudad(self, lat1, lon1, lat2, lon2):
        '''
            Esta función se encarga de obtener todos los segmentos de un
            rectángulo dado de gis2_viejo y agregarlos a la base de datos
            de gis2.

            Argumentos:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Abro la sesión con la base de datos gis2_viejo
        session_gis2_viejo = self.gis2_viejo.binded_session()

        # Obtengo la tabla calles
        calles_gis2_viejo = self.gis2_viejo.tabla('calles')

        # Obtengo la tabla calles_nombres

        # Obtengo todos los segmentos que están en el rectángulo dado
        q_obj = session_gis2_viejo.query(calles_gis2_viejo)
        q_obj = q_obj.filter(
            func.st_intersects(
                calles_gis2_viejo.geom,
                func.st_geomfromtext(
                    'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                        str(lon1), str(lat1),
                        str(lon2), str(lat1),
                        str(lon2), str(lat2),
                        str(lon1), str(lat2),
                        str(lon1), str(lat1),
                    ),
                    4326
                )
            ),
        )
        registros = q_obj.all()

        # Cierro la sesión con la base de datos gis2_viejo
        session_gis2_viejo.close()

        # Abro la sesión con la base de datos gis2
        session_gis2_nuevo = self.gis2.binded_session()

        calles_gis2_nuevo = self.gis2.tabla('calles')
        q_obj = session_gis2_nuevo.query(calles_gis2_nuevo.gid)
        q_obj = q_obj.order_by(calles_gis2_nuevo.gid.desc())
        registro = q_obj.first()
        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo obtener el último id de calles nombres')
            return rta
        id_nueva_calle = registro.gid + 1

        # Recorro los segmentos obtenidos
        for registro in registros:
            # Verifico si existe el nombre de la calle
            calles_nombres_gis2_nuevo = self.gis2.tabla('calles_nombres')
            q_obj = session_gis2_nuevo.query(
                calles_nombres_gis2_nuevo.id,
                calles_nombres_gis2_nuevo.nombre,
                calles_nombres_gis2_nuevo.id_ciudad,
            )
            q_obj = q_obj.filter(
                calles_nombres_gis2_nuevo.id == registro.id_calle_nombre
            )
            datos_calle = q_obj.first()

            # Si no existe el nombre de la calle
            # Marco la calle como desconocida
            id_calle_nombre = 0
            if datos_calle is not None:
                # Si existe la calle la guardo
                id_calle_nombre = datos_calle.id

            a = {}
            a['gid'] = id_nueva_calle
            a['id_calle_nombre'] = id_calle_nombre
            a['desde_izq'] = registro.desde_izq
            a['hasta_izq'] = registro.hasta_izq
            a['desde_der'] = registro.desde_der
            a['hasta_der'] = registro.hasta_der
            a['id_tipo'] = registro.id_tipo
            a['sentido'] = registro.sentido
            a['geom'] = registro.geom
            a['letra'] = ''
            self.gis2.agregar('calles', **a)

            # Paso al siguiente id de calles
            id_nueva_calle += 1

        # Cierro la sesión con la base de datos gis2
        session_gis2_nuevo.close()

        rta['Estado'] = True
        rta['Respuesta'] = _(u'La ciudad se pegó correctamente')
        return rta

    def arreglar_id_ciudad(self, id_ciudad, geometria_ciudad,
                                                    lat1, lon1, lat2, lon2):
        '''
            Esta función se encarga de arreglar el campo id_ciudad de las
            calles que pertenecen a la zona dada.

            Argumentos:
                id_ciudad (int)
                geometria (str)
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        rta = self.calles_ciudad(lat1, lon1, lat2, lon2)
        if rta['Estado'] is False:
            return rta

        lista_calles = rta['Respuesta']

        session = self.gis2.binded_session()
        calles = self.gis2.tabla('calles')
        calles_nombres = self.gis2.tabla('calles_nombres')

        for calle in lista_calles:
            id_segmento = calle[0]

            q_obj = session.query(
                calles.gid,
                calles.geom,
                calles.id_calle_nombre,
                func.st_intersects(
                    calles.geom,
                    geometria_ciudad,
                ).label('esta_en_geometria_ciudad')
            )
            q_obj = q_obj.filter(calles.gid == id_segmento)
            registro = q_obj.first()

            if registro is None:
                print 'No se encontro el segmento dado'
                continue

            if registro.esta_en_geometria_ciudad is False:
                print 'El segmento no está en la ciudad dada'
                continue

            q_obj2 = session.query(
                calles_nombres.id,
                calles_nombres.nombre,
                calles_nombres.id_ciudad
            )
            q_obj2 = q_obj2.filter(
                calles_nombres.id == registro.id_calle_nombre
            )

            dic = {}
            dic['id_ciudad'] = id_ciudad
            q_obj2.update(dic, synchronize_session=False)

            print 'Se actualizo la ciudad del segmento'

        session.commit()
        session.close()

        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se arregló el id_ciudad de las calles de la zona dada')
        return rta

    def modificar_calles_desconocidas(self, geometria_zona, id_calle_vieja,
                    id_calle_nueva, nombre_calle_nueva, id_ciudad_calle_nueva):
        '''
            Esta función se encarga de cambiar el nombre de desconocida
            a descononcida1 en una zona dada

            Argumentos:
                geometria_zona (str)
                id_calle_vieja (int)
                id_calle_nueva (int)
                nombre_calle_nueva (str)
                id_ciudad_calle_nueva (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        a = {}
        a['id'] = id_calle_nueva
        a['id_ciudad'] = id_ciudad_calle_nueva
        a['nombre'] = nombre_calle_nueva
        self.gis2.agregar('calles_nombres', **a)

        session = self.gis2.binded_session()

        calles = self.gis2.tabla('calles')

        q_obj = session.query(calles.gid)

        q_obj = q_obj.filter(
            and_(
                calles.id_calle_nombre == id_calle_vieja,
                func.st_intersects(
                    calles.geom,
                    geometria_zona,
                )
            )
        )
        resultado = q_obj.all()

        dic = {}
        dic['id_calle_nombre'] = id_calle_nueva
        q_obj.update(dic, synchronize_session=False)

        session.commit()
        session.close()

        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se modificaron %s calles desconocidas') \
                                                            % len(resultado)
        return rta

    def obtener_calle_equivalente(self, id_calle_gis, id_ciudad_gis,
                                                            nombre_calle_gis):
        '''
            Esta función se encarga de obtener el id de las calles de gis2
            que sea equivalente en gis.
            Se deben actualizar los datos de los clientes de los radio taxis
            que utilizaron la base de datos gis.
            Las tablas de cp2000 donde se guardan los datos de los clientes son:
            "clientes" y "clientes_direcciones".

            Argumentos:
                id_calle_gis (int)
                id_ciudad_gis (int)
                nombre_calle_gis (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['id_calle_gis'] (int)
                                        ['id_calle_gis2'] (int)
                                        ['nombre_calle_gis'] (str)
                                        ['nombre_calle_gis2'] (str)
                                        ['id_ciudad_gis'] (int)
                                        ['id_ciudad_gis2'] (int)
                                        ['puntos'] (int)
        '''

        rta = {}

        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo los datos necesario de la tabla clientes_direcciones
        clientes_direcciones = self.cp.tabla('clientes_direcciones')
        q_obj = session_cp.query(
            clientes_direcciones.IdCalle1,
            clientes_direcciones.IdCalle2,
            clientes_direcciones.Latitud,
            clientes_direcciones.Longitud,
            clientes_direcciones.Direccion,
            clientes_direcciones.IdCiudad,
            clientes_direcciones.Altura,
        )
        q_obj = q_obj.filter(
            and_(
                clientes_direcciones.IdCalle1 == id_calle_gis,
                clientes_direcciones.IdCiudad == id_ciudad_gis,
                clientes_direcciones.Latitud != 0,
                clientes_direcciones.Longitud != 0,
            )
        )
        registros = q_obj.all()

        # Cierro la sesión con la base de datos cp2000
        session_cp.close()

        # Si no se encontraron direcciones con la calle dada retorno el error
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se encontraron direcciones con la calle dada')
            return rta

        # Creo un diccionario donde se guardarán las posibles calles
        calles_posibles = {}

        # Recorro todos los registros obtenidos
        for registro in registros:
            # Si la dirección es una esquina no la tengo en cuenta
            # ya que después voy a poder reemplazar ambas calles
            # cuando obtenga por separado el id de cada una
            if registro.Altura is None:
                continue

            # Según la latitud y longitud obtengo cual sería la calle en gis2
            latitud = float(registro.Latitud / 3600)
            longitud = float(registro.Longitud / 3600)
            rta_obtener_posicion_por_latlon = \
                self.calle_postgres.obtener_posicion_por_latlon(
                    latitud,
                    longitud,
                )
            # Si no puedo obtener la calle paso al siguiente registro
            if rta_obtener_posicion_por_latlon['Estado'] is False:
                continue

            # Guardo el nombre de la calle y su id
            # Estos datos guardados son de gis2
            nombre_calle_gis2 = \
                rta_obtener_posicion_por_latlon['Respuesta']['Calle']
            id_calle_gis2 = \
                rta_obtener_posicion_por_latlon['Respuesta']['IdCalle']

            # Si no tengo el nombre de la calle de gis2 sigo con el siguiente
            if nombre_calle_gis2 is None:
                continue

            # Obtengo la centena de la altura de la dirección
            altura_centena = str(registro.Altura)[-2:]

            # Si la altura es menor a 20 o mayor a 80 no tengo en cuenta
            # los datos
            if int(altura_centena) <= 20 or int(altura_centena) >= 80:
                continue

            # Me fijo si la calle obtenida ya la tengo guardada
            # en mi diccionario
            dic = {}
            if nombre_calle_gis2 in calles_posibles:
                # Si ya la tengo le aumento 1 punto
                calles_posibles[nombre_calle_gis2]['puntos'] += 1
            else:
                # Si no la tengo la agrego con su primer punto
                dic['id_calle_gis2'] = id_calle_gis2
                dic['nombre_calle_gis2'] = nombre_calle_gis2
                dic['puntos'] = 1
                calles_posibles[nombre_calle_gis2] = dic

        # Si no encuentro calles posibles retorno el error
        if calles_posibles == {}:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo encontrar la calle equivalente')
            return rta

        # Me fijo que calle tiene más puntos
        max_puntos = 0
        for calle_posible in calles_posibles:
            if calles_posibles[calle_posible]['puntos'] > max_puntos:
                max_puntos = calles_posibles[calle_posible]['puntos']
                calle_equivalente = calles_posibles[calle_posible]

        # Le agrego la ciudad equivalente de gis2 a la calle resultante
        if id_ciudad_gis == 1:
            id_ciudad_gis2 = 324
        elif id_ciudad_gis == 2:
            id_ciudad_gis2 = 1381
        elif id_ciudad_gis == 3:
            id_ciudad_gis2 = 3772
        elif id_ciudad_gis == 4:
            id_ciudad_gis2 = 870
        else:
            id_ciudad_gis2 = None

        calle_equivalente['id_ciudad_gis'] = id_ciudad_gis
        calle_equivalente['id_ciudad_gis2'] = id_ciudad_gis2

        # Le agrego el id de la calle de gis
        calle_equivalente['id_calle_gis'] = id_calle_gis

        # Le agrego el nombre de la calle de gis
        calle_equivalente['nombre_calle_gis'] = nombre_calle_gis

        # Retorno la calle equivalente
        rta['Estado'] = True
        rta['Respuesta'] = calle_equivalente
        return rta

    def guardar_calles_equivalentes(self):
        '''
            Esta función se encarga de guardar las calles equivalente de
            gis con gis2.
            Por cada calle de gis se busca su equivalente en gis2 y se
            guarda en la tabla migracion_gis

            Funciones:
                Migraciones().obtener_calle_equivalente

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Me conecto a la base de datos gis
        session_gis = self.gis.binded_session()

        # Obtengo todas las calles de gis
        calles = self.gis.tabla('calles')
        q_obj = session_gis.query(
            calles.idCalle, calles.idCiudad, calles.Nombre
        )
        registros = q_obj.all()

        # Cierro la sesión con la base de datos gis
        session_gis.close()

        # Si no tengo calles retorno el error
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encotnraron calles')
            return rta

        # Recorro los registros obtenidos
        for registro in registros:
            print 'Calle'
            print registro.Nombre

            # Llamo a la función obtener_calle_equivalente
            rta_obtener_calle_equivalente = self.obtener_calle_equivalente(
                registro.idCalle, registro.idCiudad, registro.Nombre
            )

            print 'rta_obtener_calle_equivalente'
            print rta_obtener_calle_equivalente

            print '**********************************************************'

            # Si no se puede obtener la calle equivalente sigo con el próximo
            # registro pero guardo la calle que no se pudo agregar
            if rta_obtener_calle_equivalente['Estado'] is False:
                a = {}
                a['id_calle_gis'] = registro.idCalle
                a['nombre_calle_gis'] = registro.Nombre
                a['id_ciudad_gis'] = registro.idCiudad
                self.cp.agregar('migracion_gis', **a)
                continue

            # Guardo los datos de la calle equivalente
            calle_equivalente = rta_obtener_calle_equivalente['Respuesta']

            # Agrego la calle equivalente en la tabla migracion_gis
            a = {}
            a['id_calle_gis'] = calle_equivalente['id_calle_gis']
            a['id_calle_gis2'] = calle_equivalente['id_calle_gis2']
            a['nombre_calle_gis'] = calle_equivalente['nombre_calle_gis']
            a['nombre_calle_gis2'] = calle_equivalente['nombre_calle_gis2']
            a['id_ciudad_gis'] = calle_equivalente['id_ciudad_gis']
            a['id_ciudad_gis2'] = calle_equivalente['id_ciudad_gis2']
            a['puntos'] = calle_equivalente['puntos']
            try:
                agregar = self.cp.agregar('migracion_gis', **a)
                # Si no se puede agregar retorno el error
                if agregar is not True:
                    rta['Estado'] = False
                    rta['Respuesta'] = \
                        _(u'No se pudo agregar la calle equivalente. %s') \
                        % agregar
                    return rta
            except Exception, ex:
                # Si no se puede agregar retorno el error
                rta['Estado'] = False
                rta['Respuesta'] = \
                    _(u'No se pudo agregar la calle equivalente. %s') % ex
                return rta

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se guardaron las calles equivalentes')
        return rta

    def migrar_zona(self, id_zona, id_sucursal):
        '''
            Dado el id de una zona se deben migrar sus datos de mysql a
            postgres.
            Los datos originales están en la base de datos cp2000 en las tablas:
            zonas
            zonas_limites
            limiteszonas
            Actualmente necesitamos estos datos en la base de datos geodata
            en las tablas:
            zonas
            zonas_cp

            Argumento:
                id_zona (int)
                id_sucursal (int)

            Funciones:
                FuncionesGeo().generar_geometria_poligono

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o geom)
        '''

        rta = {}

        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo todos los puntos de la zona dada
        zonas_limites = self.cp.tabla('zonas_limites')
        limiteszonas = self.cp.tabla('limiteszonas')
        q_obj = session_cp.query(
            zonas_limites.IdSucursal,
            zonas_limites.IdZona,
            zonas_limites.IdLimite,
            limiteszonas.Lat1,
            limiteszonas.Lon1,
            limiteszonas.Lat2,
            limiteszonas.Lon2,
        )
        q_obj = q_obj.filter(
            and_(
                zonas_limites.IdZona == id_zona,
                zonas_limites.IdSucursal == id_sucursal,
            )
        )
        q_obj = q_obj.join((
            limiteszonas,
            zonas_limites.IdLimite == limiteszonas.IdLimite
        ))
        q_obj = q_obj.order_by(zonas_limites.IdLimite.asc())
        registros = q_obj.all()

        # Cierro la sesión con la base de datos gis
        session_cp.close()

        # Si no se encontraron registro retorno el error
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se encontraron los límites de la zona dada')
            return rta

        # Creo una lista para ir ordenando los puntos
        ordenados = []
        # Creo una bandera para saber si tengo que buscar el registro siguiente
        hay_siguiente = True
        # Comienzo por el primer registro
        registro = registros[0]
        # Agrego el primer registro a la lista ordenada
        ordenados.append(registro)

        # Recorro los registro hasta que no encuentre un siguiente
        while hay_siguiente:
            # Seteo la bandera en False para terminar el while
            hay_siguiente = False

            # Busco el siguiente registro del dado entre todos los registros
            for registro_siguiente in registros:
                # Si el registro ya está en la lista no lo vuelvo a agregar
                if registro_siguiente in ordenados:
                    continue

                # Me fijo si es el registro siguiente
                if (registro.Lat2 == registro_siguiente.Lat1
                        and registro.Lon2 == registro_siguiente.Lon1) \
                        or (registro.Lat1 == registro_siguiente.Lat2
                        and registro.Lon1 == registro_siguiente.Lon2) \
                        or (registro.Lat1 == registro_siguiente.Lat1
                        and registro.Lon1 == registro_siguiente.Lon1) \
                        or (registro.Lat2 == registro_siguiente.Lat2
                        and registro.Lon2 == registro_siguiente.Lon2):

                    # Agrego el registro en la lista
                    ordenados.append(registro_siguiente)
                    # Guardo los datos del registro encontrado
                    registro = registro_siguiente
                    # Si encuentro el siguiente vuelvo a setear la bandera
                    # en False para que me busque su siguiente
                    hay_siguiente = True
                    # Termino el for
                    break

        # Creo una lista para poder ir agregando los puntos
        geom = []
        for ordenado in ordenados:
            # Agrego los puntos para poder armar la geometría
            geom.append(
                [
                    float(ordenado.Lat1 / 3600),
                    float(ordenado.Lon1 / 3600),
                ]
            )

        # Agrego el primer punto al final también
        geom.append(
                [
                    float(ordenados[0].Lat1 / 3600),
                    float(ordenados[0].Lon1 / 3600),
                ]
        )

        # Intento obtener la geometría con los puntos obtenidos
        try:
            geom = func.st_geomfromtext(
                self.funciones_geo.generar_geometria_poligono(geom),
                4326
            )
        except:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo crear la geometría de la zona dada')
            return rta

        # Retorno la geometría de la zona
        rta['Estado'] = True
        rta['Respuesta'] = geom
        return rta

    def migrar_zonas(self):
        '''
            Esta función se encarga de recorrer todas las zonas y por cada
            una obtener su geometría y agregar en la base de datos geodata.

            Funciones:
                Migraciones().migrar_zona
                FuncionesGeo().generar_geometria_poligono

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo todas las zonas
        zonas = self.cp.tabla('zonas')
        q_obj = session_cp.query(
            zonas.IdZona,
            zonas.IdSucursal,
            zonas.Zona,
            zonas.Latitud,
            zonas.Longitud,
        )
        q_obj = q_obj.order_by(zonas.IdZona.asc())
        registros = q_obj.all()

        # Cierro la sesión con la base de datos gis
        session_cp.close()

        # Vacío la tabla zonas de geodata
        self.geodata.borrar('zonas')

        # Vacío la tabla zonas_cp de geodata
        self.geodata.borrar('zonas_cp')

        # Recorro las zonas obtenidas
        for registro in registros:
            # Por cada zona llamo a la función migrar_zona
            rta_migrar_zona = self.migrar_zona(
                registro.IdZona, registro.IdSucursal
            )
            # Si no se pudo obtener la geometría de la zona paso a la siguiente
            if rta_migrar_zona['Estado'] is False:
                continue

            # Guardo la geometría de la zona de cp2000
            geom = rta_migrar_zona['Respuesta']

            # Obtengo la parada de la zona si la tiene
            parada = None
            # Si la zona no tiene los datos Latitud o Longitud es porque
            # no tiene una parada marcada
            if registro.Latitud == 0 or registro.Latitud is None \
                    or registro.Longitud == 0 or registro.Longitud is None:
                parada = None
            else:
                try:
                    parada = [registro.Latitud / 3600, registro.Longitud / 3600]
                    # Intento obtener la geometría de la parada de la zona
                    parada = func.st_geomfromtext(
                        self.funciones_geo.generar_geometria_punto(parada),
                        4326
                    )
                except:
                    # Si no puedo obtener la geometría de la parada de la zona
                    # retorno el error
                    rta['Estado'] = False
                    rta['Respuesta'] = \
                        _(u'No se pudo crear la geometría de la parada de la zona %s') \
                        % registro.IdZona
                    return rta

            # Agrego la zona en la tabla zonas de geodata
            a = {}
            a['id'] = registro.IdZona
            a['id_tipo'] = 1
            a['descripcion'] = registro.Zona
            a['marcador'] = parada
            a['geom'] = geom
            try:
                agregar = self.geodata.agregar('zonas', **a)
                # Si no se puede agregar retorno el error
                if agregar is not True:
                    rta['Estado'] = False
                    rta['Respuesta'] = \
                        _(u'No se pudo agregar la zona. %s') % agregar
                    return rta
            except Exception, ex:
                # Si no se puede agregar retorno el error
                rta['Estado'] = False
                rta['Respuesta'] = _(u'No se pudo agregar la zona. %s') % ex
                return rta

            # Agrego la relación en la tabla zonas_cp
            a = {}
            a['id_zona'] = registro.IdZona
            a['id_zona_cp'] = registro.IdZona
            a['id_sucursal_cp'] = registro.IdSucursal
            try:
                agregar = self.geodata.agregar('zonas_cp', **a)
                # Si no se puede agregar retorno el error
                if agregar is not True:
                    rta['Estado'] = False
                    rta['Respuesta'] = \
                        _(u'No se pudo agregar la relación de la zona. %s') \
                                                                    % agregar
                    return rta
            except Exception, ex:
                # Si no se puede agregar retorno el error
                rta['Estado'] = False
                rta['Respuesta'] = \
                    _(u'No se pudo agregar la relación de la zona. %s') % ex
                return rta

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se migraron las zonas a geodata')
        return rta

    def eliminar_calles_sin_direcciones(self):
        '''
            Esta función se encarga de eliminar las calles que no figuren
            en ninguna dirección de ningún cliente.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Me conecto a la base de datos gis
        session_gis = self.gis.binded_session()

        # Obtengo todas las calles de gis
        calles = self.gis.tabla('calles')
        q_obj = session_gis.query(
            calles.idCalle, calles.idCiudad, calles.Nombre
        )
        calles = q_obj.all()

        # Cierro la sesión con la base de datos gis
        session_gis.close()

        # Si no tengo calles retorno el error
        if len(calles) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encotnraron calles')
            return rta

        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo la tabla clientes_direcciones
        clientes_direcciones = self.cp.tabla('clientes_direcciones')

        # Obtengo la tabla migracion_gis
        migracion_gis = self.cp.tabla('migracion_gis')

        # Recorro los registros obtenidos
        for calle in calles:
            # Obtengo los datos necesario de la tabla clientes_direcciones
            q_obj = session_cp.query(clientes_direcciones.IdCalle1)
            q_obj = q_obj.filter(
                and_(
                    or_(
                        clientes_direcciones.IdCalle1 == calle.idCalle,
                        clientes_direcciones.IdCalle2 == calle.idCalle,
                    ),
                    clientes_direcciones.IdCiudad == calle.idCiudad,
                )
            )
            registros = q_obj.all()

            # Si no se encontraron direcciones con la calle dada elimino
            # la calle de migracion_gis
            if len(registros) == 0:
                # Borro la calle que no tiene ninguna dirección
                print 'No se encontraron direcciones de la calle.'
                print calle

                q_obj_borrar = session_cp.query(migracion_gis.id_calle_gis)
                q_obj_borrar = q_obj_borrar.filter(
                    and_(
                        migracion_gis.id_calle_gis == calle.idCalle,
                        migracion_gis.id_ciudad_gis == calle.idCiudad,
                        migracion_gis.revisado == 0,
                    )
                )
                if q_obj_borrar.count() == 0:
                    print 'El registro ya fue borrado de la tabla migracion_gis'
                    print '***************************************************'
                    continue

                print '***************************************************'

                q_obj_borrar.delete(synchronize_session=False)

        # Commiteo a la base de datos cp2000
        session_cp.commit()

        # Cierro la sesión con la base de datos cp2000
        session_cp.close()

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se borraron las calles que no tienen direcciones')
        return rta

    def migrar_clientes_direcciones(self):
        '''
            Esta función se encarga de recorrer todas las IdCalles de los
            clientes y actualizarlas de gis a gis2
            .

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo la tabla clientes_direcciones
        clientes_direcciones = self.cp.tabla('clientes_direcciones')

        # Obtengo la tabla migracion_gis, en esta tabla se encuentran las
        # Idcalle en gis y su equivalente en gis2
        migracion_gis = self.cp.tabla('migracion_gis')

        # Obtengo todas las calles IdCalle1 y IdCalle2 de rosario
        #(IdCiudad = 1) de la tabla clientes_direcciones para reemplazarlas
        q_obj_ros = session_cp.query(
                                        clientes_direcciones.Telefono,
                                        clientes_direcciones.Direccion,
                                        clientes_direcciones.IdCiudad,
                                        clientes_direcciones.IdCalle1,
                                        clientes_direcciones.IdCalle2,
                                        clientes_direcciones.IdNodo,
                                        clientes_direcciones.Altura,
                                        clientes_direcciones.Actualizado
                                        )
        q_obj_ros = q_obj_ros.filter(
                                    or_(
                                        clientes_direcciones.IdCiudad == 324,
                                        clientes_direcciones.IdCiudad == 1
                                    )
        )
        calles_ros = q_obj_ros.all()

        # Obtengo todas las calles IdCalle1 y IdCalle2 de VGG (IdCalle = 2)
        q_obj_vgg = session_cp.query(
                                        clientes_direcciones.Telefono,
                                        clientes_direcciones.Direccion,
                                        clientes_direcciones.IdCiudad,
                                        clientes_direcciones.IdCalle1,
                                        clientes_direcciones.IdCalle2,
                                        clientes_direcciones.IdNodo,
                                        clientes_direcciones.Altura,
                                        clientes_direcciones.Actualizado
                                        )
        q_obj_vgg = q_obj_vgg.filter(
                                    or_(
                                        clientes_direcciones.IdCiudad == 1381,
                                        clientes_direcciones.IdCiudad == 2
                                    )
        )
        calles_vgg = q_obj_vgg.all()

        # Obtengo todas las  ciudades con IdCiudad igual a 3 y a 4
        q_obj_restantes = session_cp.query(
                                        clientes_direcciones.Telefono,
                                        clientes_direcciones.Direccion,
                                        clientes_direcciones.IdCiudad,
                                        clientes_direcciones.IdCalle1,
                                        clientes_direcciones.IdCalle2,
                                        clientes_direcciones.IdNodo,
                                        clientes_direcciones.Altura,
                                        clientes_direcciones.Actualizado
                                        )
        q_obj_restantes = q_obj_restantes.filter(
                                    or_(
                                        clientes_direcciones.IdCiudad == 3,
                                        clientes_direcciones.IdCiudad == 4,
                                    )
        )
        calles_restantes = q_obj_restantes.all()

        #Comparo para actualizar las IdCalle de clientes_direccion de
        #de rosario
        for calle_ros in calles_ros:
            dic = {}
            dic['Actualizado'] = 0
            dic['IdCiudad'] = 324

            if calle_ros.IdNodo is not None\
                    and calle_ros.IdCalle1 is None\
                    and calle_ros.IdCalle2 is None:
                dic['Actualizado'] = 3
            # Si calle_ros.IdCiudad = 324 se pidio por PTT
            elif calle_ros.IdCiudad == 324:
                dic['Actualizado'] = 3
            else:
                # Busco la calle coincidente con calle1 en migracion_gis
                q_obj1 = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj1 = q_obj1.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 1,
                            migracion_gis.id_calle_gis == calle_ros.IdCalle1,
                        )
                )

                registro1 = q_obj1.first()

                if registro1 is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic['IdCalle1'] = registro1.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    dic['Actualizado'] = 2
                    if calle_ros.IdCalle2 is None\
                         or calle_ros.IdCalle2 == 0\
                         or calle_ros.Altura is not None:
                        dic['Actualizado'] = 3

                # Busco la calle coincidente con calle2 en migracion_gis
                q_obj2 = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj2 = q_obj2.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 1,
                            migracion_gis.id_calle_gis == calle_ros.IdCalle2,
                        )
                )

                registro2 = q_obj2.first()

                if registro2 is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic['IdCalle2'] = registro2.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    if dic['Actualizado'] == 2:
                        dic['Actualizado'] = 3
                    elif dic['Actualizado'] == 0\
                                 and calle_ros.IdCalle1 is None:
                        dic['Actualizado'] = 3
                    elif dic['Actualizado'] == 0:
                        dic['Actualizado'] = 1

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up = session_cp.query(
                clientes_direcciones.Telefono,
                clientes_direcciones.Direccion,
                clientes_direcciones.IdCiudad,
                clientes_direcciones.IdCalle1,
                clientes_direcciones.IdCalle2
                )
            q_obj_up = q_obj_up.filter(
                    and_(
                      clientes_direcciones.Telefono == calle_ros.Telefono,
                      clientes_direcciones.Direccion == calle_ros.Direccion,
                          or_(
                              clientes_direcciones.IdCiudad == 1,
                              clientes_direcciones.IdCiudad == 324,
                          )
                    )
            )
            q_obj_up.update(dic, synchronize_session=False)

        # Ahora migramos las calles de VGG
        for calle_vgg in calles_vgg:
            dic_vgg = {}
            dic_vgg['Actualizado'] = 0
            dic_vgg['IdCiudad'] = 1381
            if calle_vgg.IdNodo is not None\
                    and calle_vgg.IdCalle1 is None\
                    and calle_vgg.IdCalle2 is None:
                dic_vgg['Actualizado'] = 3
            # Si calle_vgg.IdCiudad == 1381 se pidio por PTT
            elif calle_vgg.IdCiudad == 1381:
                dic_vgg['Actualizado'] = 3
            else:
                # Busco la calle coincidente con calle1 en migracion_gis
                q_obj1_vgg = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj1_vgg = q_obj1_vgg.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 2,
                            migracion_gis.id_calle_gis == calle_vgg.IdCalle1,
                        )
                )

                registro1_vgg = q_obj1_vgg.first()

                if registro1_vgg is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic_vgg['IdCalle1'] = registro1_vgg.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    dic_vgg['Actualizado'] = 2
                    if calle_vgg.IdCalle2 is None\
                         or calle_vgg.IdCalle2 == 0\
                         or calle_vgg.Altura is not None:
                        dic_vgg['Actualizado'] = 3

                # Busco la calle coincidente con calle2 en migracion_gis
                q_obj2_vgg = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj2_vgg = q_obj2_vgg.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 2,
                            migracion_gis.id_calle_gis == calle_vgg.IdCalle2,
                        )
                )

                registro2_vgg = q_obj2_vgg.first()

                if registro2_vgg is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic_vgg['IdCalle2'] = registro2_vgg.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    if dic_vgg['Actualizado'] == 2:
                        dic_vgg['Actualizado'] = 3
                    elif dic_vgg['Actualizado'] == 0\
                                 and calle_vgg.IdCalle1 is None:
                        dic_vgg['Actualizado'] = 3
                    elif dic_vgg['Actualizado'] == 0:
                        dic_vgg['Actualizado'] = 1

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up2 = session_cp.query(
                clientes_direcciones.Telefono,
                clientes_direcciones.Direccion,
                clientes_direcciones.IdCiudad,
                clientes_direcciones.IdCalle1,
                clientes_direcciones.IdCalle2
                )
            q_obj_up2 = q_obj_up2.filter(
                    and_(
                      clientes_direcciones.Telefono == calle_vgg.Telefono,
                      clientes_direcciones.Direccion == calle_vgg.Direccion,
                          or_(
                              clientes_direcciones.IdCiudad == 2,
                              clientes_direcciones.IdCiudad == 1381,
                          )
                        )
            )
            q_obj_up2.update(dic_vgg, synchronize_session=False)

        # Migramos ciudades restanes (Alvear y Baigorria)
        for calle_restante in calles_restantes:
            dic_restante = {}
            # Si IdCiudad = 3 es de Baigorria y si IdCiudad = 4 es Alvear
            if calle_restante.IdCiudad == 3:
                dic_restante['IdCiudad'] = 3772
            elif calle_restante.IdCiudad == 4:
                dic_restante['IdCiudad'] = 4142
            dic_restante['IdCalle1'] = None
            dic_restante['IdCalle2'] = None

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up3 = session_cp.query(
                clientes_direcciones.Telefono,
                clientes_direcciones.Direccion,
                clientes_direcciones.IdCiudad,
                clientes_direcciones.IdCalle1,
                clientes_direcciones.IdCalle2
                )
            q_obj_up3 = q_obj_up3.filter(
                and_(
                    clientes_direcciones.Telefono == calle_restante.Telefono,
                    clientes_direcciones.Direccion == calle_restante.Direccion,
                          or_(
                              clientes_direcciones.IdCiudad == 3,
                              clientes_direcciones.IdCiudad == 4,
                          )
                    )
            )
            q_obj_up3.update(dic_restante, synchronize_session=False)

        session_cp.commit()
        session_cp.close()

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se cambian las calles')
        return rta

    def migrar_serviciosanticipadosdetalle(self):
        '''
            Esta función se encarga de recorrer todas las direcciones de los
            clientes y actualizarlas de gis a gis2 de la base de datos cp2000
            en la tabla serviciosanticipadosdetalle
            .

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}
        # Me conecto a la base de datos cp2000bkup
        session_cp = self.cp.binded_session()

        # Obtengo la tabla clientes_direcciones
        serviciosanticipadosdetalle = \
                            self.cp.tabla('serviciosanticipadosdetalle')

        # Obtengo la tabla migracion_gis
        migracion_gis = self.cp.tabla('migracion_gis')

        # Obtengo todas las calles IdCalle1 y IdCalle2 de rosario
        q_obj_ros = session_cp.query(
                                    serviciosanticipadosdetalle.Telefono,
                                    serviciosanticipadosdetalle.Direccion,
                                    serviciosanticipadosdetalle.IdCiudad,
                                    serviciosanticipadosdetalle.IdCalle1,
                                    serviciosanticipadosdetalle.IdCalle2,
                                    serviciosanticipadosdetalle.IdNodo,
                                    serviciosanticipadosdetalle.Altura,
                                    serviciosanticipadosdetalle.Actualizado
                                        )
        q_obj_ros = q_obj_ros.filter(
                                serviciosanticipadosdetalle.IdCiudad == '1'
        )
        calles_ros = q_obj_ros.all()

        # Obtengo todas las calles IdCalle1 y IdCalle2 de VGG
        q_obj_vgg = session_cp.query(
                                    serviciosanticipadosdetalle.Telefono,
                                    serviciosanticipadosdetalle.Direccion,
                                    serviciosanticipadosdetalle.IdCiudad,
                                    serviciosanticipadosdetalle.IdCalle1,
                                    serviciosanticipadosdetalle.IdCalle2,
                                    serviciosanticipadosdetalle.IdNodo,
                                    serviciosanticipadosdetalle.Altura,
                                    serviciosanticipadosdetalle.Actualizado
                                        )
        q_obj_vgg = q_obj_vgg.filter(
                                    serviciosanticipadosdetalle.IdCiudad == '2'
        )
        calles_vgg = q_obj_vgg.all()

        # Obtengo todas las  ciudades con IdCiudad igual a 3 y a 4
        q_obj_restantes = session_cp.query(
                                        serviciosanticipadosdetalle.Telefono,
                                        serviciosanticipadosdetalle.Direccion,
                                        serviciosanticipadosdetalle.IdCiudad,
                                        serviciosanticipadosdetalle.IdCalle1,
                                        serviciosanticipadosdetalle.IdCalle2,
                                        serviciosanticipadosdetalle.IdNodo,
                                        serviciosanticipadosdetalle.Altura,
                                        serviciosanticipadosdetalle.Actualizado
                                        )
        q_obj_restantes = q_obj_restantes.filter(
                                or_(
                                    serviciosanticipadosdetalle.IdCiudad == '3',
                                    serviciosanticipadosdetalle.IdCiudad == '4',
                                )
        )
        calles_restantes = q_obj_restantes.all()

        #Comparo para actualizar las IdCalle de serviciosanticipadosdetalle
        #de rosario
        for calle_ros in calles_ros:
            dic = {}
            dic['Actualizado'] = 0
            dic['IdCiudad'] = '324'
            if calle_ros.IdNodo is not None\
                    and calle_ros.IdCalle1 is None\
                    and calle_ros.IdCalle2 is None:
                dic['Actualizado'] = 3
            # Si calle_ros.IdCalle1 > 3000 se pidio por PTT
            elif calle_ros.IdCiudad == '324':
                dic['Actualizado'] = 3
            else:
                # Busco la calle coincidente con calle1 en migracion_gis
                q_obj1 = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj1 = q_obj1.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 1,
                            migracion_gis.id_calle_gis == calle_ros.IdCalle1,
                        )
                )

                registro1 = q_obj1.first()

                if registro1 is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic['IdCalle1'] = registro1.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    dic['Actualizado'] = 2
                    if calle_ros.IdCalle2 is None\
                         or calle_ros.IdCalle2 == 0\
                         or calle_ros.Altura is not None:
                        dic['Actualizado'] = 3

                # Busco la calle coincidente con calle2 en migracion_gis
                q_obj2 = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj2 = q_obj2.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 1,
                            migracion_gis.id_calle_gis == calle_ros.IdCalle2,
                        )
                )

                registro2 = q_obj2.first()

                if registro2 is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic['IdCalle2'] = registro2.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    if dic['Actualizado'] == 2:
                        dic['Actualizado'] = 3
                    elif dic['Actualizado'] == 0\
                                 and calle_ros.IdCalle1 is None:
                        dic['Actualizado'] = 3
                    elif dic['Actualizado'] == 0:
                        dic['Actualizado'] = 1

            # Realizamos el update en la tabla serviciosanticipadosdetalle
            q_obj_up = session_cp.query(
                serviciosanticipadosdetalle.Telefono,
                serviciosanticipadosdetalle.Direccion,
                serviciosanticipadosdetalle.IdCiudad,
                serviciosanticipadosdetalle.IdCalle1,
                serviciosanticipadosdetalle.IdCalle2
                )
            q_obj_up = q_obj_up.filter(
                    and_(
                      serviciosanticipadosdetalle.Telefono ==
                                                          calle_ros.Telefono,
                      serviciosanticipadosdetalle.Direccion ==
                                                          calle_ros.Direccion,
                      serviciosanticipadosdetalle.IdCiudad == '1',
                        )
            )
            q_obj_up.update(dic, synchronize_session=False)

        #Ahora migramos las calles de VGG
        for calle_vgg in calles_vgg:
            dic_vgg = {}
            dic_vgg['Actualizado'] = 0
            dic_vgg['IdCiudad'] = '1381'
            if calle_vgg.IdNodo is not None\
                    and calle_vgg.IdCalle1 is None\
                    and calle_vgg.IdCalle2 is None:
                dic_vgg['Actualizado'] = 3
            # Si calle_vgg.IdCiudad == 1381 se pidio por PTT
            elif calle_vgg.IdCiudad == '1381':
                dic_vgg['Actualizado'] = 3
            else:
                # Busco la calle coincidente con calle1 en migracion_gis
                q_obj1_vgg = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj1_vgg = q_obj1_vgg.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 2,
                            migracion_gis.id_calle_gis == calle_vgg.IdCalle1,
                        )
                )

                registro1_vgg = q_obj1_vgg.first()

                if registro1_vgg is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic_vgg['IdCalle1'] = registro1_vgg.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    dic_vgg['Actualizado'] = 2
                    if calle_vgg.IdCalle2 is None\
                         or calle_vgg.IdCalle2 == 0\
                         or calle_vgg.Altura is not None:
                        dic_vgg['Actualizado'] = 3
                # Busco la calle coincidente con calle2 en migracion_gis
                q_obj2_vgg = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj2_vgg = q_obj2_vgg.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 2,
                            migracion_gis.id_calle_gis == calle_vgg.IdCalle2,
                        )
                )

                registro2_vgg = q_obj2_vgg.first()

                if registro2_vgg is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic_vgg['IdCalle2'] = registro2_vgg.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    if dic_vgg['Actualizado'] == 2:
                        dic_vgg['Actualizado'] = 3
                    elif dic_vgg['Actualizado'] == 0\
                                 and calle_vgg.IdCalle1 is None:
                        dic_vgg['Actualizado'] = 3
                    elif dic_vgg['Actualizado'] == 0:
                        dic_vgg['Actualizado'] = 1

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up2 = session_cp.query(
                serviciosanticipadosdetalle.Telefono,
                serviciosanticipadosdetalle.Direccion,
                serviciosanticipadosdetalle.IdCiudad,
                serviciosanticipadosdetalle.IdCalle1,
                serviciosanticipadosdetalle.IdCalle2
                )
            q_obj_up2 = q_obj_up2.filter(
                    and_(
                      serviciosanticipadosdetalle.Telefono ==
                                                           calle_vgg.Telefono,
                      serviciosanticipadosdetalle.Direccion ==
                                                           calle_vgg.Direccion,
                      serviciosanticipadosdetalle.IdCiudad == '2',
                        )
            )
            q_obj_up2.update(dic_vgg, synchronize_session=False)

        # Migramos ciudades restanes (Alvear y Baigorria)
        for calle_restante in calles_restantes:
            dic_restante = {}
            # Si IdCiudad = 3 es de Baigorria y si IdCiudad = 4 es Alvear
            if calle_restante.IdCiudad == '3':
                dic_restante['IdCiudad'] = '3772'
            elif calle_restante.IdCiudad == '4':
                dic_restante['IdCiudad'] = '4142'
            dic_restante['IdCalle1'] = None
            dic_restante['IdCalle2'] = None

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up3 = session_cp.query(
                serviciosanticipadosdetalle.Telefono,
                serviciosanticipadosdetalle.Direccion,
                serviciosanticipadosdetalle.IdCiudad,
                serviciosanticipadosdetalle.IdCalle1,
                serviciosanticipadosdetalle.IdCalle2
                )
            q_obj_up3 = q_obj_up3.filter(
                and_(
                    serviciosanticipadosdetalle.Telefono ==
                                                    calle_restante.Telefono,
                    serviciosanticipadosdetalle.Direccion ==
                                                    calle_restante.Direccion,
                          or_(
                              serviciosanticipadosdetalle.IdCiudad == '3',
                              serviciosanticipadosdetalle.IdCiudad == '4',
                          )
                    )
            )
            q_obj_up3.update(dic_restante, synchronize_session=False)

        session_cp.commit()
        session_cp.close()

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se cambian las calles')
        return rta

    def migrar_clientes(self):
        '''
            Esta función se encarga de recorrer todas las direcciones de los
            clientes y actualizarlas de gis a gis2 de la base de datos cp2000
            en la tabla clientes
            .

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo la tabla clientes
        clientes = self.cp.tabla('clientes')

        # Obtengo la tabla migracion_gis
        migracion_gis = self.cp.tabla('migracion_gis')

        # Obtengo todas las calles IdCalle1 y IdCalle2 de rosario
        q_obj_ros = session_cp.query(
                                    clientes.Telefono,
                                    clientes.Direccion,
                                    clientes.IdCiudad,
                                    clientes.IdCalle1,
                                    clientes.IdCalle2,
                                    clientes.IdNodo,
                                    clientes.Altura,
                                    clientes.Actualizado
                                        )
        q_obj_ros = q_obj_ros.filter(
                                    or_(
                                        clientes.IdCiudad == '324',
                                        clientes.IdCiudad == '1',
                                    )
        )
        calles_ros = q_obj_ros.all()

        # Obtengo todas las calles IdCalle1 y IdCalle2 de VGG
        q_obj_vgg = session_cp.query(
                                    clientes.Telefono,
                                    clientes.Direccion,
                                    clientes.IdCiudad,
                                    clientes.IdCalle1,
                                    clientes.IdCalle2,
                                    clientes.IdNodo,
                                    clientes.Altura,
                                    clientes.Actualizado
                                        )
        q_obj_vgg = q_obj_vgg.filter(
                                    or_(
                                        clientes.IdCiudad == '1381',
                                        clientes.IdCiudad == '2',
                                    )
        )
        calles_vgg = q_obj_vgg.all()

        # Obtengo todas las  ciudades con IdCiudad igual a 3 y a 4
        q_obj_restantes = session_cp.query(
                                        clientes.Telefono,
                                        clientes.Direccion,
                                        clientes.IdCiudad,
                                        clientes.IdCalle1,
                                        clientes.IdCalle2,
                                        clientes.IdNodo,
                                        clientes.Altura,
                                        clientes.Actualizado
                                        )
        q_obj_restantes = q_obj_restantes.filter(
                                    or_(
                                        clientes.IdCiudad == '3',
                                        clientes.IdCiudad == '4',
                                    )
        )
        calles_restantes = q_obj_restantes.all()

        #Comparo para actualizar las IdCalle de clientes de rosario
        for calle_ros in calles_ros:
            dic = {}
            dic['Actualizado'] = 0
            dic['IdCiudad'] = '324'
            if calle_ros.IdNodo is not None\
                    and calle_ros.IdCalle1 is None\
                    and calle_ros.IdCalle2 is None:
                dic['Actualizado'] = 3
            # Si calle_ros.IdCiudad == '324' se pidio por PTT
            elif calle_ros.IdCiudad == '324':
                dic['Actualizado'] = 3
            else:
                # Busco la calle coincidente con calle1 en migracion_gis
                q_obj1 = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj1 = q_obj1.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 1,
                            migracion_gis.id_calle_gis == calle_ros.IdCalle1,
                        )
                )

                registro1 = q_obj1.first()

                if registro1 is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic['IdCalle1'] = registro1.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    dic['Actualizado'] = 2
                    if calle_ros.IdCalle2 is None\
                         or calle_ros.IdCalle2 == 0\
                         or calle_ros.Altura is not None:
                        dic['Actualizado'] = 3

                # Busco la calle coincidente con calle2 en migracion_gis
                q_obj2 = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj2 = q_obj2.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 1,
                            migracion_gis.id_calle_gis == calle_ros.IdCalle2,
                        )
                )

                registro2 = q_obj2.first()

                if registro2 is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic['IdCalle2'] = registro2.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    if dic['Actualizado'] == 2:
                        dic['Actualizado'] = 3
                    elif dic['Actualizado'] == 0\
                                 and calle_ros.IdCalle1 is None \
                                 or calle_ros.IdCalle1 == 0:
                        dic['Actualizado'] = 3
                    elif dic['Actualizado'] == 0:
                        dic['Actualizado'] = 1

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up = session_cp.query(
                clientes.Telefono,
                clientes.Direccion,
                clientes.IdCiudad,
                clientes.IdCalle1,
                clientes.IdCalle2
                )
            q_obj_up = q_obj_up.filter(
                    and_(
                      clientes.Telefono == calle_ros.Telefono,
                      clientes.Direccion == calle_ros.Direccion,
                          or_(
                              clientes.IdCiudad == '1',
                              clientes.IdCiudad == '324',
                          )
                        )
            )
            q_obj_up.update(dic, synchronize_session=False)

        # Ahora migramos las calles de VGG
        for calle_vgg in calles_vgg:
            dic_vgg = {}
            dic_vgg['Actualizado'] = 0
            dic_vgg['IdCiudad'] = '1381'
            if calle_vgg.IdNodo is not None\
                    and calle_vgg.IdCalle1 is None\
                    and calle_vgg.IdCalle2 is None:
                dic_vgg['Actualizado'] = 3
            # Si calle_vgg.IdCiudad == '1381' se pidio por PTT
            elif calle_vgg.IdCiudad == '1381':
                dic_vgg['Actualizado'] = 3
            else:
                # Busco la calle coincidente con calle1 en migracion_gis
                q_obj1_vgg = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj1_vgg = q_obj1_vgg.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 2,
                            migracion_gis.id_calle_gis == calle_vgg.IdCalle1,
                        )
                )

                registro1_vgg = q_obj1_vgg.first()

                if registro1_vgg is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic_vgg['IdCalle1'] = registro1_vgg.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    dic_vgg['Actualizado'] = 2
                    if calle_vgg.IdCalle2 is None\
                         or calle_vgg.IdCalle2 == 0\
                         or calle_vgg.Altura is not None:
                        dic_vgg['Actualizado'] = 3

                # Busco la calle coincidente con calle2 en migracion_gis
                q_obj2_vgg = session_cp.query(migracion_gis.id_calle_gis,
                                            migracion_gis.id_calle_gis2,
                                            migracion_gis.nombre_calle_gis,
                                            migracion_gis.nombre_calle_gis2)
                q_obj2_vgg = q_obj2_vgg.filter(
                        and_(
                            migracion_gis.revisado == 1,
                            migracion_gis.id_ciudad_gis == 2,
                            migracion_gis.id_calle_gis == calle_vgg.IdCalle2,
                        )
                )

                registro2_vgg = q_obj2_vgg.first()

                if registro2_vgg is not None:
                    # Se guarda id_calle gis2 para reemplazar
                    dic_vgg['IdCalle2'] = registro2_vgg.id_calle_gis2
                    # Cambiamos el estado del campo Actualizado
                    if dic_vgg['Actualizado'] == 2:
                        dic_vgg['Actualizado'] = 3
                    elif dic_vgg['Actualizado'] == 0\
                                 and calle_vgg.IdCalle1 is None \
                                 or calle_ros.IdCalle1 == 0:
                        dic_vgg['Actualizado'] = 3
                    elif dic_vgg['Actualizado'] == 0:
                        dic_vgg['Actualizado'] = 1

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up2 = session_cp.query(
                clientes.Telefono,
                clientes.Direccion,
                clientes.IdCiudad,
                clientes.IdCalle1,
                clientes.IdCalle2
                )
            q_obj_up2 = q_obj_up2.filter(
                    and_(
                      clientes.Telefono == calle_vgg.Telefono,
                      clientes.Direccion == calle_vgg.Direccion,
                          or_(
                              clientes.IdCiudad == '2',
                              clientes.IdCiudad == '1381',
                          )
                        )
            )
            q_obj_up2.update(dic_vgg, synchronize_session=False)

        # Migramos ciudades restanes (Alvear y Baigorria)
        for calle_restante in calles_restantes:
            dic_restante = {}
            # Si IdCiudad = 3 es de Baigorria y si IdCiudad = 4 es Alvear
            if calle_restante.IdCiudad == '3':
                dic_restante['IdCiudad'] = '3772'
            elif calle_restante.IdCiudad == '4':
                dic_restante['IdCiudad'] = '4142'
            dic_restante['IdCalle1'] = None
            dic_restante['IdCalle2'] = None

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up3 = session_cp.query(
                                        clientes.Telefono,
                                        clientes.Direccion,
                                        clientes.IdCiudad,
                                        clientes.IdCalle1,
                                        clientes.IdCalle2
                )
            q_obj_up3 = q_obj_up3.filter(
                and_(
                    clientes.Telefono == calle_restante.Telefono,
                    clientes.Direccion == calle_restante.Direccion,
                          or_(
                              clientes.IdCiudad == '3',
                              clientes.IdCiudad == '4',
                          )
                    )
            )
            q_obj_up3.update(dic_restante, synchronize_session=False)

        session_cp.commit()
        session_cp.close()

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se cambian las calles')
        return rta

    def anular_calles_desconocidas(self):
        '''
            Esta función se encarga de reeemplazar los IdCalle no encontrados
            y reemplazarlas por null.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}
        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo la tabla clientes
        clientes = self.cp.tabla('clientes')

        # Obtengo la tabla serviciosanticipadosdetalle
        serviciosanticipadosdetalle = \
                            self.cp.tabla('serviciosanticipadosdetalle')

        # Obtengo la tabla clientes_direcciones
        clientes_direcciones = self.cp.tabla('clientes_direcciones')

        # Obtengo todas las calles IdCalle1 y IdCalle2 de tabla clientes
        q_obj_clianu = session_cp.query(
                                    clientes.Telefono,
                                    clientes.Direccion,
                                    clientes.IdCiudad,
                                    clientes.IdCalle1,
                                    clientes.IdCalle2,
                                    clientes.IdNodo,
                                    clientes.Altura,
                                    clientes.Actualizado
                                        )
        q_obj_clianu = q_obj_clianu.filter(
                                    or_(
                                        clientes.Actualizado == 0,
                                        clientes.Actualizado == 1,
                                        clientes.Actualizado == 2,
                                    )
        )
        calles_clianu = q_obj_clianu.all()

        for calle_clianu in calles_clianu:
            dicclianu = {}
            if calle_clianu.Actualizado == 1:
                dicclianu['IdCalle1'] = None
            elif calle_clianu.Actualizado == 2:
                dicclianu['IdCalle2'] = None
            elif calle_clianu.Actualizado == 0:
                dicclianu['IdCalle1'] = None
                dicclianu['IdCalle2'] = None

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up = session_cp.query(
                clientes.Telefono,
                clientes.Direccion,
                clientes.IdCiudad,
                clientes.IdCalle1,
                clientes.IdCalle2
                )
            q_obj_up = q_obj_up.filter(
                    and_(
                          clientes.Telefono == calle_clianu.Telefono,
                          clientes.Direccion == calle_clianu.Direccion,
                    )
            )
            q_obj_up.update(dicclianu, synchronize_session=False)

        # Obtengo las calles IdCalle1 y IdCalle2 de tabla
        #                                           serviciosanticipadosdetalle
        q_obj_servantanu = session_cp.query(
                                    serviciosanticipadosdetalle.Telefono,
                                    serviciosanticipadosdetalle.Direccion,
                                    serviciosanticipadosdetalle.IdCiudad,
                                    serviciosanticipadosdetalle.IdCalle1,
                                    serviciosanticipadosdetalle.IdCalle2,
                                    serviciosanticipadosdetalle.IdNodo,
                                    serviciosanticipadosdetalle.Altura,
                                    serviciosanticipadosdetalle.Actualizado
                                        )
        q_obj_servantanu = q_obj_servantanu.filter(
                                    or_(
                                serviciosanticipadosdetalle.Actualizado == 0,
                                serviciosanticipadosdetalle.Actualizado == 1,
                                serviciosanticipadosdetalle.Actualizado == 2,
                                    )
        )
        calles_servantanu = q_obj_servantanu.all()

        for calle_servantanu in calles_servantanu:
            dicservantanu = {}
            if calle_servantanu.Actualizado == 1:
                dicservantanu['IdCalle1'] = None
            elif calle_servantanu.Actualizado == 2:
                dicservantanu['IdCalle2'] = None
            elif calle_servantanu.Actualizado == 0:
                dicservantanu['IdCalle1'] = None
                dicservantanu['IdCalle2'] = None

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up2 = session_cp.query(
                serviciosanticipadosdetalle.Telefono,
                serviciosanticipadosdetalle.Direccion,
                serviciosanticipadosdetalle.IdCiudad,
                serviciosanticipadosdetalle.IdCalle1,
                serviciosanticipadosdetalle.IdCalle2
                )
            q_obj_up2 = q_obj_up2.filter(
                    and_(
                          serviciosanticipadosdetalle.Telefono ==
                                                    calle_servantanu.Telefono,
                          serviciosanticipadosdetalle.Direccion ==
                                                    calle_servantanu.Direccion,
                    )
            )
            q_obj_up2.update(dicservantanu, synchronize_session=False)

        # Obtengo todas las calles IdCalle1 y IdCalle2 de tabla clientes
        q_obj_clidiranu = session_cp.query(
                                    clientes_direcciones.Telefono,
                                    clientes_direcciones.Direccion,
                                    clientes_direcciones.IdCiudad,
                                    clientes_direcciones.IdCalle1,
                                    clientes_direcciones.IdCalle2,
                                    clientes_direcciones.IdNodo,
                                    clientes_direcciones.Altura,
                                    clientes_direcciones.Actualizado
                                        )
        q_obj_clidiranu = q_obj_clidiranu.filter(
                                    or_(
                                        clientes_direcciones.Actualizado == 0,
                                        clientes_direcciones.Actualizado == 1,
                                        clientes_direcciones.Actualizado == 2,
                                    )
        )
        calles_clidiranu = q_obj_clidiranu.all()

        for calle_clidiranu in calles_clidiranu:
            dicclidiranu = {}
            if calle_clidiranu.Actualizado == 1:
                dicclidiranu['IdCalle1'] = None
            elif calle_clidiranu.Actualizado == 2:
                dicclidiranu['IdCalle2'] = None
            elif calle_clidiranu.Actualizado == 0:
                dicclidiranu['IdCalle1'] = None
                dicclidiranu['IdCalle2'] = None

            # Realizamos el update en la tabla clientes_direcciones
            q_obj_up3 = session_cp.query(
                clientes_direcciones.Telefono,
                clientes_direcciones.Direccion,
                clientes_direcciones.IdCiudad,
                clientes_direcciones.IdCalle1,
                clientes_direcciones.IdCalle2
                )
            q_obj_up3 = q_obj_up3.filter(
                    and_(
                         clientes_direcciones.Telefono ==
                                                    calle_clidiranu.Telefono,
                        clientes_direcciones.Direccion ==
                                                    calle_clidiranu.Direccion,
                    )
            )
            q_obj_up3.update(dicclidiranu, synchronize_session=False)

        session_cp.commit()
        session_cp.close()

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se cambian las calles')
        return rta

    def eliminar_clientes(self):
        '''
            Esta función se encarga de eliminar los clientes mal cargados en
            la tabla clientes clintes_direcciones y serviciosanticipadosdetalles
            y que no se encuentren en la tabla
            servicioanticipadosdetalle.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}
        # Me conecto a la base de datos cp2000
        session_cp = self.cp.binded_session()

        # Obtengo la tabla clientes_direcciones
        clientes_direcciones = self.cp.tabla('clientes_direcciones')

        # Obtengo la tabla clientes
        clientes = self.cp.tabla('clientes')

        # Obtengo la tabla serviciosanticipadosdetalle
        serviciosanticipadosdetalle = \
                            self.cp.tabla('serviciosanticipadosdetalle')

        # Obtengo todas las subquery para filtrar clientes en anticipados

        sub_q_obj = session_cp.query(
                                    serviciosanticipadosdetalle.Telefono,
                                        )
        sub_q_obj = sub_q_obj.subquery()

        # Eliminamos los registros de la tabla clientes que creemos conveniente
        q_obj = session_cp.query(
                                    clientes.Telefono,
                                    )
        q_obj = q_obj.filter(
                            and_(
                                or_(
                                    clientes.Actualizado == 0,
                                    clientes.Actualizado == 1,
                                    clientes.Actualizado == 2,
                                ),
                                clientes.IdCiudad != '1',
                                clientes.IdCiudad != '2',
                                clientes.IdCiudad != '3',
                                clientes.IdCiudad != '4',
                                clientes.IdCiudad != '324',
                                clientes.IdCiudad != '1381',
                                clientes.IdCiudad != '3772',
                                clientes.IdCiudad != '4142',
                                not_(clientes.Telefono.in_(sub_q_obj)),
                            )
        )
        print 'cantidad de registros a borrados'
        print q_obj.count()

        q_obj.delete(synchronize_session=False)

        # Eliminamos los registros de la tabla clientes_direcciones
        #                                              que creemos conveniente.
        q_obj1 = session_cp.query(
                                    clientes_direcciones.Telefono,
                                    )
        q_obj1 = q_obj1.filter(
                            or_(
                                clientes_direcciones.Actualizado == 0,
                                clientes_direcciones.Actualizado == 1,
                                clientes_direcciones.Actualizado == 2,
                                )
        )
        print 'cantidad de registros borrados'
        print q_obj1.count()

        q_obj1.delete(synchronize_session=False)

        session_cp.commit()
        session_cp.close()

        # Si todo salió bien retorno un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Se cambian las calles')
        return rta

    def migrar_segmentos_osm_gis2(self, lat1, lon1, lat2, lon2,
                                                    id_calle_nombre, bd_osm):
        '''
            Esta función se encarga de insertar los segmentos de una ciudad
            de OSM a gis2.

            Argumento:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                id_calle_nombre (int)

            Funciones:
                Migraciones().calles_ciudad
                Migraciones().comparar_calles

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''
        rta = {}
        # Llamo a la función calles_ciudad para traer todos los segmentos de
        # la zona en la BD osm
        rta_calles_ciudad_osm = self.calles_ciudad_osm(
            lat1, lon1, lat2, lon2, bd_osm
        )
        # Si no se encuentra ningún segmento retorno el mensaje
        if rta_calles_ciudad_osm['Estado'] is False:
            return rta_calles_ciudad_osm

        # Guardo los segmentos obtenidos de la ciudad
        lista_calles = rta_calles_ciudad_osm['Respuesta']

        # Llamo a la función comparar_calles pasandole la lista de calles
        rta_agregar_segmentos_osm = self.agregar_segmentos_osm(
            lista_calles, lat1, lon1, lat2, lon2, id_calle_nombre
        )
        if rta_agregar_segmentos_osm is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron segmentos en la zona dada')
            return rta
        else:
            rta['Estado'] = True
            rta['Respuesta'] = _(u'Se encontraron segmentos en la zona dada')
            return rta

    def agregar_segmentos_osm(self, lista_calles, lat1, lon1, lat2, lon2,
                                                            id_calle_nombre):
        '''
            Esta función se encarga de comparar los segmentos de una ciudad
            OSM con los existentes en gis2 e insertar los faltantes.

            Argumento:
                lista_calles (lista de tuple)
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                id_calle_nombre (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}
        # Abro la sesión con la BD con gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles
        calles = self.gis2.tabla('calles')

        calles_osm_a_agregar = []
        lista_osm_dos_puntos = []
        cont_gid = 1
        #Recorremos los segmentos osm de la lista
        for calle_osm_multi in lista_calles:
            #Separamos en puntos el segmento osm para poder formar
            #segmentos de dos puntos
            conj_linea = self.funciones_geo.procesar_geometria(
                                                calle_osm_multi['geom_text'])
            #Recorremos los puntos para formar los segmentos de dos puntos
            for i in range(len(conj_linea) - 1):
                geom_text_dos_puntos = None
                geom_text_dos_puntos = (conj_linea[i], conj_linea[i + 1])

                lat_punto1 = geom_text_dos_puntos[0][1]
                long_punto1 = geom_text_dos_puntos[0][0]

                lat_punto2 = geom_text_dos_puntos[1][1]
                long_punto2 = geom_text_dos_puntos[1][0]
                #Damos vueltas los puntos ya que la función procesar geometria
                #las trae al reves
                geom_text_dos_puntos_inv = ([lat_punto1, long_punto1],
                                            [lat_punto2, long_punto2])
                #Recuperamos el nombre del segmento osm
                name_osm_dos_puntos = calle_osm_multi['name']
                #Le asignamos un id nuevo
                gid_osm_dos_puntos = cont_gid
                cont_gid = cont_gid + 1
                #Generamos el formato para poder llevarlo a geom
                multi_dos_puntos = self.funciones_geo.generar_geometria_linea(
                                                    geom_text_dos_puntos_inv)
                #Pasamos al formato geom
                geom_dos_puntos = func.st_geomfromtext(multi_dos_puntos, 4326)

                datos_osm = {}
                datos_osm['gid'] = gid_osm_dos_puntos
                datos_osm['name'] = name_osm_dos_puntos
                datos_osm['geom_text'] = multi_dos_puntos
                datos_osm['geom'] = geom_dos_puntos
                datos_osm['osm_id'] = gid_osm_dos_puntos
                #Obtenemos la lista de todos los segementos formados por
                #dos puntos
                lista_osm_dos_puntos.append(datos_osm)
        #Recorremos cada segemnto para compararlos con los de gis2
        for calle_osm in lista_osm_dos_puntos:
            print 'calle_osm'
            print calle_osm

            #Este dato se utilizara si osm esta entre los puntos de seg. gis2
            osm_entre_puntos_gis2 = False

            #Obtenemos los puntos del osm para poder calcular las distancias
            conj_linea = self.funciones_geo.procesar_geometria(
                                                        calle_osm['geom_text'])

            linea_osm = (conj_linea[0], conj_linea[len(conj_linea) - 1])

            long_punto_uno_osm = linea_osm[0][0]
            lat_punto_uno_osm = linea_osm[0][1]

            long_punto_dos_osm = linea_osm[1][0]
            lat_punto_dos_osm = linea_osm[1][1]

            #Invertir coordenadas ya que la función procesar geometria me las
            #devuelve invertida
            linea_osm_inv = (conj_linea[0], conj_linea[len(conj_linea) - 1])

            punto_aux_1 = linea_osm_inv[0][1]
            linea_osm_inv[0][1] = linea_osm_inv[0][0]
            linea_osm_inv[0][0] = punto_aux_1

            punto_aux_2 = linea_osm_inv[1][1]
            linea_osm_inv[1][1] = linea_osm_inv[1][0]
            linea_osm_inv[1][0] = punto_aux_2

            #Obtenemos el largo del segmento osm
            dist_puntos_uno_dos_osm = self.funciones_geo.mts_entre_puntos(
                                                        lat_punto_uno_osm,
                                                        long_punto_uno_osm,
                                                        lat_punto_dos_osm,
                                                        long_punto_dos_osm)

            #Este angulo es el del registro osm con respecto al ecuador.
            angulo_osm = self.NorthAzimuth(linea_osm[0], linea_osm[1])

            #Obtenemos los datos necesarios de los segmentos de gis2
            q_obj = session.query(
                calles.gid,
                func.astext(calles.geom).label('geom'),
                func.ST_HausdorffDistance(
                    calles.geom,
                    func.st_geomfromtext(calle_osm['geom'], self.proyeccion),
                    0.5,
                ).label('hausdorf'),
                func.st_distance(
                    calles.geom,
                    func.st_geomfromtext(calle_osm['geom'], self.proyeccion),
                ).label('distancia'),
                calles.id_calle_nombre,
            )
            #Ahora filtramos los segmentos gis2 dentro del area que tenemos
            #como parametro, luego los que tengan st_distance < 0.0009 con
            #respecto al segmento osm
            q_obj = q_obj.filter(
                and_(
                    func.st_intersects(
                        calles.geom,
                        func.st_geomfromtext(
                            'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                                str(lon1), str(lat1),
                                str(lon2), str(lat1),
                                str(lon2), str(lat2),
                                str(lon1), str(lat2),
                                str(lon1), str(lat1),
                            ),
                            4326
                        )
                    ),
                    func.st_distance(
                        calles.geom,
                        func.st_geomfromtext(calle_osm['geom'],
                                                            self.proyeccion),
                    ) < 0.0009,
                )
            )

            registros = q_obj.all()

            no_agregar_osm = 0
            #Recorremos los registros gis2 para compararlos con los osm y poder
            #saber si debemos agregar el segmento o no
            for registro in registros:
                se_agrega_osm = 0
                #Transformamos el geom del segmento gis2 en puntos para poder
                #comparar con los de osm
                conj_linea_gis2 = self.funciones_geo.procesar_geometria(
                                                                    registro[1])
                linea_gis2 = (conj_linea_gis2[0], conj_linea_gis2[1])

                long_punto_uno_gis2 = linea_gis2[0][0]
                lat_punto_uno_gis2 = linea_gis2[0][1]

                long_punto_dos_gis2 = linea_gis2[1][0]
                lat_punto_dos_gis2 = linea_gis2[1][1]
                #Damos vuelta los puntos ya que procesar_geometria nos los trae
                #al reves
                linea_gis2_inv = (conj_linea_gis2[0], conj_linea_gis2[1])

                punto_aux_gis2_1 = linea_gis2_inv[0][1]
                linea_gis2_inv[0][1] = linea_gis2_inv[0][0]
                linea_gis2_inv[0][0] = punto_aux_gis2_1

                punto_aux_gis2_2 = linea_gis2_inv[1][1]
                linea_gis2_inv[1][1] = linea_gis2_inv[1][0]
                linea_gis2_inv[1][0] = punto_aux_gis2_2

                #Obtenemos las distancias necesarias para poder hacer las
                #comparaciones.
                dist_p1_osm_p1_gis2 = self.funciones_geo.mts_entre_puntos(
                                                        lat_punto_uno_osm,
                                                        long_punto_uno_osm,
                                                        lat_punto_uno_gis2,
                                                        long_punto_uno_gis2)

                dist_p2_osm_p2_gis2 = self.funciones_geo.mts_entre_puntos(
                                                        lat_punto_dos_osm,
                                                        long_punto_dos_osm,
                                                        lat_punto_dos_gis2,
                                                        long_punto_dos_gis2)

                dist_p1_p2_gis2 = self.funciones_geo.mts_entre_puntos(
                                                        lat_punto_uno_gis2,
                                                        long_punto_uno_gis2,
                                                        lat_punto_dos_gis2,
                                                        long_punto_dos_gis2)

                #Para ver si se debe invertir se utilizan las sig. distancias
                dist_p1_osm_p2_gis2 = self.funciones_geo.mts_entre_puntos(
                                                        lat_punto_uno_osm,
                                                        long_punto_uno_osm,
                                                        lat_punto_dos_gis2,
                                                        long_punto_dos_gis2)

                dist_p2_osm_p1_gis2 = self.funciones_geo.mts_entre_puntos(
                                                        lat_punto_dos_osm,
                                                        long_punto_dos_osm,
                                                        lat_punto_uno_gis2,
                                                        long_punto_uno_gis2)

                #angulo_aux es el angulo del segmento gis2 con respecto
                #al ecuador.
                angulo_aux = self.NorthAzimuth(conj_linea_gis2[0],
                                                    conj_linea_gis2[1])
                #dif_angulo es el valor absoluto de la diferencia entre
                #angulo osm y gis2

                dif_angulo = abs(angulo_aux - angulo_osm)

                #si estan invertidos los puntos los damos vueltas
                if dist_p1_osm_p2_gis2 < dist_p1_osm_p1_gis2 and\
                    dist_p2_osm_p1_gis2 < dist_p2_osm_p2_gis2:
                        #asignamos las variables para poder invertir
                        aux_dist_p1_osm_p2_gis2 = dist_p1_osm_p2_gis2
                        aux_dist_p2_osm_p1_gis2 = dist_p2_osm_p1_gis2
                        aux_dist_p1_osm_p1_gis2 = dist_p1_osm_p1_gis2
                        aux_dist_p2_osm_p2_gis2 = dist_p2_osm_p2_gis2
                        #invertimos los puntos de gis2
                        dist_p1_osm_p1_gis2 = aux_dist_p1_osm_p2_gis2
                        dist_p2_osm_p2_gis2 = aux_dist_p2_osm_p1_gis2
                        dist_p1_osm_p2_gis2 = aux_dist_p1_osm_p1_gis2
                        dist_p2_osm_p1_gis2 = aux_dist_p2_osm_p2_gis2

                dist_max = 50
                #Filtramos los segmentos gis2 con angulo menor a 10 grados con
                #respecto al osm para ver si se debe agregar
                if dif_angulo < 10:
                    #Filtramos para ver si agregamos el segmento osm comparando
                    #las distancias

                    #Vemos si el segmento osm esta entre los puntos
                    #p1 y p2 del gis2
                    if (dist_p1_osm_p2_gis2 < dist_p1_p2_gis2 and
                        dist_p2_osm_p1_gis2 < dist_p1_p2_gis2):

                        osm_entre_puntos_gis2 = True
                    #Filtramos para ver si debemos agregar el osm
                    if (dist_p1_osm_p1_gis2 > dist_puntos_uno_dos_osm) and\
                        (osm_entre_puntos_gis2 is False) and\
                        (dist_p2_osm_p2_gis2 > dist_max):

                        se_agrega_osm += 1

                    if (dist_p2_osm_p2_gis2 > dist_puntos_uno_dos_osm) and\
                        (osm_entre_puntos_gis2 is False) and\
                        (dist_p1_osm_p1_gis2 > dist_max):

                        se_agrega_osm += 1

                    if (dist_p1_osm_p1_gis2 > dist_max) and\
                        (osm_entre_puntos_gis2 is False):

                        se_agrega_osm += 1

                    if (dist_p2_osm_p2_gis2 > dist_max) and\
                        (osm_entre_puntos_gis2 is False):

                        se_agrega_osm += 1
                    #Si no entro a algun filtro no se dibuja
                    if se_agrega_osm == 0:
                        no_agregar_osm += 1

            #Si no_agregar_osm es mayor a 0 no se agrega segemento osm'
            if no_agregar_osm == 0:
                calles_osm_a_agregar.append(calle_osm)
        #Agregamos los segmentos osm a gis2
        self.actualizar_osm_a_gis2(calles_osm_a_agregar, id_calle_nombre)

        session.close()
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se encontraron segmentos en la zona dada')
        return rta

    def calles_ciudad_osm(self, lat1, lon1, lat2, lon2, bd_osm):
        '''
            Esta función se encarga de obtener todos los segmentos de una
            zona dada.

            Argumento:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de tuple)
        '''

        rta = {}

        # Llamo a la función obtener_bd
        rta_obtener_bd = obtener_bd(bd_osm)
        # Si no puedo obtener la BD
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Piso la BD de OSM con la dada
        self.osm = rta_obtener_bd['Respuesta']

        # Abro la sesión con la base de datos osm
        session = self.osm.binded_session()

        # Obtengo la tabla lines
        lines = self.osm.tabla('lines')

        # Obtengo todos los segmentos de la zona dada
        q_obj2 = session.query(
            lines.gid,
            lines.name,
            func.astext(lines.the_geom).label('geom'),
            lines.the_geom,
        )

        q_obj2 = q_obj2.filter(
            and_(
                #lines.highway != 'secondary',
                lines.highway != '',
                func.st_isvalid(lines.the_geom),
                func.st_intersects(
                    lines.the_geom,
                    func.st_geomfromtext(
                        'POLYGON((%s %s, %s %s, %s %s, %s %s, %s %s))' % (
                            str(lon1), str(lat1),
                            str(lon2), str(lat1),
                            str(lon2), str(lat2),
                            str(lon1), str(lat2),
                            str(lon1), str(lat1),
                        ),
                        self.proyeccion
                    )
                ),
            )
        )

        # Guardamos todos los registros encontrados
        registros = q_obj2.all()

        lineas = []

        # Recorremos los registros para extraer los datos necesarios
        for registro in registros:
            datos = {}
            datos['gid'] = registro[0]
            datos['name'] = registro[1]
            datos['geom'] = registro[3]
            datos['geom_text'] = registro[2]
            datos['osm_id'] = registro[0]
            segmento_osm_varios_puntos = datos

            if segmento_osm_varios_puntos is not None:
                    lineas.append(segmento_osm_varios_puntos)
        # Cierro la conexión con la base de datos
        session.close()

        # Si no encuentro registros retorno el error
        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron segmentos en la zona dada')
            return rta

        # Si encontré registros retorno una lista de tuplas con los datos
        rta['Estado'] = True
        rta['Respuesta'] = lineas
        return rta

    def actualizar_osm_a_gis2(self, lista, id_calle_nombre):
        '''
            Esta función se encarga de agregar los segmentos osm a la BD gis2.

            Argumento:
                lista  (lista de tupla)
                id_calle_nombre (int) es el id de la calle con el nombre
                                        y la ciudad que queremos que se
                                        agreguen todas las calles nuevas que
                                        van a ser desconocidad

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de tuple)
        '''

        rta = {}

        if lista is None:
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo obtener calles de osm')
            return rta

        #Recorremos los segmentos del parametro lista y los agregamos a gis2
        for segm in lista:

            # Proceso la geometría.
            geom = self.funciones_geo.procesar_geometria(segm['geom'])

            # Defino el segmento.
            segmento = {
                'IdCalleNombre': id_calle_nombre,
                'IdTipo': 2,
                'Puntos': [[geom[0][1], geom[0][0]], [geom[1][1], geom[1][0]]]
            }

            # Defino el usuario admin.
            id_usuario = 1

            # Agrego el segmento.
            rta_segmento = self.operaciones.grabar_segmento(
                id_usuario, segmento
            )

            print 'agregar segmento ++'
            print rta_segmento

        rta['Estado'] = True
        rta['Respuesta'] = _(u'La ciudad se pegó correctamente')
        return rta

    def setear_calles_como_desconocidas(self, lat1, lon1, lat2, lon2,
                                            id_calle_nombre_desconocida):
        '''
            Dada una región, esta función se encarga de cambiar el nombre
            de todos los segmentos que lo companen a desconocido.

            Argumentos:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                id_calle_nombre_desconocida (int)

            Funciones:
                Migraciones().calles_ciudad
                CallesPostgres().actualizar_segmento

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Llamo al afunción calles_ciudad
        rta_calles_ciudad = self.calles_ciudad(lat1, lon1, lat2, lon2)

        # Si no encuentro los segmentos de la zona dada
        if rta_calles_ciudad['Estado'] is False:
            # Retorno el error
            return rta_calles_ciudad

        # Recorro los segmentos obtenidos
        for segmento in rta_calles_ciudad['Respuesta']:
            # Guardo el gid del segmento
            gid_segmento = segmento[0]

            print 'Seteo calle desconocida en el segmento:'
            print gid_segmento

            # Llamo a la función actualizar_segmento
            rta_actualizar_segmento = self.calle_postgres.actualizar_segmento(
                gid=gid_segmento,
                id_calle_nombre=id_calle_nombre_desconocida,
            )
            # Si no se pudo actualizar un segmento
            if rta_actualizar_segmento['Estado'] is False:
                # Retorno el error
                return rta_actualizar_segmento

        # Si se pudieron actualizar todos los segmentos
        # retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = \
            _('Se actualizaron todos los segmentos con la calle desconocida %s'
            % id_calle_nombre_desconocida)
        return rta

    def obtener_punto_osm(self, id_punto, bd_osm):
        '''
            Esta función se encarga de obtener los datos de un punto de OSM
            según el gid dado.

            Argumentos:
                id_punto (str)
                bd_osm (str)

            Funciones:
                at.orm.utiles.obtener_bd

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Llamo a la función obtener_bd
        rta_obtener_bd = obtener_bd(bd_osm)
        # Si no puedo obtener la BD
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Piso la BD de OSM con la dada
        self.osm = rta_obtener_bd['Respuesta']

        # Abro la sesión con la base de datos osm
        session = self.osm.binded_session()

        # Obtengo la tabla points
        points = self.osm.tabla('points')

        # Busco el punto dado
        q_obj = session.query(
            points.osm_id,
            points.name,
            func.astext(points.geom).label('geom'),
        )
        q_obj = q_obj.filter(points.osm_id == id_punto)

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesión con la base de datos gis2
        session.close()

        # Si no se encontró el registro
        if registro is None:
            # Retorno el error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el punto %s' % id_punto)
            return rta

        # Si se encontró el punto retorno True y sus datos
        rta['Estado'] = True
        rta['Respuesta'] = registro
        return rta

    def obtener_altura_segmento(self, gid, bd_osm, punto=1):
        '''
            Dado el gid de un segmento esta función se encarga de obtener
            su altura en base al mapa de OSM.

            Argumentos:
                gid (int)
                bd_osm (str)
                // El parámetro punto se usan para la recursividad
                // de la función, no se debe pasar como parámetros al
                // utilizar esta función
                punto (int)

            Funciones:
                at.orm.utiles.obtener_bd
                CallesPostgres().obtener_segmento
                FuncionesGeo().procesar_geometria
                FuncionesGeo().mts_entre_puntos
                Migraciones().procesar_datos_punto
                Migraciones().obtener_altura_segmento

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de int)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Llamo a la función obtener_segmento
        rta_obtener_segmento = self.calle_postgres.obtener_segmento(gid)
        # Si no puedo obtener el segmento
        if rta_obtener_segmento['Estado'] is False:
            # Retorno el error
            return rta_obtener_segmento

        # Guardo los datos del segmento dado
        segmento = rta_obtener_segmento['Respuesta']

        # Si el segmento no tiene el dato puntos
        if segmento['puntos'] is None:
            # No puedo buscar su altura, retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El segmento %s no tiene puntos cargados' % gid)
            return rta

        # Guardo el primer punto del segmento
        primer_punto = [segmento['puntos'][0][0], segmento['puntos'][0][1]]

        # Paso el primer punto a str para la SQL
        primer_punto_str = 'POINT(%s %s)' % \
                (str(segmento['puntos'][0][1]), str(segmento['puntos'][0][0]))

        # Guardo el segundo punto del segmento
        segundo_punto = [segmento['puntos'][1][0], segmento['puntos'][1][1]]

        # Paso el segundo punto a str para la SQL
        segundo_punto_str = 'POINT(%s %s)' % \
                (str(segmento['puntos'][1][1]), str(segmento['puntos'][1][0]))

        # Obtengo la distancia entre los extremos del segmento
        dist_entre_extremos = self.funciones_geo.mts_entre_puntos(
            primer_punto[0],
            primer_punto[1],
            segundo_punto[0],
            segundo_punto[1],
        )

        # Llamo a la función obtener_bd
        rta_obtener_bd = obtener_bd(bd_osm)
        # Si no puedo obtener la BD
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Piso la BD de OSM con la dada
        self.osm = rta_obtener_bd['Respuesta']

        # Abro la sesión con la base de datos osm
        session = self.osm.binded_session()

        # Obtengo la tabla points
        points = self.osm.tabla('points')

        # Busco el punto dado.
        q_obj = session.query(
            points.osm_id,
            points.name,
            points.highway,
            points.other_tags,
            func.astext(points.the_geom).label('geom'),
        )

        # Si punto es igual a 1 filtro por el primer punto
        if punto == 1:
            # Filtro por la cercanía al primer punto del segmento
            q_obj = q_obj.filter(
                func.st_distance(
                    points.the_geom,
                    func.st_geomfromtext(primer_punto_str, self.proyeccion),
                ) < 0.0003
            )
        else:
            # Si el punto es igual a 2 filtro por el segundo punto
            q_obj = q_obj.filter(
                func.st_distance(
                    points.the_geom,
                    func.st_geomfromtext(segundo_punto_str, self.proyeccion),
                ) < 0.0003
            )

        # Guardo los registros obtenidos
        registros = q_obj.all()

        # Cierro la sesión con la base de datos osm
        session.close()

        # Si no se encontró ningún punto
        if len(registros) <= 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se encontró ningún punto cercano al segmento %s' % gid)
            return rta

        # Creo una lista de puntos vacía
        lista_puntos = []

        # Creo una lista para guardar los puntos auxiliares para poder hacer
        # correcciones si es necesario
        lista_puntos_aux = []

        # Recorro los registros obtenidos
        for registro in registros:
            # Si el registro no tiene un valor en el campo other_tags
            if registro.other_tags is None:
                # Quiere decir que no tiene altura en este punto
                # Así que paso al siguiente
                continue

            # Llamo a la función procesar_datos_punto para obtener los datos
            rta_procesar_datos_punto = self.procesar_datos_punto(
                registro.other_tags
            )
            # Si no se pudo procesar los datos del punto
            if rta_procesar_datos_punto['Estado'] is False:
                # Paso al siguiente punto
                continue

            # Guardo los datos del punto
            datos_punto = rta_procesar_datos_punto['Respuesta']

            # Si la calle del punto no coincide con el nombre de la calle
            # del segmento
            if datos_punto.get('calle') != segmento['nombre_calle']:
                #print 'Nombre segmento'
                #print segmento['nombre_calle']

                #print 'Nombre calle punto'
                #print datos_punto.get('calle')

                # No tengo en cuenta este punto
                continue

            # Proceso la geometría del punto
            geom_punto = self.funciones_geo.procesar_geometria(registro.geom)
            # Ordeno los datos del punto
            geom_punto = [geom_punto[0][1], geom_punto[0][0]]

            # Si punto es igual a 1
            if punto == 1:
                # Obtengo la diferencia entre el punto cercano al primer punto
                # del segmento con el segundo punto del segmento
                dist_entre_puntos = self.funciones_geo.mts_entre_puntos(
                    geom_punto[0],
                    geom_punto[1],
                    segundo_punto[0],
                    segundo_punto[1],
                )
            else:
                # Si punto es igual a 2
                # Obtengo la diferencia entre el punto cercano al segundo punto
                # del segmento con el primer punto del segmento
                dist_entre_puntos = self.funciones_geo.mts_entre_puntos(
                    geom_punto[0],
                    geom_punto[1],
                    primer_punto[0],
                    primer_punto[1],
                )

            # Si la distancia entre un extremo y un punto es mayor
            # a la distancia entre los dos puntos del segmento
            if dist_entre_puntos > dist_entre_extremos:
                # Guardo el punto en lista_puntos_aux
                lista_puntos_aux.append(datos_punto)

                # No tengo en cuenta este punto
                continue

            # Agrego cada punto a la lista
            # junto con su distancia al extremo del segmento
            lista_puntos.append([dist_entre_puntos, datos_punto])

        # Si la lista de puntos está vacía
        if len(lista_puntos) == 0:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo obtener la altura del segmento %s' % gid)
            return rta

        # Ordeno la lista de puntos obtenido
        # por la cercanía al extremo del segmento
        lista_puntos = sorted(lista_puntos, key=itemgetter(0))

        # Si la lista de puntos tiene un solo punto
        if len(lista_puntos) == 1:
            # Agrego un segundo punto duplicando el primero
            lista_puntos.append(lista_puntos[0])

        # Me quedo con los datos de los dos primeros puntos
        datos_punto1 = lista_puntos[0][1]
        datos_punto2 = lista_puntos[1][1]

        # Guardo la primera altura obtenida
        altura1 = datos_punto1.get('altura')

        # Guardo la segunda altura obtenida
        altura2 = datos_punto2.get('altura')

        try:
            # Paso las alturas a enteros si son númericas
            altura1 = int(altura1)
            altura2 = int(altura2)
        except:
            # Si no son numéricas retorno False y un error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'Las alturas obtenidas no son numéricas')
            return rta

        # Si tengo puntos auxiliares
        if len(lista_puntos_aux) != 0:
            # Tomo la altura de un punto auxiliar para usar de referencia
            altura_aux = int(lista_puntos_aux[0]['altura'])

            # Obtengo la diferencia entre la altura1 y la altura_aux
            dif_alturas = abs(altura1 - altura_aux)

            # Si la diferencia es mayor a 50
            if dif_alturas > 50:
                # Piso la altura1 con el valor de la altura2
                # que supongo que está bien cargado
                altura1 = altura2

            # Hago lo mismo para la altura2
            # Obtengo la diferencia entre la altura2 y la altura_aux
            dif_alturas = abs(altura2 - altura_aux)

            # Si la diferencia es mayor a 50
            if dif_alturas > 50:
                # Piso la altura2 con el valor de la altura1
                # que supongo que está bien cargado
                altura2 = altura1

        # Si punto es igual a 1
        if punto == 1:
            # Llamo nuevamente a la función obtener_altura_segmento
            # para obtener las alturas del segundo punto del segmento
            rta_obtener_altura_segmento = self.obtener_altura_segmento(
                gid, bd_osm, 2
            )

            # Si no se pudo obtener la altura del segundo punto
            if rta_obtener_altura_segmento['Estado'] is False:
                # Retorno el error
                return rta_obtener_altura_segmento

            # Guardo la tercera altura obtenida
            altura3 = rta_obtener_altura_segmento['Respuesta'][0]

            # Guardo la cuarta altura obtenida
            altura4 = rta_obtener_altura_segmento['Respuesta'][1]

            # Si todo salió bien retorno True y las dos alturas obtenidas
            rta['Estado'] = True
            rta['Respuesta'] = [altura1, altura2, altura3, altura4]
            return rta

        # Si todo salió bien retorno True y la altura obtenida
        rta['Estado'] = True
        rta['Respuesta'] = [altura1, altura2]
        return rta

    def procesar_datos_punto(self, datos_punto):
        '''
            Esta función se encarga de devolver de una mejor forma los datos
            de un punto de OSM, estos se obtienen del campo other_tags
            de la tabla points

            Argumentos:
                datos_punto (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['pais'] (str)
                                        ['provincia'] (str)
                                        ['ciudad'] (str)
                                        ['codigo_postal'] (str)
                                        ['calle'] (str)
                                        ['altura'] (str)
        '''

        # Creo el diccionario a retornar
        rta = {}

        # En este diccionario voy a guardar los datos procesados
        dic = {}

        # Separo los datos por una coma
        datos_separados = datos_punto.split(',')

        # Recorro los datos separados
        for dato_separado in datos_separados:
            # Vuelvo a separar los datos obtenidos
            dato_valor = dato_separado.split('=>')

            # Tengo que tener un campo y su valor
            if len(dato_valor) != 2:
                # Si no es así, paso al siguiente
                continue

            # Si el campo se llama "addr:country"
            if dato_valor[0] == '"addr:country"':
                # Guardo el pais del punto
                dic['pais'] = dato_valor[1].replace('"', '')
                # Paso al siguiente dato
                continue

            # Si el campo se llama "addr:state"
            if dato_valor[0] == '"addr:state"':
                # Guardo la provincia del punto
                dic['provincia'] = dato_valor[1].replace('"', '')
                # Paso al siguiente dato
                continue

            # Si el campo se llama "addr:city"
            if dato_valor[0] == '"addr:city"':
                # Guardo la ciudad del punto
                dic['ciudad'] = dato_valor[1].replace('"', '')
                # Paso al siguiente dato
                continue

            # Si el campo se llama "addr:postcode"
            if dato_valor[0] == '"addr:postcode"':
                # Guardo el código postal del punto
                dic['codigo_postal'] = dato_valor[1].replace('"', '')
                # Paso al siguiente dato
                continue

            # Si el campo se llama "addr:street"
            if dato_valor[0] == '"addr:street"':
                # Guardo la calle del punto
                dic['calle'] = dato_valor[1].replace('"', '')
                # Paso la calle a latin1
                dic['calle'] = dic['calle'].encode("latin1")
                # Paso al siguiente dato
                continue

            # Si el campo se llama "addr:housenumber"
            if dato_valor[0] == '"addr:housenumber"':
                # Guardo la altura del punto
                dic['altura'] = dato_valor[1].replace('"', '')
                # Paso al siguiente dato
                continue

        # Si el diccionario está vacío
        if dic == {}:
            # Retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo procesar los datos del punto')
            return rta

        # Si no pude obtener la altura retorno False y un mensaje
        if dic.get('altura') is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo obtener la altura')
            return rta

        # Si todo salió bien retorno True y los datos en un diccionario
        rta['Estado'] = True
        rta['Respuesta'] = dic
        return rta

    def migrar_alturas(self, lat1, lon1, lat2, lon2, bd_osm):
        '''
            Esta función se encarga de migrar las alturas de OSM a nuestra
            BD gis2

            Argumento:
                lat1 (float)
                lon1 (float)
                lat2 (float)
                lon2 (float)
                bd_osm (str)

            Funciones:
                Migraciones().calles_ciudad
                Migraciones().obtener_altura_segmento
                CallesPostgres().actualizar_segmento

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Llamo a la función calles_ciudad
        rta_calles_ciudad = self.calles_ciudad(lat1, lon1, lat2, lon2)
        # Si no se encuentra ningún segmento retorno el mensaje
        if rta_calles_ciudad['Estado'] is False:
            return rta_calles_ciudad

        # Guardo los segmentos obtenidos de la ciudad
        lista_segmentos = rta_calles_ciudad['Respuesta']

        # Recorro los segmento obtenidos
        for segmento in lista_segmentos:
            # Guardo el gid del segmento
            gid = segmento[0]

            # Llamo a la función obtener_altura_segmento
            rta_obtener_altura_segmento = self.obtener_altura_segmento(
                gid, bd_osm
            )
            print 'rta_obtener_altura_segmento'
            print rta_obtener_altura_segmento

            # Si no se obtivieron las alturas
            if rta_obtener_altura_segmento['Estado'] is False:
                # Paso al siguiente segmento
                continue

            # Guardo las alturas
            desde_izq = rta_obtener_altura_segmento['Respuesta'][0]
            desde_der = rta_obtener_altura_segmento['Respuesta'][1]
            hasta_izq = rta_obtener_altura_segmento['Respuesta'][2]
            hasta_der = rta_obtener_altura_segmento['Respuesta'][3]

            # El id uno es el del admin.
            id_usuario = 1

            # Defino el segmento.
            segmento = {
                'Id': gid,
                'DesdeDer': desde_der,
                'HastaDer': hasta_der,
                'DesdeIzq': desde_izq,
                'HastaIzq': hasta_izq
            }

            # Llamo a la función actualizar_segmento
            rta_actualizar_segmento = self.operaciones.grabar_segmento(
                id_usuario, segmento
            )
            # Si no se pudo actualizar un segmento
            if rta_actualizar_segmento['Estado'] is False:
                # Retorno el error
                return rta_actualizar_segmento

        # Si todo salió bien, retorno True y un mensaje
        rta = {}
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se migraron las alturas de las calles')
        return rta

    def obtener_ciudad_del_segmento(self, gid, bd_osm, ids_ciudades):
        '''
            Esta función se encarga de obtener de que ciudad pertence
            un segmento dado.

            Argumentos:
                gid (int)
                bd_osm (str)
                ids_ciudades (dict)
                            ['id_ciudad_osm'] (int)
                            // El valor es el id de gis2
                            // y la llave es el valor del id de OSM

            Funciones:
                at.orm.utiles.obtener_bd
                CallePostgres().obtener_segmento

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o int)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Llamo a la función obtener_segmento
        rta_obtener_segmento = self.calle_postgres.obtener_segmento(gid)
        # Si no puedo obtener el segmento
        if rta_obtener_segmento['Estado'] is False:
            # Retorno el error
            return rta_obtener_segmento

        # Guardo los datos del segmento dado
        segmento = rta_obtener_segmento['Respuesta']

        # Si el segmento no tiene el dato puntos
        if segmento['puntos'] is None:
            # No puedo buscar su altura, retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'El segmento %s no tiene puntos cargados' % gid)
            return rta

        # Me quedo con un punto del segmeto para poder comparar en la SQL
        punto_segmento_str = 'POINT(%s %s)' % \
                (str(segmento['puntos'][0][1]), str(segmento['puntos'][0][0]))

        # Llamo a la función obtener_bd
        rta_obtener_bd = obtener_bd(bd_osm)
        # Si no puedo obtener la BD
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Piso la BD de OSM con la dada
        self.osm = rta_obtener_bd['Respuesta']

        # Abro la sesión con la base de datos osm
        session = self.osm.binded_session()

        # Obtengo la tabla multipolygons
        multipolygons = self.osm.tabla('multipolygons')

        # Busco el punto dado
        q_obj = session.query(multipolygons.osm_id, multipolygons.name)

        # El tipo del polígono debe ser 'boundary'
        # y debe estar dentro de la geometría de alguna ciudad
        q_obj = q_obj.filter(
            and_(
                multipolygons.type == 'boundary',
                or_(
                    func.st_within(
                        multipolygons.the_geom,
                        func.st_geomfromtext(
                            punto_segmento_str, self.proyeccion
                        ),
                    ),
                    func.st_intersects(
                        multipolygons.the_geom,
                        func.st_geomfromtext(
                            punto_segmento_str, self.proyeccion
                        ),
                    ),
                )
            )
        )

        # Guardo el registro obtenido
        registro = q_obj.first()

        # Cierro la sesión con la base de datos osm
        session.close()

        # Si no se encontró la ciudad
        if registro is None:
            # Retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró la ciudad del segmento')
            return rta

        # Declaro id_ciudad en None
        id_ciudad = None

        # Me fijo a que id corresponde la ciudad de OSM en gis2
        # Para eso recorro el diccionario ids_ciudades
        for llave in ids_ciudades.keys():
            # Comparo el osm_id con las llave del diccionario ids_ciudades
            if llave == str(registro.osm_id):
                # Si concide tomo la ciudad de gis2
                id_ciudad = ids_ciudades[llave]

        # Si no se pudo obtener el id de la ciudad
        if id_ciudad is None:
            # Retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo encontrar el id de la ciudad de gis2')
            return rta

        # Si todo salió bien retorno True y el id de la ciudad
        rta['Estado'] = True
        rta['Respuesta'] = id_ciudad
        return rta

    def validar_geometria(self, bd_muni):
        '''
            Esta función se encarga de validar si la tabla "lines" de una base
            de datos dada cuanta con los datos de la geometría bien cargados

            Argumentos:
                bd_muni (str)
        '''

        # Llamo a la función obtener_bd
        rta_obtener_bd = obtener_bd(bd_muni)
        # Si no puedo obtener la BD
        if rta_obtener_bd['Estado'] is False:
            # Retorno el error
            return rta_obtener_bd

        # Piso la BD de MUNI con la dada
        self.muni = rta_obtener_bd['Respuesta']

        # Abro la sesión con la base de datos muni
        session = self.muni.binded_session()

        # Obtengo la tabla lines
        lines = self.muni.tabla('lines')

        # Obtengo el gid de cada registro
        q_obj = session.query(lines.gid)

        # Guardo todos los registros de la tabla
        registros = q_obj.all()

        # Cierro la sesión con la base de datos osm
        session.close()

        # Recorro los registros obtenidos
        for registro in registros:
            # Guardo el gid del registro
            gid = registro.gid

            print 'gid'
            print gid

            # Abro la sesión con la base de datos muni
            session = self.muni.binded_session()

            # Obtengo la tabla lines
            lines = self.muni.tabla('lines')

            # Obtengo el gid de cada registro
            q_obj = session.query(lines.gid, func.astext(lines.the_geom))

            # Filtro utilizando la función de postgis isvalid
            q_obj = q_obj.filter(
                and_(
                    lines.gid == gid,
                    func.st_isvalid(lines.the_geom),
                )
            )

            # Guardo el registro que se quiere verificar
            linea = q_obj.first()
            print linea

            # Cierro la sesión con la base de datos osm
            session.close()

    def cambiar_id_ciudad(self, id_ciudad_actual, id_ciudad_nuevo):
        '''
            Esta función se encarga de cambiar el id ciudad de todas las calles
            que lo contienen por otro.

            Argumentos:
                id_ciudad_actual (int)
                id_ciudad_nuevo (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro un diccionario a retornar
        rta = {}

        # Abro la sesión con la base de datos gis2
        session = self.gis2.binded_session()

        # Obtengo la tabla calles_nombres
        calles_nombres = self.gis2.tabla('calles_nombres')

        # Busco la calles a actualizar
        q_obj = session.query(calles_nombres.id)

        # Filtro por el id actual de la ciudad que tienen las calles asignado
        q_obj = q_obj.filter(calles_nombres.id_ciudad == id_ciudad_actual)

        # Seteo el nuevo id ciudad de las calles
        dic = {}
        dic['id_ciudad'] = id_ciudad_nuevo

        # Actualizo la base de datos
        rta_update = q_obj.update(dic, synchronize_session=False)

        # Commiteo a la base de datos
        session.commit()

        # Cierro la sesión con la base de datos
        session.close()

        # Retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se cambió la ciudad en %s calles') % rta_update
        return rta
