# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import or_
from sqlalchemy.sql.expression import func
from sqlalchemy.sql.expression import and_

from at.map import _
from at.map.utiles.funciones_geo import FuncionesGeo
from at.utiles.validaciones_generales import ValidacionesGenerales
from at.orm.decorators import DBSwitch


@DBSwitch('geodata')
class Zonas(ValidacionesGenerales):

    def __init__(self):
        #super(Zonas, self).__init__()
        self.f_geo = FuncionesGeo()

    def __parse(self, estado, respuesta):
        ''' Funcion para parsear la respuesta de cada funcion'''

        return {'Estado': estado, 'Respuesta': respuesta}

    def obtener_tipo(self, id_tipo):
        ''' Funcion para obener un tipo de zona especifico

        Argumentos:
            id_tipo: int
        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_tipo)
        except:
            return self.__parse(False, _(u'El id ingresado no es valido'))
        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas_tipos = self.geodata.tabla('zonas_tipos')
        # Intentamos obtener el tipo
        q_obj = session.query(tabla_zonas_tipos)
        q_obj = q_obj.filter(tabla_zonas_tipos.id == id_tipo)
        # Obtenemos
        zona_tipo_tmp = q_obj.first()
        # Cerramos sesion
        session.close()
        # Si es vacio, salimos
        if zona_tipo_tmp is None:
            return self.__parse(
                False, _(u'No existe el tipo de zona buscada')
            )
        # Sino, devolvemos correctamente
        return self.__parse(
            True, zona_tipo_tmp.descripcion.encode('latin1')
        )

    def listar_tipos(self):
        ''' Funcion para listar los tipos de zonas del sistema

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: list[[int, str]]
        '''
        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas_tipos = self.geodata.tabla('zonas_tipos')
        # Intentamos obtener el tipo
        q_obj = session.query(tabla_zonas_tipos)
        # Obtenemos
        zonas_tipo_tmp = q_obj.all()
        session.close()
        # Generamos el buffer
        zonas_tipo = []
        # Recorremos
        for zona_tipo_tmp in zonas_tipo_tmp:
            zonas_tipo.append({
                'Id': zona_tipo_tmp.id,
                'Descripcion': zona_tipo_tmp.descripcion.encode('latin1')
            })
        # Sino, devolvemos correctamente
        return self.__parse(True, zonas_tipo)

    def grabar(self, id_tipo, descripcion, puntos=None, marcador=None,
               id_zona=None):
        ''' Funcion para grabar una zona en la base de datos

        Argumentos:
            id_tipo: int
            descripcion: str
            geometria: list[list[float, float]] (lat, lon)

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_tipo)
            self.validar_string_no_vacio(descripcion)
            if id_zona is not None:
                self.validar_entero(id_zona)
        except Exception, error:
            return self.__parse(
                False, _(u'Los datos ingresados no son validos.')
            )
        try:
            if puntos is not None:
                self.f_geo.validar_puntos_geometria(puntos, True)
            if marcador is not None:
                self.f_geo.validar_punto_geometria(marcador, True)
        except:
            return self.__parse(
                False, _(u'Los datos de la geometria no son validos.')
            )

        # Validamos el id_tipo de la zona
        rta_tipo = self.obtener_tipo(id_tipo)
        # Si no existe, salimos
        if not rta_tipo['Estado']:
            return rta_tipo
        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas = self.geodata.tabla('zonas')
        if marcador is not None:
            # No es un punto, asi que vamos a vaciar
            if len(marcador) < 2:
                marcador = None
            else:
                marcador = func.st_geomfromtext(
                    self.f_geo.generar_geometria_punto(marcador),
                    4326
                )
        if puntos is not None:
            # No es un poligono, asi que vamos a vaciar
            if len(puntos) < 3:
                puntos = None
            else:
                # Antes de armar los datos vamos a cerrar el poligono de la zona
                if puntos[0] != puntos[-1]:
                    puntos.append(puntos[0])
                # Ahora si lo' armamo'
                puntos = func.st_geomfromtext(
                    self.f_geo.generar_geometria_poligono(puntos),
                    4326
                )
        # Armamos diccionario de datos
        datos_zona = {
            'id_tipo': id_tipo,
            'descripcion': descripcion,
        }

        # Solo modifico si puntos tiene información.
        if puntos is not None:
            datos_zona['geom'] = puntos

        # Solo modifico si marcador tiene información.
        if marcador is not None:
            datos_zona['marcador'] = marcador

        # Como no hay id, agregamos la zona
        if id_zona is None:
            r = self.geodata.agregar('zonas', **datos_zona)
            # Vemos si salio algo mal
            if r is not True:
                session.rollback()
                session.close()
                return self.__parse(
                    False, _(u'No se pudo agregar el la zona.')
                )
            # Sino, guardamos y cerramos el session
            session.commit()
            session.close()
        # Si nos pasan el id, vemos si existe
        else:

            # Pedimos el interno que conicida
            q_obj = session.query(tabla_zonas)
            q_obj = q_obj.filter(tabla_zonas.id == id_zona)
            # Obtenemos
            zona_tmp = q_obj.first()
            if zona_tmp is None:
                session.close()

                datos_zona['id'] = id_zona
                r = self.geodata.agregar('zonas', **datos_zona)
                # Vemos si salio algo mal
                if r is not True:
                    return self.__parse(
                        False, _(u'No se pudo agregar el la zona.')
                    )

                return self.__parse(
                    True, _(u'Se agregó correctamente la zona.')
                )
            # Como encontramos, vamos a actualizar
            try:
                q_obj.update(datos_zona, synchronize_session=False)
                session.commit()
                session.close()
            except:
                session.rollback()
                session.close()
                # Salimos, indicando que fallo!!!
                return self.__parse(
                    False, _(u'No se pudo actualizar la zona.')
                )
        # Como llegamos hasta aca, salimos bien!!!
        return self.__parse(True, _(u'Se grabó correctamente la zona'))

    def borrar(self, id_zona):
        ''' Funcion para borrar una zona especifica del sistema
        Argumentos:
            id_zona: int
        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_zona)
        except:
            return self.__parse(False, u'Los datos ingresados no son validos.')
        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas = self.geodata.tabla('zonas')
        # Pedimos el interno que conicida
        q_obj = session.query(tabla_zonas)
        q_obj = q_obj.filter(tabla_zonas.id == id_zona)
        # Obtenemos
        zona_tmp = q_obj.first()
        if zona_tmp is None:
            session.close()
            return self.__parse(False, _(u'No existe la zona buscada'))
        # Como encontramos, vamos a borrar
        try:
            q_obj.delete(synchronize_session=False)
            session.commit()
            session.close()
        except:
            session.rollback()
            session.close()
            return self.__parse(
                False,
                _(u'No se pudo borrar la zona de id %s' % str(id_zona)
            ))
        # Sino, salimos bien
        return self.__parse(True, _(u'Se borro correctamente la zona'))

    def listar(self, id_tipo=None, id_sucursal=None):
        ''' Funcion para listar las zonas del sistema. Si se pasa el id
        del tipo como argumento, entonces solo se listaran las zonas
        de ese tipo.

        Argumentos:
            id_tipo: int
            id_sucursal: int

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''

        # Generamos la session y la tabla
        session = self.geodata.binded_session()

        tabla_zonas = self.geodata.tabla('zonas')

        tabla_zonas_cp = self.geodata.tabla('zonas_cp')

        # Pedimos el interno que conicida
        q_obj = session.query(
            tabla_zonas.id,
            tabla_zonas.id_tipo,
            tabla_zonas.descripcion,
            func.st_astext(tabla_zonas.marcador).label('marcador'),
            tabla_zonas_cp.id_zona_cp,
            tabla_zonas_cp.id_sucursal_cp
        )

        # Si nos pasan el tipo, filtramos por el mismo
        if id_tipo is not None:
            # Validamos el id_tipo de la zona
            rta_tipo = self.obtener_tipo(id_tipo)
            # Si no existe, salimos
            if not rta_tipo['Estado']:
                session.close()
                return rta_tipo
            q_obj = q_obj.filter(tabla_zonas.id_tipo == id_tipo)

        # Armo el join con la sucursal dada
        q_obj = q_obj.outerjoin((
            tabla_zonas_cp,
            and_(
                tabla_zonas.id == tabla_zonas_cp.id_zona,
                or_(
                    tabla_zonas_cp.id_sucursal_cp == id_sucursal,
                    id_sucursal == 0
                )
            )
        ))

        # Ordenamos
        q_obj = q_obj.order_by(tabla_zonas.id.asc())
        # Obtenemos
        zonas_tmp = q_obj.all()

        # Cerramos la sesion
        session.close()
        # Creamos el buffer que vamos a deovlver
        zonas = []
        for zona_tmp in zonas_tmp:
            zona = {}
            zona['Id'] = int(zona_tmp.id)
            zona['IdTipo'] = int(zona_tmp.id_tipo)
            zona['Descripcion'] = zona_tmp.descripcion.encode('latin1')
            zona['Marcador'] = None
            # Si tengo un marcador lo devuelvo
            if zona_tmp.marcador is not None:
                puntos = self.f_geo.procesar_geometria(zona_tmp.marcador)
                zona['Marcador'] = [puntos[0][1], puntos[0][0]]
            # Agregamos
            zonas.append(zona)
        # Salimos con el listado de zonas
        return self.__parse(True, zonas)

    def obtener(self, id_zona):
        ''' Funcion para obtener una zona del sistema.

        Argumentos:
            id_zona: int
        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_zona)
        except:
            return self.__parse(
                False,
                _(u'El id ingresado no es valido.')
            )
        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas = self.geodata.tabla('zonas')
        # Pedimos el interno que coincida
        q_obj = session.query(
            tabla_zonas.id,
            tabla_zonas.id_tipo,
            tabla_zonas.descripcion,
            func.st_astext(tabla_zonas.marcador).label('marcador'),
            func.st_astext(tabla_zonas.geom).label('geom'),
        ).filter(
            tabla_zonas.id == id_zona
        )
        # Buscamos
        zona_tmp = q_obj.first()
        # Cerramos la session
        session.close()
        # Si fallamos, salimos
        if zona_tmp is None:
            return self.__parse(False, _(u'No se encontró la zona buscada'))
        # Sino, parseamos
        zona = {}
        zona['Id'] = int(zona_tmp.id)
        zona['IdTipo'] = int(zona_tmp.id_tipo)
        zona['Descripcion'] = zona_tmp.descripcion.encode('latin1')
        # Procesamos el marcador
        zona['Marcador'] = []
        if zona_tmp.marcador is not None:
            # Guardamos la condicion
            puntos = self.f_geo.procesar_geometria(zona_tmp.marcador)
            for i in range(len(puntos)):
                puntos[i] = [puntos[i][1], puntos[i][0]]
            # Si salio todo bien, agregamos el punto
            if len(puntos) == 1:
                zona['Marcador'] = puntos[0]
        # Procesamos la geometria
        zona['Geometria'] = []
        if zona_tmp.geom is not None:
            # Guardamos la condicion
            puntos = self.f_geo.procesar_geometria(zona_tmp.geom)
            for i in range(len(puntos)):
                puntos[i] = [puntos[i][1], puntos[i][0]]
            zona['Geometria'] = puntos
        # Buscamos la zona cp
        zona['ZonaCp'] = None
        rta_zona_cp = self.obtener_zona_cp(id_zona)
        if rta_zona_cp['Estado']:
            zona['ZonaCp'] = rta_zona_cp['Respuesta']
        return self.__parse(True, zona)

    def obtener_por_punto(self, punto, id_tipo=None):
        ''' Funcion para obtener un listado de las zonas que contienen al
        punto indicado.

        Argumentos:
            punto: list[float, float]

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str o list[id: int]
        '''

        # Validamos los datos de entrada
        try:
            self.f_geo.validar_punto_geometria(punto)
            if id_tipo is not None:
                self.validar_entero(id_tipo)
        except:
            return self.__parse(
                False, _(u'Los datos ingresados no son validos.')
            )
        # Buscamos el tipo, si no es vacio
        if id_tipo is not None:
            # Validamos el id_tipo de la zona
            rta_tipo = self.obtener_tipo(id_tipo)
            # Si no existe, salimos
            if not rta_tipo['Estado']:
                return rta_tipo
        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas = self.geodata.tabla('zonas')
        punto = func.st_geomfromtext(
            self.f_geo.generar_geometria_punto(punto),
            4326
        )
        q_obj = session.query(tabla_zonas.id).filter(or_(
           func.ST_Within(punto, tabla_zonas.geom),
           func.ST_Intersects(punto, tabla_zonas.geom)
        ))
        # Si nos especificaron el id tipo, filtramos por el mismo.
        if id_tipo is not None:
            q_obj = q_obj.filter(tabla_zonas.id_tipo == id_tipo)
        # Buscamos
        zonas_tmp = q_obj.all()
        # Cerramos la session
        session.close()
        # Recolectamos las zonas
        id_zonas = []
        for zona_tmp in zonas_tmp:
            id_zonas.append(int(zona_tmp.id))
        # Salimos bien
        return self.__parse(True, id_zonas)

    def grabar_zona_cp(self, id_zona, id_sucursal_cp, id_zona_cp):
        ''' Funcion para asociar una zona del sistema a una zona de
        call processor

        Argumentos:
            id_zona: int
            id_zona_cp: int
            id_sucursal_cp: int

        Retorna:
            dic['Estado']: bool
            dic['Respuesta']: string
        '''
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_zona)
            self.validar_entero(id_zona_cp)
            self.validar_entero(id_sucursal_cp)
        except:
            return self.__parse(
                False, _(u'Los datos ingresados no son validos.')
            )
        # Vemos que la zona de geodata exista
        rta_zona = self.obtener(id_zona)
        if not rta_zona['Estado']:
            return rta_zona
        # Intentamos borrar la zona si ya existe
        session = self.geodata.binded_session()
        tabla_zonas = self.geodata.tabla('zonas_cp')
        # Pedimos el interno que conicida
        q_obj = session.query(tabla_zonas)
        q_obj = q_obj.filter(tabla_zonas.id_zona == id_zona)
        # Obtenemos
        zona_tmp = q_obj.first()
        if zona_tmp is not None:
            # Como encontramos, vamos a borrar
            try:
                q_obj.delete(synchronize_session=False)
                session.commit()
                session.close()
            except:
                session.rollback()
                session.close()
                return self.__parse(
                    False,
                    _(u'No se pudo borrar la zona de id %s' % str(id_zona)
                ))
        # Ahora lo agregamos
        datos_zona_cp = {
            'id_zona': id_zona,
            'id_zona_cp': id_zona_cp,
            'id_sucursal_cp': id_sucursal_cp
        }
        # Agregamos la zona
        r = self.geodata.agregar('zonas_cp', **datos_zona_cp)
        # Vemos si salio algo mal
        if r is not True:
            session.rollback()
            session.close()
            return self.__parse(
                False, _(u'No se pudo agregar el la zona.')
            )
        # Sino, guardamos y cerramos el session
        session.commit()
        session.close()
        # Salimos
        return self.__parse(
            True, _(u'Se agrego correctamente la zona cp.')
        )

    def obtener_zona_cp(self, id_zona, id_sucursal=None):
        ''' Funcion para obtener una zona del sistema.

        Argumentos:
            id_zona: int
        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_zona)
            if id_sucursal is not None:
                self.validar_entero(id_sucursal)
        except:
            return self.__parse(
                False,
                _(u'El id ingresado no es valido.')
            )
        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas_cp = self.geodata.tabla('zonas_cp')
        # Pedimos el interno que coincida
        q_obj = session.query(
            tabla_zonas_cp
        ).filter(
            tabla_zonas_cp.id_zona == id_zona
        )
        # Si hay id sucursal, filtramos
        if id_sucursal is not None:
            q_obj = q_obj.filter(tabla_zonas_cp.id_sucursal_cp == id_sucursal)
        # Buscamos
        zona_tmp = q_obj.first()
        # Cerramos la session
        session.close()
        # Si fallamos, salimos
        if zona_tmp is None:
            return self.__parse(False, _(u'No se encontró la zona cp buscada'))
        # Sino, parseamos
        zona = {
            'IdZona': int(zona_tmp.id_zona),
            'IdZonaCp': int(zona_tmp.id_zona_cp),
            'IdSucursalCp': int(zona_tmp.id_sucursal_cp)
        }
        # Salimos
        return self.__parse(True, zona)

    def borrar_zona_cp(self, id_zona):
        ''' Funcion que toma como argumento una zona de geodata y borra
        la zona_cp asociada. Destruye la relacion entre ambas

        Argumentos:
            id_zona: int

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str
        '''
        # Validamos los datos de entrada
        try:
            self.validar_entero(id_zona)
        except:
            return self.__parse(
                False, _(u'Los datos ingresados no son validos.')
            )

        # Intentamos borrar la zona si ya existe
        session = self.geodata.binded_session()
        tabla_zonas = self.geodata.tabla('zonas_cp')
        # Pedimos el interno que conicida
        q_obj = session.query(tabla_zonas)
        q_obj = q_obj.filter(tabla_zonas.id_zona == id_zona)
        # Obtenemos
        zona_tmp = q_obj.first()
        if zona_tmp is None:
            return self.__parse(
                False,
                _(u'La zona indicada no tiene zona cp %s' % str(id_zona)
            ))
        # Como encontramos, vamos a borrar
        try:
            q_obj.delete(synchronize_session=False)
            session.commit()
            session.close()
            return self.__parse(True, _(
                u'Se borró correctamente la zona cp de la zona con id %s' % str(
                id_zona
            )))
        except:
            session.rollback()
            session.close()
            return self.__parse(
                False,
                _(u'No se pudo borrar la zona de id %s' % str(id_zona)
            ))

    def obtener_zonas_cp_por_punto(self, punto, id_sucursal=None):
        ''' Funcion que nos devolverá la zona cp en la que se encuentre un
        punto determinado. Esto siempre y cuando halla alguna zona dibujada
        y asociada a una zona cp.

        Argumentos:
            punto: list[lat, lon]
            id_sucursal: int o None

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: str o list[list[id_suc:int, id_zona:int]]
        '''

        # Validamos los datos de entrada
        try:
            self.f_geo.validar_punto_geometria(punto)
            if id_sucursal is not None:
                self.validar_entero(id_sucursal)
        except:
            return self.__parse(
                False, _(u'Los datos ingresados no son validos.')
            )
        # BUscamos en que zona de geodata se encuetra
        rta_por_punto = self.obtener_por_punto(punto)
        # Si fallamos, salimos
        if not rta_por_punto['Estado']:
            return rta_por_punto
        # Sino, vemos si tenemos puntos
        id_zonas = rta_por_punto['Respuesta']
        if len(id_zonas) == 0:
            return self.__parse(False, _(u'No se encontró ninguna zona'))
        # Recorremos las zonas
        zonas_cp = []
        for id_zona in id_zonas:
            rta_zona_cp = self.obtener_zona_cp(id_zona, id_sucursal)
            if rta_zona_cp['Estado']:
                zonas_cp.append(rta_zona_cp['Respuesta'])
        # Si no encontramos zonas cp, salimos
        if len(zonas_cp) == 0:
            return self.__parse(
                False, _(u'Se encontraron zonas pero ninguna zona cp')
            )
        # Sino, salimos correctamente
        return self.__parse(True, zonas_cp)

    def listar_con_geom(self, longitud_primero=False):
        ''' Esta función devuelve una lista de listas con el id_zona,
         descripcion, marcadores, geometría de todas las zonas.

        Retorna:
            rta['Estado']: bool
            rta['Respuesta']: [[]]
        '''

        # Creo el diccionario que contendrá el estado y la respuesta de la
        # función.
        rta = {}
        # Esta lista contendrá las zonas encontradas(listas).
        zonas = []

        # Generamos la session y la tabla
        session = self.geodata.binded_session()
        tabla_zonas = self.geodata.tabla('zonas')
        # Determinamos los campos a obtener.
        q_obj = session.query(
            tabla_zonas.id,
            tabla_zonas.descripcion,
            func.st_astext(tabla_zonas.marcador).label('marcador'),
            func.st_astext(tabla_zonas.geom).label('geom'),
        )
        # Cierro la sesión.
        session.close()

        # En el caso de que no encontremos ninguna zona.
        if int(q_obj.count()) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró ninguna zona')
            return rta

        # Si la consulta nos duvuelve 1 o más registros, obtenemos todos los
        # registros.
        zonas_tmp = q_obj.all()

        # Procesamos zona por zona
        for zona_tmp in zonas_tmp:
            # Creo una lista por cada zona.
            zona = []
            # En la posición 0 estará el ID
            zona.append(int(zona_tmp.id))
            # En la posición 1 estará la descripción
            zona.append(zona_tmp.descripcion.encode('latin1'))
            # En la posición 2 estará el marcador.
            # Determino que sea una lista vacia por si esa zona no tiene
            # marcadores.
            zona.append([])
            if zona_tmp.marcador is not None:
                # Guardamos la condicion
                puntos = self.f_geo.procesar_geometria(zona_tmp.marcador)
                for i in range(len(puntos)):
                    # [[long, lat], [lon, lat]]
                    if longitud_primero is True:
                        puntos[i] = [puntos[i][1], puntos[i][0]]
                    else:
                        puntos[i] = [puntos[i][0], puntos[i][1]]
                # Si salio todo bien, agregamos el punto
                if len(puntos) == 1:
                    zona[2] = puntos[0]
            # En la posición 3 estará la geometría de la zona.
            zona.append([])
            if zona_tmp.geom is not None:
                # Guardamos la condicion
                puntos = self.f_geo.procesar_geometria(zona_tmp.geom)
                for i in range(len(puntos)):
                    if longitud_primero is True:
                        puntos[i] = [puntos[i][1], puntos[i][0]]
                    else:
                        puntos[i] = [puntos[i][0], puntos[i][1]]
                zona[3] = [puntos]
            # Agregamos la zona a la lista general.
            zonas.append(zona)
        # Retorno la respuesta.
        rta['Estado'] = True
        rta['Respuesta'] = zonas
        return rta

