# -*- coding: utf-8 -*-

from sqlalchemy.orm.query import Query

from at.orm import _
from at.orm.mapper import ATMapper


class ABM(ATMapper):
    ''' Clase para Altas Bajas y Modificaciones utilizando ATMapper. ABM hereda
    de Gateway, cual es un singleton.
    '''

    def __get_tabla(self, tabla):
        ''' Metodo que obtiene un objeto de tipo tabla de la instanciacion
        del objeto.'''

        try:
            return self.db[tabla]
        except Exception as e:
            return e

    def tabla(self, tabla):
        return self.__get_tabla(tabla)

    def agregar(self, tabla, **kwargs):

        # obtiene la tabla
        tabla = self.__get_tabla(tabla)

        # lee uno por uno las claves
        for kw in kwargs.keys():

            # si el campo no pertenece a la tabla arroja una excepcion
            if not hasattr(tabla, kw):
                raise Exception('Campo %s no pertenece a la tabla' % kw)

        new_obj = tabla()
        for kw in kwargs.keys():
            setattr(new_obj, kw, kwargs[kw])

        session = self.binded_session()

        try:
            session.add(new_obj)
            session.commit()
            session.close()
            return True
        except Exception as e:
            session.rollback()
            session.close()
            return e

    def __filter_equals(self, q_obj, field, value):
        if not isinstance(q_obj, Query):
            raise Exception('Query Object no admitido')
        q_obj = q_obj.filter(field == value)
        return q_obj

    def __filter_minor_equals(self, q_obj, field, value):
        if not isinstance(q_obj, Query):
            raise Exception('Query Object no admitido')
        q_obj = q_obj.filter(field <= value)
        return q_obj

    def __filter_major_equals(self, q_obj, field, value):
        if not isinstance(q_obj, Query):
            raise Exception('Query Object no admitido')
        q_obj = q_obj.filter(field >= value)
        return q_obj

    def __filter_diff(self, q_obj, field, value):
        if not isinstance(q_obj, Query):
            raise Exception('Query Object no admitido')
        q_obj = q_obj.filter(field != value)
        return q_obj

    def borrar(self, tabla, **kwargs):
        '''
        Borra un registro o registros que concuerden con el criterio pasado
        como argumento.

        Args:
        tabla(str): ej: 'clientes'
        **kwargs(dct): ej: datos = {'Telefono':('==', '4112233'),}
        Returns:
        True en caso de borrar satisfactoriamente el registro
        None en caso de no encontrar ningun registro, ergo no borra nada.
        Raises:
        Exception con un mensaje aclarando el tipo de error.

        Ejemplo:
            abm = ABM()
            datos = {'Telefono':('==', '4112233'),
                      'Nombre':('==', 'Juan Perez')}
            abm.borrar('clientes', **datos)
        '''

        __ops = {'==': self.__filter_equals,
                 '<=': self.__filter_minor_equals,
                 '>=': self.__filter_major_equals,
                 '!=': self.__filter_diff,
                 }

        tabla = self.__get_tabla(tabla)
        for kw in kwargs.keys():
            if not hasattr(tabla, kw):
                raise Exception('Campo %s no pertenece a la tabla' % kw)

        session = self.binded_session()
        q_obj = session.query(tabla)
        for kw in kwargs.keys():
            if isinstance(kwargs[kw], tuple):
                if len(kwargs[kw]) != 2:
                    session.close()
                    raise Exception('Formato no admitido en el operador')
                op = kwargs[kw][0]
                if op not in __ops.keys():
                    session.close()
                    raise Exception('Operador desconocido')
                field = getattr(tabla, kw)
                value = kwargs[kw][1]
                q_obj = __ops[op](q_obj, field, value)

        if int(q_obj.count()) == 0:
            session.close()
            return None

        try:
            for obj in q_obj:
                session.delete(obj)
            session.commit()
            session.close()
            return True
        except Exception as e:
            session.rollback()
            session.close()
            return e

    def actualizar(self, tabla, **kwargs):

        __ops = {'==': self.__filter_equals,
                 '<=': self.__filter_minor_equals,
                 '>=': self.__filter_major_equals,
                 '!=': self.__filter_diff,
                 }

        __op_found = False

        tabla = self.__get_tabla(tabla)
        for kw in kwargs.keys():
            if not hasattr(tabla, kw):
                raise Exception('Campo %s no pertenece a la tabla' % kw)

        session = self.binded_session()
        q_obj = session.query(tabla)

        for kw in kwargs.keys():
            if __op_found is False:
                if isinstance(kwargs[kw], tuple):
                    __op_found = True
                    if len(kwargs[kw]) != 3:
                        session.close()
                        raise Exception('Formato no admitido en el operador')
                    op = kwargs[kw][0]
                    if op not in __ops.keys():
                        session.close()
                        raise Exception('Operador desconocido')
                    field = getattr(tabla, kw)
                    value = kwargs[kw][1]
                    q_obj = __ops[op](q_obj, field, value)

        target = q_obj.first()
        if target is None:
            session.close()
            return

    def insertar(self, nombre_tabla, valor_campo=None, **kwargs):
        '''
            Esta función es la nueva versión de la función "agregar".
            Ahora va a retornar Estado y Respuesta.
            Se le pueda pasar un parámetro para que retorne el valor
            de ese campo del registro agregado.

            Argumentos:
                tabla (str)
                valor_campo (str) // No es obligatorio
                **kwargs (dict)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
                    ['ValorCampo'] // El tipo de dato según el campo de la tabla
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Obtengo la tabla dada
        tabla = self.__get_tabla(nombre_tabla)

        # Recorro los campos dados
        for kw in kwargs.keys():
            # Si el campo no existe en la tabla
            if not hasattr(tabla, kw):
                # Retorno False y un mensaje
                rta['Estado'] = False
                rta['Respuesta'] = _(
                    u'El campo "%s" no pertenece a la tabla "%s"'
                ) % (kw, nombre_tabla)
                return rta

        # Setea los campos dados a la tabla dada
        new_obj = tabla()
        for kw in kwargs.keys():
            setattr(new_obj, kw, kwargs[kw])

        # Abro la sesión con la base de datos
        session = self.binded_session()

        # Intento insertar el registro
        try:
            # Inserto el registro en la tabla
            session.add(new_obj)

            # Realizo el commit con la base de datos
            session.commit()

            # Si recibo un valor en el parámetro campo
            if valor_campo is not None:
                # Obtengo el valor pedido del registro recién agregado
                valor_campo = getattr(new_obj, valor_campo)
            else:
                # Si no, lo dejo en None
                valor_campo = None

            # Cierro la sesión con la base de datos
            session.close()

            # Retorno True y un mensaje
            rta['Estado'] = True
            rta['Respuesta'] = _(
                u'Se insertó el registro en la tabla "%s" de forma correcta'
            ) % nombre_tabla
            rta['ValorCampo'] = valor_campo
            return rta
        except Exception as error:
            # Si no se pudo insertar el registro realizo un rollback
            session.rollback()

            # Cierro la sesión con la base de datos
            session.close()

            # Retorno False y un mensaje de error
            rta['Estado'] = False
            rta['Respuesta'] = str(error)
            return rta
