# -*- coding: utf-8 -*-

import os
import configobj
import inspect

from at.orm import _
from at.orm.abm import ABM
from at.orm.config import databases

# Usamos el singleton para crear solamente un objeto ABM para cada base
# de datos. Abriendo asi solo 20 conexiones por proceso x bd


def obtener_bd(nombre_bd):
    '''
        Esta función se encarga de retornar o no la configuración para
        conectarse a una base de datos.

        Argumentos:
            nombre_bd (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str u object)
    '''

    # Declaro el diccionario a retornar
    rta = {}

    # Seteo en None los datos para conectarse a la base de datos dada
    datos_bd = None

    # Recorro las los datos de conexión con las bases de datos
    # que tiene el servidor
    for database in databases:
        # Si encuentro los datos de conexión de la base de datos dada
        if database['name'] == nombre_bd:
            # Guardo sus datos de conexión
            datos_bd = database
            # Termino el for
            break

    # Si no tengo los datos de pideturemis
    if datos_bd is None:
        # Retorno el siguiente error
        rta['Estado'] = False
        rta['Respuesta'] = _(
            u'No se encontró la configuracion para conectarse a la base de datos %s'
        ) % nombre_bd
        return rta

    # Si tengo los datos de conexión con la base de datos
    # Retorno True y la forma de conexión con la base de datos dada
    rta['Estado'] = True
    rta['Respuesta'] = ABM(datos_bd)
    return rta


def cambiar_bd(clase, nombre_bd_actual, nombre_bd_nueva, validar=False):
    '''
        Esta función se encarga de cambiar la base de datos de una clase y de
        todas las clases que se importen o se utilicen.

        Argumentos:
            clase (class)
            nombre_bd_actual (str)
            nombre_bd_nueva (str)
            validar (bool)

        Funciones:
            obtener_bd
            cambiar_bd

        Retorna:
            None o dict
                    ['Estado'] False
                    ['Respuesta'] str
    '''

    # Recorro los objetos de la clase
    for objeto in dir(clase):
        # Obtengo cada objeto
        obj = getattr(clase, objeto)

        # Si el objeto es una clase
        if inspect.isclass(obj) is True:
            # No lo tengo en cuenta y paso al siguiente
            continue

        # Si el objeto es la base de datos
        if objeto == nombre_bd_actual:
            # Obtengo la configuración con la nueva base de datos
            rta_obtener_bd = obtener_bd(nombre_bd_nueva)

            # Si recibo el parámetro validar en True
            if validar is True:
                # Tengo que ver si se pudo cambiar la base de datos o no
                if rta_obtener_bd['Estado'] is False:
                    # Si no se pudo obtener la base de datos nueva
                    # Retorno el error
                    return rta_obtener_bd

            # Si puedo obtener la configuración con la nueva base de datos
            if rta_obtener_bd['Estado'] is True:
                # Cambio la base de datos actual por la nueva
                setattr(clase, nombre_bd_actual, rta_obtener_bd['Respuesta'])
            # Paso al siguiente objeto
            continue

        # Si el objeto es una instancia de la clase
        if hasattr(obj, nombre_bd_actual) \
                or str(type(obj)) == "<type 'instance'>":
            # Llamo nuevamente a la función para hacer lo mismo
            # con cada objeto de la clase principal
            cambiar_bd(obj, nombre_bd_actual, nombre_bd_nueva)
            # Paso al siguiente objeto
            continue


def seleccionar_bd(clase, nombre_bd, empresa, base_local):
    '''
        Esta función se encarga de seleccionar que base de datos
        se va a utilizar, si va a ser la base local o una base de
        datos según la empresa recibida.

        Argumentos:
            clase (class)
            nombre_bd (str)
            empresa (str)
            base_local (bool)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Declaro el diccionario a retornar
    rta = {}

    # Si base_local es True
    if base_local is True:
        # Utilizo la base local, no cambio nada
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Utilizando la base local %s') % nombre_bd
        return rta

    # Si el valor base_local es False
    # Tengo que usar la base dada pero de la empresa especificada
    nombre_bd_empresa = nombre_bd + '_' + empresa
    # Cambio la base dada por la base de la empresa
    # Le paso validar True para que si no existe la bd me retorne el error
    rta_cambiar_bd = cambiar_bd(
        clase, nombre_bd, nombre_bd_empresa, validar=True
    )
    # print("rta:::",rta_cambiar_bd)
    # Si al querer cambiar la BD no me retorna None
    if rta_cambiar_bd is not None:
        # Es porque no se pudo cambiar de BD, retorno el error
        return rta_cambiar_bd

    # Si se pudo seleccionar la BD según la empresa
    # Retorno True y un mensaje que indica que BD se está usando
    rta['Estado'] = True
    rta['Respuesta'] = _(u'Utilizando la base de datos %s') % nombre_bd_empresa
    return rta


def obtener_datos_conexion_bd(nombre_bd):
    '''
        Esta función se encarga de retornar los datos de conexión una base de
        datos dada.

        Argumentos:
            nombre_bd (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str o dict)
                                    'engine' (str)
                                    'name' (str)
                                    'dbname' (str)
                                    'user' (str)
                                    'passwd' (str)
                                    'changeme' (str)
                                    'host' (str)
                                    'port' (str)
    '''

    # Declaro el diccionario a retornar
    rta = {}

    # Defino datos_bd en None por si no puedo obtener los datos de conexión
    datos_bd = None

    # Recorro las los datos de conexión con las bases de datos
    # que tiene el servidor
    for database in databases:
        # Si encuentro los datos de conexión de la base de datos dada
        if database['dbname'] == nombre_bd:
            # Guardo sus datos de conexión
            datos_bd = database
            # Termino el for
            break

    # Si no se encontraron los datos de conexión
    if datos_bd is None:
        # Retorno False y un mensaje de error
        rta['Estado'] = False
        rta['Respuesta'] = _(
            u'No se pudieron obtener los datos de conexión para la base de datos %s'
        ) % nombre_bd
        return rta

    # Si tengo los datos de conexión de la bd retorno True y un mensaje
    rta['Estado'] = True
    rta['Respuesta'] = datos_bd.dict()
    return rta


def recargar_databases():
    '''
        Esta función se encarga de volver a cargar las bases de datos.
        Agrega cada base de datos en la lista "databases".
        Si la base ya está agregada no hace nada, si es una base que no está
        en la lista la agrega.

        Retorna
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Declaro el diccionario a retornar
    rta = {}

    # Obtengo la ruta donde están los archivos .db
    ruta_dbs = '/var/lib/at/etc/dbs'
    if not os.path.isdir(ruta_dbs):
        # Si no existe la ruta retorno False y un mensaje
        rta['Estado'] = False
        rta['Respuesta'] = _(u'No se encontró la carpeta /var/lib/at/etc/dbs')
        return rta

    # Obtengo todos los archivos que están en la carpeta "dbs"
    archivos = os.listdir(ruta_dbs)

    # Recorro los archivos obtenidos
    for archivo in archivos:
        # Si el archivo no termina en .db
        if archivo.endswith('.db') is False:
            # Paso al siguiente archivo
            continue

        # Concateno la ruta del archivo con su nombre
        ruta_archivo = os.path.join(ruta_dbs, archivo)

        # Obtengo los datos de cada archivo
        datos_archivo = configobj.ConfigObj(ruta_archivo)

        # Si el archivo ya está en la lista "databases"
        if datos_archivo in databases:
            # Paso al siguiente archivo
            continue

        # Agrego los datos del archivo a la lista "databases"
        databases.append(datos_archivo)

    # Si se pudo recargar las bases de datos retorno True y un mensaje
    rta['Estado'] = True
    rta['Respuesta'] = _(u'Se volvieron a cargar las bases de datos')
    return rta
