# -*- coding: utf-8 -*-

import os
from zipfile import ZipFile
from configobj import ConfigObj

from at.utiles import _
from at.utiles.conversiones import respuesta


def es_archivo_zip(archivo):
    try:
        # Intenta crear el objeto zip
        archivo_zip = ZipFile(archivo)
        # Lo testeamos
        rta = archivo_zip.testzip()
        # Si hay respuesta, es por que fallo algo, no es zip o esta mal
        if rta is not None:
            return False
        # Sino salimos bien
        return True
    except:
        # Como tiro excepcion, salimos con un false
        return False


def crear_archivo_zip(ruta_archivo, ruta_destino, nombre_zip, carpeta_raiz,
                    carpetas_excluidas=[], archivos_excluidos=[],
                    nueva_extension=''):
    '''
        Esta función se encarga de crear un archivo ZIP.

        Argumentos:
            ruta_archivo (str path)
            ruta_destino (str path)
            nombre_zip (str)
            carpeta_raiz (str)
            carpetas_excluidas (list de str)
            archivos_excluidos (list de str)
            nueva_extension (str .ext)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    rta = {}

    # valido que la ruta destino sea válida
    if os.path.isdir(ruta_destino) is False:
        rta['Estado'] = False
        rta['Respuesta'] = _(u'La ruta destino no es válida')
        return rta

    if os.path.isfile(ruta_archivo):
        # creo el archivo ZIP
        mi_zip = ZipFile(os.path.join(ruta_destino, nombre_zip), 'w')

        # intento escribo el archivo en el ZIP
        try:
            mi_zip.write(ruta_archivo, os.path.basename(ruta_archivo))
            rta['Estado'] = True
            rta['Respuesta'] = _(u'El archivo ZIP ha sido creado correctamente')
        except Exception as ex:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo crear el archivo ZIP')

        # cierro el ZIP
        mi_zip.close()
        return rta

    # valido que la ruta a comprimir sea válida
    if os.path.isdir(ruta_archivo) is False:
        rta['Estado'] = False
        rta['Respuesta'] = _(u'La ruta a comprimir no es válida')
        return rta

    # cuento la cantidad de caracteres que tiene la ruta original
    cant_ruta_original = len(ruta_archivo)

    # no agrego la carpeta misma que se está zipiando si es que está en el
    # mismo directorio
    carpetas_excluidas.append(os.path.basename(ruta_destino))

    # no agrego el archivo que se está zipiando si es que está en el mismo
    # directorio
    archivos_excluidos.append(nombre_zip)

    # creo el archivo ZIP
    mi_zip = ZipFile(os.path.join(ruta_destino, nombre_zip), 'w')

    # recorro los archivos a comprimir
    for root, dirs, files in os.walk(ruta_archivo):
        # saco las carpetas que no están incluidas en el ZIP
        for carpeta_excluida in carpetas_excluidas:
            if carpeta_excluida in dirs:
                dirs.remove(carpeta_excluida)

        for nombre_archivo in files:
            # no agrego al ZIP los archivos que no están incluidos
            if nombre_archivo in archivos_excluidos:
                continue

            # obtengo la ruta del archivo a agregar al ZIP
            ruta_archivo = os.path.join(root, nombre_archivo)

            if nueva_extension != '':
                # obtengo el nombre y la extensión del archivo por separado
                nombre, extension = os.path.splitext(ruta_archivo)
                # le cambio la extensión del archivo a .map
                nombre_archivo = nombre.split("/")[-1] + nueva_extension

            # creo la nueva ruta del archivo dentro del ZIP
            nueva_ruta = os.path.join(carpeta_raiz, root[cant_ruta_original:],
                                                            nombre_archivo)

            # intento escribo el archivo en el ZIP
            try:
                mi_zip.write(ruta_archivo, nueva_ruta)
            except Exception as ex:
                print(ex)

    # cierro el ZIP
    mi_zip.close()

    # retorno que el archivo ZIP fue creado correctamente
    rta['Estado'] = True
    rta['Respuesta'] = _(u'El archivo ZIP ha sido creado correctamente')
    return rta


def leer_configobj(ruta_archivo):
    '''
        Esta función se encarga de devolver los datos con formato de ConfigObj
        de un archivo dado.

        Argumentos:
            ruta_archivo (str)

        Retorno:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str o dict)
    '''

    # Declaro el diccionario a retornar
    rta = {}

    try:
        # Si no existe el archivo
        if not os.path.isfile(ruta_archivo):
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No existe el archivo %s') % ruta_archivo
            return rta

        # Obtengo los datos del archivo
        # unrepr es para obtener el valor según su tipo de dato
        datos = ConfigObj(ruta_archivo, unrepr=True)

        # Paso los datos como un diccionario
        datos = datos.dict()

        # Si tengo los datos del archivo retorno True y el diccionario
        rta['Estado'] = True
        rta['Respuesta'] = datos
        return rta
    except Exception as error:
        # Si no se pudo leer el archivo con configobg retorno False y el error
        rta['Estado'] = False
        rta['Respuesta'] = 'No se pudo leer el archivo con configobj. %s' \
                           % error
        return rta


def escribir(texto, ruta_archivo, sobrescribir=False):
    '''
        Esta función se encarga de escribir un texto dado en un archivo.
        Si el archivo no existe se crea y si existe se agrega el texto al
        final del archivo.

        Parámetros:
            texto (str)
            ruta_archivo (str)
            sobrescribir (bool)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Si se debe sobrescribir el archivo
    if sobrescribir is True:
        # El modo va a ser "w" (escritura)
        modo = 'w'
    else:
        # Si no, va a ser "a" (agregar)
        modo = 'a'

    # Intento abrir el archivo según la ruta dada
    try:
        archivo = open(ruta_archivo, modo)
    except:
        # Si no existe la ruta retorno False y un mensaje
        return respuesta(False, _(u'No se encontró la ruta %s') % ruta_archivo)

    try:
        # Escribo el texto dado en el archivo
        archivo.write(texto.encode('utf8'))
    except:
        # NOTA: En Python 3 ya no se usa encode en los "str"
        archivo.write(texto)

    # Cierro el archivo
    archivo.close()

    # Si todo salió bien retorno True y un mensaje
    return respuesta(True, _(u'Se escribió el archivo de forma correcta'))


def reemplazar(texto_actual, texto_nuevo, ruta_archivo):
    '''
        Esta función se encarga de reemplazar un texto por otro
        en un archivo dado.

        Parámetros:
            texto (str)
            ruta_archivo (str)
            sobrescribir (bool)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Intento abrir el archivo según la ruta dada en modo lectura
    try:
        archivo_original = open(ruta_archivo, 'r')
    except:
        # Si no existe la ruta retorno False y un mensaje
        return respuesta(False, _(u'No se encontró la ruta %s') % ruta_archivo)

    # Guardo todo el texto del archivo
    texto = archivo_original.read()

    # Ya cierro el archivo original
    archivo_original.close()

    # De todo el contenido del archivo reemplzado el texto nuevo por el actual
    texto = texto.replace(texto_actual, texto_nuevo)

    # Vuevo a abrir el mismo archivo pero en modo escritura
    archivo_nuevo = open(ruta_archivo, 'w')

    # Escribo el archivo con todo el texto ya reemplazado
    archivo_nuevo.write(texto)

    # Cierro el nuevo archivo ya modificado
    archivo_nuevo.close()

    # Si todo salió bien retorno True y un mensaje
    return respuesta(True, _(u'Se reemplazó el archivo de forma correcta'))


def eliminar(ruta_archivo):
    '''
        Esta función se encarga de eliminar un archivo según la ruta dada.

        Parámetros:
            ruta_archivo (str)

        Retorna:
            dict
                ['Estado'] (bool)
                ['Respuesta'] (str)
    '''

    # Intento eliminar el archivo según la ruta dada
    try:
        os.remove(ruta_archivo)
    except:
        # Si no se pudo eliminar el archivo retorno False y un mensaje
        return respuesta(
            False,
            _(u'No se pudo eliminar el archivo %s') % ruta_archivo
        )

    # Si todo salió bien retorno True y un mensaje
    return respuesta(True, _(u'Se eliminó el archivo de forma correcta'))
