# -*- coding: utf-8 -*-

import time

from at.utiles import _
from at.utiles.singleton import Singleton


class ClientesConectados(object):
    '''
        Esta clase contiene funciones sobre los clientes que se conectan al
        servidor.
    '''

    def __init__(self):
        # Instancio el Singleton
        self.singleton = Singleton()

        # Si no fue creada la variable "clientes_conectados" en el Singleton
        if not hasattr(self.singleton, 'clientes_conectados'):
            # Creo la lista "clientes_conectados" en el Singleton
            self.singleton.clientes_conectados = []

        # Si no fue creada la variable "clientes_para_desconectar"
        # en el Singleton
        if not hasattr(self.singleton, 'clientes_para_desconectar'):
            # Creo la lista "clientes_para_desconectar" en el Singleton
            self.singleton.clientes_para_desconectar = []

    def grabar(self, empresa, usuario, id_sesion,
               hash_generado, actualizar_hash, permisos='', datos_extra=None,
               actualizar_datos_extra=False, ultima_consulta=None):
        '''
            Esta función se encarga de agregar un cliente en la lista
            de clientes conectados.

            Argumentos:
                empresa (str)
                usuario (str)
                id_sesion (str)
                hash_generado (str)
                actualizar_hash (bool)
                permisos (str)
                datos_extra (dict)
                actualizar_datos_extra (bool)
                ultima_consulta (dict)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Lo primero que hago es obtener la fecha y hora actual del sistema
        fecha_hora_actual = time.strftime('%Y-%m-%d %H:%M:%S')

        # Obtengo el time actual
        time_actual = time.time()

        # Si recibo la última consulta
        if ultima_consulta is not None:
            # Agrego al diccionario la hora actual
            # en el formato 'yyyy-mm-dd HH:MM:SS'
            ultima_consulta['fecha_hora'] = fecha_hora_actual

        # Primero recorro la lista de clientes conectados
        for cliente_conectado in self.singleton.clientes_conectados:
            # Si el cliente ya está en la lista
            if cliente_conectado['empresa'] == empresa \
                    and cliente_conectado['usuario'] == usuario:
                # Actualizo la sesión del cliente
                cliente_conectado['id_sesion'] = id_sesion
                # Actualizo la fecha y hora del último ping
                cliente_conectado['fecha_hora_ping'] = time_actual

                # Si actualizar_hash es True
                if actualizar_hash is True:
                    # Actualizo el hash del cliente
                    cliente_conectado['hash'] = hash_generado
                    cliente_conectado['fecha_hora_hash'] = time_actual

                # Si actualizar_datos_extra es True
                if actualizar_datos_extra is True:
                    cliente_conectado['datos_extra'] = datos_extra

                # Actualizo la última consulta realizada por el cliente
                cliente_conectado['ultima_consulta'] = ultima_consulta

                # No hago más nada, retorno True y un mensaje
                rta['Estado'] = True
                rta['Respuesta'] = _(
                    u'Se actualizó el cliente: %s'
                ) % cliente_conectado
                return rta

        # Si el cliente no está en la lista lo agrego
        cliente = {}
        cliente['empresa'] = empresa
        cliente['usuario'] = usuario
        cliente['id_sesion'] = id_sesion
        cliente['fecha_hora_ping'] = time_actual
        cliente['hash'] = hash_generado
        cliente['fecha_hora_hash'] = time_actual
        cliente['permisos'] = permisos
        cliente['datos_extra'] = datos_extra
        cliente['ultima_consulta'] = ultima_consulta
        self.singleton.clientes_conectados.append(cliente)

        # Retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se agregó el cliente: %s') % cliente
        return rta

    def eliminar(self, id_sesion):
        '''
            Esta función se encarga de eliminar un cliente de la lista de
            clientes conectados.

            Argumentos:
                id_sesion (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Primero recorro la lista de clientes conectados
        for cliente_conectado in self.singleton.clientes_conectados:
            # Busco el cliente que tenga el id_sesion a eliminar
            if cliente_conectado['id_sesion'] == id_sesion:
                # Elimino el cliente que coincide con el id_sesion dado
                self.singleton.clientes_conectados.remove(cliente_conectado)
                # Termino el for
                break

        # Retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se eliminó el cliente con sesión %s') % id_sesion
        return rta

    def listar(self, validar_ping, validar_hash=True, tiempo_maximo_hash=12):
        '''
            Esta función se encarga de lista los clientes conectados y de
            sacar de la lista a los que hace un tiempo que no envían ping.

            Argumentos:
                validar_ping (bool)
                validar_hash (bool)
                tiempo_maximo_hash (int) HORAS
                funcion_desconectar (func)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Recorro los clientes conectados
        for cliente_conectado in self.singleton.clientes_conectados:

            # Si validar_hash es True.
            if validar_hash is True:

                # Calculo el tiempo máximo en horas.
                tiempo_limite = 60 * 60 * tiempo_maximo_hash

                # Valido si el hash caducó.
                if time.time() - cliente_conectado['fecha_hora_hash'] >\
                        tiempo_limite:
                    # Saco al cliente de la lista de cliente conectados
                    self.singleton.clientes_conectados.remove(cliente_conectado)
                    # Agrego el cliente a la lista para desconectar
                    self.singleton.clientes_para_desconectar.append(
                        cliente_conectado
                    )

            # Si validar ping es True
            if validar_ping is True:
                # Si el cliente hace más de 5 minutos que no envía un PING
                if time.time() - cliente_conectado['fecha_hora_ping'] > 300:
                    # Saco al cliente de la lista de cliente conectados
                    self.singleton.clientes_conectados.remove(cliente_conectado)
                    # Agrego el cliente a la lista para desconectar
                    self.singleton.clientes_para_desconectar.append(
                        cliente_conectado
                    )

        # Retorno True y la lista de clientes conectados con su total
        rta['Estado'] = True
        rta['Respuesta'] = 'CLIENTES: %s - TOTAL: %s' % (
            self.singleton.clientes_conectados,
            len(self.singleton.clientes_conectados)
        )
        rta['ClientesConectados'] = len(self.singleton.clientes_conectados)
        return rta

    def validar_cliente(self, hash_cliente, permisos=None):
        '''
            Esta función se encarga de validar si el hash del cliente es valido.

            Argumentos:
                hash_cliente (str)
                permisos (list de str)

            Retorna:
                dict
                    ['Estado'] (True o None)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Recorro los clientes conectados
        for cliente_conectado in self.singleton.clientes_conectados:
            # Busco el cliente según el hash dado
            if cliente_conectado['hash'] == hash_cliente:
                # Si el hash del cliente tiene más de 12 horas
                if time.time() - cliente_conectado['fecha_hora_hash'] > 43200:
                    # Saco al cliente de la lista de cliente conectados
                    self.singleton.clientes_conectados.remove(cliente_conectado)
                    # Retorno None para indicar que el cliente ya no es valido
                    rta['Estado'] = None
                    rta['Respuesta'] = _(u'El hash del cliente se ha vencido')
                    return rta
                else:
                    # Si permisos no es None
                    if permisos is not None:
                        # Me fijo si el permiso del cliente conectado
                        # no está en los permisos recibidos como parámetro
                        if cliente_conectado['permisos'] not in permisos:
                            # Si el cliente no tiene permisos
                            # saco al cliente de la lista de cliente conectados
                            self.singleton.clientes_conectados.remove(
                                cliente_conectado
                            )
                            # Retorno None
                            # y un mensaje para desconectar al cliente
                            rta['Estado'] = None
                            rta['Respuesta'] = _(u'Permisos no válidos')
                            return rta

                    # Si el hash está dentro de las 12 horas
                    # retorno True y un mensaje
                    rta['Estado'] = True
                    rta['Respuesta'] = _(u'Cliente válido')
                    return rta

        # Si el hash dado no pertenece a ningún cliente
        # retorno None y un mensaje
        rta['Estado'] = None
        rta['Respuesta'] = _(u'Cliente no válido')
        return rta

    def obtener(self, id_sesion=None, empresa=None, usuario=None, hash=None):
        '''
            Esta función se encarga de obtener un cliente entre todos los
            clientes conectados, se puede buscar por el id_sesion o por la
            empresa y el id del usuario.
            También se puede buscar al cliente por su hash.

            Argumentos:
                id_sesion (str)
                empresa (str)
                id_usuario (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o dict)
                                        ['empresa'] (str)
                                        ['usuario'] (int)
                                        ['id_sesion'] (str)
                                        ['fecha_hora_ping'] (time)
                                        ['hash'] (str)
                                        ['fecha_hora_hash'] (time)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Por defecto dejo el cliente en None
        cliente = None

        # Primero recorro la lista de clientes conectados
        for cliente_conectado in self.singleton.clientes_conectados:
            # Busco el cliente que tenga el id_sesion dado
            if cliente_conectado['id_sesion'] == id_sesion:
                # Guardo el cliente obtenido
                cliente = cliente_conectado
                # Como ya encontré al cliente termino el for
                break

            # Busco el cliente que tenga la empresa y el usuario dado
            if cliente_conectado['empresa'] == empresa \
                    and cliente_conectado['usuario'] == usuario:
                # Guardo el cliente obtenido
                cliente = cliente_conectado
                # Como ya encontré al cliente termino el for
                break

            # Busco el cliente que tenga el hash dado
            if cliente_conectado['hash'] == hash:
                # Guardo el cliente obtenido
                cliente = cliente_conectado
                # Como ya encontré al cliente termino el for
                break

        # Si no se encontró el cliente con los datos dados
        if cliente is None:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se pudo obtener el cliente')
            return rta

        # Retorno True y los datos del cliente
        rta['Estado'] = True
        rta['Respuesta'] = cliente
        return rta

    def validar_ultima_consulta(self, hash, consulta):
        '''
            Esta función se encarga de validar la última consulta del cliente.

            Argumentos:
                hash (str)
                consulta (dict)

            Retorna:
                ['Estado'] (bool)
                ['Respuesta'] (str)
        '''

        # Lo primero que hago es obtener la fecha y hora actual del sistema
        fecha_hora_actual = time.strftime('%Y-%m-%d %H:%M:%S')

        # Obtengo el cliente según su HASH
        rta_obtener_cliente = self.obtener(hash=hash)
        # Si no se pudo obtener el cliente
        if rta_obtener_cliente['Estado'] is False:
            # Retorno el error
            return rta_obtener_cliente

        # Declaro el diccionario a retornar
        rta = {}

        # Si se pudo obtener el cliente lo guardo
        cliente = rta_obtener_cliente['Respuesta']

        # Agrego al diccionario la hora actual
        # en el formato 'yyyy-mm-dd HH:MM:SS'
        consulta['fecha_hora'] = fecha_hora_actual

        # Si la última consulta del cliente es igual a la que tenía guardada
        # como tengo agregada la fecha y hora en la consulta se va a validar
        # que el usuario no realice la misma consulta en el mismo segundo
        if cliente['ultima_consulta'] == consulta:
            # Retorno False y un mensaje
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'La consulta es igual a la anterior realizada en el mismo segundo'
            )
            return rta

        # Si la consulta no es repetida en el mismo segundo
        # retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(u'La consulta es válida')
        return rta
