# -*- coding: utf-8 -*-

from datetime import timedelta
from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_
from twisted.python import log

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.utiles.password import Password
from at.utiles.log import start_custom_log
from at.orm.decorators import DBSwitch

# Iniciamos nuestro propio log
start_custom_log(
    '/var/lib/at/logs/at.utiles/at.utiles', 'comandos'
)


@DBSwitch('pidetutaxi', 'pidetutaxi_central', 'pideturemis')
class Comandos(object):
    '''
        Esta clase contiene funciones útiles de los comandos.
    '''

    def __init__(self, servidor=False):
        self.dth = DateTimeHelper()
        self.password = Password()

        self.servidor = servidor

        if self.servidor is False:
            self.pidetutaxi = self.pidetutaxi_central

        if self.servidor == 'PTR':
            self.pidetutaxi = self.pideturemis

    def actualizar_estado(self, hash_comando, estado, error='', sum_cant=True):
        '''
            Esta función se encarga de actualizar el estado del comando dado.

            Argumentos:
                hash_comando (str)
                estado (int)
                error (str)
                sum_cant (bool)

            Funciones:
                DataTimeHelper.get_now()

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        session = self.pidetutaxi.binded_session()
        comandos = self.pidetutaxi.tabla('comandos')
        q_obj = session.query(
            comandos.id_comando,
            comandos.fecha_hora,
            comandos.estado,
            comandos.cant_envios,
        )
        q_obj = q_obj.filter(
            and_(
                comandos.hash_comando == hash_comando,
                or_(
                    comandos.estado == 0,
                    comandos.estado == 1,
                    comandos.estado == 2,
                )
            )
        )

        registro = q_obj.first()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró el comando dado')
            session.close()
            return rta

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        dic = {}
        dic['estado'] = estado
        dic['error'] = error
        if estado == 2 and sum_cant:
            dic['cant_envios'] = registro.cant_envios + 1
        if registro.estado == 2 and estado == 2:
            dic['fecha_hora'] = fecha_hora_actual + timedelta(seconds=15)

        # Si el estado recibido es 3 actualizo el campo fecha_confirmacion
        # con la fecha y hora actual
        if estado == 3:
            dic['fecha_confirmacion'] = fecha_hora_actual

        q_obj.update(dic, synchronize_session=False)
        session.commit()

        session.close()

        rta['Estado'] = True
        rta['Respuesta'] = _(u'Se ha actualizado el estado del comando')
        return rta

    def grabar(self, id_usuario=None, telefono='', nombre='', id_servicio=None,
                    id_ciudad=None, id_calle1=None, id_calle2=None, altura=None,
                    letra_altura='', depto='', aire=0, espacio_en_baul=0,
                    observaciones='', id_movil='', id_flota=None, mensaje='',
                    id_accion=0, origen=None, destino=None, estado=1,
                    empresa='', patente='', licencia='',
                    latitud_movil=0, longitud_movil=0,
                    fecha_hora_pos_movil=None, latitud_usuario=None,
                    longitud_usuario=None, fecha_hora_pos_usuario=None,
                    id_usuario_a_seguir=None, nombre_usuario_a_seguir=None,
                    id_estado_usuario_a_seguir=1,
                    id_subestado_usuario_a_seguir=1, moviles_cercanos='',
                    motivo_estado_usuario='', dni_chofer=None, chofer='',
                    puntaje_chofer=None, con_cta_cte=0, hash_comando='',
                    solicitado_para_a_seguir='', telefono_usuario_a_seguir='',
                    telefono_empresa='', fecha_subio_al_movil=None,
                    fecha_fin_servicio=None, tiempo_de_viaje=None,
                    velocidad_movil=0, dif_ult_pos_movil=None, datos_ctacte=''):
        '''
            Esta función se encarga de grabar un registro en la tabla comandos.

            Argumentos:
                id_usuario (int)
                telefono (str)
                nombre (str)
                id_servicio (int)
                id_ciudad (int)
                id_calle1 (int)
                id_calle2 (int)
                altura (int)
                letra_altura (str)
                depto (str)
                aire (int)
                espacio_en_baul (int)
                observaciones (str)
                id_movil (str)
                id_flota (int)
                mensaje (str)
                id_accion (int)
                origen (int)
                destino (int)
                estado (int)
                empresa (str)
                patente (str)
                licencia (str)
                latitud_movil (float)
                longitud_movil (float)
                fecha_hora_pos_movil (datetime)
                latitud_usuario (float)
                longitud_usuario (float)
                fecha_hora_pos_usuario (datetime)
                id_usuario_a_seguir (int)
                nombre_usuario_a_seguir (str)
                id_estado_usuario_a_seguir (int)
                id_subestado_usuario_a_seguir (int)
                moviles_cercanos (str)
                motivo_estado_usuario (str)
                dni_chofer (int)
                chofer (str)
                puntaje_chofer (int)
                con_cta_cte (int)
                hash_comando (str)
                solicitado_para_a_seguir (str)
                telefono_usuario_a_seguir (str)
                telefono_empresa (str)
                fecha_subio_al_movil (datetime)
                fecha_fin_servicio (datetime)
                tiempo_de_viaje (datetime)
                velocidad_movil (int)
                dif_ult_pos_movil (datetime)
                datos_ctacte (str) // debe ser un diccionario

            Funciones:
                DataTimeHelper.get_now()
                Password().generar_hash

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Obtengo la fecha y hora actual
        fecha_hora_actual = self.dth.get_now()

        # Si no recibo un hash lo genero
        if hash_comando == '':
            hash_comando = self.password.generar_hash(24) + '_' + str(
                                        fecha_hora_actual).replace(' ', '_')

        # Agrego un comando
        dic = {}
        dic['id_usuario'] = id_usuario
        dic['telefono'] = telefono
        dic['nombre'] = nombre
        dic['id_servicio'] = id_servicio
        dic['id_ciudad'] = id_ciudad
        dic['id_calle1'] = id_calle1
        dic['id_calle2'] = id_calle2
        dic['altura'] = altura
        dic['letra_altura'] = letra_altura
        dic['depto'] = depto
        dic['aire'] = aire
        dic['espacio_en_baul'] = espacio_en_baul
        dic['observaciones'] = observaciones
        dic['id_movil'] = id_movil
        dic['id_flota'] = id_flota
        dic['mensaje'] = mensaje
        dic['id_accion'] = id_accion
        dic['origen'] = origen
        dic['destino'] = destino
        dic['fecha_hora'] = fecha_hora_actual
        dic['hash_comando'] = hash_comando
        dic['error'] = ''
        dic['estado'] = estado
        dic['empresa'] = empresa
        dic['patente'] = patente
        dic['licencia'] = licencia
        dic['latitud_movil'] = latitud_movil
        dic['longitud_movil'] = longitud_movil
        dic['fecha_hora_pos_movil'] = fecha_hora_pos_movil
        dic['latitud_usuario'] = latitud_usuario
        dic['longitud_usuario'] = longitud_usuario
        dic['fecha_hora_pos_usuario'] = fecha_hora_pos_usuario
        dic['id_usuario_a_seguir'] = id_usuario_a_seguir
        dic['nombre_usuario_a_seguir'] = nombre_usuario_a_seguir
        dic['id_estado_usuario_a_seguir'] = id_estado_usuario_a_seguir
        dic['id_subestado_usuario_a_seguir'] = id_subestado_usuario_a_seguir
        dic['moviles_cercanos'] = moviles_cercanos
        dic['motivo_estado_usuario'] = motivo_estado_usuario
        dic['dni_chofer'] = dni_chofer
        dic['chofer'] = chofer
        dic['puntaje_chofer'] = puntaje_chofer
        dic['con_cta_cte'] = con_cta_cte
        dic['solicitado_para_a_seguir'] = solicitado_para_a_seguir
        dic['telefono_usuario_a_seguir'] = telefono_usuario_a_seguir
        dic['telefono_empresa'] = telefono_empresa
        dic['fecha_subio_al_movil'] = fecha_subio_al_movil
        dic['fecha_fin_servicio'] = fecha_fin_servicio
        dic['tiempo_de_viaje'] = tiempo_de_viaje
        dic['velocidad_movil'] = velocidad_movil
        dic['dif_ult_pos_movil'] = dif_ult_pos_movil
        dic['datos_ctacte'] = datos_ctacte

        # Guardo la fecha de creación, esta no se va a modificar como el campo
        # fecha_hora que puede ser modificado
        dic['fecha_creacion'] = fecha_hora_actual

        agregar = self.pidetutaxi.agregar('comandos', **dic)
        if agregar is not True:
            # Grabo en un LOG el error
            log.msg(
                'No se pudo guardar el comando. ERROR: %s - hash_comando recibido: %s'
                % (agregar, hash_comando),
                mode='comandos',
            )

            rta['Estado'] = False
            rta['Respuesta'] = \
                _(u'No se pudo agregar el comando a la base de datos')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = _(u'El comando se pudo agregar correctamente')
        return rta

    def existe(self, hash_comando):
        '''
            Esta función se encarga de validar si el comando existe en la base
            de datos o no.

            Argumento:
                hash_comando (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        session = self.pidetutaxi.binded_session()
        comandos = self.pidetutaxi.tabla('comandos')
        q_obj = session.query(comandos.id_comando)
        q_obj = q_obj.filter(comandos.hash_comando == hash_comando)

        if q_obj.count() == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'Comando nuevo')
        else:
            rta['Estado'] = True
            rta['Respuesta'] = _(u'Comando existente')

        session.close()
        return rta

    def finalizar(self, id_servicio):
        '''
            Esta función se encarga de finalizar todos los comandos que todavía
            no se han enviado del servicio dado.

            Argumentos:
                id_servicio (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        rta = {}

        # Obtengo los comandos del servicio que todavía no fueron finalizados
        # Hay comandos que si o si se deben enviar para que se finalice el
        # servicio en el celular o en el radio taxi
        session = self.pidetutaxi.binded_session()
        comandos = self.pidetutaxi.tabla('comandos')
        q_obj = session.query(comandos.id_comando)
        q_obj = q_obj.filter(
            and_(
                comandos.id_servicio == id_servicio,
                or_(
                    comandos.estado == 1,
                    comandos.estado == 2,
                ),
                comandos.id_accion != 105,
                comandos.id_accion != 108,
                comandos.id_accion != 112,
                comandos.id_accion != 114,
                comandos.id_accion != 202,
            )
        )
        registro = q_obj.first()

        # Si no encuentro ningún comando para finalizar retorno el mensaje
        if registro is None:
            session.close()
            rta['Estado'] = False
            rta['Respuesta'] = _(u'Los comandos ya estaban finalizados')
            return rta

        # Actualizo los comandos que no fueron finalizados
        # Los paso al estado 5 para que no se sigan enviando al usuario
        dic = {}
        dic['estado'] = 5
        dic['error'] = \
            _(u'Se finaliza el comando porque el servicio ya está finalizado')
        q_obj.update(dic, synchronize_session=False)
        session.commit()
        session.close()

        # Retorno el mensaje
        rta['Estado'] = True
        rta['Respuesta'] = \
            _(u'Todos los comandos del servicio fueron finalizados')
        return rta
