# -*- coding: utf-8 -*-

from at.utiles import _
from at.utiles.singleton import Singleton
from at.orm.utiles import seleccionar_bd


class EmpresasConectadas(object):
    '''
        Esta clase contiene funciones sobre las empresas que se conectan al
        servidor.
    '''

    def __init__(self):
        # Instancio el Singleton
        self.singleton = Singleton()

        # Si no fue creada la variable "dic_empresas" en el Singleton
        if not hasattr(self.singleton, 'dic_empresas'):
            # Creo el diccionario "dic_empresas" en el Singleton
            self.singleton.dic_empresas = {}

    def grabar_clase(self, nombre_empresa, nombre_clase, clase, empresa_local):
        '''
            Esta función se encarga de grabar una clase a una empresa en el
            diccionario de empresas.

            Argumentos:
                nombre_empresa (str)
                nombre_clase (str)
                clase (object)
                empresa_local (bool)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro un diccionario a retornar
        rta = {}

        # Si es una empresa local, o sea, que el servidor corre en la empresa
        if empresa_local is True:
            # Llamo a la empresa "local"
            nombre_empresa = 'local'

        # Si la empresa no fue agregada al diccionario
        if self.singleton.dic_empresas.get(nombre_empresa) is None:
            # Creo un diccionario donde se van a guardar
            # las clases de la empresa
            dic_clases = {}
            dic_clases[nombre_clase] = clase
            # Guardo la clase en la llave de la empresa
            self.singleton.dic_empresas[nombre_empresa] = dic_clases

            # Si se pudo agregar la empresa con su clase al diccionario
            # Retorno True y un mensaje
            rta['Estado'] = True
            rta['Respuesta'] = _(
                u'Se grabó correctamente la clase %s de la empresa %s'
            ) % (nombre_clase, nombre_empresa)
            return rta

        # Si la empresa ya existe en el diccionario agrego la clase dada
        self.singleton.dic_empresas[nombre_empresa][nombre_clase] = clase

        # Si se pudo modificar el diccionario de empresas
        # retorno True y un mensaje
        rta['Estado'] = True
        rta['Respuesta'] = _(
            u'Se grabó correctamente la clase %s de la empresa %s'
        ) % (nombre_clase, nombre_empresa)
        return rta

    def obtener_clase(self, nombre_empresa, nombre_clase, empresa_local):
        '''
            Esta función se encarga de obtener la clase dada de la empresa dada.

            Argumentos:
                nombre_empresa (str)
                nombre_clase (str)
                empresa_local (bool)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o class)
        '''

        # Declaro un diccionario a retornar
        rta = {}

        # Si es una empresa local, o sea, que el servidor corre en la empresa
        if empresa_local is True:
            # El nombre de la empresa va a ser siempre "local"
            nombre_empresa = 'local'

        # Me fijo si tengo el nombre de la empresa en el diccionario de empresas
        if self.singleton.dic_empresas.get(nombre_empresa) is None:
            # Si la empresa no está en el diccionario retorno False y el error
            rta['Estado'] = False
            rta['Respuesta'] = _(
                u'No se encontró la empresa %s'
            ) % nombre_empresa
            return rta

        # Me fijo si la empresa dada tiene la clase dada
        if self.singleton.dic_empresas[nombre_empresa].get(nombre_clase) is None:
            # Si no se encontró la clase dada retorno False y el error
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró la clase %s') % nombre_clase
            return rta

        # Obtengo la clase dada según el nombre de la empresa recibido
        clase_empresa = \
            self.singleton.dic_empresas[nombre_empresa][nombre_clase]

        # Retorno True y la clase de la empresa
        rta['Estado'] = True
        rta['Respuesta'] = clase_empresa
        return rta

    def listar(self):
        '''
            Esta función se encarga de listar todas las empresas conectadas
            que están guardadas en el diccionario de empresas que está en el
            singleton.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Retorno True y la lista de empresas conectadas con su total
        rta['Estado'] = True
        rta['Respuesta'] = 'EMPRESAS: %s - TOTAL: %s' % (
            self.singleton.dic_empresas,
            len(self.singleton.dic_empresas)
        )
        return rta

    def obtener_clase_bd(
            self, clase, nombre_empresa=None, nombre_bd_a_cambiar='cp'):
        '''
            Esta función se encarga de obtener la clase de la empresa.
            Si la clase no está instancia y guardada en la empresa se debe
            agregar junto con los demás datos de la empresa dada.

            Argumentos:
                clase (object) // clase sin instanciar
                nombre_empresa (str)
                nombre_bd_a_cambiar (str)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str u object // clase instanciada)
        '''

        # Declaro el diccionario a retornar
        rta = {}

        # Si no tengo un nombre de empresa
        if nombre_empresa is None:
            # Solo instancio la clase dada
            clase_instanciada = clase()
            # Retorno True y la clase instanciada
            rta['Estado'] = True
            rta['Respuesta'] = clase_instanciada
            return rta

        # Obtengo el nombre completo de la clase dada
        nombre_clase = clase.__module__ + "." + clase.__name__

        # Obtengo la clase de la empresa dada
        rta_obtener_clase = self.obtener_clase(
            nombre_empresa, nombre_clase, False
        )
        # Si se pudo obtener la clase de la empresa
        if rta_obtener_clase['Estado'] is True:
            # Retorno True y la clase de la empresa
            # ya instanciada y con la BD ya pisada
            clase_instanciada = rta_obtener_clase['Respuesta']
            rta['Estado'] = True
            rta['Respuesta'] = clase_instanciada
            return rta

        # Si la empresa no tiene la clase que se va a usar
        # Instancio la clase dada
        clase_instanciada = clase()

        # Cambio la BD por la que le corresponde a la empresa
        rta_seleccionar_bd = seleccionar_bd(
            clase_instanciada, nombre_bd_a_cambiar, nombre_empresa, False
        )
        # Si no se pudo seleccionar la BD
        if rta_seleccionar_bd['Estado'] is False:
            # Retorno el error
            return rta_seleccionar_bd

        # Grabo la clase en la empresa
        # La clase guardada ya tiene la BD pisada por la de la empresa
        self.grabar_clase(
            nombre_empresa, nombre_clase, clase_instanciada, False
        )

        # Retorno True y la clase de la empresa con la BD pisado
        rta['Estado'] = True
        rta['Respuesta'] = clase_instanciada
        return rta
