# -*- coding: utf-8 -*-

import sys
import traceback
from sqlalchemy.sql.expression import and_

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.orm.decorators import DBSwitch


def obtener_traceback():
    exc_type, exc_value, exc_traceback = sys.exc_info()
    return str(traceback.extract_tb(exc_traceback))


@DBSwitch('cp')
class Errores(object):
    '''
        Clase que contiene funciones para manejar los errores.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()

    def guardar_error(self, aplicacion, descripcion, id_sucursal=0,
                        nombre_funcion=None, id_movil=None, telefono=None,
                        hora_servicio=None):
        '''
            Esta función graba en la tabla errorlog los errores ocurridos.

            Argumentos:
                aplicacion (str)
                descripcion (str)
                nombre_funcion (str)
                id_sucursal (int)
                id_movil (str)
                telefono (str)
                hora_servicio (str)

            Funciones:
                get_now

            Retorna:
                dic['Estado'] (bool)
                dic['Respuesta'] (str)
        '''

        dic = {}

        fecha_hora_actual = self.dth.get_now()

        d = {}
        d['FechaHora'] = fecha_hora_actual
        d['Aplicacion'] = aplicacion
        d['FunctionName'] = nombre_funcion
        d['ErrorDescription'] = descripcion
        d['IdSucursal'] = id_sucursal
        d['IdMovil'] = id_movil
        d['Telefono'] = telefono
        d['FechaHoraServicio'] = hora_servicio

        agregar = self.cp.agregar('errorlog', **d)
        if agregar is True:
            dic['Estado'] = True
            dic['Respuesta'] = _(u'Se ha guardado el error.')
        else:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No se pudo guardar el error.')

    def listar_errores(self, aplicacion, id_sucursal=0, limite=5):
        '''
            Esta función debe devolver los últimos 5 errores de una aplicación
            dada.

            Argumento:
                aplicacion (str)
                id_sucursal (int)
                limite (int)

            Retorna:
                dic['Estado'] (bool)
                dic['Respuesta'] (list o str)
        '''

        dic = {}

        session = self.cp.binded_session()

        errorlog = self.cp.tabla('errorlog')

        q_obj = session.query(
                            errorlog.IdError,
                            errorlog.FechaHora,
                            errorlog.FunctionName,
                            errorlog.ErrorDescription,
                            errorlog.IdMovil,
                            errorlog.Telefono,
                            errorlog.FechaHoraServicio,
                            )

        q_obj = q_obj.filter(and_(
                                errorlog.Aplicacion == aplicacion,
                                errorlog.IdSucursal == id_sucursal,
                                )
                            )

        q_obj = q_obj.order_by(errorlog.IdError.desc())

        q_obj = q_obj.limit(limite)

        if q_obj.count() > 0:
            registros = q_obj.all()

            errores = []
            for registro in registros:
                if registro.FechaHora is not None:
                    registro.FechaHora = registro.FechaHora.strftime(
                                                        "%d/%m/%Y %H:%M:%S")

                if registro.FechaHoraServicio is not None:
                    registro.FechaHoraServicio = registro.FechaHoraServicio.strftime("%H:%M:%S")

                error = [
                    registro.IdError,
                    registro.FechaHora,
                    registro.FunctionName,
                    registro.ErrorDescription,
                    registro.IdMovil,
                    registro.Telefono,
                    registro.FechaHoraServicio,
                    ]

                errores.append(error)

            dic['Estado'] = True
            dic['Respuesta'] = errores
        else:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No se encontraron errores.')

        session.close()

        return dic

    def borrar_errores(self, aplicacion, id_sucursal=0):
        '''
            Esta función debe borrar todos los registros que están en la
            tabla errorlog de la aplicación y la sucursal dada.

            Argumento:
                aplicacion (str)
                id_sucursal (int)

            Retorna:
                dic['Estado'] (bool)
                dic['Respuesta'] (str)
        '''

        dic = {}

        try:
            dato = {}
            dato['Aplicacion'] = ('==', aplicacion)
            dato['IdSucursal'] = ('==', id_sucursal)
            self.cp.borrar('errorlog', **dato)
            dic['Estado'] = True
            dic['Respuesta'] = _(u'Se han borrado los errores.')
        except:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No se pudieron borrar los errores.')

        return dic
