# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_
from sqlalchemy.sql.expression import or_

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.utiles.conversiones import respuesta
from at.orm.decorators import DBSwitch


@DBSwitch('cp')
class Feriados(object):
    '''
        Esta clase contiene funciones para administrar los días feriados.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()

    def obtener(self):
        '''
            Esta función se encarga de obtener todos los días feriados del año
            actual.

            Funciones:
                at.utiles.conversiones -> respuesta
                at.utiles.servidores -> DateTimeHelper -> get_year

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str) o ['Dato'] (list de str // yyyy-mm-dd)
        '''

        # Obtengo el año actual
        anio_actual = self.dth.get_year()
        # Paso el año actual a str
        anio_actual = str(anio_actual)

        # Abro la sesión con la base de datos
        session = self.cp.binded_session()

        # Obtengo la tabla "feriados"
        feriados = self.cp.tabla('feriados')

        # Obtengo la fecha de cada feriado
        q_obj = session.query(feriados.Fecha)

        # Ordeno por fecha de forma ascendente
        q_obj = q_obj.order_by(feriados.Fecha.asc())

        # Filtro para obtener los feriados del año actual
        # También tengo en cuenta los feriados con el año 1900, esto quiere
        # decir que se repite todos los años
        q_obj = q_obj.filter(
            or_(
                and_(
                    feriados.Fecha >= anio_actual + '-01-01',
                    feriados.Fecha <= anio_actual + '-12-31'
                ),
                and_(
                    feriados.Fecha >= '1900-01-01',
                    feriados.Fecha <= '1900-12-31'
                )
            )
        )

        # Guardo los registros obtenidos
        registros = q_obj.all()

        # Cierro la sesión con la base de datos
        session.close()

        # Si no se encontró ningún registro
        if len(registros) == 0:
            # Retorno False y un mensaje de error
            return respuesta(
                False, _(u'No se pudieron obtener los días feriados')
            )

        # Creo una lista para guardar todos los feriados del año actual
        feriados = []

        # Recorro los registros
        for registro in registros:
            # Agrego cada registro a la lista de feriados
            feriados.append(str(registro.Fecha))

        # Retorno True y la lista de feriados
        return respuesta(True, dato=feriados)

    def __es_feriado(self, fecha):
        '''
            Esta función indica si una fecha dada es un día feriado o no.

            Argumentos:
                fecha (str) // yyyy-mm-dd

            Funciones:
                at.utiles.conversiones -> respuesta
                at.utiles.feriados -> Feriados -> obtener

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo todos los feriados del año actual
        rta = self.obtener()
        # Si no se pudieron obtener los feriados
        if rta['Estado'] is False:
            # Retorno False y un mensaje
            return rta

        # Guardo los feriados del año
        feriados = rta['Dato']

        # Recorro los feriados obtenidos
        for feriado in feriados:
            # Si el feriado tiene el año "1900" quiere decir que sirve para
            # todos los años
            if feriado[:4] == '1900':
                # Tomo el año de la fecha dada
                feriado = fecha[:4] + feriado[4:]

            # Si la fecha es igual a una fecha que es feriado
            if feriado == fecha:
                # Retorno True y un mensaje
                return respuesta(True, _(u'Es feriado'))

        # Si la fecha no está entre la lista de feriados retorno False
        return respuesta(False, _(u'No es feriado'))

    def es_feriado(self):
        '''
            Esta función determina si la fecha actual es un día feriado o no.

            Funciones:
                at.utiles.servidores -> DateTimeHelper -> get_date
                at.utiles.feriados -> Feriados -> __es_feriado

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Obtengo la fecha actual en formato str
        fecha_actual = self.dth.get_date(date_str=True)

        # Me fijo si la fecha actual es un día feriado o no
        rta = self.__es_feriado(fecha_actual)
        return rta
