# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.orm.decorators import DBSwitch
try:
    from at.map.pais import PaisPostgres
except:
    print('>> warn: No podemos importar at.map')


@DBSwitch('cp')
class Flotas(object):
    '''
        Esta clase contiene funciones útiles de las flotas.
    '''

    def __init__(self):
        # Intentamos instanciar
        try:
            self.pais = PaisPostgres()
        except:
            self.pais = None

        # Dth
        self.dth = DateTimeHelper()

    def conectadas(self):
        '''
            Esta función se encarga de devolver un listado de las flotas que
            están conectadas.

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de int)
        '''

        rta = {}

        session = self.cp.binded_session()

        flotas = self.cp.tabla('flotas')
        q_obj = session.query(flotas.IdFlota)
        q_obj = q_obj.filter(flotas.Conectada == 1)
        q_obj = q_obj.order_by(flotas.IdFlota.asc())

        registros = q_obj.all()

        session.close()

        if len(registros) == 0:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontraron flotas conectadas')
            return rta

        lista = []
        for registro in registros:
            lista.append(registro.IdFlota)

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def listar(self):
        '''
            Esta función se encarga de listar todas las flotas existentes
            en la base de datos cp2000

            Retorna:
                list de dict
                        ['id_flota'] (int)
                        ['nombre'] (str)
        '''

        rta = {}

        session = self.cp.binded_session()

        flotas = self.cp.tabla('flotas')
        q_obj = session.query(flotas.IdFlota, flotas.Nombre)
        q_obj = q_obj.order_by(flotas.IdFlota.asc())

        registros = q_obj.all()

        session.close()

        lista = []
        for registro in registros:
            dic = {}
            dic['id_flota'] = registro.IdFlota
            try:
                dic['nombre'] = registro.Nombre.decode('latin1')
            except:
                # NOTA: En Python 3 no se usa decode en los "str"
                dic['nombre'] = registro.Nombre
            lista.append(dic)

        rta['Estado'] = True
        rta['Respuesta'] = lista
        return rta

    def obtener(self, id_flota, id_cliente=None, id_sucursal=None):
        '''
            Esta función se encarga de devolver todos los datos de una flota
            dada.

            Argumentos:
                id_flota (int)
                id_cliente (int)
                id_sucursal (int)

            Rertorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (dict)
                                    ['IdFlota'] (int)
                                    ['Nombre'] (str)
                                    ['UltConexion'] (datetime)
                                    ['Orden'] (int)
                                    ['Latitud'] (float)
                                    ['Longitud'] (float)
                                    ['id_cliente'] (int)
                                    ['id_sucursal'] (int)
                                    ['IdCiudad'] (int)
                                    ['usuario_cliente'] (str)
                                    ['Conectada'] (int)
                                    ['Tipo'] (int)
                                    ['IdSucursalEmpresa'] (int)
        '''

        rta = {}

        session = self.cp.binded_session()

        flotas = self.cp.tabla('flotas')

        q_obj = session.query(flotas)

        if id_cliente is not None and id_sucursal is not None:
            q_obj = q_obj.filter(
                and_(
                    flotas.id_cliente == id_cliente,
                    flotas.id_sucursal == id_sucursal,
                )
            )
        else:
            q_obj = q_obj.filter(flotas.IdFlota == id_flota)

        registro = q_obj.first()

        session.close()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró la flota dada')
            return rta

        datos = {}
        datos['id_flota'] = registro.IdFlota
        datos['nombre'] = registro.Nombre
        datos['ult_conexion'] = registro.UltConexion
        datos['orden'] = registro.Orden
        datos['latitud'] = registro.Latitud
        datos['longitud'] = registro.Longitud
        datos['id_cliente'] = registro.id_cliente
        datos['id_sucursal'] = registro.id_sucursal
        datos['id_ciudad'] = registro.IdCiudad
        datos['usuario_cliente'] = registro.usuario_cliente
        datos['conectada'] = registro.Conectada
        datos['tipo'] = registro.Tipo
        datos['id_sucursal_empresa'] = registro.IdSucursalEmpresa

        rta['Estado'] = True
        rta['Respuesta'] = datos
        return rta

    def obtener_fecha_hora_flota(self, id_flota):
        '''
            Esta función se encarga de obtener la fecha y hora según la zona
            horaria de la ciudad de la flota dada.

            Argumentos:
                id_flota (int)

            Funciones:
                Flotas().obtener
                DateTimeHelper().obtener_fecha_hora_ciudad

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o datetime)
        '''

        # Obtengo los datos de la flota dada
        rta_obtener_flota = self.obtener(id_flota)
        if rta_obtener_flota['Estado'] is False:
            return rta_obtener_flota

        # Guardo la ciudad de la flota
        id_ciudad_flota = rta_obtener_flota['Respuesta']['id_ciudad']

        # Llamo a la función obtener_fecha_hora_ciudad
        rta_obtener_fecha_hora_ciudad = self.dth.obtener_fecha_hora_ciudad(
                                                                id_ciudad_flota)

        # Retorno la respuesta de la función obtener_fecha_hora_ciudad
        return rta_obtener_fecha_hora_ciudad
