# -*- coding: utf-8 -*-

import time
from multiprocessing import Process
from multiprocessing import Manager

from at.utiles.hilos import ejecutar_hilo


def ejecutar_proceso(funcion, parametros=(), daemon=True, func_rta=None):
    '''
        Este método se encarga de ejecutar una función en otro proceso.

        Argumentos:
            funcion (function)
            parametros (tuple)
            deamon (bool)
            func_rta (function)

        Retorna:
            None
    '''

    # Si tengo una función de respuesta
    if func_rta is not None:
        # Agrego a la función principal un decorador con la función de respuesta
        funcion = con_rta(funcion, func_rta)

    # Creo un proceso con la función recibida y sus parámetros
    proceso = Process(
        target=funcion,
        args=parametros,
    )
    # Si es un demonio
    if daemon is True:
        # Seteo deamon en True para que se muera junto con la función
        proceso.daemon = True
    # Inicio el hilo
    proceso.start()


def ejecutar_tarea(funcion, parametros, segundos):
    '''
        Este método se encarga de ejecutar una función como una tarea
        en un subproceso cada el tiempo que se le indique en segundos.

        Argumentos:
            funcion (function)
            parametros (tuple)
            segundos (int)

        Funciones:
            ejecutar_proceso

        Retorna:
            None
    '''

    # Creo una función
    def tarea():
        # Esta se va a ejecutar infinitamente
        while True:
            # Cada tanto tiempo según los segundos seteados
            time.sleep(segundos)
            # Ejecuto la función dada con sus parámetros
            funcion(*parametros)
    # Ejecuto la función "tarea" en un subproceso
    ejecutar_proceso(tarea)


def ejecutar_multiples_procesos(funciones, daemon=True,
                                func_fin_procesos=None, valor_rta=None,
                                max_procesos=4):
    '''
        Esta función se encarga de ejecutar varias funciones al mismo tiempo.
        Si se le pasa una función de respuesta se va a ejecutar cuando terminen
        todos los procesos.
        Si se le pasa un valor de respuesta se le debe pasar también una función
        de respuesta que reciba un parámetro, esta se va a ejecutar recibiendo
        el valor pasado.
        Se puede setear cuantos procesos se van a ejecutar en simultáneo.

        Argumentos:
            funciones (list de list) [[function, ()], [function, ()]]
                    // Se le debe pasar cada función con sus parámetros
            deamon (bool)
            func_fin_procesos (function)
                    // Si no tengo valor_rta la función no recibe parámetros
            valor_rta
                    // Si recibo un valor de respuesta la función debe recibir
                    // un parámetro
            max_procesos (int)
                    // Cantidad de procesos que se quieren ejecutar al mismo
                    // tiempo, debe ser mayor o igual a 1

        Retorna:
            None
    '''

    # Si max_procesos es menor o igual a 0
    if max_procesos <= 0:
        # Seteo max_procesos en 1
        max_procesos = 1

    # Ejecuto las funciones en un hilo para que puedan ser controladas
    # y saber cuando terminan todas
    ejecutar_hilo(
        __hilo_multiprocesos,
        parametros=(
            funciones,
            daemon,
            func_fin_procesos,
            valor_rta,
            max_procesos
        ),
        daemon=daemon
    )


# FUNCIONES PRIVADAS ##########################################################


def __hilo_multiprocesos(funciones, daemon, func_fin_procesos, valor_rta,
                         max_procesos):
    '''
        Este función se ejecuta en un hilo y se encarga de ejecutar
        varias funciones y esperar que terminen para luego ejecutar
        una función de respuesta o no.

        Argumentos:
            funciones (list de list) [[function, ()], [function, ()]]
                    // Se le debe pasar cada función con sus parámetros
            deamon (bool)
            func_fin_procesos (function)
                    // Si no tengo valor_rta la función no recibe parámetros
            valor_rta
                    // Si recibo un valor de respuesta la función debe recibir
                    // un parámetro
            max_procesos (int)

        Retorna:
            None
    '''

    # Creo una lista con Manager de multiprocessing
    # Esta lista se comparte entre procesos
    lista_de_procesos = Manager().list()

    # Armo una lista con el id único para identificar cada función
    for funcion in funciones:
        lista_de_procesos.append(id(funcion[0]))

    # Si la cantidad de procesos totales pendientes es menor a la
    # cantidad de procesos que se deben ejecutar
    if len(lista_de_procesos) < max_procesos:
        # Indico que se deben ejecutar todos los procesos juntos
        max_procesos = len(lista_de_procesos)

    # Seteo el número de proceso en 0
    num_proceso = 0

    # Recorro todas las funciones recibidas
    for funcion in funciones:
        # Ejecuto cada función en un proceso y paso la lista de procesos
        __ejecutar_multiproceso(
            funcion[0],
            funcion[1],
            daemon,
            lista_de_procesos
        )

        # Aumento 1 para indicar que paso al siguiente proceso
        num_proceso += 1

        # Si el número de proceso es igual al máximo de procesos
        # que se deben ejecutar
        if num_proceso == max_procesos:
            # Espero que terminen los procesos que mandé a ejecutar
            # Guardo en una variable fija la cantidad de procesos totales
            cant_procesos = len(lista_de_procesos)
            while True:
                # Si ya no hay procesos para ejecutar
                if len(lista_de_procesos) == 0:
                    # Dejo de esperar
                    break

                # Si se ejecutaron todos los procesos pendientes
                if len(lista_de_procesos) == cant_procesos - max_procesos:
                    # Dejo de esperar para pasar a los siguientes pendientes
                    break

                # Duermo un segundo mientras espero que terminen los procesos
                time.sleep(1)

            # Si la cantidad de procesos totales pendientes es menor a la
            # cantidad de procesos que se deben ejecutar
            if len(lista_de_procesos) < max_procesos:
                # Indico que se deben ejecutar todos los procesos restantes
                max_procesos = len(lista_de_procesos)

            # Reseteo a 0 el número de procesos para ejecutar los siguientes
            num_proceso = 0

    # Si no tengo una función de respuesta
    # para cuando terminen todos los procesos
    if func_fin_procesos is None:
        # No retorno nada
        return

    # Si no tengo un valor fijo de respuesta
    if valor_rta is None:
        # Ejecuto la función de respuesta sin pasarle parámetros
        return func_fin_procesos()

    # Si tengo la función de respuesta y un valor fijo
    # ejecuto la función de respuesta pasandole el valor fijo
    return func_fin_procesos(valor_rta)


def __ejecutar_multiproceso(funcion, parametros, daemon, lista_de_procesos):
    '''
        Este método se encarga de ejecutar una función como un multiproceso.

        Argumentos:
            funcion (function)
            parametros (tuple)
            deamon (bool)
            lista_de_procesos (list de int)

        Retorna:
            None
    '''

    # Agrego a la función un decorador para eliminarla de la lista de procesos
    # cuando termine su ejecución
    funcion = validar_fin_proceso(funcion, lista_de_procesos)

    # Ejecuto cada función en un proceso distinto
    ejecutar_proceso(
        funcion,
        parametros,
        daemon=daemon
    )


###############################################################################


# DECORADORES #################################################################

# Este decorador ejecuta una función de respuesta al terminar la ejecución
# de la función principal
def con_rta(func, func_rta):
    def ejecutar(*args, **kwargs):
        # Ejecuto la función decorada
        rta = func(*args, **kwargs)
        # Ejecuto la función de respuesta pasandole como parámetro el resultado
        # de la función original
        func_rta(rta)
        return rta
    return ejecutar


# Este decorador elimina de la lista de procesos al procesos que fue finalizado
def validar_fin_proceso(func, lista_de_procesos):
    def ejecutar(*args, **kwargs):
        # Ejecuto la función original
        rta = func(*args, **kwargs)
        # Ni bien termina la función la elimino de la lista de procesos
        lista_de_procesos.remove(id(func))
        return rta
    return ejecutar

###############################################################################
