# -*- coding: utf-8 -*-

from at.utiles import _
from at.utiles.conversiones import respuesta
from at.utiles import validar
from at.orm.decorators import DBSwitch


@DBSwitch('cp')
class Sucursales(object):
    '''
        Esta clase contiene funciones para trabajar con las sucursales de
        la base de datos cp2000.
    '''

    def obtener_id_sucursales(self):
        '''
            Esta función se encarga de obtener el id de todas las sucursales
            que existan en la base de datos del cliente.

            Retorna:
                list de int
        '''

        session = self.cp.binded_session()

        sucursales = self.cp.tabla('sucursales')
        q_obj = session.query(sucursales.Sucursal)
        q_obj = q_obj.order_by(sucursales.Sucursal.asc())

        registros = q_obj.all()

        sucursales = []
        for registro in registros:
            sucursales.append(registro.Sucursal)

        session.close()

        return sucursales

    def obtener_licencia_empresa(self, id_sucursal):
        '''
            Esta función se encarga de obtener el número de licencia de una
            empresa dada su número de sucursal.

            Argumentos:
                id_sucursal (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (int o str)
        '''

        dic = {}

        session = self.cp.binded_session()

        sucursales = self.cp.tabla('sucursales')
        q_obj = session.query(sucursales.LicenciaEmpresa)
        q_obj = q_obj.filter(sucursales.Sucursal == id_sucursal)
        registro = q_obj.first()

        if registro is None:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No se pudo obtener el número de licencia de la empresa')
        else:
            dic['Estado'] = True
            dic['Respuesta'] = registro.LicenciaEmpresa

        session.close()

        return dic

    def obtener_datos(self, id_sucursal):
        '''
            Esta función se encarga de obtener todos los datos de una sucursal
            dada.

            Argumentos:
                id_sucursal (int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (object o str)
                                    .Sucursal (int)
                                    .Nombre (str)
                                    .CierreDeTruno (datetime)
                                    .idMapa (int)
                                    .LicenciaEmpresa (int)
                                    .Despachando (int)
        '''

        rta = {}

        session = self.cp.binded_session()

        sucursales = self.cp.tabla('sucursales')
        q_obj = session.query(sucursales)
        q_obj = q_obj.filter(sucursales.Sucursal == id_sucursal)
        registro = q_obj.first()
        session.close()

        if registro is None:
            rta['Estado'] = False
            rta['Respuesta'] = _(u'No se encontró la sucursal')
            return rta

        rta['Estado'] = True
        rta['Respuesta'] = registro

        return rta

    def activar_desactivar_despacho(self, id_sucursal, activar):
        '''
            Esta función se encarga de modificar el campo "Despachando"
            de la tabla "sucursales" y ponerlo en 0 o 1.

            Argumento:
                id_sucursal (int)
                activar (int) // 1 = despachando;   0 = no despachando

            Funciones:
                at.utiles -> conversiones -> respuesta
                at.utiles -> validar -> numero_entero_positivo

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        # Valido el dato "id_sucursal" recibido
        rta = validar.numero_entero_positivo(
            id_sucursal, nombre_dato='id_sucursal'
        )
        if rta['Estado'] is False:
            return rta

        # Valido el dato "activar" recibido
        rta = validar.numero_entre(activar, 0, 1, nombre_dato='activar')
        if rta['Estado'] is False:
            return rta

        # Abro la sesión con la BD "cp2000"
        session = self.cp.binded_session()

        # Obtengo la tabla "sucursales"
        sucursales = self.cp.tabla('sucursales')

        # Obtengo la sucursal dada
        q_obj = session.query(sucursales.Sucursal)

        # Filtro por el id de la sucursal dado
        q_obj = q_obj.filter(sucursales.Sucursal == id_sucursal)

        # Actualizo el campo "Despachando" de la tabla "sucursales"
        dic = {}
        dic['Despachando'] = activar

        # Realizo el update con la BD
        rta_update = q_obj.update(dic)

        # Realizo el commit
        session.commit()

        # Cierro la sesión con la BD
        session.close()

        # Si no se encontró la sucursal dada
        if rta_update == 0:
            # Retorno False y un mensaje
            return respuesta(
                False,
                _(u'No se pudo cambiar el estado del despacho de la sucursal %s') % id_sucursal
            )

        # Si el valor de activar es 1
        if activar == 1:
            # Quiere decir que se activó el despacho
            mensaje = _(
                u'Se activa el despacho para la sucursal %s'
            ) % id_sucursal
        else:
            # Si no recibo el valor 1 quiere decir que se desactivó el despacho
            mensaje = _(
                u'Se desactiva el despacho para la sucursal %s'
            ) % id_sucursal

        # Si se pudo modificar el estado del despacho retorno True y un mensaje
        return respuesta(True, mensaje)
