# -*- coding: utf-8 -*-
from at.utiles import tokens
from datetime import datetime, timedelta
from jose import jws
from jose.utils import base64url_encode
import json

import time
import unittest



class TestTokens(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_generar(self):
        # Fallamos con los datos
        rta = tokens.generar(2, 2, 2, 2, 2, 2, 2)
        self.assertFalse(rta['Estado'])

        # Generamos datos
        hash = 'hash_unico_del_token'
        clave = '1234567890987654321'
        datos = {'id': 'asd'}
        emisora = 'PideTuTaxi'
        receptora = 'AtPagos'

        # Generamos correctamente
        rta = tokens.generar(datos, clave, hash, emisora, receptora)
        self.assertTrue(rta['Estado'])

        # Generamos con limite de tiempo de 10 minutos
        rta = tokens.generar(datos, clave, hash, emisora, receptora, 10)
        self.assertTrue(rta['Estado'])

        # Generamos con inicio de tiempo
        rta = tokens.generar(datos, clave, hash, emisora, receptora, 10, datetime.now())
        self.assertTrue(rta['Estado'])

        # Generamos con inicio de tiempo e ip
        rta = tokens.generar(datos, clave, hash, emisora, receptora, 10, datetime.now(), '127.0.0.1')
        self.assertTrue(rta['Estado'])

    def test_procesar(self):
        # Generamos datos
        hash = 'hash_unico_del_token'
        clave = '1234asdlasd567890987654321'
        datos = {'id': 'asd'}
        emisora = 'PideTuTaxi'
        receptora = 'AtPagos'

        # Generamos correctamente
        rta = tokens.generar(datos, clave, hash, emisora, receptora)
        self.assertTrue(rta['Estado'])

        # Obtenemos el token
        token = rta['Respuesta']

        # Vamos a procesar mal
        rta = tokens.procesar(1, 1, 1, 1, 1)
        self.assertFalse(rta['Estado'])

        # Vamos a procesar mal por la clave
        rta = tokens.procesar(token, '1a2323', hash, emisora, receptora)
        self.assertFalse(rta['Estado'])

        # Vamos a procesar mal por el hash
        rta = tokens.procesar(token, clave, '12312s3', emisora, receptora)
        self.assertFalse(rta['Estado'])

        # Vamos a procesar mal por la emisora
        rta = tokens.procesar(token, clave, hash, 'asd232', receptora)
        self.assertFalse(rta['Estado'])

        # Vamos a procesar mal por la receptora
        rta = tokens.procesar(token, clave, hash, emisora, '23423asdas')
        self.assertFalse(rta['Estado'])

        # Procesamos bien
        rta = tokens.procesar(token, clave, hash, emisora, receptora)
        self.assertTrue(rta['Estado'])
        self.assertEqual(rta['Respuesta']['id'], datos['id'])

        # Generamos correctamente
        rta = tokens.generar(datos, clave, hash, emisora, receptora, ip='192.168.1.1')
        self.assertTrue(rta['Estado'])

        # Obtenemos el token
        token = rta['Respuesta']

        # Procesamos con error por que llego desde otra ip
        rta = tokens.procesar(token, clave, hash, emisora, receptora, ip='192.168.1.0')
        self.assertFalse(rta['Estado'])

        # Procesamos bien
        rta = tokens.procesar(token, clave, hash, emisora, receptora, ip='192.168.1.1')
        self.assertTrue(rta['Estado'])
        self.assertEqual(rta['Respuesta']['id'], datos['id'])
        self.assertEqual(rta['Respuesta']['ip'], '192.168.1.1')

        fecha_hora_ayer = datetime.now() - timedelta(days=1)

        # Generamos correctamente un token para ayer. Asi que va a fallar
        rta = tokens.generar(datos, clave, hash, emisora, receptora, ip='192.168.1.1', fecha_hora_inicio=fecha_hora_ayer)
        self.assertTrue(rta['Estado'])

        # Obtenemos el token
        token = rta['Respuesta']

        # Procesamos bien
        rta = tokens.procesar(token, clave, hash, emisora, receptora, ip='192.168.1.1')
        self.assertFalse(rta['Estado'])

        # Generamos correctamente un token para ayer. pero con un dia de vida
        rta = tokens.generar(datos, clave, hash, emisora, receptora, ip='192.168.1.1',
            fecha_hora_inicio=fecha_hora_ayer, tiempo_expiracion=1440
        )
        self.assertTrue(rta['Estado'])

        # Obtenemos el token
        token = rta['Respuesta']

        # Procesamos bien
        rta = tokens.procesar(token, clave, hash, emisora, receptora, ip='192.168.1.1')
        self.assertTrue(rta['Estado'])

        # Generamos correctamente un token que dure menos de un segundo
        rta = tokens.generar(datos, clave, hash, emisora, receptora, ip='192.168.1.1',
            tiempo_expiracion=.0003
        )
        self.assertTrue(rta['Estado'])

        # Obtenemos el token
        token = rta['Respuesta']

        # NOs dormimos
        time.sleep(1)

        # Procesamos con error, porque ya expiro el token
        rta = tokens.procesar(token, clave, hash, emisora, receptora, ip='192.168.1.1')
        self.assertFalse(rta['Estado'])

        # Generamos correctamente un token con una IP
        rta = tokens.generar(datos, clave, hash, emisora, receptora, ip='192.168.1.1')
        self.assertTrue(rta['Estado'])

        token = rta['Respuesta']

        # Obtenemos el dato original
        data_orig = token.split('.')[1]

        # Modificamos el token, para meterle nuestra ip
        data = jws.get_unverified_claims(token)
        data = json.loads(data.decode('utf-8'))

        # LE metemos otra ip
        data['ip'] = '1.1.1.1'

        # Encodeamos todo
        json_data = json.dumps(
            data,
            separators=(',', ':'),
        ).encode('utf-8')
        data = base64url_encode(json_data)

        # Validamos que son diferentes
        self.assertNotEqual(data_orig, data)
        # Armammos el nuevo token
        aux = token.split('.')
        # Creamos el nuevo token
        new_token = '%s.%s.%s' % (aux[0], data, aux[2])

        # Validamos que son diferentes
        self.assertNotEqual(new_token, token)

        # Validmos que el data este bien armado
        data = jws.get_unverified_claims(new_token)

        # ARREGLAR: Este test no funciona con Python 3
        # data_decode = data.decode('utf-8')
        # print('+++ data_decode')
        # print(data_decode)
        # print(type(data_decode))
        #
        # data = json.loads(data_decode)
        # print('+++ data')
        # print(data)
        # print(type(data))
        # self.assertEqual(data['ip'], '1.1.1.1')
        #
        # # Cuando lo procesamos fallamos, por que no es valido
        # rta = tokens.procesar(new_token, clave, hash, emisora, receptora, ip='1.1.1.1')
        # self.assertFalse(rta['Estado'])


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestTokens))
    return suite
