# -*- coding: utf-8 -*-
from at.utiles import _
from at.utiles import validar

from jose import jwt, jws
import time
import json

# Tiempo maximo por defecto, 30 min.
_TIEMPO_EXPIRACION = 30
_ALGORITMO = 'HS512'


def __parse(estado, respuesta, exception=None):
    '''
    :param estado:
    :param respuesta:
    :return:
    '''
    rta = {
        'Estado': estado,
        'Respuesta': respuesta
    }
    if exception is not None:
        rta['Exception'] = exception
    return rta


def generar(datos, clave, hash, aplicacion_emisora, aplicacion_receptora, tiempo_expiracion=None,
            fecha_hora_inicio=None, ip=None):
    '''
    Funcion para generar un token de datos a partir de informacion recibida
    :param clave: La clave con la cual firmar el token
    :param id: El id unico del token
    :param datos: Los datos a meter en el token
    :param aplicacion_emisora: La aplicacion emisora
    :param aplicacion_receptora: La aplicacion receptora
    :param fecha_inicio: La fecha en la que inicia a ser valido
    :param tiempo_expiracion: El timpo en minutos que durara el token
    :param ip:
    :return:
    '''

    # Validamos
    rta = validar.diccionario(datos)
    if not rta['Estado']:
        return rta

    rta = validar.string(clave)
    if not rta['Estado']:
        return rta

    rta = validar.string(hash)
    if not rta['Estado']:
        return rta

    rta = validar.string(aplicacion_emisora)
    if not rta['Estado']:
        return rta

    rta = validar.string(aplicacion_receptora)
    if not rta['Estado']:
        return rta

    # Validamos datos opcionales
    if tiempo_expiracion is not None:
        rta = validar.numero_positivo(tiempo_expiracion)
        if not rta['Estado']:
            return rta
    else:
        # Sino, lo metemos por defecto
        tiempo_expiracion = _TIEMPO_EXPIRACION

    if fecha_hora_inicio is not None:
        rta = validar.fecha_hora(fecha_hora_inicio)
        if not rta['Estado']:
            return rta

        # Pasamos a segundos
        fecha_hora_inicio = int(time.mktime(fecha_hora_inicio.timetuple()))
    else:
        fecha_hora_inicio = int(time.time())

    if ip is not None:
        rta = validar.string(ip)
        if not rta['Estado']:
            return rta

    # Vamos a procesar los claims
    # Issuer: el que emite el token
    datos['iss'] = aplicacion_emisora
    # Audience: para quien va el token
    datos['aud'] = aplicacion_receptora
    # Not Before: El dato desde cuando empieza a ser valido
    datos['nbf'] = fecha_hora_inicio
    # Expiration: El tiempo en el que va a dejar de ser valido
    datos['exp'] = fecha_hora_inicio + tiempo_expiracion * 60
    # Issued At : El tiempo en el que fue creado
    datos['iat'] = int(time.time())

    # Vemos si metemos la ip
    if ip is not None:
        datos['ip'] = ip

    # Encodeamos
    try:
        return __parse(True, jwt.encode(datos, clave, algorithm=_ALGORITMO, access_token=hash))
    except Exception as e:
        return __parse(False, _(u'Fallo al generar el token.'), str(e))


def procesar(token, clave, hash, aplicacion_emisora, aplicacion_receptora, ip=None, limpiar=True):
    '''
    Funcion para generar un token de datos a partir de informacion recibida
    :param clave: La clave con la cual firmar el token
    :param id: El id unico del token
    :param datos: Los datos a meter en el token
    :param aplicacion_emisora: La aplicacion emisora
    :param aplicacion_receptora: La aplicacion receptora
    :param ip:
    :param limpiar: Si debemos limpiar los datos o no
    :return:
    '''

    # Validamos
    rta = validar.string(token)
    if not rta['Estado']:
        return rta

    rta = validar.string(clave)
    if not rta['Estado']:
        return rta

    rta = validar.string(hash)
    if not rta['Estado']:
        return rta

    rta = validar.string(aplicacion_emisora)
    if not rta['Estado']:
        return rta

    rta = validar.string(aplicacion_receptora)
    if not rta['Estado']:
        return rta

    rta = validar.booleano(limpiar)
    if not rta['Estado']:
        return rta

    try:
        jws.verify(token, clave, algorithms=_ALGORITMO)
    except Exception as e:
        return __parse(False, _('El token no es válido.'), str(e))

    try:
        datos = jwt.decode(token, clave, algorithms=_ALGORITMO, audience=aplicacion_receptora,
            issuer=aplicacion_emisora, access_token=hash)

        # Validamos la ip, si la hay
        if ip is not None:
            if datos.get('ip') is not None:
                if datos['ip'] != ip:
                    return __parse(False, _(u'La ip no coincide con la incluida en el token.'))

        # Si hay que limpiar
        if limpiar:
            try:
                del datos['iss']
            except:
                pass

            try:
                del datos['aud']
            except:
                pass

            try:
                del datos['nbf']
            except:
                pass

            try:
                del datos['exp']
            except:
                pass

            try:
                del datos['iat']
            except:
                pass

            try:
                del datos['at_hash']
            except:
                pass
        # Salimos
        return __parse(True, datos)

    except Exception as e:
        return __parse(False, _('El token no se pudo procesar. Verificar la excepción.'), str(e))


def obtener_datos(token):
    '''
    Funcion para obtener DE FORMA INSEGURA y sin veirifcar, los
    datos de un token. Esto solo se debe usar en caso de
    querer ver que tiene un token. No es seguro. Para eso usar
    la funcion procesar()
    :param token: El token del cual queremos sacar los datos
    :return:
    '''

    # Validamos
    rta = validar.string(token)
    if not rta['Estado']:
        return rta

    #####################################################
    ##### ATENCION: OBTENEMOS SIN VALIDAR.
    ##### DATOS NO SEGUROS!!!!!!!!!!!!!!!!!!!!!
    ######################################################
    try:
        data = jwt.get_unverified_claims(token)
    except Exception as e:
        return __parse(False, _('El token no es válido.'), str(e))

    # Sino, salimos con el data
    return __parse(True, data)
