# -*- coding: utf-8 -*-

from sqlalchemy.sql.expression import and_

from at.utiles import _
from at.utiles.servidores import DateTimeHelper
from at.orm.decorators import DBSwitch


@DBSwitch('cp')
class TraficoDeDatos(object):
    '''
        Esta clase contiene funciones para interactuar con la tabla
        trafico_de_datos de cp2000.
    '''

    def __init__(self):
        self.dth = DateTimeHelper()

    def guardar_trafico_por_dia(self, aplicacion, kb_recibidos, kb_enviados):
        '''
            Esta función debe crear un registro en la tabla trafico_de_datos
            según el día actual, si ya existe este registro debe actualizar
            los valores de los campos kb_recibidos y kb_enviados.

            Argumentos:
                aplicacion (str)
                kb_recibidos (float)
                kb_enviados (float)

            Funciones:
                get_now

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str)
        '''

        dic = {}

        fecha_hora_actual = self.dth.get_now()
        fecha_hora_desde = fecha_hora_actual.strftime("%Y-%m-%d 00:00:00")
        fecha_hora_hasta = fecha_hora_actual.strftime("%Y-%m-%d 23:59:59")

        session = self.cp.binded_session()

        trafico_de_datos = self.cp.tabla('trafico_de_datos')
        q_obj = session.query(trafico_de_datos.kb_recibidos,
                            trafico_de_datos.kb_enviados)
        q_obj = q_obj.filter(
                    and_(
                        trafico_de_datos.aplicacion == aplicacion,
                        trafico_de_datos.fecha_hora_desde == fecha_hora_desde,
                        trafico_de_datos.fecha_hora_hasta == fecha_hora_hasta,
                    )
                )
        registro = q_obj.first()

        if registro is None:
            d = {}
            d['fecha_hora_desde'] = fecha_hora_desde
            d['fecha_hora_hasta'] = fecha_hora_hasta
            d['kb_recibidos'] = kb_recibidos
            d['kb_enviados'] = kb_enviados
            d['aplicacion'] = aplicacion
            agregar = self.cp.agregar('trafico_de_datos', **d)

            if agregar is not True:
                dic['Estado'] = False
                dic['Respuesta'] = _(u'No se pudo agregar un nuevo registro en la tabla trafico_de_datos')
                session.close()
                return dic

            dic['Estado'] = True
            dic['Respuesta'] = _(u'Se agrego un nuevo registro en la tabla trafico_de_datos')
            session.close()
            return dic

        u = {}
        u['kb_recibidos'] = registro.kb_recibidos + kb_recibidos
        u['kb_enviados'] = registro.kb_enviados + kb_enviados
        q_obj.update(u)
        session.commit()

        dic['Estado'] = True
        dic['Respuesta'] = _(u'Se ha actualizado un registro en la tabla trafico_de_datos')
        session.close()
        return dic

    def obtener_trafico(self, aplicacion, orden='asc', limite=None):
        '''
            Esta función se encarga de devolver todos los registros de la
            tabla trafico_de_datos filtrando por la aplicación dada.

            Argumento:
                aplicacion(str)
                orden (str)
                limite (None o int)

            Retorna:
                dict
                    ['Estado'] (bool)
                    ['Respuesta'] (str o list de dict)
                                                ['aplicacion'] (str)
                                                ['fecha_hora_desde'] (str)
                                                ['fecha_hora_hasta'] (str)
                                                ['kb_recibidos'] (float)
                                                ['kb_enviados'] (float)
                                                ['dia'] (str)
        '''

        dic = {}

        session = self.cp.binded_session()

        trafico_de_datos = self.cp.tabla('trafico_de_datos')
        q_obj = session.query(trafico_de_datos.aplicacion,
                            trafico_de_datos.fecha_hora_desde,
                            trafico_de_datos.fecha_hora_hasta,
                            trafico_de_datos.kb_recibidos,
                            trafico_de_datos.kb_enviados)

        q_obj = q_obj.filter(trafico_de_datos.aplicacion == aplicacion)

        if orden == 'asc':
            q_obj = q_obj.order_by(trafico_de_datos.fecha_hora_desde.asc())
        else:
            q_obj = q_obj.order_by(trafico_de_datos.fecha_hora_desde.desc())

        if limite is not None:
            q_obj = q_obj.limit(limite)

        if q_obj.count() == 0:
            dic['Estado'] = False
            dic['Respuesta'] = _(u'No se encontraron registros en la tabla trafico_de_datos')
            session.close()
            return dic

        registros = q_obj.all()

        filas = []
        for registro in registros:
            d = {}
            d['aplicacion'] = registro.aplicacion
            d['fecha_hora_desde'] = registro.fecha_hora_desde.strftime("%d/%m/%Y %H:%M:%S")
            d['fecha_hora_hasta'] = registro.fecha_hora_hasta.strftime("%d/%m/%Y %H:%M:%S")
            d['kb_recibidos'] = registro.kb_recibidos
            d['kb_enviados'] = registro.kb_enviados
            d['dia'] = registro.fecha_hora_desde.strftime("%d/%m/%Y")
            filas.append(d)

        dic['Estado'] = True
        dic['Respuesta'] = filas
        session.close()
        return dic
